/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
  See OTBCopyright.txt for details.


     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbApplicationCLIParser_h
#define __otbApplicationCLIParser_h

#include <iostream>
#include <vector>
#include <string>
#include <map>
#include <cstdlib>

#include "itkProcessObject.h"
#include "otbMacro.h"

#include "otbApplicationDescriptor.h"
#include "otbApplicationOptions.h"

namespace otb
{

/**
 * \class ApplicationCLIParser
 */
class ITK_EXPORT ApplicationCLIParser : public otb::ApplicationDescriptor
{
public:
  typedef ApplicationCLIParser  Self;
  typedef itk::LightObject              Superclass;
  typedef itk::SmartPointer<Self>       Pointer;
  typedef itk::SmartPointer<const Self> ConstPointer;

  itkNewMacro(Self);
  itkTypeMacro(ApplicationCLIParser, itk::LightObject);

  bool
  ParseCommandLine(int argc, char *argv[],
                   bool reportFailedMsg,
                   bool failOnUnknownTrailingParameters,
                   ApplicationOptionsResult::Pointer result);

  void PrintUsage(std::ostream& os) const;
  void PrintVersion(std::ostream& os) const;

protected:
  ApplicationCLIParser();
  virtual ~ApplicationCLIParser() {}

private:
  bool FindOption(const std::string&, int& index);
};

}

#endif // __otbApplicationCLIParser_h_
