/*=========================================================================

 Program:   ORFEO Toolbox
 Language:  C++
 Date:      $Date$
 Version:   $Revision$


 Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
 See OTBCopyright.txt for details.

 This code comes from a David Dubois contribution.

 This software is distributed WITHOUT ANY WARRANTY; without even
 the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 PURPOSE.  See the above copyright notices for more information.

 =========================================================================*/
#ifndef __otbApplicationFLTKGui_h
#define __otbApplicationFLTKGui_h

#include <iostream>
#include <vector>
#include <string>
#include <map>
#include <cstdlib>

#include "itkProcessObject.h"
#include "otbMacro.h"

#include "otbApplicationDescriptor.h"
#include "otbApplicationOptions.h"

// *DD* Needed include files for ParseGUI *DD*
#include <FL/Fl_Return_Button.H>
#include <FL/Fl_File_Chooser.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Check_Button.H>
#include <FL/Fl_Input.H>
#include <FL/Fl.H>

namespace otb
{

/**
 * \class ApplicationFLTKGui
 */
class ITK_EXPORT ApplicationFLTKGui: public otb::ApplicationDescriptor
{
public:
  typedef ApplicationFLTKGui Self;
  typedef itk::LightObject Superclass;
  typedef itk::SmartPointer<Self> Pointer;
  typedef itk::SmartPointer<const Self> ConstPointer;

  itkNewMacro(Self)
;
  itkTypeMacro(ApplicationFLTKGui, itk::LightObject)
;

  bool
  ParseGui(bool reportFailedMsg, bool failOnUnknownTrailingParameters, ApplicationOptionsResult::Pointer result);

protected:
  ApplicationFLTKGui();
  virtual ~ApplicationFLTKGui()
  {
  }

  // *DD* Callbacks needed for the FLTK buttons for ParseGUI *DD*
  void cb_BrowseButton_i(Fl_Button*, void* value)
  {
    BrowseButton((long) value);
  }
  static void cb_BrowseButton(Fl_Button* o, void* v)
  {
    ((ApplicationFLTKGui *) (o->parent()->user_data()))->cb_BrowseButton_i(o, v);
  }
  void cb_OkButton_i(Fl_Return_Button*, void* value)
  {
    OkButton();
  }
  static void cb_OkButton(Fl_Return_Button* o, void* v)
  {
    ((ApplicationFLTKGui *) (o->parent()->user_data()))->cb_OkButton_i(o, v);
  }

private:

  bool FindOption(const std::string&, int& index);

  bool TryParseGUI(ApplicationOptionsResult * outResult, bool reportFailedMsg, bool failOnUnknownTrailingParameters);

  // Methods for the two types of buttons DD
  void BrowseButton(long currentOption);
  void OkButton();

  // Method to parse a string
  std::vector<std::string>  Tokenize(std::string currentParameterLine);

  // *DD* Lists of FL objects to keep track and delete *DD*
  typedef std::vector<Fl_Input *>  ListInputType;
  typedef std::vector<Fl_Button *> ListButtonType;
  typedef std::vector<Fl_Check_Button *> ListCheckButtonType;
  
  ListInputType m_InputList;
  ListButtonType m_ButtonList;
  ListCheckButtonType m_CheckButtonList;
  
  // *DD* Main FL window *DD*
  Fl_Window *inputwindow;
  // *DD* Simple toggle to know when user pressed the OK button *DD*
  bool m_StartParsing;

};

}

#endif // __otbApplicationFLTKGui_h_
