/*=========================================================================

 Program:   ORFEO Toolbox
 Language:  C++
 Date:      $Date$
 Version:   $Revision$


 Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
 See OTBCopyright.txt for details.


 This software is distributed WITHOUT ANY WARRANTY; without even
 the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 PURPOSE.  See the above copyright notices for more information.

 =========================================================================*/
#include "otbApplicationDescriptor.h"
#include "itkMacro.h"

#include <assert.h>
#include <iostream>
#include "otbConfigure.h" // For OTB_VERSION_STRING generated by ccmake
namespace otb
{

ApplicationDescriptor::ApplicationDescriptor()
{
  AddOption("help", "Help", "h", 0, false, Self::Help);
  AddOption("version", "Version", "v", 0, false, Self::Version);
  //AddOptionNParams("--OTBTesting", "Testing purposes only.", "-OTBTesting", false);
}

void ApplicationDescriptor::AddInputImage(bool Mandatory)
{
  AddOption("InputImage", "Input image file name ", "in", 1, Mandatory, Self::InputImage);
}

void ApplicationDescriptor::AddOutputImage(bool Mandatory)
{
  AddOption("OutputImage", "Output image file name ", "out", 1, Mandatory, Self::OutputImage);
}

void ApplicationDescriptor::AddOption(std::string name, std::string comment, std::string Synonym,
                                             int nParameters, bool Mandatory, ApplicationOptionType optionType)
{
  // Create a structure for the command
  OptionType option;
  option.CommonName = name;
  option.Description = comment;
  option.Synonym = Synonym;
  option.NumberOfParameters = nParameters;
  option.NumberOfParametersFixed = true;
  option.Mandatory = Mandatory;
  option.Found = false;
  option.Type = optionType;

  // Add the option to the map
  m_OptionList.push_back(option);

}

void ApplicationDescriptor::AddOptionNParams(std::string name, std::string comment, std::string Synonym,
                                                    bool Mandatory, ApplicationOptionType optionType)
{
  // Create a structure for the command
  OptionType option;
  option.CommonName = name;
  option.Description = comment;
  option.Synonym = Synonym;
  option.NumberOfParameters = -1;
  option.NumberOfParametersFixed = false;
  option.Mandatory = Mandatory;
  option.Found = false;
  option.Type = optionType;

  // Add the option to the map
  m_OptionList.push_back(option);

}

void ApplicationDescriptor::PrintUsage(std::ostream& os) const
{
  os << std::endl;

  if (!m_Description.empty())
    {
    os << m_Description << std::endl << std::endl;
    }

  os << " Usage : " << m_Name << std::endl;
  // Compute the max width for option display
  int largeurmax(-1);
  unsigned int i;

  for (i = 0; i < m_OptionList.size(); ++i)
    {
    int largeur = m_OptionList[i].CommonName.size() + m_OptionList[i].Synonym.size();
    if (largeur > largeurmax) largeurmax = largeur;
    }

  // Check that all required arguments are present on the command line
  for (i = 0; i < m_OptionList.size(); ++i)
    {
    int largeur = m_OptionList[i].CommonName.size() + m_OptionList[i].Synonym.size();
    os << "      ";
    if (m_OptionList[i].Mandatory == false)
      os << "[";
    else os << " ";
    os << m_OptionList[i].CommonName;
    if (m_OptionList[i].Synonym.empty() == false)
      {
      os << "|" << m_OptionList[i].Synonym;
      }
    if (m_OptionList[i].Mandatory == false)
      os << "]";
    else os << " ";
    //Align the text
    for (int b = largeur; b < largeurmax; b++)
      os << " ";
    os << "  :  " << m_OptionList[i].Description;
    if (m_OptionList[i].NumberOfParametersFixed == true)
      {
      switch (m_OptionList[i].NumberOfParameters)
        {
        case 0:
          break;
        case 1:
          os << "  (" << m_OptionList[i].NumberOfParameters << " parameter)";
          break;
        default:
          os << "  (" << m_OptionList[i].NumberOfParameters << " parameters)";
          break;
        }
      }
    else
      {
      os << "  (N parameters)";
      }
    os << std::endl;
    }
  os << std::endl;
}

void ApplicationDescriptor::PrintVersion(std::ostream& os) const
{
  os << " OTB Version : " << OTB_VERSION_STRING << std::endl;
}

}
