/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
  See OTBCopyright.txt for details.


     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#include "otbApplicationOptions.h"
#include "itkMacro.h"

#include <assert.h>
#include <iostream>
#include "otbConfigure.h" // For OTB_VERSION_STRING generated by ccmake

namespace otb
{

ApplicationOptionsResult* ApplicationOptionsResult
::Clone() const
{
  ApplicationOptionsResult::Pointer result = new ApplicationOptionsResult;
  result->m_OptionMap = this->m_OptionMap;
  return result;
}

void ApplicationOptionsResult
::PrintSelf(std::ostream& /*os*/ /*, itk::Indent indent*/) const
{

}

bool ApplicationOptionsResult
::IsOptionPresent(std::string option) const
{
  return m_OptionMap.count(option) > 0;
}

bool ApplicationOptionsResult
::IsOptionInputImagePresent(void) const
{
  return IsOptionPresent("InputImage");
}
bool ApplicationOptionsResult
::IsOptionOutputImagePresent(void) const
{
  return IsOptionPresent("OutputImage");
}

bool ApplicationOptionsResult
::IsOptionOTBTestingPresent(void) const
{
  return IsOptionPresent("OTBTesting");
}

std::string ApplicationOptionsResult
::GetParameterString(std::string option, unsigned int number) const
{
  if (this->IsOptionPresent(option) == false)
    {
    itk::OStringStream msg;
    msg << "GetParameterString(): The following '" << option << "' option is unknown !!";
    itk::ExceptionObject e(__FILE__, __LINE__);
    e.SetDescription(msg.str().c_str());
    throw e;
    }
  OptionMapType::const_iterator it = m_OptionMap.begin();
  it = m_OptionMap.find(option);
  ParameterArrayType pat = (*it).second;
  std::string        lString = pat[number];
  return lString;
}

std::string ApplicationOptionsResult
::GetInputImage(void) const
{
  return GetParameterString("InputImage");
}

std::string ApplicationOptionsResult
::GetOutputImage(void) const
{
  return GetParameterString("OutputImage");
}

int ApplicationOptionsResult
::GetNumberOfParameters(std::string option)
{
  assert(IsOptionPresent(option));
  return m_OptionMap[option].size();
}

void ApplicationOptionsResult
::Clear()
{
  m_OptionMap.clear();
}

void ApplicationOptionsResult
::AddOption(const std::string& option)
{
  if (m_OptionMap.count(option) == 0)
    {
    m_OptionMap[option] = ParameterArrayType();
    }
}

void ApplicationOptionsResult
::RemoveOption(const std::string& option)
{
  if (m_OptionMap.count(option) != 0)
    {
    m_OptionMap.erase(option);
    }
}

void ApplicationOptionsResult
::AddParameter(const std::string& option, const std::string& parameter)
{
  m_OptionMap[option].push_back(parameter);
}

void ApplicationOptionsResult
::SetParameter(const std::string& option, const std::string& parameter, unsigned int parameterIndex)
{
  if (parameterIndex >= m_OptionMap[option].size())
    m_OptionMap[option].resize(parameterIndex + 1);

  m_OptionMap[option][parameterIndex] = parameter;
}
}
