/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
  See OTBCopyright.txt for details.


     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbApplicationOptions_h
#define __otbApplicationOptions_h

#include <iostream>
#include <vector>
#include <string>
#include <map>
#include <cstdlib>

#include "itkProcessObject.h"
#include "otbMacro.h"

namespace otb
{

/**
 * \class ApplicationOptionsResult
 * \brief Object returned by ApplicationOptions
 *
 * \see ApplicationOptions
 */
class ITK_EXPORT ApplicationOptionsResult : public itk::LightObject
{
public:
  typedef ApplicationOptionsResult Self;
  typedef itk::LightObject               Superclass;
  typedef itk::SmartPointer<Self>        Pointer;
  typedef itk::SmartPointer<const Self>  ConstPointer;

  itkNewMacro(Self);
  itkTypeMacro(ApplicationOptionsResult, itk::ProcessObject);

  ApplicationOptionsResult* Clone() const;

  /** Check whether the option was passed in or not */
  bool IsOptionPresent(std::string option) const;

  /** Check whether the input image option was passed in or not */
  bool IsOptionInputImagePresent(void) const;

  /** Check whether the output image option was passed in or not */
  bool IsOptionOutputImagePresent(void) const;

  /** Check whether the OTBTesting option was passed in or not */
  bool IsOptionOTBTestingPresent(void) const;

  /** Get one of the parameters to the option */
//  const char *GetOptionParameter(const char *option, unsigned int number = 0);
  int GetNumberOfParameters(std::string option);

  void PrintSelf(std::ostream& os /*, itk::Indent indent*/) const;

#define otbApplicationOptionsResultGetParameterMacro(name, type)                                                       \
  virtual type GetParameter ## name (std::string option, unsigned int number = 0) const                                \
                                       {                                                                               \
                                       std::string parameter = this->GetParameterString(option, number);               \
                                       type        lValeur;                                                            \
                                       std::stringstream flux;                                                         \
                                       flux << parameter;                                                              \
                                       flux >> lValeur;                                                                \
                                       return lValeur;                                                                 \
                                       }

  otbApplicationOptionsResultGetParameterMacro(Char, char);
  otbApplicationOptionsResultGetParameterMacro(Short, short);
  otbApplicationOptionsResultGetParameterMacro(UShort, unsigned short);
  otbApplicationOptionsResultGetParameterMacro(Int, int);
  otbApplicationOptionsResultGetParameterMacro(UInt, unsigned int);
  otbApplicationOptionsResultGetParameterMacro(Long, long);
  otbApplicationOptionsResultGetParameterMacro(ULong, unsigned long);
  otbApplicationOptionsResultGetParameterMacro(Float, float);
  otbApplicationOptionsResultGetParameterMacro(Double, double);

  std::string GetParameterString(std::string option, unsigned int number = 0) const;

  std::string GetInputImage(void) const;
  std::string GetOutputImage(void) const;

//protected:
  ApplicationOptionsResult() {};
  virtual ~ApplicationOptionsResult() {}

//private:

  template<typename TypeValeur>
  TypeValeur GetParameter(std::string option, unsigned int number = 0) const;

  typedef std::vector<std::string>                  ParameterArrayType;
  typedef std::map<std::string, ParameterArrayType> OptionMapType;

  void Clear();
  void AddOption(const std::string & option);
  void RemoveOption(const std::string & option);
  void SetParameter(const std::string & option, const std::string & parameter, unsigned int parameterIndex);
  void AddParameter(const std::string & option, const std::string & parameter);

  OptionMapType m_OptionMap;

};


}

#endif // __otbApplicationOptions_h_
