/*=========================================================================

 Program:   ORFEO Toolbox
 Language:  C++
 Date:      $Date$
 Version:   $Revision$


 Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
 See OTBCopyright.txt for details.


 This software is distributed WITHOUT ANY WARRANTY; without even
 the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 PURPOSE.  See the above copyright notices for more information.

 =========================================================================*/
#include "otbApplicationExecutionThread.h"

namespace otb
{

ApplicationExecutionThread::ApplicationExecutionThread(QObject* parent)
: QThread(parent)
{
}

ApplicationExecutionThread::~ApplicationExecutionThread()
{
  wait();
}

void ApplicationExecutionThread::Execute(otb::Application* appli, otb::ApplicationOptionsResult* options)
{
  m_Appli = appli;

  // clone the options object to avoid race condition
  m_Options = options->Clone();

  start();
}

void ApplicationExecutionThread::run()
{
  try
  {
    m_Appli->Execute(m_Options);
  }
  catch( std::exception& std_ex )
  {
    const QString& msg = QString(std_ex.what());
    emit ReportError( msg );
  }
  catch( ... )
  {
    const QString& msg = QString("An unknown error occurred during processing");
    emit ReportError( msg );
  }
}

}
