/*=========================================================================

 Program:   ORFEO Toolbox
 Language:  C++
 Date:      $Date$
 Version:   $Revision$


 Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
 See OTBCopyright.txt for details.


 This software is distributed WITHOUT ANY WARRANTY; without even
 the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 PURPOSE.  See the above copyright notices for more information.

 =========================================================================*/
#ifndef __otbApplicationQtWidget_h
#define __otbApplicationQtWidget_h

#include <QtGui>
#include "otbMacro.h"
#include "otbApplication.h"
#include "otbProgressLabel.h"
#include "otbApplicationExecutionThread.h"

namespace otb
{

/**
 * \class ApplicationQtWidget
 */
class ITK_EXPORT ApplicationQtWidget: public QWidget
{
  Q_OBJECT

public:
  ApplicationQtWidget(otb::Application* appli);
  ~ApplicationQtWidget();

  QString GetName();
  QString GetDescription();
  void CreateGui();

protected slots:
  virtual bool Execute();

  virtual void StartExecute();
  virtual void AbortExecute();
  virtual void EndExecute();

  void SetActiveOption( const QString& option );
  void SetActiveOptionValue( const QString& value );
  void SetActiveOptionBoolValue( int );
  void SetActiveEnumOptionValue( const QString& value );
  void SelectFile(QWidget* inputWidget);
  void SelectDir(QWidget* inputWidget);

  void CatchError(const QString& msg);

protected:
  typedef otb::ApplicationDescriptor::ListOptionType ListOptionType;
  typedef ListOptionType::value_type OptionType;

  virtual QWidget* CreateHeader();
  virtual QWidget* CreateInputWidgets();
  virtual QWidget* CreateFooter();

  virtual QWidget* CreateInputWidgetLabel (const OptionType& option);
  virtual QWidget* CreateInputWidgetParam (const OptionType& option);

  virtual QWidget* CreateInputImageWidget (const OptionType& option);
  virtual QWidget* CreateOutputImageWidget(const OptionType& option);
  virtual QWidget* CreateFileNameWidget   (const OptionType& option);
  virtual QWidget* CreateDirNameWidget    (const OptionType& option);
  virtual QWidget* CreateStringWidget     (const OptionType& option);
  virtual QWidget* CreateRealWidget       (const OptionType& option);
  virtual QWidget* CreateIntWidget        (const OptionType& option);
  virtual QWidget* CreateBoolWidget       (const OptionType& option);
  virtual QWidget* CreateStringListWidget (const OptionType& option);

  otb::Application::Pointer              m_Application;
  otb::ApplicationDescriptor::Pointer    m_Descriptor;
  otb::ApplicationOptionsResult::Pointer m_Options;
  std::string                            m_ActiveOption;

  QSignalMapper *                        m_TextChangedMapper;
  QSignalMapper *                        m_CheckboxChangedMapper;
  QSignalMapper *                        m_SelectFileMapper;
  QSignalMapper *                        m_SelectDirMapper;

  otb::ProgressLabel *                   m_ProgressLabel;
  QPushButton *                          m_ExecButton;

  ApplicationExecutionThread*            m_ExecutionThread;
  bool m_ErrorCaught;
};

}

#endif // __otbApplicationQtWidget_h_
