
// ./otbActiveLearning ~/OTB/trunk/OTB-Data/Examples/qb_RoadExtract.tif output.tif

#include "otbVectorImage.h"
#include "otbImageFileReader.h"
#include "otbImageFileWriter.h"
#include "otbImage.h"
#include "otbSVMSampleListModelEstimator.h"
#include "otbSVMClassifier.h"
#include "itkListSample.h"
#include "itkFixedArray.h"

#include "itkImageRandomIteratorWithIndex.h"
#include "otbSVMMarginSampler.h"
#include "otbSVMImageClassificationFilter.h"
#include "otbVectorRescaleIntensityImageFilter.h"
#include "otbStandardImageViewer.h"

#include "itkMultiThreader.h"

typedef float PixelType;
typedef otb::VectorImage<PixelType, 2> ImageType;
typedef otb::ImageFileReader<ImageType> ReaderType;

typedef unsigned int LabeledPixelType;
typedef otb::Image<LabeledPixelType, 2> LabelImageType;

typedef itk::VariableLengthVector<PixelType>                                SampleType;
typedef itk::Statistics::ListSample<SampleType>                             ListSampleType;
typedef otb::SVMClassifier<ListSampleType, LabeledPixelType>                     ClassifierType;
typedef itk::FixedArray<LabeledPixelType, 1>                                TrainingSampleType;
typedef itk::Statistics::ListSample<TrainingSampleType>                     TrainingListSampleType;
typedef otb::SVMSampleListModelEstimator<ListSampleType, TrainingListSampleType> ModelEstimatorType;
typedef ModelEstimatorType::Pointer ModelEstimatorPointerType;
typedef ModelEstimatorType::ModelType SVMModelType;

typedef otb::SVMMarginSampler<ListSampleType, SVMModelType>                  SVMMarginSamplerType;
typedef itk::ImageRandomIteratorWithIndex<ImageType> IteratorType;

typedef ImageType::IndexType IndexType;

typedef  otb::SVMImageClassificationFilter<ImageType, LabelImageType, LabelImageType> ClassificationFilterType;

typedef otb::ImageFileWriter<LabelImageType> WriterType;

typedef otb::VectorRescaleIntensityImageFilter<ImageType, ImageType> RescaleType;

typedef otb::StandardImageViewer<ImageType> ViewerType;

static ITK_THREAD_RETURN_TYPE RunViewer_static(void * t)
{
  Fl::run();
  return 0;
}


int QueryLabel(ViewerType* viewer, const IndexType& index)
{
  viewer->SetCenter(index);
//  viewer->Update();
//  Fl::check();
  std::cout << "Note: close visualization when ready\n";
  std::cout << "Enter the label for index: " << index;
  std::cout << "\n";

  int l = 0;
  std::cin >> l;

  return l;
}

int main(int argc, char* argv[])
{
  ReaderType::Pointer reader = ReaderType::New();
  reader->SetFileName(argv[1]);
  reader->Update();

  RescaleType::Pointer rescaler = RescaleType::New();
  rescaler->SetInput(reader->GetOutput());
  rescaler->SetClampThreshold(0);

  ImageType::PixelType min(reader->GetOutput()->GetNumberOfComponentsPerPixel()) , max(reader->GetOutput()->GetNumberOfComponentsPerPixel());
  min.Fill(0);
  max.Fill(1);
  rescaler->SetOutputMinimum(min);
  rescaler->SetOutputMaximum(max);
  rescaler->Update();

  ImageType::Pointer image = rescaler->GetOutput();

  IteratorType it(image, image->GetLargestPossibleRegion());
  ListSampleType::Pointer listSample = ListSampleType::New();
  std::vector<IndexType> indexes;

  it.GoToBegin();
  unsigned int nSamples = 1000;
  for (unsigned int i = 0; i < nSamples; ++i)
    {
    listSample->PushBack(it.Get());
    indexes.push_back(it.GetIndex());
    ++it;
    }

  //printing
  for (unsigned int i = 0; i < nSamples; ++i)
    {
    std::cout << indexes[i] << ": " << listSample->GetMeasurementVector(i) << "\n";
    }


  ListSampleType::Pointer trainingSamples = ListSampleType::New();
  TrainingListSampleType::Pointer trainingLabels = TrainingListSampleType::New();

  unsigned int nbInitialSamples = 0;
  unsigned int i = 0;

  ViewerType::Pointer viewer = ViewerType::New();
  viewer->SetImage(image);
  viewer->Update();
  typedef itk::MultiThreader ThreaderType;
  ThreaderType::Pointer threader = ThreaderType::New();
  threader->SpawnThread(RunViewer_static, NULL);
 
  while(nbInitialSamples < 10)
    {
    IndexType index = indexes[i];
    int label = QueryLabel(viewer, index);
    if (label >= 0)
      {
      trainingSamples->PushBack(listSample->GetMeasurementVector(i));
      TrainingSampleType labelVector;
      labelVector[0]=label;
      trainingLabels->PushBack(labelVector);
      ++nbInitialSamples;
      }
//otherwise, the user just don't know what it is
    ++i;
    }

  SVMMarginSamplerType::Pointer marginSampler = SVMMarginSamplerType::New();
   unsigned int nMargingSamplesPerIteration = 2;
 
 marginSampler->SetNumberOfCandidates(nMargingSamplesPerIteration);
 marginSampler->SetSample(listSample);
 
  for(int k = 0; k<4; ++k)
    {
    ModelEstimatorPointerType estimator = ModelEstimatorType::New();
    estimator->SetInputSampleList(trainingSamples);
    estimator->SetTrainingSampleList(trainingLabels);

    estimator->Update();

    marginSampler->SetModel(estimator->GetModel());
    marginSampler->Update();

    for(unsigned int j=0; j<nMargingSamplesPerIteration; ++j)
      {
      int i = marginSampler->GetMarginSamples()[j];
      IndexType index = indexes[i];
      int label = QueryLabel(viewer, index);
    if (label >= 0)
      {
      trainingSamples->PushBack(listSample->GetMeasurementVector(i));
      TrainingSampleType labelVector;
      labelVector[0]=label;
      trainingLabels->PushBack(labelVector);
      }

      }
    }

  ModelEstimatorPointerType estimator = ModelEstimatorType::New();
  estimator->SetInputSampleList(trainingSamples);
  estimator->SetTrainingSampleList(trainingLabels);

  estimator->Update();


  ClassificationFilterType::Pointer classifier = ClassificationFilterType::New();
  classifier->SetInput(rescaler->GetOutput());
  classifier->SetModel(estimator->GetModel());

  WriterType::Pointer writer = WriterType::New();
  writer->SetFileName(argv[2]);
  writer->SetInput(classifier->GetOutput());
  writer->Update();

  return EXIT_SUCCESS;
}
