/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
  See OTBCopyright.txt for details.


     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbBlackImage_h
#define __otbBlackImage_h

#include "itkImageToImageFilter.h"
#include "itkImageRegionIteratorWithIndex.h"
#include "itkConstNeighborhoodIterator.h"
#include "itkProcessObject.h"

namespace otb
{
/** \class BlackImage
 * \brief Create a black Image in streaming
 *
 */
template <class TImage >
class ITK_EXPORT BlackImage
      : public itk::ImageToImageFilter<TImage, TImage>
{
public:
  /** Standard class typedefs. */
  typedef BlackImage           Self;
  typedef itk::ImageToImageFilter<TImage, TImage>  Superclass;
  typedef itk::SmartPointer<Self>                             Pointer;
  typedef itk::SmartPointer<const Self>                       ConstPointer;

  /** Method for creation through the object factory. */
  itkNewMacro(Self);

  /** Run-time type information (and related methods). */
  itkTypeMacro(BlackImage, ImageToImageFilter);

  /** Some convenient typedefs. */
  typedef TImage ImageType;
  typedef typename ImageType::SizeType SizeType;
  typedef typename ImageType::IndexType IndexType;
  typedef typename ImageType::PixelType PixelType;
  typedef typename ImageType::InternalPixelType InternalPixelType;

  typedef typename Superclass::OutputImageRegionType OutputImageRegionType;
  typedef typename Superclass::OutputImagePointer OutputImagePointer;


  typedef itk::ProcessObject ProcessObjectType;


  OutputImageRegionType GetRegion()
  {
    return m_Region;
  };
  void SetRegion(OutputImageRegionType reg)
  {
    m_Region = reg;
  };
  void SetSize(SizeType size)
  {
    m_Region.SetSize(size);
  };
  void SetIndex(IndexType id)
  {
    m_Region.SetIndex(id);
  };
  SizeType GetSize()
  {
    return m_Region.GetSize();
  };
  IndexType GetIndex()
  {
    return m_Region.GetIndex();
  };
  //itkGetMacro(NumberOfComponentsPerPixel, int);
  //itkSetMacro(NumberOfComponentsPerPixel, int);
  InternalPixelType GetValue()
  {
    return m_Value;
  };
  void SetValue( InternalPixelType val )
  {
    m_Value = val;
  };

protected:
  /**
   * Constructor
   */
  BlackImage();
  /**
   * Destructor
   */
  virtual ~BlackImage() {};

  /** BlackImage can be implemented as a multithreaded filter.
   * Therefore, this implementation provides a ThreadedGenerateData() routine
   * which is called for each processing thread. The output image data is
   * allocated automatically by the superclass prior to calling
   * ThreadedGenerateData().  ThreadedGenerateData can only write to the
   * portion of the output image specified by the parameter
   * "outputRegionForThread"
   *
   * \sa ImageToImageFilter::ThreadedGenerateData(),
   *     ImageToImageFilter::GenerateData()  */
  virtual void ThreadedGenerateData(const OutputImageRegionType& outputRegionForThread, int threadId );

  /**
   * Pad the input requested region by radius
   */
  virtual void GenerateOutputInformation(void);

private:
  BlackImage(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented


  OutputImageRegionType m_Region;
  //int m_NumberOfComponentsPerPixel;
  InternalPixelType m_Value;
};

} // namespace otb

#ifndef OTB_MANUAL_INSTANTIATION
#include "otbBlackImage.txx"
#endif


#endif
