/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
  See OTBCopyright.txt for details.


     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbBlackImage_txx
#define __otbBlackImage_txx

#include "otbBlackImage.h"
#include "itkImageRegionIterator.h"
#include "itkConstNeighborhoodIterator.h"
#include "itkNeighborhoodAlgorithm.h"
#include "itkProgressReporter.h"
#include "itkZeroFluxNeumannBoundaryCondition.h"
#include "itkNeighborhoodAlgorithm.h"

namespace otb
{
/**
 * Constructor
 */
template <class TImage>
BlackImage<TImage>
::BlackImage()
{
  this->SetNumberOfRequiredInputs( 0 );
  SizeType size;
  size[0] = 1;
  size[1] = 1;
  IndexType index;
  index[0] = 0;
  index[1] = 0;
  m_Region.SetSize(size);
  m_Region.SetIndex(index);
  //m_NumberOfComponentsPerPixel = 1;
  m_Value = itk::NumericTraits<InternalPixelType>::Zero;
}


template <class TImage>
void
BlackImage<TImage>
::GenerateOutputInformation()
{
  Superclass::GenerateOutputInformation();

  // get pointers to the input and output
  typename Superclass::OutputImagePointer      outputPtr = this->GetOutput();


  if ( !outputPtr)
    return;

  outputPtr->SetLargestPossibleRegion( m_Region );

  //outputPtr->SetNumberOfComponentsPerPixel( m_NumberOfComponentsPerPixel );
};


/**
 * ThreadedGenerateData Performs the neighborhood-wise operation
 */
template <class TImage >
void
BlackImage<TImage>
::ThreadedGenerateData( const OutputImageRegionType &outputRegionForThread, int threadId)
{
  OutputImagePointer outputPtr = this->GetOutput(0);

  // support progress methods/callbacks
  itk::ProgressReporter progress(this, threadId, outputRegionForThread.GetNumberOfPixels());

  itk::ImageRegionIterator<ImageType> outputIt( outputPtr, outputRegionForThread );
  outputIt.GoToBegin();

  //PixelType pix;
  //pix.SetSize(m_NumberOfComponentsPerPixel);
  //pix.Fill(m_Value);

  while ( ! outputIt.IsAtEnd() )
  {
    outputIt.Set( m_Value );

    ++outputIt;
    progress.CompletedPixel();
  }
}

} // end namespace otb

#endif
