/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
  See OTBCopyright.txt for details.


     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbChangeOpacityActionHandler_h
#define __otbChangeOpacityActionHandler_h

#include "otbImageWidgetActionHandler.h"

namespace otb
{
/** \class ChangeOpacityActionHandler
*   \brief Implements basic Scroll, Full and Zoom Gl components transparency.
*
*   \sa ImageWidgetController
*   \sa ImageWidgetActionHandler
*/

template <class TModel, class TView>
class ChangeOpacityActionHandler
  : public ImageWidgetActionHandler
{
public:
  /** Standard class typedefs */
  typedef ChangeOpacityActionHandler Self;
  typedef ImageWidgetActionHandler               Superclass;
  typedef itk::SmartPointer<Self>                Pointer;
  typedef itk::SmartPointer<const Self>          ConstPointer;
  
  /** Method for creation through the object factory */
  itkNewMacro(Self);
  
  /** Runtime information */
  itkTypeMacro(ChangeOpacityActionHandler, ImageWidgetActionHandler);

  /** Model typedefs */
  typedef TModel                         ModelType;
  typedef typename ModelType::Pointer    ModelPointerType;
  typedef typename ModelType::RegionType RegionType;

  /** View typedefs */
  typedef TView                          ViewType;
  typedef typename ViewType::Pointer     ViewPointerType;


  /** Handle widget event
   * \param widgetId The id of the moved widget
   * \param event The event
   * \return The handling return code
   */
  virtual bool HandleWidgetEvent(const std::string & widgetId, int event)
    {
      if( m_View.IsNotNull() && m_Model.IsNotNull() )
       {
         if( Fl::event_ctrl() )
           {
             switch (event)
              {
              case FL_MOUSEWHEEL:
                {
                  int dy = Fl::event_dy();
                  if(dy>0)
                    {
                     m_Opacity = std::min(1., m_Opacity+0.05);
                    }
                  else if(dy<0)
                    {
                     m_Opacity = std::max(0.01, m_Opacity-0.05);
                    }
                  if(dy!=0)
                    {
                     m_Model->ChangeOpacity(m_Opacity);
                    }
                  return true;
                }
              case FL_PUSH:
                {
                  if ( Fl::event_button3() )
                    {
                     m_Model->ChangeGlRepresentation();
                     return true;
                    }
                  break;
                }
              }
           }
       }
      return false;
    }
 
       
  
  /** Set/Get the pointer to the view */
  itkSetObjectMacro(View, ViewType);
  itkGetObjectMacro(View, ViewType);

  /** Set/Get the pointer to the model */
  itkSetObjectMacro(Model, ModelType);
  itkGetObjectMacro(Model, ModelType);

  /** Set/Get transparency ratio */
  itkSetMacro(Opacity, double);
  itkGetMacro(Opacity, double);

protected:
  /** Constructor */
  ChangeOpacityActionHandler() : m_View(), m_Model(), m_Opacity(1)
  {}

  /** Destructor */
  virtual ~ChangeOpacityActionHandler(){}
  /** Printself method */
  void PrintSelf(std::ostream& os, itk::Indent indent) const
  {
    Superclass::PrintSelf(os, indent);
  }
 
private:
  ChangeOpacityActionHandler(const Self&);    // purposely not implemented
  void operator=(const Self&); // purposely not implemented

  // Pointer to the view
  ViewPointerType m_View;
  
  // Pointer to the model
  ModelPointerType m_Model;
  
  /** Scale Ratio */
  double m_Opacity;

}; // end class
} // end namespace otb
#endif


