/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
  See OTBCopyright.txt for details.


     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbFocusingController_h
#define __otbFocusingController_h

#include "otbImageWidgetController.h"
#include "otbSelectAreaActionHandler.h"
//#include "otbFocusingControllerInterface.h"

namespace otb
{
/** \class FocusingController
 *
 *
 */
template<class TFocusingModel, class TFocusingView>
class FocusingController
: public itk::Object
{
  public:
  /** Standard class typedefs */
  typedef FocusingController                       Self;
  typedef itk::Object                              Superclass;
  typedef itk::SmartPointer<Self>                  Pointer;
  typedef itk::SmartPointer<const Self>            ConstPointer;

  typedef ImageWidgetController                    ControllerType;
  typedef ControllerType::Pointer                  ControllerPointerType;

  typedef TFocusingModel                           FocusingModelType;
  typedef typename FocusingModelType::Pointer      FocusingModelPointerType;
  typedef TFocusingView                            FocusingViewType;
  typedef typename FocusingViewType::Pointer       FocusingViewPointerType;
  typedef typename FocusingViewType::ImageWidgetType ImageWidgetType;
  //typedef typename FocusingViewType::ImageViewType ImageViewType;
  //typedef typename ImageViewType::ImageWidgetType  ImageWidgetType;
    
  typedef SelectAreaActionHandler<FocusingModelType, ImageWidgetType> SelectAreaHandlerType;
  typedef typename SelectAreaHandlerType::Pointer                     SelectAreaHandlerPointerType;

  /** Standards macros */
  itkNewMacro(Self);

  /** Runtime information */
  itkTypeMacro(FocusingController, itk::Object);

  /** Set controllers */
  itkSetObjectMacro(MasterController, ControllerType);
  itkSetObjectMacro(WorkingController, ControllerType);

  /** Get controllers */
  itkGetObjectMacro(MasterController, ControllerType);
  itkGetObjectMacro(WorkingController, ControllerType);

  /** Focusing Model accessors */
  itkSetObjectMacro(FocusingModel, FocusingModelType);
  itkGetObjectMacro(FocusingModel, FocusingModelType);

  /** Focusing View accessors */
  itkSetObjectMacro(FocusingView, FocusingViewType);
  itkGetObjectMacro(FocusingView, FocusingViewType);

  virtual void AddSelectAreaHandler();

 protected:
  /** Constructor */
  FocusingController();
  /** Destructor */
  ~FocusingController(){};

 private:
  ControllerPointerType    m_MasterController;
  ControllerPointerType    m_WorkingController;
  FocusingViewPointerType  m_FocusingView;
  FocusingModelPointerType m_FocusingModel;

}; // end class
} // end namespace otb

#ifndef  OTB_MANUAL_INSTANTIATION
#include "otbFocusingController.txx"
#endif

#endif
