/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbFineRegistrationApplicationController_h
#define __otbFineRegistrationApplicationController_h

#include "otbFineRegistrationApplicationControllerInterface.h"
#include "otbFineRegistrationApplicationViewGUI.h"

namespace otb
{
class FineRegistrationApplicationController
      : public FineRegistrationApplicationControllerInterface
{
public:
  /** Standard class typedefs */
  typedef FineRegistrationApplicationController Self;
  typedef itk::Object                       Superclass;
  typedef itk::SmartPointer<Self>           Pointer;
  typedef itk::SmartPointer<const Self>     ConstPointer;

  typedef otb::FineRegistrationApplicationViewGUI                    ViewType;
  typedef ViewType::Pointer                                          ViewPointerType;
  typedef FineRegistrationApplicationControllerInterface::IndexType  IndexType;
  typedef FineRegistrationApplicationControllerInterface::RadiusType RadiusType;

  /** Standard type macros */
  itkTypeMacro(FineRegistrationApplicationController, Superclass);
  itkNewMacro(Self);

  /** Set the view */
  itkSetObjectMacro(View, ViewType);

  /** User action */
  virtual void OpenImagePair(const char * moving, const char * fixed);
  virtual void IndexClicked(const IndexType & index);
  virtual void ChangeNormalizationFactor(double factor);
  virtual void SaveDisparityMap(const char * filename);
  virtual void SaveRegisteredImage(const char * filename);
  virtual void ChangeMaxDeformationValue(double minVal, double maxVal);
  virtual void Run();
  virtual void SetNumberOfIterations(int nbIter);
  virtual void SetRadius(unsigned int rad, int id);
  void SetRadius(RadiusType rad);

protected:
  /** Constructor */
  FineRegistrationApplicationController();
  /** Destructor */
  ~FineRegistrationApplicationController();

private:
  FineRegistrationApplicationController(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

  /** Pointer to the view */
  ViewPointerType m_View;
  /** bool to inhib multi access extraction*/
  bool m_IsExtractRunning;

};
} //end namespace otb

#endif
