/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbFineRegistrationApplicationControllerInterface_h
#define __otbFineRegistrationApplicationControllerInterface_h

#include "itkObject.h"
#include "otbFineRegistrationApplicationModel.h"

namespace otb
{
class FineRegistrationApplicationControllerInterface
      : public itk::Object
{
public:
  /** Standard class typedefs */
  typedef FineRegistrationApplicationControllerInterface Self;
  typedef itk::Object                      Superclass;
  typedef itk::SmartPointer<Self>          Pointer;
  typedef itk::SmartPointer<const Self>    ConstPointer;

  typedef FineRegistrationApplicationModel::IndexType   IndexType;
  typedef FineRegistrationApplicationModel::RadiusType  RadiusType;

  /** Standard type macros */
  itkTypeMacro(FineRegistrationApplicationControllerInterface, Superclass);

  /** Users actions */
  virtual void OpenImagePair(const char * moving, const char * fixed) = 0;
  virtual void IndexClicked(const IndexType & index) = 0;
  virtual void ChangeNormalizationFactor(double factor) = 0;
  virtual void SaveDisparityMap(const char * filename) = 0;
  virtual void SaveRegisteredImage(const char * filename) = 0;
  virtual void ChangeMaxDeformationValue(double minVal, double maxVal) = 0;
  virtual void Run() = 0;
  virtual void SetNumberOfIterations(int nbIter)=0;
  virtual void SetRadius(unsigned int rad, int id)=0;

protected:
  /** Constructor */
  FineRegistrationApplicationControllerInterface()
  {
    m_Model = FineRegistrationApplicationModel::GetInstance();
  }
  /** Destructor */
  ~FineRegistrationApplicationControllerInterface() {}

  /** The instance of the model (for commodity) */
  FineRegistrationApplicationModel * m_Model;

private:
  FineRegistrationApplicationControllerInterface(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented
};
} // end namespace otb

#endif
