/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include "otbFineRegistrationApplicationViewGUI.h"
#include <FL/Fl_File_Chooser.H>
#include "otbFltkFilterWatcher.h"
#include "otbMacro.h"
#include "otbVisualizationMacro.h"


namespace otb
{

FineRegistrationApplicationViewGUI::FineRegistrationApplicationViewGUI()
{
  m_FineRegistrationApplicationModel = FineRegistrationApplicationModel::GetInstance();
  m_FineRegistrationApplicationModel->RegisterListener(this);
  m_ImageToVectorImageCaster = ImageToVImageCasterType::New();

  this->Build();
  wOpenPairWindow->hide();
  wMainWindow->show();

  m_InterfaceColor[0]=1;
  m_InterfaceColor[1]=0;
  m_InterfaceColor[2]=0;
  m_InterfaceColor[3]=1;

  bNbIterations->value(m_FineRegistrationApplicationModel->GetNumberOfIterations());
  bXRadius->value(m_FineRegistrationApplicationModel->GetRadius()[0]);
  bYRadius->value(m_FineRegistrationApplicationModel->GetRadius()[1]);
  bXMax->value(m_FineRegistrationApplicationModel->GetMaxDeformationValue()[0]);
  bYMax->value(m_FineRegistrationApplicationModel->GetMaxDeformationValue()[1]);

  /* Start running */
  this->StartRunning();
  this->UpdateInterface();
}


void FineRegistrationApplicationViewGUI::SetFineRegistrationApplicationController(FineRegistrationApplicationControllerInterface * controller)
{
  m_FineRegistrationApplicationController = controller;
}

void FineRegistrationApplicationViewGUI::RefreshViewer()
{
  if (m_FineRegistrationApplicationModel->GetState() == NO_IMAGES)
  {
    return;
  }

  BoxType::Pointer box = BoxType::New();
  SizeType boxSize;
  IndexType boxIndex;

  // Defining the full image widget
  if (m_FullImageWidget.IsNull())
  {
    m_FullImageWidget  = FullWidgetType::New();
    m_FullImageWidget->SetInput(m_FineRegistrationApplicationModel->GetImageColorComposition());
    m_FullImageWidget->Init(gImageViewer->x(), gImageViewer->y(), gImageViewer->w(), gImageViewer->h(),"Full resolution images");
    gImageViewer->box(FL_NO_BOX);
    gImageViewer->add(m_FullImageWidget);
    m_FullImageWidget->Show();
  }
  else
  {
    m_FullImageWidget->SetInput(m_FineRegistrationApplicationModel->GetImageColorComposition());
    m_FullImageWidget->Init(gImageViewer->x(), gImageViewer->y(), gImageViewer->w(), gImageViewer->h(),"Full resolution images");
    m_FullImageWidget->ClearBufferedRegion();
  }

  m_FullImageWidget->GetTransferFunctionList()->GetNthElement(0)->SetLowerBound(m_FineRegistrationApplicationModel->GetMin());
  m_FullImageWidget->GetTransferFunctionList()->GetNthElement(0)->SetUpperBound(m_FineRegistrationApplicationModel->GetMax());
  m_FullImageWidget->GetTransferFunctionList()->GetNthElement(1)->SetLowerBound(m_FineRegistrationApplicationModel->GetMin());
  m_FullImageWidget->GetTransferFunctionList()->GetNthElement(1)->SetUpperBound(m_FineRegistrationApplicationModel->GetMax());
  m_FullImageWidget->GetTransferFunctionList()->GetNthElement(2)->SetLowerBound(m_FineRegistrationApplicationModel->GetMin());
  m_FullImageWidget->GetTransferFunctionList()->GetNthElement(2)->SetUpperBound(m_FineRegistrationApplicationModel->GetMax());

  // Defining the box of the viewed full region
  boxSize[0]=(m_FineRegistrationApplicationModel->GetCurrentRegion().GetSize()[0]);
  boxSize[1]=(m_FineRegistrationApplicationModel->GetCurrentRegion().GetSize()[1]);
  boxIndex[0]=(m_FineRegistrationApplicationModel->GetCurrentRegion().GetIndex()[0]-1);
  boxIndex[1]=(m_FineRegistrationApplicationModel->GetCurrentRegion().GetIndex()[1]+1);
  box->SetSize(boxSize);
  box->SetIndex(boxIndex);
  box->SetColor(m_InterfaceColor);

  // defining the scroll widget
  if (m_ScrollWidget.IsNull())
  {
    m_ScrollWidget  = ScrollWidgetType::New();
    m_ScrollWidget->SetController(m_FineRegistrationApplicationController);
    m_ScrollWidget->SetInput(m_FineRegistrationApplicationModel->GetQuicklookColorComposition());
    m_ScrollWidget->Init(gScroll->x(), gScroll->y(), gScroll->w(), gScroll->h(),"Scroll");
    m_ScrollWidget->SetFormOverlayVisible(true);
    gScroll->add(m_ScrollWidget);
    gScroll->box(FL_NO_BOX);
    m_ScrollWidget->GetFormList()->PushBack(box);
    m_ScrollWidget->GetTransferFunctionList()->SetNthElement(0, m_FullImageWidget->GetTransferFunctionList()->GetNthElement(0));
    m_ScrollWidget->GetTransferFunctionList()->SetNthElement(1, m_FullImageWidget->GetTransferFunctionList()->GetNthElement(1));
    m_ScrollWidget->GetTransferFunctionList()->SetNthElement(2, m_FullImageWidget->GetTransferFunctionList()->GetNthElement(2));
    m_ScrollWidget->Show();
  }
  else
  {
    m_ScrollWidget->GetFormList()->SetNthElement(0, box);
    m_ScrollWidget->SetInput(m_FineRegistrationApplicationModel->GetQuicklookColorComposition());
    m_ScrollWidget->ClearBufferedRegion();
  }
  m_ScrollWidget->SetSubSamplingRate(m_FineRegistrationApplicationModel->GetShrinkFactor());

  m_ScrollWidget->redraw();
  m_FullImageWidget->redraw();

  if (m_FineRegistrationApplicationModel->GetState() == NO_DEFORMATION)
  {
    return;
  }


  // Defining the full image widget
  if (m_FullDeformationWidget.IsNull())
  {
    m_FullDeformationWidget  = FullWidgetType::New();
    m_FullDeformationWidget->SetInput(m_FineRegistrationApplicationModel->GetDeformationFieldColorComposition());
    m_FullDeformationWidget->Init(gDeformationViewer->x(), gDeformationViewer->y(), gDeformationViewer->w(), gDeformationViewer->h(),"Full resolution images");
    gDeformationViewer->box(FL_NO_BOX);
    gDeformationViewer->add(m_FullDeformationWidget);
    m_FullDeformationWidget->Show();
  }
  else
  {
    m_FullDeformationWidget->SetInput(m_FineRegistrationApplicationModel->GetDeformationFieldColorComposition());
    m_FullDeformationWidget->Init(gDeformationViewer->x(), gDeformationViewer->y(), gDeformationViewer->w(), gDeformationViewer->h(),"Full resolution images");
    m_FullDeformationWidget->ClearBufferedRegion();
  }

  double maxX = m_FineRegistrationApplicationModel->GetMaxDeformationValue()[0];
  double maxY = m_FineRegistrationApplicationModel->GetMaxDeformationValue()[1];
  double maxI = vcl_sqrt(maxX*maxX + maxY*maxY);


  m_FullDeformationWidget->GetTransferFunctionList()->GetNthElement(0)->SetLowerBound(-maxX);
  m_FullDeformationWidget->GetTransferFunctionList()->GetNthElement(0)->SetUpperBound(maxX);
  m_FullDeformationWidget->GetTransferFunctionList()->GetNthElement(1)->SetLowerBound(-maxY);
  m_FullDeformationWidget->GetTransferFunctionList()->GetNthElement(1)->SetUpperBound(maxY);
  m_FullDeformationWidget->GetTransferFunctionList()->GetNthElement(2)->SetLowerBound(-maxI);
  m_FullDeformationWidget->GetTransferFunctionList()->GetNthElement(2)->SetUpperBound(maxI);

  m_FullDeformationWidget->redraw();
}

void FineRegistrationApplicationViewGUI
::Notify()
{
  this->RefreshViewer();
  this->UpdateInterface();
}

void FineRegistrationApplicationViewGUI
::UpdateInterface()
{
  switch (m_FineRegistrationApplicationModel->GetState())
  {
  case NO_IMAGES:
    bSaveDisparityMap->deactivate();
    bSaveRegisteredImage->deactivate();
    bMoving->deactivate();
    bFixed->deactivate();
    bResampled->deactivate();
    bXDeformation->deactivate();
    bYDeformation->deactivate();
    bIntensity->deactivate();
    bRun->deactivate();
    if (m_ScrollWidget.IsNotNull())
    {
      m_ScrollWidget->hide();
    }
    if (m_FullImageWidget.IsNotNull())
    {
      m_FullImageWidget->hide();
    }
    if (m_FullDeformationWidget.IsNotNull())
    {
      m_FullDeformationWidget->hide();
    }
    break;

  case NO_DEFORMATION:
    bSaveDisparityMap->activate();
    bSaveRegisteredImage->activate();
    bMoving->activate();
    bFixed->activate();
    bResampled->activate();
    bXDeformation->deactivate();
    bYDeformation->deactivate();
    bIntensity->deactivate();
    bRun->activate();
    if (m_ScrollWidget.IsNotNull())
    {
      m_ScrollWidget->show();
    }
    if (m_FullImageWidget.IsNotNull())
    {
      m_FullImageWidget->show();
    }
    if (m_FullDeformationWidget.IsNotNull())
    {
      m_FullDeformationWidget->hide();
    }
    break;

  case PARTIAL_DEFORMATION:
    bSaveDisparityMap->activate();
    bSaveRegisteredImage->activate();
    bMoving->activate();
    bFixed->activate();
    bResampled->activate();
    bXDeformation->activate();
    bYDeformation->activate();
    bIntensity->activate();
    bRun->activate();
    if (m_ScrollWidget.IsNotNull())
    {
      m_ScrollWidget->show();
    }
    if (m_FullImageWidget.IsNotNull())
    {
      m_FullImageWidget->show();
    }
    if (m_FullDeformationWidget.IsNotNull())
    {
      m_FullDeformationWidget->show();
    }
    break;
  }
}

void FineRegistrationApplicationViewGUI
::OpenFixedBrowserCallback()
{
  const char * filename = NULL;

  filename = fl_file_chooser("Pick an image file", "*.*",".");

  if (filename == NULL || !strcmp(filename, ""))
  {
    otbMsgDebugMacro(<<"Empty file name!");
    return;
  }
  vFixedImagePath->value(filename);

}


void FineRegistrationApplicationViewGUI
::OpenMovingBrowserCallback()
{
  const char * filename = NULL;

  filename = fl_file_chooser("Pick an image file", "*.*",".");

  if (filename == NULL || !strcmp(filename, ""))
  {
    otbMsgDebugMacro(<<"Empty file name!");
    return;
  }
  vMovingImagePath->value(filename);

}


void FineRegistrationApplicationViewGUI
::OpenPairOkCallback()
{
  m_FineRegistrationApplicationController->OpenImagePair(vMovingImagePath->value(), vFixedImagePath->value());
  wOpenPairWindow->hide();
}


void FineRegistrationApplicationViewGUI
::OpenPairCancelCallback()
{
  wOpenPairWindow->hide();
}


void
FineRegistrationApplicationViewGUI
::QuitCallback()
{
  this->StopRunning();
}

void
FineRegistrationApplicationViewGUI
::NumberOfIterationsCallback()
{
  m_FineRegistrationApplicationController->SetNumberOfIterations( static_cast<unsigned int>(bNbIterations->value()) );
}


void
FineRegistrationApplicationViewGUI
::XNCCRadiusCallback()
{
  m_FineRegistrationApplicationController->SetRadius(static_cast<unsigned int>(bXRadius->value()), 0);
}


void
FineRegistrationApplicationViewGUI
::YNCCRadiusCallback()
{
  m_FineRegistrationApplicationController->SetRadius(static_cast<unsigned int>(bYRadius->value()), 1 );
}

void
FineRegistrationApplicationViewGUI
::ShowFixedMovingResampledImage()
{
  m_FullImageWidget->SetInput(m_FineRegistrationApplicationModel->GetImageColorComposition());

  if ( bFixed->value()==1 && bMoving->value() == 0 && bResampled->value() == 0)
  {
    m_FullImageWidget->SetRedChannelIndex(0);
    m_FullImageWidget->SetGreenChannelIndex(0);
    m_FullImageWidget->SetBlueChannelIndex(0);
  }
  else if ( bFixed->value()==0 && bMoving->value() == 1 && bResampled->value() == 0)
  {
    m_FullImageWidget->SetRedChannelIndex(1);
    m_FullImageWidget->SetGreenChannelIndex(1);
    m_FullImageWidget->SetBlueChannelIndex(1);
  }
  else  if ( bFixed->value()== 0 && bMoving->value() == 0 && bResampled->value() == 1)
  {
    m_FullImageWidget->SetRedChannelIndex(2);
    m_FullImageWidget->SetGreenChannelIndex(2);
    m_FullImageWidget->SetBlueChannelIndex(2);
  }
  else  if ( bFixed->value()==1 && bMoving->value() == 1 && bResampled->value() == 0)
  {
    m_FullImageWidget->SetRedChannelIndex(0);
    m_FullImageWidget->SetGreenChannelIndex(1);
    m_FullImageWidget->SetBlueChannelIndex(1);
  }
  else  if ( bFixed->value()==1 && bMoving->value() == 0 && bResampled->value() == 1)
  {
    m_FullImageWidget->SetRedChannelIndex(0);
    m_FullImageWidget->SetGreenChannelIndex(2);
    m_FullImageWidget->SetBlueChannelIndex(2);
  }
  else  if ( bFixed->value()==0 && bMoving->value() == 1 && bResampled->value() == 1)
  {
    m_FullImageWidget->SetRedChannelIndex(1);
    m_FullImageWidget->SetGreenChannelIndex(2);
    m_FullImageWidget->SetBlueChannelIndex(2);
  }
  else
  {
    m_FullImageWidget->SetRedChannelIndex(0);
    m_FullImageWidget->SetGreenChannelIndex(1);
    m_FullImageWidget->SetBlueChannelIndex(2);
  }
  m_FullImageWidget->ClearBufferedRegion();
  m_FullImageWidget->redraw();
}


void
FineRegistrationApplicationViewGUI
::ShowFixedCallback()
{
  if ( bFixed->value()==0 && bMoving->value()==0 && bResampled->value()==0 )
  {
    bFixed->value(1);
    return;
  }
  this->ShowFixedMovingResampledImage();
}

void
FineRegistrationApplicationViewGUI
::ShowMovingCallback()
{
  if ( bFixed->value()==0 && bMoving->value()==0 && bResampled->value()==0 )
  {
    bMoving->value(1);
    return;
  }
  this->ShowFixedMovingResampledImage();
}

void
FineRegistrationApplicationViewGUI
::ShowResampledCallback()
{
  if ( bFixed->value()==0 && bMoving->value()==0 && bResampled->value()==0 )
  {
    bResampled->value(1);
    return;
  }
  this->ShowFixedMovingResampledImage();
}

void
FineRegistrationApplicationViewGUI
::ShowXYIntensityDeformation()
{
  m_FullDeformationWidget->SetInput(m_FineRegistrationApplicationModel->GetDeformationFieldColorComposition());


  if ( bXDeformation->value()==1 && bYDeformation->value() == 0 && bIntensity->value() == 0)
  {
    m_FullDeformationWidget->SetRedChannelIndex(0);
    m_FullDeformationWidget->SetGreenChannelIndex(0);
    m_FullDeformationWidget->SetBlueChannelIndex(0);
  }
  else if ( bXDeformation->value()==0 && bYDeformation->value() == 1 && bIntensity->value() == 0)
  {
    m_FullDeformationWidget->SetRedChannelIndex(1);
    m_FullDeformationWidget->SetGreenChannelIndex(1);
    m_FullDeformationWidget->SetBlueChannelIndex(1);
  }
  else  if ( bXDeformation->value()== 0 && bYDeformation->value() == 0 && bIntensity->value() == 1)
  {
    m_FullDeformationWidget->SetRedChannelIndex(2);
    m_FullDeformationWidget->SetGreenChannelIndex(2);
    m_FullDeformationWidget->SetBlueChannelIndex(2);
  }
  else  if ( bXDeformation->value()==1 && bYDeformation->value() == 1 && bIntensity->value() == 0)
  {
    m_FullDeformationWidget->SetRedChannelIndex(0);
    m_FullDeformationWidget->SetGreenChannelIndex(1);
    m_FullDeformationWidget->SetBlueChannelIndex(1);
  }
  else  if ( bXDeformation->value()==1 && bYDeformation->value() == 0 && bIntensity->value() == 1)
  {
    m_FullDeformationWidget->SetRedChannelIndex(0);
    m_FullDeformationWidget->SetGreenChannelIndex(2);
    m_FullDeformationWidget->SetBlueChannelIndex(2);
  }
  else  if ( bXDeformation->value()==0 && bYDeformation->value() == 1 && bIntensity->value() == 1)
  {
    m_FullDeformationWidget->SetRedChannelIndex(1);
    m_FullDeformationWidget->SetGreenChannelIndex(2);
    m_FullDeformationWidget->SetBlueChannelIndex(2);
  }
  else
  {
    m_FullDeformationWidget->SetRedChannelIndex(0);
    m_FullDeformationWidget->SetGreenChannelIndex(1);
    m_FullDeformationWidget->SetBlueChannelIndex(2);
  }
  m_FullDeformationWidget->ClearBufferedRegion();
  m_FullDeformationWidget->redraw();
}


void
FineRegistrationApplicationViewGUI
::ShowXDeformationCallback()
{
  if ( bXDeformation->value()==0 && bYDeformation->value()==0 && bIntensity->value()==0)
  {
    bXDeformation->value(1);
    return;
  }
  this->ShowXYIntensityDeformation();
}


void
FineRegistrationApplicationViewGUI
::ShowYDeformationCallback()
{
  if ( bXDeformation->value()==0 && bYDeformation->value()==0 && bIntensity->value()==0)
  {
    bYDeformation->value(1);
    return;
  }
  this->ShowXYIntensityDeformation();
}


void
FineRegistrationApplicationViewGUI
::ShowIntensityCallback()
{
  if ( bXDeformation->value()==0 && bYDeformation->value()==0 && bIntensity->value()==0)
  {
    bIntensity->value(1);
    return;
  }
  this->ShowXYIntensityDeformation();
}


void
FineRegistrationApplicationViewGUI
::SaveDisparityMapCallback()
{
  const char * filename = NULL;

  filename = fl_file_chooser("Save Disparity Map as", "*.*",".");
  if (filename == NULL || !strcmp(filename, ""))
  {
    otbMsgDebugMacro(<<"Empty file name!");
    return;
  }
  m_FineRegistrationApplicationController->SaveDisparityMap(filename);
}

void
FineRegistrationApplicationViewGUI
::SaveRegisteredImageCallback()
{
  const char * filename = NULL;

  filename = fl_file_chooser("Save Registered Image as", "*.*",".");
  if (filename == NULL || !strcmp(filename, ""))
  {
    otbMsgDebugMacro(<<"Empty file name!");
    return;
  }

  m_FineRegistrationApplicationController->SaveRegisteredImage(filename);
}


void
FineRegistrationApplicationViewGUI
::ChangeMaxDeformationValueCallback()
{
  m_FineRegistrationApplicationController->ChangeMaxDeformationValue(bXMax->value(), bYMax->value());
}

void
FineRegistrationApplicationViewGUI
::RunCallback()
{
  m_FineRegistrationApplicationController->Run();
  this->ShowFixedMovingResampledImage();
  this->ShowXYIntensityDeformation();
}

void
FineRegistrationApplicationViewGUI
::OpenPairCallback()
{
  wOpenPairWindow->show();
}
}
