/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbFineRegistrationApplicationViewGUI_h
#define __otbFineRegistrationApplicationViewGUI_h

// Disabling deprecation warning
#ifdef _MSC_VER
#pragma warning(push)
#pragma warning(disable:4996)
#endif
#include "otbFineRegistrationApplicationViewGroup.h"
#ifdef _MSC_VER
#pragma warning(pop)
#endif

#include "otbListenerBase.h"
#include "otbFineRegistrationApplicationModel.h"
#include "otbFineRegistrationApplicationControllerInterface.h"

#include "otbInteractiveScrollWidget.h"
#include "otbFullResolutionImageWidget.h"
#include "otbImageWidgetBoxForm.h"
#include "otbImageToVectorImageCastFilter.h"
#include "otbBlackImage.h"
#include "otbConfigure.h"

#include "itkObject.h"

#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Tabs.H>

#include "otbVisualizationMacro.h"

namespace otb
{
/** \class FineRegistrationApplicationViewGUI
 *
 */
class FineRegistrationApplicationViewGUI
      : public ListenerBase, public FineRegistrationApplicationViewGroup, public itk::Object
{
public:
  /** Standard class typedefs */
  typedef FineRegistrationApplicationViewGUI Self;
  typedef itk::Object                    Superclass;
  typedef itk::SmartPointer<Self>        Pointer;
  typedef itk::SmartPointer<const Self>  ConstPointer;

  // Pixel type
  typedef FineRegistrationApplicationModel::PixelType                                                  PixelType;
  typedef FineRegistrationApplicationModel::ImageType                                                  SingleImageType;
  typedef InteractiveScrollWidget<PixelType, FineRegistrationApplicationControllerInterface>            ScrollWidgetType;
  typedef FullResolutionImageWidget<PixelType>                                                         FullWidgetType;
  typedef FullWidgetType::ImageType                                                                    ImageType;
  typedef ImageType::IndexType                                                                         IndexType;
  typedef ImageType::SizeType                                                                          SizeType;
  typedef ImageType::RegionType                                                                        RegionType;
  typedef ImageWidgetBoxForm                                                                           BoxType;
  typedef BoxType::ColorType                                                                           ColorType;
  typedef ImageToVectorImageCastFilter<SingleImageType, ImageType>                                     ImageToVImageCasterType;

  /** Standards macros */
  itkNewMacro(Self);
  itkTypeMacro(FineRegistrationApplicationViewGUI, Object);

  /** Set the controller */
  virtual void SetFineRegistrationApplicationController(FineRegistrationApplicationControllerInterface * controller);

  /** Event from the model */
  virtual void Notify();

  /** Buttons callback */
  virtual void OpenFixedBrowserCallback();
  virtual void OpenMovingBrowserCallback();
  virtual void OpenPairOkCallback();
  virtual void OpenPairCancelCallback();
  virtual void ShowFixedCallback();
  virtual void ShowMovingCallback();
  virtual void ShowResampledCallback();
  virtual void NumberOfIterationsCallback();
  virtual void XNCCRadiusCallback();
  virtual void YNCCRadiusCallback();
  virtual void ShowXDeformationCallback();
  virtual void ShowYDeformationCallback();
  virtual void ShowIntensityCallback();
  virtual void RunCallback();
  virtual void QuitCallback();
  virtual void OpenPairCallback();
  virtual void SaveRegisteredImageCallback();
  virtual void SaveDisparityMapCallback();
  virtual void ChangeMaxDeformationValueCallback();

  virtual void ShowFixedMovingResampledImage();
  void ShowXYIntensityDeformation();

  itkGetObjectMacro(FullImageWidget, FullWidgetType);
  itkGetObjectMacro(FullDeformationWidget, FullWidgetType);
  itkGetObjectMacro(ScrollWidget, ScrollWidgetType);

  /** Standard Running macro */
  otbRunningMacro();

protected:
  /** Constructor */
  FineRegistrationApplicationViewGUI();
  /** Destructor */
  ~FineRegistrationApplicationViewGUI() {};

  /** Refresh the image widgets */
  virtual void RefreshViewer();
  virtual void UpdateInterface();

private:
  FineRegistrationApplicationViewGUI(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

  /** Pointer to the model */
  FineRegistrationApplicationModel::Pointer m_FineRegistrationApplicationModel;
  /** Pointer to the controller */
  FineRegistrationApplicationControllerInterface::Pointer m_FineRegistrationApplicationController;

  ColorType m_InterfaceColor;

  /// Vector cast filters
  ScrollWidgetType::Pointer         m_ScrollWidget;
  FullWidgetType::Pointer           m_FullImageWidget;
  FullWidgetType::Pointer           m_FullDeformationWidget;
  ImageToVImageCasterType::Pointer  m_ImageToVectorImageCaster;

  /* Flag to Enable/Disable the window (set to disable when QuitCallback method has been activated */
  bool m_enableWindow;
};
}//end namespace otb

#endif
