/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include "otbFineRegistrationApplicationModel.h"
#include "otbFineRegistrationApplicationViewGUI.h"
#include "otbFineRegistrationApplicationController.h"
#include "otbMsgReporter.h"
#include "otbConfigure.h"
#include <FL/Fl.H>

#include "otbCommandLineArgumentParser.h"

#include "otbI18n.h"

using namespace otb;

int main(int argc, char* argv[])
{
  otbI18nMacro();
  typedef CommandLineArgumentParser ParserType;
  ParserType::Pointer parser = ParserType::New();

  //parser->AddInputImage(false); //Optionnal parameter
  parser->AddOption("--MovingImage","Input Moving Image","-mov", 1, false);
  parser->AddOption("--FixedImage","Input Fixed Image","-fix", 1, false);

  typedef otb::CommandLineArgumentParseResult ParserResultType;
  ParserResultType::Pointer  parseResult = ParserResultType::New();

  typedef otb::MsgReporter ReporterType;
  try
  {
    parser->ParseCommandLine(argc, argv, parseResult);
  }
  catch ( itk::ExceptionObject & err )
  {
    std::string descriptionException = err.GetDescription();
    if (descriptionException.find("ParseCommandLine(): Help Parser") != std::string::npos)
    {
      return EXIT_SUCCESS;
    }
    if (descriptionException.find("ParseCommandLine(): Version Parser") != std::string::npos)
    {
      return EXIT_SUCCESS;
    }
    return EXIT_FAILURE;
  }

  FineRegistrationApplicationModel::Pointer model = FineRegistrationApplicationModel::GetInstance();
  FineRegistrationApplicationController::Pointer controller = FineRegistrationApplicationController::New();
  FineRegistrationApplicationViewGUI::Pointer view = FineRegistrationApplicationViewGUI::New();
  otb::MsgReporter::GetInstance()->SetTitle("FineRegistration application");

  view->SetFineRegistrationApplicationController(controller);
  controller->SetView(view);

  if (  parseResult->IsOptionPresent("--MovingImage") && parseResult->IsOptionPresent("--FixedImage") )
  {
    controller->OpenImagePair(parseResult->GetParameterString("--FixedImage").c_str(), parseResult->GetParameterString("--MovingImage").c_str() );
  }

  otbFlRunMacro(view->IsRunning());

  return EXIT_SUCCESS;
}
