/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/

#include "otbImageToDBRegistrationController.h"
#include "otbMsgReporter.h"

namespace otb
{

ImageToDBRegistrationController
::ImageToDBRegistrationController()
{
  // Build a "visu"controller
   m_FocusingVisuController = FocusingVisuControllerType::New();
   m_PixelModel             = PixelDescriptionModelType::New();
   m_PixelView              = PixelDescriptionViewType::New();
   m_PixelModelWorking      = PixelDescriptionModelType::New();
   m_PixelViewWorking       = PixelDescriptionViewType::New();
}

ImageToDBRegistrationController
::~ImageToDBRegistrationController()
{}

void
ImageToDBRegistrationController
::AddHandlers()
{
  m_FocusingVisuController->SetFocusingModel(m_Model->GetFocusingVisuModel());
  m_FocusingVisuController->SetFocusingView(m_View->GetFocusingVisuView());
  m_FocusingVisuController->AddSelectAreaHandler();

  ResizingHandlerType::Pointer resizingHandlerM = ResizingHandlerType::New();
  resizingHandlerM->SetModel(m_Model->GetMasterVisuModel());
  resizingHandlerM->SetView(m_View->GetMasterVisuView());
 this->GetMasterVisuController()->AddActionHandler(resizingHandlerM);
  
  ResizingHandlerType::Pointer resizingHandlerW = ResizingHandlerType::New();
  resizingHandlerW->SetModel(m_Model->GetWorkingVisuModel());
  resizingHandlerW->SetView(m_View->GetWorkingVisuView());
  m_FocusingVisuController->GetWorkingController()->AddActionHandler(resizingHandlerW);
  
  // Add the change scaled region handler
  ChangeScaledRegionHandlerType::Pointer changeScaledHandler =ChangeScaledRegionHandlerType::New();
  changeScaledHandler->SetModel(m_Model->GetWorkingVisuModel());
  changeScaledHandler->SetView(m_View->GetWorkingVisuView());
  m_FocusingVisuController->GetWorkingController()->AddActionHandler(changeScaledHandler);
  
  // Add the change scaled handler
  ChangeScaleHandlerType::Pointer changeScaleHandler =ChangeScaleHandlerType::New();
  changeScaleHandler->SetModel(m_Model->GetWorkingVisuModel());
  changeScaleHandler->SetView(m_View->GetWorkingVisuView());
  m_FocusingVisuController->GetWorkingController()->AddActionHandler(changeScaleHandler);
  
  // Add the change region action handler
  ChangeRegionHandlerType::Pointer changeRegionHandler = ChangeRegionHandlerType::New();
  changeRegionHandler->SetModel(m_Model->GetWorkingVisuModel());
  changeRegionHandler->SetView(m_View->GetWorkingVisuView());
  m_FocusingVisuController->GetWorkingController()->AddActionHandler(changeRegionHandler);
  
  // Add the arrow keys move action handler
  ArrowKeyMoveActionHandlerType::Pointer arrowKeyMoveHandler = ArrowKeyMoveActionHandlerType::New();
  arrowKeyMoveHandler->SetModel(m_Model->GetWorkingVisuModel());
  arrowKeyMoveHandler->SetView(m_View->GetWorkingVisuView());
  m_FocusingVisuController->GetWorkingController()->AddActionHandler(arrowKeyMoveHandler);

  // Add the opacity management
  ChangeOpacityHandlerType::Pointer changeOpacityHandler = ChangeOpacityHandlerType::New();
  changeOpacityHandler->SetModel(m_Model->GetFocusingVisuModel());
  changeOpacityHandler->SetView(m_View->GetFocusingVisuView());
  m_FocusingVisuController->GetWorkingController()->AddActionHandler(changeOpacityHandler);
  m_FocusingVisuController->GetMasterController()->AddActionHandler(changeOpacityHandler);
 
  // Pixel description
  // master scroll
  m_PixelModel->SetLayers(m_Model->GetMasterVisuModel()->GetLayers());
  PixelDescriptionActionHandlerType::Pointer pixelActionHandler = PixelDescriptionActionHandlerType::New();
  pixelActionHandler->SetView(m_View->GetMasterVisuView());
  pixelActionHandler->SetModel(m_PixelModel);
  m_FocusingVisuController->GetMasterController()->AddActionHandler(pixelActionHandler);
  m_PixelView->SetModel(m_PixelModel);
  // Build a pixel description view
  m_PixelView->SetModel(m_PixelModel);
  m_View->gPixelValue->add(m_PixelView->GetPixelDescriptionWidget());
  m_View->gPixelValue->resizable(m_PixelView->GetPixelDescriptionWidget());
  m_View->gPixelValue->show();
  m_PixelView->GetPixelDescriptionWidget()->show();
  m_PixelView->GetPixelDescriptionWidget()->resize(m_View->gPixelValue->x(), m_View->gPixelValue->y(), m_View->gPixelValue->w(), m_View->gPixelValue->h());
    
  // working scroll
  m_PixelModelWorking->SetLayers(m_Model->GetWorkingVisuModel()->GetLayers());
  PixelDescriptionActionHandlerType::Pointer pixelActionHandler2 = PixelDescriptionActionHandlerType::New();
  pixelActionHandler2->SetView(m_View->GetWorkingVisuView());
  pixelActionHandler2->SetModel(m_PixelModelWorking);
  m_FocusingVisuController->GetWorkingController()->AddActionHandler(pixelActionHandler2);
  m_PixelViewWorking->SetModel(m_PixelModelWorking);
  // Build a pixel description view
  m_PixelViewWorking->SetModel(m_PixelModelWorking);
  m_View->gPixelValueW->add(m_PixelViewWorking->GetPixelDescriptionWidget());
  m_View->gPixelValueW->resizable(m_PixelViewWorking->GetPixelDescriptionWidget());
  m_View->gPixelValueW->show();
  m_PixelViewWorking->GetPixelDescriptionWidget()->show();
  m_PixelViewWorking->GetPixelDescriptionWidget()->resize(m_View->gPixelValueW->x(), m_View->gPixelValueW->y(), m_View->gPixelValueW->w(), m_View->gPixelValueW->h());
}


void
ImageToDBRegistrationController
::OpenInputImage(const char * filename)
{
  
  try
  {
    std::string strFilename = filename;
    m_Model->SetInputFileName( strFilename );
    m_Model->OpenImage();
  }
  catch (itk::ExceptionObject & err)
  {
    MsgReporter::GetInstance()->SendError(err.GetDescription());
  }
}


void
ImageToDBRegistrationController
::SetOutputFileName( const char * name )
{
  try
  {
    m_Model->SetOutputFileName( name );
  }
  catch (itk::ExceptionObject & err)
  {
    MsgReporter::GetInstance()->SendError(err.GetDescription());
  }
}


void
ImageToDBRegistrationController
::LoadVectorData( const char * name )
{
  try
  {
    m_Model->LoadVectorData( name );
  }
  catch (itk::ExceptionObject & err)
  {
    MsgReporter::GetInstance()->SendError(err.GetDescription());
  }
}


void
ImageToDBRegistrationController
::SetDEMDirectory( const char * name )
{
  m_Model->SetDEMDirectory( name );
}

void
ImageToDBRegistrationController
::UseDEM(bool b)
{
  try
  {
    m_Model->UseDEM( b );
  }
    catch (itk::ExceptionObject & err)
  {
    MsgReporter::GetInstance()->SendError(err.GetDescription());
  }
}

void
ImageToDBRegistrationController
::ChangeVectorDataColor( unsigned int id, ColorType color)
{
  m_Model->ChangeVectorDataColor( id, color );
}


void
ImageToDBRegistrationController
::ClearAllVectorData()
{
  m_Model->ClearAllVectorData();
}

void
ImageToDBRegistrationController
::ClearVectorData( int id )
{
  m_Model->ClearVectorData( static_cast<unsigned int>(id) );
}


void
ImageToDBRegistrationController::
ChangeColor( ColorType color)
{
  m_Model->ChangeColor( color );
}


void
ImageToDBRegistrationController
::SaveVectorData(const char * filename, InputImageEnumType type )
{
  try
    {
      m_Model->SetOutputFileName( filename );
      m_Model->SaveVectorData( type );
      MsgReporter::GetInstance()->SendMsg("Save Vector Data done.");
    }
  catch (itk::ExceptionObject & err)
    {
      MsgReporter::GetInstance()->SendError(err.GetDescription());
    }
}

void
ImageToDBRegistrationController
::SaveImage(const char * filename)
{
  try
    {
      m_Model->SaveImage( filename );
      MsgReporter::GetInstance()->SendMsg("Save Image done.");
    }
  catch (itk::ExceptionObject & err)
    {
      MsgReporter::GetInstance()->SendError(err.GetDescription());
    }
}


void
ImageToDBRegistrationController
:: RunOverExtract(OutputEnumType type)
{
  try
    {
      int id = m_View->dVDList->value();
      if( m_View->dVDList->size()==0 )
       {
         itkExceptionMacro(<<"No Vector Data available");
       }
      else if( id==0 )
       {
         itkExceptionMacro(<<"No Vector Data selected");
       }
    
      m_Model->SetColinearityThreshold( m_View->gColinearityThresh->value());
      m_Model->SetValidSegmentThreshold( m_View->gValidSegmentThresh->value());
      m_Model->SetTripletMaxDistance( m_View->gTripletMaxDist->value());

      m_Model->RunOverExtract(type, id-1);


      itk::OStringStream oss;
      oss.str("");
      oss<<m_Model->GetTransform()->GetParameters();
      m_View->tTransform->value( oss.str().c_str() );

      MsgReporter::GetInstance()->SendMsg("Transformation computed.");
    }
  catch (itk::ExceptionObject & err)
    {
      MsgReporter::GetInstance()->SendError(err.GetDescription());
    }
}


void
ImageToDBRegistrationController
:: Run(std::string name, OutputEnumType type)
{
  try
    {
      int id = m_View->dVDList->value();
      if(type == otb::DATABASE)
       {
         if( m_View->dVDList->size()==0 )
           {
             itkExceptionMacro("No Vector Data available");
           }
         else if( id==0 )
           {
             itkExceptionMacro("No Vector Data selected");
           }
         for(unsigned int i=0; i<m_Model->GetDBListId().size(); i++)
           {
             if( static_cast<int>(m_Model->GetDBListId()[i])==id-1 )
              itkExceptionMacro("No valid Vector Data, can't run over an result!...");
           }
       }

      this->SetOutputFileName(name.c_str());
  
    m_Model->Run(type, id-1);
    MsgReporter::GetInstance()->SendMsg("Full proceeded.");

    }
  catch (itk::ExceptionObject & err)
    {
      MsgReporter::GetInstance()->SendError(err.GetDescription());
    }
}


void
ImageToDBRegistrationController
:: ChangeAlpha()
{
  if( m_View->gDBRef->value() == 1 )
    {
      double alpha = m_View->vAlpha->value();
      try
       {
         m_Model->ChangeAlpha(alpha);
       }
      catch (itk::ExceptionObject & err)
       {
         MsgReporter::GetInstance()->SendError(err.GetDescription());
       }
    }
}


} // end namespace otb

