/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbImageToDBRegistrationController_h
#define __otbImageToDBRegistrationController_h

//#include "otbImageToDBRegistrationEnum.h"
#include "otbImageToDBRegistrationControllerInterface.h"
#include "otbImageToDBRegistrationViewGUI.h"
#include "otbWidgetResizingActionHandler.h"
#include "otbChangeScaledExtractRegionActionHandler.h"
#include "otbChangeExtractRegionActionHandler.h"
#include "otbChangeScaleActionHandler.h"
#include "otbSelectAreaActionHandler.h"
#include "otbArrowKeyMoveActionHandler.h"
#include "otbChangeOpacityActionHandler.h"
#include "otbPixelDescriptionActionHandler.h"
#include "otbPixelDescriptionModel.h"
#include "otbPixelDescriptionView.h"


namespace otb
{
class ITK_EXPORT ImageToDBRegistrationController
      : public ImageToDBRegistrationControllerInterface
{
public:
  /** Standard class typedefs */
  typedef ImageToDBRegistrationController          Self;
  typedef ImageToDBRegistrationControllerInterface Superclass;
  typedef itk::SmartPointer<Self>                  Pointer;
  typedef itk::SmartPointer<const Self>            ConstPointer;
  
  typedef ImageToDBRegistrationViewGUI      ViewType;
  typedef ViewType::Pointer                 ViewPointerType;
  typedef Superclass::InputImageType        InputImageType;
  typedef Superclass::InputImagePointerType InputImagePointerType;
  typedef Superclass::ModelType             ModelType;
  typedef ModelType::IndexType              IndexType;
  typedef ModelType::SizeType               SizeType;
  typedef ModelType::OffsetType             OffsetType;
  
  /** Visualization typedefs*/
  // Model and view
  typedef ModelType::FocusingVisuModelType           FocusingVisuModelType;
  typedef FocusingVisuModelType::Pointer             FocusingVisuModelPointerType;
  typedef ViewType::FocusingVisuViewType             FocusingVisuViewType;
  typedef FocusingVisuViewType::Pointer              FocusingVisuViewPointerType;
  typedef Superclass::FocusingVisuControllerType     FocusingVisuControllerType;
  typedef FocusingVisuControllerType::Pointer        FocusingVisuControllerPointerType;
  typedef ModelType::VisuModelType                   VisuModelType;
  typedef VisuModelType::Pointer                     VisuModelPointerType;
  typedef FocusingVisuViewType::ImageViewType        VisuViewType;
  typedef VisuViewType::Pointer                      VisuViewPointerType;
  typedef VisuViewType::ImageWidgetType              VisuImageWidgetType;
  typedef VisuImageWidgetType::Pointer               VisuImageWidgetPointerType;
  typedef FocusingVisuControllerType::ControllerType VisuControllerType;
  typedef VisuControllerType::Pointer                VisuControllerPointerType;
  typedef FocusingVisuModelType::OutputImageType     OutputImageType;

  // Handlers
  typedef WidgetResizingActionHandler<VisuModelType, VisuViewType>                ResizingHandlerType;
  typedef ChangeScaledExtractRegionActionHandler<VisuModelType, VisuViewType>     ChangeScaledRegionHandlerType;
  typedef ChangeExtractRegionActionHandler<VisuModelType, VisuViewType>           ChangeRegionHandlerType;
  typedef ChangeScaleActionHandler<VisuModelType, VisuViewType>                   ChangeScaleHandlerType;
  typedef SelectAreaActionHandler<FocusingVisuModelType, VisuImageWidgetType>    SelectAreaHandlerType;
  typedef ArrowKeyMoveActionHandler<VisuModelType, VisuViewType>                 ArrowKeyMoveActionHandlerType;
  typedef ChangeOpacityActionHandler<FocusingVisuModelType, FocusingVisuViewType> ChangeOpacityHandlerType;

  typedef PixelDescriptionModel<OutputImageType>                                 PixelDescriptionModelType;
  typedef PixelDescriptionActionHandler<PixelDescriptionModelType, VisuViewType> PixelDescriptionActionHandlerType;
  typedef PixelDescriptionView<PixelDescriptionModelType>                        PixelDescriptionViewType;

  /** Standard type macros */
  itkTypeMacro(ImageToDBRegistrationController, Superclass);
  itkNewMacro(Self);

  VisuControllerPointerType GetMasterVisuController()
  {
    return m_FocusingVisuController->GetMasterController();
  }
  VisuControllerPointerType GetWorkingVisuController()
  {
    return m_FocusingVisuController->GetWorkingController();
  }
  virtual FocusingVisuControllerPointerType GetFocusingVisuController()
  {
    return m_FocusingVisuController;
  }


  itkSetObjectMacro(View, ViewType);
  itkGetObjectMacro(View, ViewType);
  void AddHandlers();


  virtual void OpenInputImage(const char * filename);
  virtual void SetOutputFileName( const char * name );
  virtual void LoadVectorData( const char * name );
  virtual void SetDEMDirectory( const char * name );
  virtual void UseDEM( bool b );
  virtual void RunOverExtract(OutputEnumType type);
  virtual void Run(std::string name, OutputEnumType type);
  virtual void ChangeVectorDataColor( unsigned int id, ColorType color);
  virtual void ChangeColor( ColorType color);
  virtual void ClearAllVectorData();
  virtual void ClearVectorData( int id );
  virtual void SaveVectorData(const char * filename, InputImageEnumType type);
  virtual void SaveImage(const char * filename);
  virtual void ChangeAlpha();


protected:
  /** Constructor */
  ImageToDBRegistrationController();
  /** Destructor */
  virtual ~ImageToDBRegistrationController();

private:
  ImageToDBRegistrationController(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

  /** Pointer to the view */
  ViewPointerType m_View;
  /** POinter to the Focusing controller */
  FocusingVisuControllerPointerType m_FocusingVisuController;
  /** For pixel description */
  PixelDescriptionModelType::Pointer m_PixelModel;
  PixelDescriptionViewType::Pointer  m_PixelView;
  PixelDescriptionModelType::Pointer m_PixelModelWorking;
  PixelDescriptionViewType::Pointer  m_PixelViewWorking;


};
} //end namespace otb

#endif
