/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbImageToDBRegistrationControllerInterface_h
#define __otbImageToDBRegistrationControllerInterface_h

#include "otbImageToDBRegistrationModel.h"
#include "itkProcessObject.h"
#include "otbFocusingController.h"
#include "otbImageToDBRegistrationEnum.h"

namespace otb
{
class ITK_EXPORT  ImageToDBRegistrationControllerInterface
: public itk::ProcessObject
{
public:
  /** Standard class typedefs */
  typedef ImageToDBRegistrationControllerInterface Self;
  typedef itk::ProcessObject                   Superclass;
  typedef itk::SmartPointer<Self>              Pointer;
  typedef itk::SmartPointer<const Self>        ConstPointer;

  typedef ImageToDBRegistrationModel        ModelType;
  typedef ModelType::FocusingVisuModelType  FocusingVisuModelType;
  typedef ModelType::FocusingVisuViewType   FocusingVisuViewType;
  typedef ModelType::InputImageType         InputImageType;
  typedef ModelType::InputImagePointerType  InputImagePointerType;
  typedef ModelType::IndexType              IndexType;
  typedef ModelType::SizeType               SizeType;
  typedef FocusingVisuModelType::ColorType  ColorType;

/** NewVisu */
  typedef FocusingController<FocusingVisuModelType, FocusingVisuViewType> FocusingVisuControllerType;
  typedef FocusingVisuControllerType::Pointer                             FocusingVisuControllerPointerType;


  /** Standard type macros */
  itkTypeMacro(ImageToDBRegistrationControllerInterface, Superclass);

  /** Users actions */
  virtual void OpenInputImage(const char * filename) =0;
  virtual void SetOutputFileName( const char * name ) =0;
  virtual void LoadVectorData( const char * name ) =0;
  virtual void SetDEMDirectory( const char * name ) =0;
  virtual void UseDEM( bool b ) =0;
  virtual void RunOverExtract(OutputEnumType type) =0;
  virtual void Run(std::string name, OutputEnumType type) =0;
  virtual void ChangeVectorDataColor( unsigned int id, ColorType color) =0;
  virtual void ChangeColor( ColorType color) =0;
  virtual void ClearAllVectorData() =0;
  virtual void ClearVectorData( int id ) =0;
  virtual void SaveVectorData(const char * filename, InputImageEnumType type) =0;
  virtual void SaveImage(const char * filename) =0;
  virtual void ChangeAlpha() =0;

  virtual FocusingVisuControllerPointerType GetFocusingVisuController()=0;

protected:
  /** Constructor */
  ImageToDBRegistrationControllerInterface()
  {
    m_Model = ModelType::GetInstance();
  }
  /** Destructor */
  ~ImageToDBRegistrationControllerInterface() {};

  /** The instance of the model (for commodity) */
  ModelType * m_Model;

private:
  ImageToDBRegistrationControllerInterface(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented
};
} // end namespace otb

#endif
