/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
  See OTBCopyright.txt for details.


     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITYor FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/

#ifndef __otbSignificantLineVectorDataToTriplet_h
#define __otbSignificantLineVectorDataToTriplet_h

#include  "itkProcessObject.h"
#include  "itkSize.h"
#include  "itkVector.h"
#include  "itkPoint.h"

namespace otb
{
  /** \class SignificantLineVectorDataToTriplet
   *  \brief this class computes the coordicate of three non colinear
   *         segments
   *         Update Method have to be explicitly called by the user
   */

template <class TVectorData, class TImage>
class ITK_EXPORT SignificantLineVectorDataToTriplet :
  public itk::ProcessObject
  {
  public:
    /** Standard class typedefs. */
    typedef SignificantLineVectorDataToTriplet                     Self;
    typedef itk::ProcessObject                                            Superclass;
    typedef itk::SmartPointer<Self>                                Pointer;
    typedef itk::SmartPointer<const Self>                          ConstPointer;

    /** Run-time type information (and related methods). */
    itkTypeMacro(SignificantLineVectorDataToTriplet, itk::ProcessObject);
    
    /** Method for creation through the object factory. */
    itkNewMacro(Self);

    /** VectorData TypeDef*/
    typedef TVectorData                                         VectorDataType;
    typedef typename VectorDataType::Pointer                    VectorDataPointerType;
    typedef typename VectorDataType::DataNodeType               DataNodeType;
    typedef typename DataNodeType::Pointer                      DataNodePointerType;
    typedef typename DataNodeType::LineType::VertexType         VertexType;
    typedef typename DataNodeType::LineType::VertexListType     VertexListType;
    
    typedef typename VectorDataType::DataTreeType::TreeNodeType InternalTreeNodeType;
    typedef typename InternalTreeNodeType::ChildrenListType     ChildrenListType;
    
    /** ImageType*/
    typedef TImage                                               ImageType;
    typedef typename ImageType::Pointer                          ImagePointerType;
    
    typedef std::vector<VertexType>                              LineVectorType;
    typedef std::vector<LineVectorType>                          VertexVectorType;

    typedef itk::Vector<double, 2>                               SpacingType;
    typedef itk::Point<double, 2>                                OriginType;
    typedef itk::Size<2>                                         SizeType;

    //Set The vectorData
    itkSetObjectMacro(Image, ImageType);

    //Set The origin
    itkSetMacro(Origin, OriginType);
    
    //Set/Get Colinearitythreshol
    itkSetMacro(ColinearityThreshold, double);
    itkGetMacro(ColinearityThreshold, double);
    
    //Get the list of tuplets
    VertexVectorType GetTupletOutput()
      {
       return m_TupletVector;
      }

     //Get OutputVectorData
     itkGetObjectMacro(OutputVectorData, VectorDataType);

     //
     virtual void SetVectorData(VectorDataType*  vector);
     virtual VectorDataType * GetInputVectorData();
     virtual VectorDataType * GetOutput();
     
  protected:
    virtual void GenerateData();
    virtual void ProcessNode(InternalTreeNodeType * source);
    virtual double ComputeSegmentOrientation(VertexType currentVertex, VertexType nextVertex);
    virtual VertexType ComputeLineIntersection(typename VertexVectorType::iterator it1, typename VertexVectorType::iterator it2 );
    virtual void ProcessTuplets();
    virtual double ComputeSegmenLength(VertexType currentVertex, VertexType nextVertex);
    virtual bool IsPresent(LineVectorType tuplet);
    
    SignificantLineVectorDataToTriplet();
    virtual ~SignificantLineVectorDataToTriplet() {};
    
  private:
    SignificantLineVectorDataToTriplet(const Self&); //purposely not implemented
    void operator=(const Self&); //purposely not implemented

    double                        m_ColinearityThreshold;
    VertexVectorType              m_LineVector;
    VertexVectorType              m_TupletVector;
    OriginType                    m_Origin;

    VectorDataPointerType         m_OutputVectorData;
    ImagePointerType              m_Image;
  };


}
#ifndef ITK_MANUAL_INSTANTIATION
#include "otbSignificantLineVectorDataToTriplet.txx"
#endif

#endif
