/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
  See OTBCopyright.txt for details.


     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITYor FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/

#ifndef __otbVectorDataToHistogramFilter_h
#define __otbVectorDataToHistogramFilter_h

#include  "itkProcessObject.h"
#include  "itkSize.h"
#include  "itkHistogram.h"

namespace otb
{
  /** \class VectorDataToHistogramFilter
   *  \brief this class computes an histogram orientation.
   *         For every Segment, it computes its orientation and length then
   *         accumulate in the bin theta the length L ( Bin[theta ] += L )
   */

template <class TVectorData, class THistogram>
class ITK_EXPORT VectorDataToHistogramFilter :
  public itk::ProcessObject
  {
  public:
    /** Standard class typedefs. */
    typedef VectorDataToHistogramFilter                            Self;
    typedef itk::ProcessObject                                     Superclass;
    typedef itk::SmartPointer<Self>                                Pointer;
    typedef itk::SmartPointer<const Self>                          ConstPointer;

    /** Run-time type information (and related methods). */
    itkTypeMacro(VectorDataToHistogramFilter, itk::ProcessObject);
    
    /** Method for creation through the object factory. */
    itkNewMacro(Self);

    /** VectorData TypeDef*/
    typedef TVectorData                                         VectorDataType;
    typedef typename VectorDataType::Pointer                    VectorDataPointerType;
    typedef typename VectorDataType::DataNodeType               DataNodeType;
    typedef typename DataNodeType::Pointer                      DataNodePointerType;
    typedef typename DataNodeType::PolygonType::VertexType      VertexType;
    typedef typename DataNodeType::PolygonType::VertexListType  VertexListType;
    
    typedef typename VectorDataType::DataTreeType::TreeNodeType InternalTreeNodeType;
    typedef typename InternalTreeNodeType::ChildrenListType     ChildrenListType;

    /** Histogram*/
    
    typedef THistogram                          HistogramType;
    typedef typename HistogramType::Pointer     HistogramPointerType;

    //Origin
    typedef itk::Point<double, 2>                                 PointType;
    
    //Get the histogram
    itkGetObjectMacro(Histogram, HistogramType);

    //Get/Set origin
    itkSetMacro(Origin, PointType);
    
    //Get Mean
    itkGetMacro(Mean, double);
    //Get Variance
    itkGetMacro(Sigma, double);
    //Get the inverse histogram
    virtual HistogramPointerType GetInverseHistogram();
    
    virtual void SetVectorData(VectorDataType * vector);
    virtual const VectorDataType  * GetVectorData();
    virtual  VectorDataType* GetOutput();

  protected:
    virtual void GenerateData();
    virtual void ProcessNode(InternalTreeNodeType * source);
    virtual void VertexToHistogram(typename VertexListType::ConstPointer vertexlist);
    virtual void UpdateHistogram(VertexType currentVertex, VertexType nextVertex);
    virtual void ComputeHistogramVariance();
    
    VectorDataToHistogramFilter();
    virtual ~VectorDataToHistogramFilter() {};
    
  private:
    VectorDataToHistogramFilter(const Self&); //purposely not implemented
    void operator=(const Self&); //purposely not implemented
    
    double                        m_Sigma;
    double                        m_Mean;
    itk::Size<1>                  m_Size;
    PointType                     m_Origin;

    VectorDataPointerType         m_VectorData;
    HistogramPointerType          m_Histogram;
    HistogramPointerType          m_InverseHistogram;
  };


}
#ifndef ITK_MANUAL_INSTANTIATION
#include "otbVectorDataToHistogramFilter.txx"
#endif

#endif
