/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/

#include "otbImageToDBRegistrationViewGUI.h"
#include <FL/Fl_File_Chooser.H>
#include <FL/Fl_Color_Chooser.H>
#include <ctime>
#include "otbMacro.h"
#include "itkExceptionObject.h"
#include "otbMsgReporter.h"
#include "otbImageToDBRegistrationEnum.h"

namespace otb
{

ImageToDBRegistrationViewGUI
::ImageToDBRegistrationViewGUI()
{
  // Model pointer and listener registration
  m_ImageToDBRegistrationModel = ImageToDBRegistrationModel::GetInstance();
  m_ImageToDBRegistrationModel->RegisterListener(this);
  m_FocusingVisuView = FocusingVisuViewType::New();
  m_FocusingVisuView->SetFocusingModel( m_ImageToDBRegistrationModel->GetFocusingVisuModel() );

  srand((unsigned)time(0));
  m_LastPath = "";
  m_MasterViewSelected = true;
  // Init with blue
  m_ClassColor.Fill(0);
  m_ClassColor[2]=1.;
  m_ClassColor[3]=1;

  m_Counter = 0;
  m_ResultInListId.clear();

  this->Build();
  this->BuildDEM();
  
  this->Show();

  dROIColor->color(FL_BLUE);
  dVDList->selection_color(FL_BLUE);
}


ImageToDBRegistrationViewGUI
::~ImageToDBRegistrationViewGUI()
{
}

/*
void
ImageToDBRegistrationViewGUI
::LaunchSudo()
{
  Sudoku s;
  s.show();
  s.load_game();
  Fl::run();
}
*/

void
ImageToDBRegistrationViewGUI
::InitWidgets()
{
  assert(m_ImageToDBRegistrationController.IsNotNull() && "The controller is not created");

  // MasterScroll
  gMasterScroll->add(m_FocusingVisuView->GetMasterScrollWidget());
  gMasterScroll->resizable(m_FocusingVisuView->GetMasterScrollWidget());
  gMasterScroll->box(FL_NO_BOX);
  m_FocusingVisuView->GetMasterScrollWidget()->show();
  m_FocusingVisuView->GetMasterScrollWidget()->resize( gMasterScroll->x(), gMasterScroll->y(), gMasterScroll->w(), gMasterScroll->h() );

  // WorkingScroll
  gWorkingScroll->add(m_FocusingVisuView->GetWorkingScrollWidget());
  gWorkingScroll->resizable(m_FocusingVisuView->GetWorkingScrollWidget());
  gWorkingScroll->box(FL_NO_BOX);
  m_FocusingVisuView->GetWorkingScrollWidget()->show();
  m_FocusingVisuView->GetWorkingScrollWidget()->resize( gWorkingScroll->x(), gWorkingScroll->y(), gWorkingScroll->w(), gWorkingScroll->h() );
     

  // WorkingFull
  gWorkingFull->add(m_FocusingVisuView->GetWorkingFullWidget());
  gWorkingFull->resizable(m_FocusingVisuView->GetWorkingFullWidget());
  gWorkingFull->box(FL_NO_BOX);
  m_FocusingVisuView->GetWorkingFullWidget()->hide();
  m_FocusingVisuView->GetWorkingFullWidget()->resize( gWorkingFull->x(), gWorkingFull->y(), gWorkingFull->w(), gWorkingFull->h() );

  gMasterScroll->show();
  gWorkingScroll->show();
  gWorkingFull->hide();
}

void
ImageToDBRegistrationViewGUI
::OpenImage()
{
  const char * cfname = fl_file_chooser("Pick an image file", "*.*", m_LastPath.c_str());
  Fl::check();
  wMainWindow->redraw();
  if (cfname == NULL || strlen(cfname) < 1) return;

  m_LastPath = itksys::SystemTools::GetFilenamePath(cfname);
  this->ClearAll();
  m_ImageToDBRegistrationController->OpenInputImage(cfname);
  mSaveFull->deactivate();

}


void
ImageToDBRegistrationViewGUI
::InitActivations()
{
  mDEM->deactivate();
  mImportVector->activate();
}


void
ImageToDBRegistrationViewGUI
::UpdateInformation()
{
  std::string imName = m_ImageToDBRegistrationModel->GetInputFileName();
  itk::OStringStream oss;
  oss.str("");
  oss<<"Image To Data Base Registration Application";
  oss<<" : "<<imName.substr(imName.find_last_of("/")+1, imName.size());
  oss<<" ("<<m_ImageToDBRegistrationModel->GetInputImage()->GetNumberOfComponentsPerPixel();
  if(m_ImageToDBRegistrationModel->GetInputImage()->GetNumberOfComponentsPerPixel() != 1)
    oss<<" bands , ";
  else
    oss<<" band , ";

  oss<<m_ImageToDBRegistrationModel->GetInputImage()->GetLargestPossibleRegion().GetSize()<<")";
  wMainWindow->label(oss.str().c_str());
}


void
ImageToDBRegistrationViewGUI
::Show()
{
  wMainWindow->show();
}

void
ImageToDBRegistrationViewGUI
::SwitchScroll()
{
  if( m_MasterViewSelected == true )
    {
      m_FocusingVisuView->GetMasterScrollWidget()->hide();
      gMasterScroll->hide();
      m_FocusingVisuView->GetWorkingFullWidget()->show();
      gWorkingFull->show();
    }
  else
    {
      m_FocusingVisuView->GetWorkingFullWidget()->hide();
      gWorkingFull->hide();
      m_FocusingVisuView->GetMasterScrollWidget()->show();
      gMasterScroll->show();
    }
  m_MasterViewSelected = !m_MasterViewSelected;
}


void
ImageToDBRegistrationViewGUI
::SaveExtract()
{
  if( gImageRef->value() == 1 )
    {
      const char * cfname = fl_file_chooser("Save as...", "*.{shp, kml}", m_LastPath.c_str());
      Fl::check();
      wMainWindow->redraw();
      if (cfname == NULL || strlen(cfname)<1) return;

      m_LastPath = itksys::SystemTools::GetFilenamePath(cfname);
      m_ImageToDBRegistrationController->SaveVectorData(cfname, otb::EXTRACT);
    }
  // correct the image
  else if( gDBRef->value() == 1 )
    {
      const char * cfname = fl_file_chooser("Output Image Name", "*.*", m_LastPath.c_str());
      Fl::check();
      wMainWindow->redraw();
      if (cfname == NULL || strlen(cfname)<1)
       {
         return;
       }
      m_ImageToDBRegistrationController->SaveImage(cfname);
    }

}


void
ImageToDBRegistrationViewGUI
::OpenVector()
{
  const char * cfname = fl_file_chooser("Open Vector Data...", "*.{shp, kml}", m_LastPath.c_str());
  Fl::check();
  wMainWindow->redraw();
  if (cfname == NULL || strlen(cfname)<1)
    {
      return;
    }
  // Set a random color to the vector data
  this->SetRandomColor();
  m_ImageToDBRegistrationController->LoadVectorData(cfname);
  std::string name = cfname;
  dVDList->add(name.substr( name.find_last_of("/")+1, name.size() ).c_str() );
  dVDList->redraw();
  dVDList->value(dVDList->size());
  dVDList->redraw();
  this->UpdateListSelectionColor();
  mDEM->activate();
}


void
ImageToDBRegistrationViewGUI
::ClearAll()
{
  m_ImageToDBRegistrationController->ClearAllVectorData();
  m_Counter = 0;
  dVDList->clear();
  m_ResultInListId.clear();
  dVDList->redraw();
}

void
ImageToDBRegistrationViewGUI
::Delete()
{
  int id = dVDList->value();
  if( id != 0 )
    {
      m_ImageToDBRegistrationController->ClearVectorData(id-1);
      dVDList->remove(id);
      // select another line in the list
      if( id <= dVDList->size() )
       {
         dVDList->value(id);
       }
      else
       {
         dVDList->value(dVDList->size());
       }

      // Supress the corresponding m_ResultInListId element
      unsigned int i = 0;
      bool ok = false;
      while( i<m_ResultInListId.size() && ok==false )
       {
         if(static_cast<int>(m_ResultInListId[i])==id)
           {
             m_ResultInListId.erase(m_ResultInListId.begin()+i);
             ok = true;
           }
         i++;
       }
      if (ok==true)
       {
         // Shift the m_ResultInListId higher values of 1
         for(unsigned int k=0; k<m_ResultInListId.size(); k++)
           {
             if(static_cast<int>(m_ResultInListId[k])>id)
              m_ResultInListId[k]--;
           }
       }

      this->UpdateListSelectionColor();
    }
}


void
ImageToDBRegistrationViewGUI
::UpdateDEMSettings()
{
  if( gUseInputDEM->value()==1 || gUseOutputDEM->value()==1 )
    {
      m_ImageToDBRegistrationController->SetDEMDirectory( gDEMPath->value() );
      m_ImageToDBRegistrationController->UseDEM(true);
    }
  else
    {
      m_ImageToDBRegistrationController->UseDEM(false);
    }
}


void
ImageToDBRegistrationViewGUI
::ChangeROIColor()
{
  m_ClassColor = m_ImageToDBRegistrationModel->GetFocusingVisuModel()->GetColor();
  double r = (double)m_ClassColor[0];
  double g = (double)m_ClassColor[1];
  double b = (double)m_ClassColor[2];

  int ok = fl_color_chooser("Changed class color:", r, g, b);

  if (ok)
    {
      m_ClassColor[0]=(float)r;
      m_ClassColor[1]=(float)g;
      m_ClassColor[2]=(float)b;
      
      fl_color(static_cast<unsigned char>(255*m_ClassColor[0]),
              static_cast<unsigned char>(255*m_ClassColor[1]),
              static_cast<unsigned char>(255*m_ClassColor[2]));
      
      dROIColor->color(fl_color());
      dVDList->selection_color(fl_color());
      
      if(dVDList->value() != 0)
       {
         dVDList->redraw();
         m_ImageToDBRegistrationController->ChangeVectorDataColor( dVDList->value()-1, m_ClassColor );
       }
      else
       {
         m_ImageToDBRegistrationController->ChangeColor( m_ClassColor );
       }
    }
}


void
ImageToDBRegistrationViewGUI
::ChangeReference( int i )
{
  if(i==0)
    {
      gDBRef->value( 1-gImageRef->value() );
      m_FocusingVisuView->SetGLComponentsStatus(true, m_ImageToDBRegistrationModel->GetDBListId());
      if(m_ImageToDBRegistrationModel->GetWorkingVisuModel()->GetNumberOfLayers()>1)
       m_ImageToDBRegistrationModel->GetWorkingVisuModel()->GetLayer(1)->SetVisible(false);
      m_ImageToDBRegistrationModel->GetWorkingVisuModel()->Update();

    }
  else if(i==1)
    {
      gImageRef->value( 1-gDBRef->value() );
      m_FocusingVisuView->SetGLComponentsStatus(false, m_ImageToDBRegistrationModel->GetDBListId());
      if(m_ImageToDBRegistrationModel->GetWorkingVisuModel()->GetNumberOfLayers()>1)
       m_ImageToDBRegistrationModel->GetWorkingVisuModel()->GetLayer(1)->SetVisible(true);
     m_ImageToDBRegistrationModel->GetWorkingVisuModel()->Update();
    }
}

void
ImageToDBRegistrationViewGUI
::UpdateListSelectionColor()
{
  if( dVDList->size()!=0 && dVDList->value()<=static_cast<int>(m_ImageToDBRegistrationModel->GetColorList().size()) )
    {
      ColorType color;
      color = m_ImageToDBRegistrationModel->GetColorList()[dVDList->value()-1];
      fl_color(static_cast<unsigned char>(255*color[0]),
              static_cast<unsigned char>(255*color[1]),
              static_cast<unsigned char>(255*color[2]));
      dVDList->selection_color(fl_color());
      dVDList->redraw();
      dROIColor->color(fl_color());
      dROIColor->redraw();
    }
}

void
ImageToDBRegistrationViewGUI
::Notify()
{
  if( m_ImageToDBRegistrationModel->GetHasNewInput()==true )
    {
      gMasterScroll->remove(m_FocusingVisuView->GetMasterScrollWidget());
      gWorkingScroll->remove(m_FocusingVisuView->GetWorkingScrollWidget());
      gWorkingFull->remove(m_FocusingVisuView->GetWorkingFullWidget());
      this->UpdateInformation();
      this->InitWidgets();
      this->InitActivations();
    }
  if( m_ImageToDBRegistrationModel->GetExtractHasNewDB()==true )
    {
      itk::OStringStream oss;
      oss.str("");
      oss<<"Result ";
      oss<<m_Counter+1;
      oss<<" :";
      oss<<gColinearityThresh->value();
      oss<<" , ";
      oss<<gValidSegmentThresh->value();
       oss<<" , ";
      oss<<gTripletMaxDist->value();

      m_Counter++;
      dVDList->add(oss.str().c_str() );
      dVDList->value(dVDList->size());
      this->UpdateListSelectionColor();

      m_ResultInListId.push_back(dVDList->size());
      mDEM->activate();
      mSaveFull->activate();
    }
  if( m_ImageToDBRegistrationModel->GetFullHasNewDB()==true )
    {
      itk::OStringStream oss;
      oss.str("");
      oss<<"Full Result: ";
      oss<<gColinearityThresh->value();
      oss<<" , ";
      oss<<gValidSegmentThresh->value();
      oss<<" , ";
      oss<<gTripletMaxDist->value();
      
      dVDList->add(oss.str().c_str() );
      dVDList->value(dVDList->size());
      this->UpdateListSelectionColor();
    }
  // remove all "Results VD" informations
  if( m_ImageToDBRegistrationModel->GetFocusingVisuModel()->GetNewExtractVDDisplayed() == true )
    {
      // Sort in decrease values the vector to erase it from the end (insertion sort)
      unsigned int j;
      for(unsigned int i=0; i<m_ResultInListId.size(); ++i)
       {
         unsigned int val = m_ResultInListId[i];
         for( j=i; (j>0 && m_ResultInListId[j-1]<val); --j )
           {
             m_ResultInListId[j] = m_ResultInListId[j-1];
           }
         m_ResultInListId[j] = val;
       }

      // Now erase. CAUTION : remove change the vector size...
      for(unsigned int k=0; k<m_ResultInListId.size(); k++)
       {
         dVDList->remove(m_ResultInListId[k]);
       }
      dVDList->redraw();
      m_ResultInListId.clear();
      m_Counter = 0;
      mSaveExtract->activate();
    }
}


void
ImageToDBRegistrationViewGUI
::RunOverExtract()
{

  // correct the DB
  if( gImageRef->value() == 1 )
    {
      // Set a random color to the vector data
      this->SetRandomColor();
      m_ImageToDBRegistrationController->RunOverExtract(otb::DATABASE);
    }
  // correct the image
  else if( gDBRef->value() == 1 )
    {
      m_ImageToDBRegistrationController->RunOverExtract(otb::IMAGE);
    }
}

void
ImageToDBRegistrationViewGUI
::Run()
{
  // correct the DB
  if( gImageRef->value() == 1 )
    {
      const char * cfname = fl_file_chooser("Save as...", "*.{shp, kml}", m_LastPath.c_str());
      Fl::check();
      wMainWindow->redraw();
      if (cfname == NULL || strlen(cfname) < 1) return;

      m_LastPath = itksys::SystemTools::GetFilenamePath(cfname);
      // Set a random color to the vector data
      this->SetRandomColor();
      m_ImageToDBRegistrationController->Run(cfname, otb::DATABASE);
    }
  // correct the image
  else if( gDBRef->value() == 1 )
    {
      const char * cfname = fl_file_chooser("Output Image Name", "*.*", m_LastPath.c_str());
      Fl::check();
      wMainWindow->redraw();
      if (cfname == NULL || strlen(cfname)<1)
       {
         return;
       }
      m_ImageToDBRegistrationController->Run(cfname, otb::IMAGE);
    }
}

void
ImageToDBRegistrationViewGUI
::ChangeAlpha()
{
  m_ImageToDBRegistrationController->ChangeAlpha();
}

void
ImageToDBRegistrationViewGUI
::SetRandomColor()
{
  // Update Vector Data color
  m_ClassColor[0]=rand()/(RAND_MAX+1.0);
  m_ClassColor[1]=rand()/(RAND_MAX+1.0);
  m_ClassColor[2]=rand()/(RAND_MAX+1.0);
  m_ClassColor[3]=1.0;
  m_ImageToDBRegistrationController->ChangeColor( m_ClassColor );
}

void
ImageToDBRegistrationViewGUI
::Quit()
{
  wMainWindow->hide();
  wDEM->hide();
  MsgReporter::GetInstance()->Hide();
}


}


