/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbImageToDBRegistrationViewGUI_h
#define __otbImageToDBRegistrationViewGUI_h


#include "otbListenerBase.h"

// Disabling deprecation warning
#ifdef _MSC_VER
#pragma warning(push)
#pragma warning(disable:4996)
#endif
#include "otbImageToDBRegistrationViewGroup.h"
#ifdef _MSC_VER
#pragma warning(pop)
#endif

#include <FL/Fl_Group.H>
#include "otbImageToDBRegistrationModel.h"
#include "otbImageToDBRegistrationControllerInterface.h"


namespace otb
{
/** \class ImageToDBRegistrationViewGUI
 *
 */
class ITK_EXPORT ImageToDBRegistrationViewGUI
  : public ListenerBase, public ImageToDBRegistrationViewGroup, public itk::Object
{
public:
  /** Standard class typedefs */
  typedef ImageToDBRegistrationViewGUI          Self;
  typedef itk::Object   Superclass;
  typedef itk::SmartPointer<Self>       Pointer;
  typedef itk::SmartPointer<const Self> ConstPointer;

  /** Standards macros */
  itkNewMacro(Self);
  itkTypeMacro(ImageToDBRegistrationViewGUI, itk::Object);


  // Pixel type
  typedef double PixelType;

  typedef ImageToDBRegistrationModel                            ImageToDBRegistrationModelType;
  typedef ImageToDBRegistrationModelType::FocusingVisuModelType FocusingVisuModelType;
  typedef FocusingVisuModelType::Pointer                        FocusingVisuModelPointerType;
  typedef ImageToDBRegistrationModelType::FocusingVisuViewType  FocusingVisuViewType;
  typedef FocusingVisuViewType::Pointer                         FocusingVisuViewPointerType;
  typedef FocusingVisuModelType::ImageLayerRenderingModelType   VisuModelType;
  typedef VisuModelType::Pointer                                VisuModelPointerType;
  typedef FocusingVisuViewType::ImageViewType                   VisuViewType;
  typedef VisuViewType::Pointer                                 VisuViewPointerType;
  typedef VisuViewType::ImageWidgetType                         VisuImageWidgetType;
  typedef VisuImageWidgetType::Pointer                          VisuImageWidgetPointerType;
  typedef FocusingVisuModelType::ColorType                      ColorType;


  /** Set the controller */
  itkGetObjectMacro(ImageToDBRegistrationController, ImageToDBRegistrationControllerInterface);

  void SetImageToDBRegistrationController(ImageToDBRegistrationControllerInterface *pController)
  {
    m_ImageToDBRegistrationController = pController;
   // Set the model
    m_FocusingVisuView->SetFocusingController(m_ImageToDBRegistrationController->GetFocusingVisuController());
  }

  itkGetMacro(FocusingVisuView, FocusingVisuViewPointerType);
  VisuViewPointerType GetWorkingVisuView(){
    return m_FocusingVisuView->GetWorkingViewer();
  }
  VisuViewPointerType GetMasterVisuView(){
    return m_FocusingVisuView->GetMasterViewer();
  }
  VisuImageWidgetPointerType GetMasterScrollWidget(){
    return m_FocusingVisuView->GetMasterScrollWidget();
  }

  void InitWidgets();
  void UpdateInformation();

  /** Standard Running macro */
  otbRunningMacro();

  void Show();

  /** Constructor */
  ImageToDBRegistrationViewGUI();

  /** Callbacks */
  virtual void Quit();
  virtual void RunOverExtract();
  virtual void ClearAll();
  virtual void Delete();
  virtual void SwitchScroll();

protected:

  /** Destructor */
  virtual ~ImageToDBRegistrationViewGUI();
  /** Callbacks */
  virtual void Notify();
  virtual void OpenImage();
  virtual void OpenVector();
  virtual void UpdateDEMSettings();
  //virtual void LaunchSudo();
  virtual void ChangeROIColor();
  virtual void UpdateListSelectionColor();
  virtual void ChangeReference( int i );
  virtual void Run();
  virtual void SaveExtract();
  virtual void ChangeAlpha();
  virtual void SetRandomColor();


  void InitActivations();

 private:
  ImageToDBRegistrationViewGUI(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

  /** Pointer to the model */
  ImageToDBRegistrationModel::Pointer m_ImageToDBRegistrationModel;
  /** Pointer to the controller */
  ImageToDBRegistrationControllerInterface::Pointer m_ImageToDBRegistrationController;
  /** Store the last path. */
  std::string m_LastPath;
  /** Pointer to the focusing visu view. */
  FocusingVisuViewPointerType                 m_FocusingVisuView;
  /** Which kind of view is selected. */
  bool m_MasterViewSelected;
  /** Color */
  ColorType m_ClassColor;
  /** DB Result number */
  unsigned int m_Counter;
  /** Index list of Result */
  std::vector<unsigned int> m_ResultInListId;
};
}//end namespace otb

#endif
