/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/

#include "itkExceptionObject.h"
#include "otbImageToDBRegistrationController.h"
#include "otbImageToDBRegistrationViewGUI.h"
#include "otbImageFileReader.h"
#include "otbCommandLineArgumentParser.h"
#include "otbMsgReporter.h"

#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>

#include "otbI18n.h"

int main(int argc, char* argv[])
{
  otbI18nMacro();
  // Parse command line parameters
  typedef otb::CommandLineArgumentParser ParserType;
  ParserType::Pointer parser = ParserType::New();

  parser->AddInputImage(false); //Optionnal parameter
  parser->AddOptionNParams("--DataBase","Data Base path","-DB", false);
  parser->AddOption("--DEM","DEM directory path","-DEM", 1, false);

  typedef otb::CommandLineArgumentParseResult ParserResultType;
  ParserResultType::Pointer  parseResult = ParserResultType::New();

  try
  {
    parser->ParseCommandLine(argc, argv, parseResult);
  }
  catch ( itk::ExceptionObject & err )
  {
    std::string descriptionException = err.GetDescription();
    if (descriptionException.find("ParseCommandLine(): Help Parser") != std::string::npos)
    {
      std::cout << "WARNING : output file pixels are converted in 'unsigned char'" << std::endl;
      return EXIT_SUCCESS;
    }
    if (descriptionException.find("ParseCommandLine(): Version Parser") != std::string::npos)
    {
      return EXIT_SUCCESS;
    }
    return EXIT_FAILURE;
  }

  typedef otb::ImageToDBRegistrationController ControllerType;
  typedef otb::ImageToDBRegistrationViewGUI ViewType;

  ControllerType::Pointer controller = ControllerType::New();
  ViewType::Pointer view = ViewType::New();
  controller->SetView(view);
  controller->AddHandlers();
  view->SetImageToDBRegistrationController(controller);

  otb::MsgReporter::GetInstance()->SetTitle("Image To Data Base Registration Application");

  view->InitWidgets();
  view->Show();

  if ( parseResult->IsOptionInputImagePresent() )
    controller->OpenInputImage(parseResult->GetInputImage().c_str());
  if ( parseResult->IsOptionPresent("--DataBase") )
    {
      for(int i = 0; i<parseResult->GetNumberOfParameters("--DataBase"); i++ )
        {
       std::string name = parseResult->GetParameterString("--DataBase", i);
       controller->LoadVectorData(name.c_str());
       view->dVDList->add(name.substr( name.find_last_of("/")+1, name.size() ).c_str() );
       view->dVDList->value(view->dVDList->size());

       view->mDEM->activate();
       }
    }
  if ( parseResult->IsOptionPresent("--DEM") )
    {
      if ( parseResult->IsOptionPresent("--DataBase") )
       {
         view->gDEMPath->value(parseResult->GetParameterString("--DEM").c_str());
         view->gDEMPath->redraw();
         view->gUseInputDEM->value(1);
         view->gUseOutputDEM->value(1);
         controller->SetDEMDirectory(parseResult->GetParameterString("--DEM").c_str());
         controller->UseDEM(true);
       }
      else
       std::cout<<"DEM can't be loaded without DataBase..."<<std::endl;
    }


  Fl::check();

  otbGenericMsgDebugMacro(<<"Running GUI ...");
  Fl::run();


  return EXIT_SUCCESS;

}
