/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/

#include "otbLandCoverMapController.h"
#include <FL/fl_ask.H>
namespace otb
{

LandCoverMapController
::LandCoverMapController()
{}

LandCoverMapController
::~LandCoverMapController()
{}


void
LandCoverMapController
::OpenInputImage(const char * filename)
{
  std::string strFilename = filename;

  try
  {
    m_Model->SetInputImage( strFilename );
  }
  catch (itk::ExceptionObject & err)
  {
    const char* msg = err.GetDescription();
    fl_alert("%s", msg);
  }

}

void
LandCoverMapController
::SetOutputFileName( const char * name )
{
  m_Model->SetOutputFileName( name );
}

void
LandCoverMapController
::SetFeatureFileName( const char * name )
{
  m_Model->SetFeatureFileName( name );
}


void
LandCoverMapController
::SaveOutput()
{
  this->CheckDescriptors();
  m_Model->SaveResult(); //GenerateOutputImage();
}


void
LandCoverMapController
::SaveFeatureImage()
{
//  this->CheckDescriptors();

  typedef std::vector<bool> VectorOfFilters;
  typedef VectorOfFilters::iterator VectorIterator;

  VectorOfFilters filters = m_Model->GetSelectedFilters();

  VectorIterator filterIterator = filters.begin();

  while ( filterIterator != filters.end() )
  {
    *filterIterator=true;
    ++filterIterator;
  }


  m_Model->SetSelectedFilters( filters );


  m_Model->SaveFeatureImage();
}


void
LandCoverMapController
::LoadModel( const char * modelName)
{
  m_Model->LoadModel( modelName );
}

void
LandCoverMapController
::ResetModel()
{
  std::vector<bool> selectedList( m_Model->GetSelectedFilters().size(), true);
  m_Model->SetSelectedFilters(selectedList);
}


void
LandCoverMapController
::CheckDescriptors()
{
  std::vector<int> ckeckedList(m_View->guiSelection->nchecked(), 0);
  int j = 1;
  while ( j<=m_View->guiSelection->nitems() )
  {
    if (m_View->guiSelection->checked(j) != 1)
    {
      m_Model->ChangeSelectedFilterStatus(j-1);
    }
    j++;
  }
}


void
LandCoverMapController
::IndexClicked(const IndexType & index)
{
  ViewType::RegionType region = m_View->GetViewer()->GetFullWidget()->GetViewedRegion();
  IndexType newIndex;
  newIndex[0]=index[0]*m_Model->GetShrinkFactor()-region.GetSize()[0]/2;
  newIndex[1]=index[1]*m_Model->GetShrinkFactor()-region.GetSize()[1]/2;
  m_View->GetViewer()->GetFullWidget()->SetUpperLeftCorner(newIndex);
  m_View->GetViewer()->GetFullWidget()->SetUpperLeftCorner(newIndex);
  //m_View->GetFullImageWidget()->redraw();
  //m_View->GetScrollWidget()->redraw();

  m_View->Notify();
}

} // end namespace otb
