/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbLandCoverMapController_h
#define __otbLandCoverMapController_h

#include "otbLandCoverMapControllerInterface.h"
#include "otbLandCoverMapViewGUI.h"

namespace otb
{
class ITK_EXPORT LandCoverMapController
      : public LandCoverMapControllerInterface
{
public:
  /** Standard class typedefs */
  typedef LandCoverMapController          Self;
  typedef LandCoverMapControllerInterface Superclass;
  typedef itk::SmartPointer<Self>          Pointer;
  typedef itk::SmartPointer<const Self>    ConstPointer;

  typedef LandCoverMapViewGUI          ViewType;
  typedef ViewType::Pointer                 ViewPointerType;
  typedef Superclass::InputImageType        InputImageType;
  typedef Superclass::InputImagePointerType InputImagePointerType;
  typedef ModelType::IndexType              IndexType;
  typedef ModelType::SizeType               SizeType;

  /** Standard type macros */
  itkTypeMacro(LandCoverMapController, Superclass);
  itkNewMacro(Self);

  itkSetObjectMacro(View, ViewType);

  virtual void OpenInputImage(const char * filename);
  virtual void SetOutputFileName( const char * name );
  virtual void SetFeatureFileName( const char * name );
  virtual void SaveOutput();
  virtual void SaveFeatureImage();
  virtual void ResetModel();
  virtual void LoadModel( const char * modelName);

  virtual void IndexClicked(const IndexType & index);

protected:
  /** Constructor */
  LandCoverMapController();
  /** Destructor */
  virtual ~LandCoverMapController();
  /** Chack selected desciptors*/
  void CheckDescriptors();

private:
  LandCoverMapController(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

  /** Pointer to the view */
  ViewPointerType m_View;


};
} //end namespace otb

#endif
