/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbLandCoverMapControllerInterface_h
#define __otbLandCoverMapControllerInterface_h

#include "otbLandCoverMapModel.h"
#include "otbImageViewerFullResolutionEventsInterface.h"

namespace otb
{
class ITK_EXPORT  LandCoverMapControllerInterface
      : public ImageViewerFullResolutionEventsInterface
{
public:
  /** Standard class typedefs */
  typedef LandCoverMapControllerInterface Self;
  typedef itk::Object                      Superclass;
  typedef itk::SmartPointer<Self>          Pointer;
  typedef itk::SmartPointer<const Self>    ConstPointer;

  typedef LandCoverMapModel           ModelType;
  typedef ModelType::InputImageType        InputImageType;
  typedef ModelType::InputImagePointerType InputImagePointerType;
  typedef ModelType::IndexType             IndexType;
  typedef ModelType::SizeType              SizeType;

  /** Standard type macros */
  itkTypeMacro(LandCoverMapControllerInterface, Superclass);

  /** Users actions */
  virtual void OpenInputImage(const char * filename) =0;
  virtual void SaveOutput() =0;
  virtual void ResetModel() =0;
  virtual void SetOutputFileName( const char * name ) =0;
  virtual void LoadModel( const char * modelName)=0;
  virtual void IndexClicked(const IndexType & index) = 0;

protected:
  /** Constructor */
  LandCoverMapControllerInterface()
  {
    m_Model = ModelType::GetInstance();
  }
  /** Destructor */
  ~LandCoverMapControllerInterface() {};

  /** The instance of the model (for commodity) */
  ModelType * m_Model;

private:
  LandCoverMapControllerInterface(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented
};
} // end namespace otb

#endif
