/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbLandCoverMapViewGUI_h
#define __otbLandCoverMapViewGUI_h


#include "otbListenerBase.h"
// Disabling deprecation warning
#ifdef _MSC_VER
#pragma warning(push)
#pragma warning(disable:4996)
#endif
#include "otbLandCoverMapView.h"
#ifdef _MSC_VER
#pragma warning(pop)
#endif
#include "otbLandCoverMapModel.h"
#include "otbLandCoverMapControllerInterface.h"
#include "itkMinimumMaximumImageCalculator.h"
#include "otbImageToVectorImageCastFilter.h"
#include "otbImageViewerBase.h"
#include "otbObjectList.h"
#include "otbFullResolutionImageWidget.h"
#include "otbMultiChannelExtractROI.h"
#include "otbVectorRescaleIntensityImageFilter.h"
#include "itkCastImageFilter.h"

#include "otbLandCoverMapControllerInterface.h"
/*#include "otbInteractiveScrollWidget.h"
#include "otbFullResolutionImageWidget.h"*/
#include "otbImageWidgetBoxForm.h"

namespace otb
{
/** \class LandCoverMapViewGUI
 *
 */
class ITK_EXPORT LandCoverMapViewGUI
      : public ListenerBase, public LandCoverMapView, public itk::Object
{
public:
  /** Standard class typedefs */
  typedef LandCoverMapViewGUI          Self;
  typedef itk::Object   Superclass;
  typedef itk::SmartPointer<Self>       Pointer;
  typedef itk::SmartPointer<const Self> ConstPointer;

  /** Standards macros */
  itkNewMacro(Self);
  itkTypeMacro(LandCoverMapViewGUI, itk::Object);


  // Pixel type
  typedef double PixelType;
  /*typedef InteractiveScrollWidget<PixelType, LandCoverMapControllerInterface>      ScrollWidgetType; */
  typedef FullResolutionImageWidget<PixelType>                                    FullWidgetType;
  typedef FullWidgetType::ImageType                                               ImageType;
  typedef ImageWidgetBoxForm                                                                           BoxType;
  typedef ImageViewerBase<PixelType>                 ViewerType;
  typedef ViewerType::Pointer                        ViewerPointerType;
  //typedef ViewerType::Pointer                        ViewerPointerType;
  //typedef ViewerType::FullWidgetType                 FullWidgetType;
  //typedef FullWidgetType::OverlayImageType           OverlayImageType;
  //typedef ViewerType::ImageType                      ImageType;
  typedef ImageType::PixelType                       VectorPixelType;
  typedef ImageType::RegionType                      RegionType;
  typedef RegionType::SizeType                       SizeType;
  typedef RegionType::IndexType                      IndexType;
  typedef ImageType::PointType                       PointType;
  typedef ImageType::SpacingType                     SpacingType;
  typedef ImageType::Pointer                         ImagePointerType;
  /*
  typedef ViewerType::SingleImageType                SingleImageType;
  typedef SingleImageType::SizeType                  SingleSizeType;
  typedef SingleImageType::IndexType                 SingleIndexType;
  typedef SingleImageType::PointType                 SinglePointType;
  typedef SingleImageType::SpacingType               SingleSpacingType;
  typedef SingleImageType::Pointer                   SingleImagePointerType;
  */
  typedef LandCoverMapModel                                      LandCoverMapModelType;
  //typedef LandCoverMapModelType::SingleImageType                 ModelSingleImageType;
  //typedef LandCoverMapModelType::LabeledVectorImageType          ModelLabeledVectorImageType;

  typedef VectorRescaleIntensityImageFilter<ImageType, ImageType> RescaleFilterType;
  typedef RescaleFilterType::Pointer                             RescaleFilterPointerType;

  typedef std::vector<std::string>                               StringVectorType;

  /** Set the controller */
  itkSetObjectMacro(LandCoverMapController, LandCoverMapControllerInterface);
  itkGetObjectMacro(LandCoverMapController, LandCoverMapControllerInterface);
  /** Get the Viewer */
  itkGetObjectMacro(Viewer, ViewerType);

  /*  itkGetObjectMacro(ScrollWidget, ScrollWidgetType);
    itkGetObjectMacro(FullImageWidget, FullWidgetType); */

  /** Event from the model */
  virtual void Notify();

  void Show();

  /** Constructor */
  LandCoverMapViewGUI();


protected:

  /** Destructor */
  virtual ~LandCoverMapViewGUI();

  /** Refresh the image widgets */
  virtual void RefreshViewer();

  /** Center the widgets */
  virtual void OpenImage();
  virtual void LoadModel();
  virtual void Quit();
  virtual void OK();
  virtual void ChangeOpacity();

private:
  LandCoverMapViewGUI(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

  void UpdateInformation();
  void UpdateChannelSelection();
  //  void InitInterface();

  /** Pointer to the model */
  LandCoverMapModel::Pointer m_LandCoverMapModel;
  /** Pointer to the controller */
  LandCoverMapControllerInterface::Pointer m_LandCoverMapController;

  std::string         m_FileName;
  std::string         m_ViewerName;
  std::string         m_LastPath;
  std::string         m_TemplateViewerName;
  std::string         m_DisplayedLabel;
  std::string         m_UndisplayedLabel;
  bool                m_DisplayStatus;
  int                 m_FeatureType;
  bool                m_HadViews;

  RescaleFilterPointerType          m_Rescaler;
  ViewerPointerType                 m_Viewer;


};
}//end namespace otb

#endif
