/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/

#include "itkExceptionObject.h"
#include "otbLandCoverMapController.h"
#include "otbCommandLineArgumentParser.h"

#include "otbI18n.h"

int main(int argc, char* argv[])
{
  otbI18nMacro();
  // Parse command line parameters
  typedef otb::CommandLineArgumentParser ParserType;
  ParserType::Pointer parser = ParserType::New();

  parser->AddInputImage();
  parser->AddOutputImage();

  typedef otb::CommandLineArgumentParseResult ParserResultType;
  ParserResultType::Pointer  parseResult = ParserResultType::New();

  try
  {
    parser->ParseCommandLine(argc, argv, parseResult);
  }
  catch ( itk::ExceptionObject & err )
  {
    std::string descriptionException = err.GetDescription();
    if (descriptionException.find("ParseCommandLine(): Help Parser") != std::string::npos)
    {
      std::cout << "WARNING : output file pixels are converted in 'unsigned char'" << std::endl;
      return EXIT_SUCCESS;
    }
    if (descriptionException.find("ParseCommandLine(): Version Parser") != std::string::npos)
    {
      return EXIT_SUCCESS;
    }
    return EXIT_FAILURE;
  }

  typedef otb::LandCoverMapController ControllerType;

  ControllerType::Pointer controller = ControllerType::New();

  controller->OpenInputImage( parseResult->GetInputImage().c_str() );
  controller->SetFeatureFileName( parseResult->GetOutputImage().c_str() );
  controller->SaveFeatureImage();


  return EXIT_SUCCESS;

}
