/*=========================================================================

  $Author: allain $
  $Date: 2005/06/06 16:16:56 $
  $Revision: 1.2 $
  $Id: ConfigurationFileReader.h, v 1.2 2005/06/06 16:16:56 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/ConfigurationFileReader.h, v $
  $Log: ConfigurationFileReader.h, v $
  Revision 1.2  2005/06/06 16:16:56  allain
  Gave new method names, add commentary and implement new params


=========================================================================*/

#ifndef __ConfigurationFileReader_H
#define __ConfigurationFileReader_H

#include <iostream>
#include <string>

#include <fstream>
#include <sstream>

/** Class ConfigurationFileReader
    This class enables to load the registration parameters from a text
    file.
    There is only one input method: ReadConfigFile(...).
    All the output method return the information needed by
    RegistrationMethod class (and input in this class thanks to
    PireoApplication class).
*/
class ConfigurationFileReader
{
public:
  /** Constructor.*/
  ConfigurationFileReader();
  /** Destructor.*/
  ~ConfigurationFileReader();

  /** Return true if the file could be read, false otherwise.*/
  bool ReadConfigFile(const char* filename);

  /** Return the registration number of levels.*/
  int GetNumberOfLevels()
  {
    return NumberOfLevels;
  }
  /** Return the registration transform type.*/
  unsigned int GetTransformType()
  {
    return trans;
  }
  /** Return the registration interpolator type.*/
  unsigned int GetInterpolatorType()
  {
    return interp;
  }
  /** Return the registration metric type.*/
  unsigned int GetMetricType()
  {
    return metric;
  }
  /** Return the registration optimizer type.*/
  unsigned int GetOptimizerType()
  {
    return optim;
  }
  /** Return optimizer minimize status.*/
  unsigned int GetMinimizeStatus()
  {
    return minimize;
  }
  /** Return the maximum step length in case ...*/
  double GetMaxStepLength()
  {
    return maxStepLength;
  }
  /** Return the minimum step length in case ...*/
  double GetMinStepLength()
  {
    return minStepLength;
  }
  /** Return the registration number of iterations.*/
  int GetNumberOfIterations()
  {
    return NumberOfIterations;
  }

protected:
  /** Registration number of levels.*/
  int NumberOfLevels;
  /** Registration transform type.*/
  unsigned int trans;
  /** registration interpolator type.*/
  unsigned int interp;
  /** Registration metric type.*/
  unsigned int metric;
  /** Registration optimizer type.*/
  unsigned int optim;

  /** Maximum step length of the optimizer.*/
  double maxStepLength;
  /** Minimum step length of the optimizer.*/
  double minStepLength;
  /** Minimze metric ?.*/
  unsigned int minimize;
  /** Registration number of iterations.*/
  int NumberOfIterations;
};

#endif
