
#include "GenericRegistration.h"
#include "string"
#include "otbMacro.h"

#include "itkRegularStepGradientDescentOptimizer.h"

GenericRegistration::GenericRegistration()
{
  // Instantiate multi-resolution registration
  registration  = RegistrationType::New();
  // Instantiate pyramidal images
  fixedPyramid = FixedImagePyramidType::New();
  movingPyramid = MovingImagePyramidType::New();

  //resample = ResampleFilterType::New();

  // Default value for the number of levels of the registration
  // process
  m_NumberOfLevels = 1;

  g=0;
}

GenericRegistration::~GenericRegistration()
{
}

void GenericRegistration::SetNumberOfLevels(int l)
{
  m_NumberOfLevels = l;
}

void GenericRegistration::SetFixedImage(ImageType2D::Pointer m_Im)
{
  m_FixedIm = m_Im;
}

void GenericRegistration::SetMovingImage(ImageType2D::Pointer m_Im)
{
  m_MovingIm = m_Im;
}

GenericRegistration::ImageType2D::Pointer
GenericRegistration::GetMovingImage()
{
  return m_MovingIm;
}

GenericRegistration::ImageType2D::Pointer
GenericRegistration::GetFixedImage()
{
  return m_FixedIm;
}


GenericRegistration::ParametersType
GenericRegistration::GetParams()
{
  return Params;
}


void GenericRegistration::SetTransform(TransformType::Pointer T)
{
  transform = T;
  finalTransform = T;
  initialParams = transform->GetParameters();
  Params = transform->GetParameters();

  registration->SetTransform(    transform     );
}

void GenericRegistration::SetMetric(MetricType::Pointer M)
{
  metric = M;
  registration->SetMetric( metric  );
}

void GenericRegistration::SetOptimizer(OptimizerType::Pointer O)
{
  optimizer = O;
  registration->SetOptimizer(     optimizer     );
// itk::RegularStepGradientDescentOptimizer* RSGO = dynamic_cast< itk::RegularStepGradientDescentOptimizer* >( registration->GetOptimizer() );
  //std::cout<<"OPTIM gene :"<< RSGO->GetMaximumStepLength()<<std::endl;
}

void GenericRegistration::SetInterpolator(InterpolatorType::Pointer I)
{
  interpolator = I;
  registration->SetInterpolator(  interpolator  );
}

void GenericRegistration::SetRegistrationTransformParameters(ParametersType t)
{
  Params = t;
}

GenericRegistration::ImageType2D::Pointer
GenericRegistration::DoRegister()
{
  // Insert all parameters into ITK registration object
  /*registration->SetOptimizer(     optimizer     );
  registration->SetTransform(    transform     );
  registration->SetInterpolator(  interpolator  );
  registration->SetMetric( metric  ); */

  registration->SetNumberOfLevels(m_NumberOfLevels);
  registration->SetFixedImagePyramid(fixedPyramid);
  registration->SetMovingImagePyramid(movingPyramid);

  registration->SetFixedImage( m_FixedIm );
  registration->SetMovingImage( m_MovingIm );

  registration->SetFixedImageRegion( m_FixedIm->GetBufferedRegion() );

  registration->SetInitialTransformParameters(Params);
  // Display initial transform paramters
  std::cout << "Initial Transform Parameters " << std::endl;
  std::cout << Params << std::endl;

  try
  {
    registration->StartRegistration();
  }
  catch ( itk::ExceptionObject & err )
  {
    std::cout << "ExceptionObject caught !" << std::endl;
    std::cout << err << std::endl;
  }

  // When registration process is done, get back the last transform
  // parameters.
  OptimizerType::ParametersType finalParameters =
    registration->GetLastTransformParameters();
  std::cout <<"Final Transform Parameters " << std::endl;
  std::cout << finalParameters << std::endl;
  finalTransform->SetParameters( finalParameters );

  ResampleFilterType::Pointer resample = ResampleFilterType::New();
  // Resample moving image thanks to the last registration transform.
  resample->SetTransform( finalTransform );
  resample->SetInput( m_MovingIm );
  resample->SetSize(    m_FixedIm->GetLargestPossibleRegion().GetSize() );
  resample->SetOutputOrigin(  m_FixedIm->GetOrigin() );
  resample->SetOutputSpacing( m_FixedIm->GetSpacing() );
  resample->SetDefaultPixelValue( 100 );
  try
  {
    resample->Update();
  }
  catch ( itk::ExceptionObject & err )
  {
    std::cout << "ExceptionObject caught !" << std::endl;
    std::cout << err << std::endl;
  }
  //WriteResult(resample);
  return resample->GetOutput();
}

GenericRegistration::TransformType::Pointer
GenericRegistration::GetFinalTransform()
{
  return finalTransform;
}

GenericRegistration::ImageType2D::Pointer
GenericRegistration::GetRegistrationResult()
{
  ParametersType finalParameters = optimizer->GetCurrentPosition();

  /*TransformType::ParametersType toto(finalTransform->GetNumberOfParameters());
  toto[0] = finalParameters[0];
  toto[1] = finalParameters[1]; */
  //g++;

  finalTransform->SetParameters( finalParameters );

  ResampleFilterType::Pointer resample = ResampleFilterType::New();
  resample->SetTransform( finalTransform );
  resample->SetInput( m_MovingIm );
  resample->SetSize(    m_FixedIm->GetLargestPossibleRegion().GetSize() );
  resample->SetOutputOrigin(  m_FixedIm->GetOrigin() );
  resample->SetOutputSpacing( m_FixedIm->GetSpacing() );
  resample->SetDefaultPixelValue( 100 );
  try
  {
    resample->Update();
  }
  catch ( itk::ExceptionObject & err )
  {
    std::cout << "ExceptionObject caught !" << std::endl;
    std::cout << err << std::endl;
  }
  //WriteResult(resample);
  return resample->GetOutput();
}

void GenericRegistration::WriteResult(ResampleFilterType::Pointer r)
{
  typedef otb::ImageFileWriter< otb::Image<unsigned char, 2> >  ImageWriterType;
  ImageWriterType::Pointer writer = ImageWriterType::New();
  std::string mystr;
  itk::OStringStream oss;
  oss <<"Deformed"<<"_"<<g<<".png";
  g++;
  mystr=oss.str();
  std::cout << mystr << std::endl;
  writer->SetFileName((oss.str()).c_str());

  typedef itk::RescaleIntensityImageFilter< ImageType2D, otb::Image<unsigned char, 2> >    CastFilterType;
  CastFilterType::Pointer caster = CastFilterType::New();
  caster->SetInput(r->GetOutput());
  caster->SetOutputMinimum(0);
  caster->SetOutputMaximum(255);

  writer->SetInput(caster->GetOutput());

  try
  {
    writer->Update();
  }
  catch ( itk::ExceptionObject & err )
  {
    std::cout << "ExceptionObject caught !" << std::endl;
    std::cout << err << std::endl;
  }
}
