/*=========================================================================

  $Author: allain $
  $Date: 2005/07/12 15:20:13 $
  $Revision: 1.5 $
  $Id: Grid.cxx, v 1.5 2005/07/12 15:20:13 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/Grid.cxx, v $
  $Log: Grid.cxx, v $
  Revision 1.5  2005/07/12 15:20:13  allain
  Add some functionalities

  Revision 1.4  2005/07/11 08:48:00  allain
  grid debugged

  Revision 1.3  2005/07/06 16:07:14  allain
  still a bug with the grid

  Revision 1.2  2005/06/06 16:16:56  allain
  Gave new method names, add commentary and implement new params

  Revision 1.1  2005/04/27 13:15:11  allain
  Change most of class names

  Revision 1.4  2005/03/24 16:14:28  allain
  Pre-Processing structure done

  Revision 1.3  2005/03/23 17:34:11  allain
  Pre-Processing is working

  Revision 1.2  2005/03/22 17:40:18  allain
  Grid is working and Pre-processing starts

  Revision 1.1  2005/03/15 17:28:47  allain
  Add Grid class


=========================================================================*/

#include "Grid.h"

Grid::Grid()
{
  // Unstructured grid that contains grid points and grid cells
  vgrid = vtkUnstructuredGrid::New();
  // Unstructured grid that contains grid points and grid cells used
  // to change values and to initialize grid
  m_PointSet = vtkUnstructuredGrid::New();

  // Glyph that represent grid point
  circle = vtkGlyphSource2D::New();
  circle->SetGlyphTypeToCircle();

  glyph = vtkGlyph3D::New();
  glyph->SetInput(vgrid);
  // Grid points are represented as circles
  glyph->SetSource(circle->GetOutput());
  glyph->SetScaleModeToScaleByVector();
  glyph->OrientOn();
  glyph->SetVectorModeToUseVector();
  glyph->SetColorModeToColorByVector();

  // Grid points mapper
  sgridMapper = vtkPolyDataMapper::New();
  sgridMapper->SetInput(glyph->GetOutput());

  // Grid point actor
  sgridActor = vtkActor::New();
  sgridActor->SetMapper(sgridMapper);
  sgridActor->GetProperty()->SetColor(1, 0, 0);

  mapper = vtkDataSetMapper::New();
  mapper->SetInput(vgrid);

  actor = vtkActor::New();
  actor->SetMapper(mapper);
  actor->GetProperty()->SetDiffuseColor( 1, .4, .5);
  actor->GetProperty()->SetRepresentationToWireframe();

  ren = vtkRenderer::New();
  ren->AddActor(actor);
  ren->AddActor(sgridActor);
  ren->SetBackground(0.5, 0.5, 0.5);

  NumberOfPointsAlongAxes = 5;
}

Grid::~Grid()
{
  m_PointSet->Delete();
  sgridActor->Delete();
  sgridMapper->Delete();
  mapper->Delete();
  actor->Delete();
  ren->Delete();
  vgrid->Delete();
  glyph->Delete();
  circle->Delete();

}

vtkRenderer* Grid::GetRenderer()
{
  return ren;
}

void Grid::SetBackgroundColor(float r, float g, float b)
{
  ren->SetBackground(r, g, b);
}

void Grid::SetGridPointColor(float r, float g, float b)
{
  sgridActor->GetProperty()->SetColor(r, g, b);
}

void Grid::SetImageDimensions(int dimx, int dimy)
{
  dims = new int[2];
  dims[0] = dimx;
  dims[1] = dimy;
}

void Grid::SetParameter(int NbOfPointsAlongAxes)
{
  NumberOfPointsAlongAxes = NbOfPointsAlongAxes;
}

void Grid::Initialize()
{
  int i, j;

  NbPoints = NumberOfPointsAlongAxes*NumberOfPointsAlongAxes;

  vtkPoints* points = vtkPoints::New();
  points->Allocate(NbPoints);

  double stepx = (double)dims[0]/(double)(NumberOfPointsAlongAxes-1);
  double stepy = (double)dims[1]/(double)(NumberOfPointsAlongAxes-1);

#define min(a, b) (((a)<(b))?(a):(b))
  int s = min(static_cast<int>(stepx), static_cast<int>(stepy));
  // Scale circle representing the point in order for it to be visible
  circle->SetScale(s/5);
#undef min

  // Create a rectangular grid for initialization
  int h =0;
  for (i = 0; i < NumberOfPointsAlongAxes; i++)
  {
    for (j = 0; j < NumberOfPointsAlongAxes; j++)
    {
      points->InsertPoint(h, static_cast<int>(i*stepx), static_cast<int>(j*stepy), 0);
      h++;
    }
  }

  vtkCellArray* cells = vtkCellArray::New();
  // Create rectangular grid cells
  for (i=0; i < NumberOfPointsAlongAxes-1; i++)
  {
    for (j=0; j < NumberOfPointsAlongAxes-1; j++)
    {
      cells->InsertNextCell(4);
      cells->InsertCellPoint( i*NumberOfPointsAlongAxes + j);
      cells->InsertCellPoint( i*NumberOfPointsAlongAxes + (j+1));
      cells->InsertCellPoint( (i+1)*NumberOfPointsAlongAxes + (j+1));
      cells->InsertCellPoint( (i+1)*NumberOfPointsAlongAxes + j);
    }
  }

  int numCells = (int)cells->GetNumberOfCells();
  int *types = new int[numCells];
  for (i=0; i<numCells; i++)
    types[i] = VTK_POLYGON;

  // Insert grid points and grid cells
  m_PointSet->SetPoints(points);
  m_PointSet->SetCells(types, cells);

  // Clean up vtk objects (no vtkSmartPointer ... )
  cells->Delete();
  points->Delete();

}

void Grid::SetRenderer()
{
  if (sgridActor!=NULL)
    ren->RemoveActor(sgridActor);
  if (actor!=NULL)
    ren->RemoveActor(actor);

  // Update grid points
  vgrid->SetPoints(m_PointSet->GetPoints());
  vgrid->SetCells(VTK_POLYGON, m_PointSet->GetCells());

  // Insert grid actor to renderer
  ren->AddActor(sgridActor);
  ren->AddActor(actor);
}
