/*=========================================================================

  $Author: allain $
  $Date: 2005/06/06 16:16:56 $
  $Revision: 1.2 $
  $Id: Grid.h, v 1.2 2005/06/06 16:16:56 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/Grid.h, v $
  $Log: Grid.h, v $
  Revision 1.2  2005/06/06 16:16:56  allain
  Gave new method names, add commentary and implement new params

  Revision 1.1  2005/04/27 13:15:11  allain
  Change most of class names

  Revision 1.3  2005/03/23 17:34:11  allain
  Pre-Processing is working

  Revision 1.2  2005/03/22 17:40:18  allain
  Grid is working and Pre-processing starts

  Revision 1.1  2005/03/15 17:28:47  allain
  Add Grid class


=========================================================================*/

#ifndef __GRID_H__
#define __GRID_H__

/* C++ Libraries*/
#include "math.h"
#include "stdlib.h"

/* VTK Libraries*/
#include "vtkRenderer.h"
#include "vtkActor.h"
#include "vtkUnstructuredGrid.h"
#include "vtkPolygon.h"
#include "vtkDataSetMapper.h"
#include "vtkCellArray.h"
#include "vtkPoints.h"
#include "vtkGlyphSource2D.h"
#include "vtkPolyDataMapper.h"
#include "vtkGlyph3D.h"
#include "vtkProperty.h"
#include "vtkRenderWindowInteractor.h"
#include "vtkRenderWindow.h"

/** Class Grid
    Take care of everything concerning the renderer of the grid.
    In order to generate a grid, we need to:
    - input the dimensions of the image (method SetImageDimensions)
    - input the desired number of point in the grid along each axe
    (method SetParameter)
    - and then intialize the grid with Initialize() method.
    As the grid is only here to visualize the registration process at
    each iteration, the grid is set from the PireoObserverManager
    class. The grid object instantiated in PireoApplication class is only
    used by this class to update the grid (see UpdateGrid() method in
    PireoApplication class).
    Everything input in grid is done from the PireoObserverManager
    object and the only element return by grid (i.e the renderer) is
    used by PireoApplication object, where grid is encapsulated).
*/
class Grid
{
public:
  /** Constructor: instantiate all VTK objects needed.*/
  Grid();
  /** Destructor.*/
  ~Grid();

  /** Set the color of the points in the grid (not working yet).*/
  void SetGridPointColor(float r, float g, float b);
  /** Set the color of the background of the grid (not working yet).*/
  void SetBackgroundColor(float r, float g, float b);

  /** Return renderer corresponding to the grid.*/
  vtkRenderer* GetRenderer();
  /** Set grid renderer from all the input information.*/
  void SetRenderer();
  /** Need to input the dimensions of image on which the grid is
  created*/
  void SetImageDimensions(int dimx, int dimy);
  /** Set the only parameter of this object i.e the number of points
  in the grid along each axe*/
  void SetParameter(int NbOfPointsAlongAxes);
  /** Generate the grid from the number of points set and the image
  dimensions*/
  void Initialize();
  /** Return the VTK point set that contains grid points and grid cells*/
  vtkUnstructuredGrid* GetPointSet()
  {
    return m_PointSet;
  }

protected:
  /** VTK point set*/
  vtkUnstructuredGrid* m_PointSet;
  /** VTK unstructuredGrid*/
  vtkUnstructuredGrid* vgrid;
  /** VTK glyph (used to generate grid points)*/
  vtkGlyphSource2D* circle;
  /** VTK glyph (used to generate grid points)*/
  vtkGlyph3D* glyph;
  /** VTK grid points mapper */
  vtkPolyDataMapper *sgridMapper;
  /** VTK grid points actor*/
  vtkActor *sgridActor;
  /** VTK grid cells mapper*/
  vtkDataSetMapper* mapper;
  /** VTK grid cells actor*/
  vtkActor* actor;
  /** VTK grid renderer*/
  vtkRenderer* ren;

  /** Image dimensions*/
  int* dims;
  /** Number of points in the grid*/
  int NbPoints;
  /** Number of points in the grid along each axe*/
  int NumberOfPointsAlongAxes;

};

#endif

