/*=========================================================================

  $Author: allain $
  $Date: 2005/07/13 13:33:50 $
  $Revision: 1.18 $
  $Id: PireoApplication.cxx, v 1.18 2005/07/13 13:33:50 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/PireoApplication.cxx, v $
  $Log: PireoApplication.cxx, v $
  Revision 1.18  2005/07/13 13:33:50  allain
  Pause button

  Revision 1.17  2005/07/12 15:20:13  allain
  Add some functionalities

  Revision 1.16  2005/07/06 16:07:14  allain
  still a bug with the grid

  Revision 1.15  2005/07/06 13:49:16  allain
  Update

  Revision 1.14  2005/07/06 13:32:21  allain
  Blender/Resampled Images changed

  Revision 1.13  2005/07/04 15:46:33  allain
  Add results auto save

  Revision 1.12  2005/06/24 15:40:09  allain
  Add metric values graph and started registration levels observer

  Revision 1.11  2005/06/20 15:16:21  allain
  Add ITK registration parameters in the save text file

  Revision 1.10  2005/06/16 15:28:08  allain
  Add new functionalities

  Revision 1.9  2005/06/15 14:41:20  allain
  Add registration level observer

  Revision 1.8  2005/06/14 15:38:03  allain
  Translation, Affine, OPOEO are working now

  Revision 1.7  2005/06/10 15:56:42  allain
  Add some functionalities for registration

  Revision 1.6  2005/06/06 16:16:56  allain
  Gave new method names, add commentary and implement new params

  Revision 1.5  2005/05/09 15:52:57  allain
  Add pause and stop buttons for generic registration

  Revision 1.4  2005/05/04 09:16:09  allain
  Add generic registration class

  Revision 1.3  2005/05/02 15:33:01  allain
  Generic Registration and Configuration File installed

  Revision 1.2  2005/04/29 13:44:53  allain
  Add Save Registration Parameters Method

  Revision 1.1  2005/04/27 13:15:11  allain
  Change most of class names

  Revision 1.14  2005/03/24 16:14:28  allain
  Pre-Processing structure done

  Revision 1.13  2005/03/23 17:34:11  allain
  Pre-Processing is working

  Revision 1.12  2005/03/22 17:40:18  allain
  Grid is working and Pre-processing starts

  Revision 1.11  2005/03/21 14:35:09  allain
  Interface modified

  Revision 1.10  2005/03/16 17:09:08  allain
  Add some commentaries

  Revision 1.9  2005/03/15 17:28:47  allain
  Add Grid class

  Revision 1.8  2005/03/15 11:04:47  allain
  Add Vector Field Window

  Revision 1.7  2005/03/08 13:11:39  allain
  Set MultiResolutionImageRegistrationMethod

  Revision 1.6  2005/03/07 16:17:27  allain
  Registration with GDAL image starts working

  Revision 1.5  2005/03/04 13:49:42  allain
  GLSliceView file modified for 2D Image

  Revision 1.4  2005/03/03 16:52:44  allain
  Registration start and its observer

  Revision 1.3  2005/02/25 15:33:10  allain
  Transparency Window Set

  Revision 1.2  2005/02/23 17:48:29  allain
  ITK To VTK IMage start

  Revision 1.1  2005/02/23 16:12:14  allain
  Add missing files


=========================================================================*/

#include "PireoApplication.h"

#ifdef _MSC_VER
#pragma warning(push)
#pragma warning(disable:4996)
#endif

/********************************************************************/
/*                        Constructor                               */
/********************************************************************/
PireoApplication::PireoApplication()
{
  // Read Image and keep information in SmartPointer type itk::Image
  // Use itkGDALIO to read data not recognized by ITK
  m_pireofilereader = new PireoFileReader();
  // Help to generate new zoomed image (image which represents a region
  // of the image loaded) and to manage also the cursors to be able to
  // move around the image
  m_zoom = new ZoomManager();
  // Manage the blending of the two images : fixed and moving
  m_transWin = new TransparencyWindowManager();
  // Take care of the ITK To VTK Transform
  m_save = new PireoFileWriter();
  // Give a sample of transforms to apply to an image (shrink filter,
  // resample filter, cast filter...)
  m_pireoFilter = new PireoFiltering();
  // Manage the registration process
  m_registration = new RegistrationMethod();
  // Manage the Vector Field that is displayed during the
  // registration process
  m_vectorField = new VectorField();
  // Manage the grid that is displayed during the registration process
  m_grid = new Grid();
  // Manage pre-processing filter
  m_filter = new PreProcessFiltering();
  // Enable loading registration parameters from a file
  m_configurationfilereader = new ConfigurationFileReader();

  // Set the renderer of the first VTK Window (down-left window)
  // This renderer is responsible for displaying the two registred
  // images in transparency
  m_ren = vtkRenderer::New();
  m_ren->SetAmbient(0.1, 0.1, 0.1);
  m_ren->TwoSidedLightingOn();
  m_ren->LightFollowCameraOn();

  // Set the renderer of the second VTK Window (down-right)
  // This renderer is responsible for displaying either the vector
  // field or hte grid during the registration process
  m_VecRen = vtkRenderer::New();
  m_VecRen->SetAmbient(0.1, 0.1, 0.1);
  m_VecRen->TwoSidedLightingOn();
  m_VecRen->LightFollowCameraOn();

  // Set first render Window
  m_renwin = vtkRenderWindow::New();
  m_renwin->AddRenderer(m_ren);
  // Set second render Window
  m_VecWin = vtkRenderWindow::New();
  m_VecWin->AddRenderer(m_VecRen);

  // Booleen used to know if the view in transparency has already been
  // displayed
  WinStatu = false;
}

/********************************************************************/
/*                        Destructor                                */
/********************************************************************/
PireoApplication::~PireoApplication()
{
  delete m_pireofilereader;
  delete m_zoom;
  delete m_transWin;
  delete m_save;
  delete m_pireoFilter;
  delete m_registration;
  delete m_vectorField;
  delete m_grid;
  delete m_filter;
  delete m_configurationfilereader;

  m_ren->Delete();
  m_VecRen->Delete();
  m_renwin->Delete();
  m_VecWin->Delete();
}

void PireoApplication::SetMetricObserver(bool a)
{
  m_registration->GetObserver()->DisplayMetric(a);
}

void PireoApplication::AutomaticSave(const char* filename, int n)
{
  //std::string ext(filename);
  // ext = ext.substr(ext.rfind("."), ext.length());

  //if( strcmp((char*)ext.c_str(),"")!=0 )
  // return;

  m_registration->GetObserver()->SetAutoSave(true);

  m_saveFilename = filename;
  m_saveType = n;
}

void PireoApplication::DisableAutoSave()
{
  m_registration->GetObserver()->SetAutoSave(false);
}

void PireoApplication::AutomaticSave(int n)
{
  std::stringstream oss;

  if (n < 10)
    oss <<m_saveFilename<<"_00"<<n<<".jpg";
  else if (n < 100)
    oss <<m_saveFilename<<"_0"<<n<<".jpg";
  else
    oss <<m_saveFilename<<"_"<<n<<".jpg";

  if (m_saveType==0)
    m_save->SaveAsITKImage(m_pireofilereader->GetImage(4), (oss.str()).c_str());
  // Save the down-sampled moving image
  else if (m_saveType==1)
    m_save->SaveAsITKImage(m_pireofilereader->GetImage(5), (oss.str()).c_str());
  // Save the deformed image given at the end of the registration process
  else if (m_saveType==2)
  {
    m_save->SaveAsITKImage(
      this->GetPireoFiltering()->CastImageToInitialState(
        this->GetRegistration()->GetRegistrationResult()), (oss.str()).c_str());
  }
  else if (m_saveType==3)
    m_save->SaveAsVTKImage(m_transWin->GetBlender(), (oss.str()).c_str());
}

/********************************************************************/
/*      Set registration parameters from a text file                */
/********************************************************************/
void PireoApplication::ReadRegistrationParametersFromFile(const char* filename)
{
  // Read configuration file
  m_configurationfilereader->ReadConfigFile(filename);

  // Set read parameters into registration method
  m_registration->SetNumberOfLevels(m_configurationfilereader->GetNumberOfLevels());
  m_registration->SetNumberOfIterations(m_configurationfilereader->GetNumberOfIterations());
  // Set registration transform type
  if (m_configurationfilereader->GetTransformType()==0)
    m_registration->SetTransformType(AFFINE);
  else if (m_configurationfilereader->GetTransformType()==1)
    m_registration->SetTransformType(SCALE);
  else if (m_configurationfilereader->GetTransformType()==2)
    m_registration->SetTransformType(TRANSLATION);
  else if (m_configurationfilereader->GetTransformType()==3)
    m_registration->SetTransformType(CENTEREDRIGID);
  // Set registration interpolator type
  if (m_configurationfilereader->GetInterpolatorType()==0)
    m_registration->SetInterpolatorType(LINEAR);
  else if (m_configurationfilereader->GetInterpolatorType()==1)
    m_registration->SetInterpolatorType(NEARESTNEIGHBOR);
  else if (m_configurationfilereader->GetInterpolatorType()==2)
    m_registration->SetInterpolatorType(BSPLINE);
  // Set registration metric type
  if (m_configurationfilereader->GetMetricType()==0)
    m_registration->SetMetricType(MUTUALINFO);
  else if (m_configurationfilereader->GetMetricType()==1)
    m_registration->SetMetricType(MATTESMUTUALINFO);
  else if (m_configurationfilereader->GetMetricType()==2)
    m_registration->SetMetricType(MEANSQUARE);
  else if (m_configurationfilereader->GetMetricType()==3)
    m_registration->SetMetricType(MEANRECIPROCALSQUARE);
  else if (m_configurationfilereader->GetMetricType()==4)
    m_registration->SetMetricType(CORRELATION);
  // Set registration optimizer type
  if (m_configurationfilereader->GetOptimizerType()==0)
    m_registration->SetOptimizerType(RSG);
  if (m_configurationfilereader->GetOptimizerType()==1)
    m_registration->SetOptimizerType(CG);
  if (m_configurationfilereader->GetOptimizerType()==2)
    m_registration->SetOptimizerType(GD);
  if (m_configurationfilereader->GetOptimizerType()==3)
    m_registration->SetOptimizerType(OPOE);

}

/********************************************************************/
/*      Launch pre-process that filters image before registration   */
/********************************************************************/
void PireoApplication::ViewPreProcessWindow()
{
  // Display the pre-processing window
  m_filter->Show();
  // Update the parameter values of the chosen filter
  m_filter->UpdateParameters();
  // Apply pre-processing
  this->ApplyPreProcessFilter();
}

void PireoApplication::ApplyPreProcessFilter()
{
  // Check if fixed image has been loaded and if it is selected for
  // pre-processing
  if (m_pireofilereader->FixedImageLoaded() &&
      ((m_filter->SelectFixed())))
  {

    // Set fixed zoomed Image after pre_process
    m_pireofilereader->
    SetImage(m_filter->DoFiltering(
               m_pireofilereader->GetImage(2)), 6);
    // Set fixed resampled image after pre-process
    m_pireofilereader->
    SetImage(m_filter->DoFiltering(
               m_pireofilereader->GetImage(4)), 8);
  }
  // Check if moving image has been loaded and if it is selected for
  // pre-processing
  if (m_pireofilereader->MovingImageLoaded() &&
      ((m_filter->SelectMoving())))
  {
    // Set moving zoomed image after pre_process
    m_pireofilereader->
    SetImage(m_filter->DoFiltering(
               m_pireofilereader->GetImage(3)), 7);
    // Set moving resampled image after pre-process
    m_pireofilereader->
    SetImage(m_filter->DoFiltering(
               m_pireofilereader->GetImage(5)), 9);
  }
}

/********************************************************************/
/*             To get back the real size of the VTK Window          */
/*        (window where the images are shown in transparency)       */
/********************************************************************/
void PireoApplication::GetVTKWindowSize(int* SizeWin)
{
  m_pireoFilter->SetWinDimensions(SizeWin[0], SizeWin[1]);
}

/********************************************************************/
/*             Load image chosen by user                            */
/********************************************************************/
bool PireoApplication::OpenFile(const char* file, bool TypeIm)
{
  // Read image chosen and implement also zoomed image
  if (m_pireofilereader->ReadImage(file, TypeIm))
    return true;

  // Generate resampled images
  if (TypeIm)
  {
    // If image size exceeds VTK window size, resample image
    /*if( (m_pireofilereader->GetImage(0)->GetLargestPossibleRegion()
    .GetSize()[0] > m_pireoFilter->GetWinDimensionX() &&
    (m_pireofilereader->GetImage(0)->GetLargestPossibleRegion()
    .GetSize()[1] > m_pireoFilter->GetWinDimensionY()) ) )*/
    m_pireofilereader->
    SetImage(m_pireoFilter
             ->ResampleToGUIWindowSize(m_pireofilereader
                                       ->GetImage(0)), 4);
    /* else
     m_pireofilereader->SetImage(m_pireofilereader->GetImage(0), 4); */

    m_pireofilereader->SetImage(m_pireofilereader->GetImage(4), 8);

  }
  else
  {
    // If image size exceeds VTK window size, resample image
    /*if( (m_pireofilereader->GetImage(1)->GetLargestPossibleRegion()
    .GetSize()[0] > m_pireoFilter->GetWinDimensionX() &&
    (m_pireofilereader->GetImage(1)->GetLargestPossibleRegion()
    .GetSize()[1] > m_pireoFilter->GetWinDimensionY()) ) )*/
    m_pireofilereader
    ->SetImage(m_pireoFilter
               ->ResampleToGUIWindowSize(m_pireofilereader
                                         ->GetImage(1)), 5);
    /* else
     m_pireofilereader->SetImage(m_pireofilereader->GetImage(1), 5); */

    m_pireofilereader->SetImage(m_pireofilereader->GetImage(5), 9);
  }

  if (m_pireofilereader->FixedImageLoaded() && m_pireofilereader->MovingImageLoaded())
    InputImagesIntoBlender();

  return false;
}

/********************************************************************/
/*      Read a new 1024x1024 region of the full resolution image    */
/*      This region is centered around the point clicked by mouse   */
/*      (point shown as intersection of the green lines in HMI)     */
/********************************************************************/
void PireoApplication::UpdateZoomedImage(float px, float py, bool Type)
{
  // Be sure that the image we are working on has already been loaded
  if (((!m_pireofilereader->FixedImageLoaded()) && Type==true)||
      ((!m_pireofilereader->MovingImageLoaded()) && Type==false))
    return;

  // Type image we are working on (fixed or moving)
  m_pireofilereader->SetImageType(Type);
  // Set Parameters to ZoomImage class to calculate in pixel the new
  // origin of zoomed image
  m_zoom->SetResampledImageDimensions(m_pireoFilter->GetWinDimensionX(),
                                      m_pireoFilter->GetWinDimensionY());
  m_zoom->SetFullResolutionImageDimensions(m_pireofilereader->GetFullResolutionImageDimensionX(),
      m_pireofilereader->GetFullResolutionImageDimensionY());
  m_zoom->SetFullResolutionImageSpacing(m_pireofilereader->GetFullResolutionImageSpacingX(),
                                        m_pireofilereader->GetFullResolutionImageSpacingY());
  m_zoom->SetResampledImageSpacing(m_pireofilereader->GetResampledImageSpacingX(),
                                   m_pireofilereader->GetResampledImageSpacingY());
  if (!m_zoom->SetOrigin(px, py))
    return;
  // Put the new calculated origin into ViewerImageComputation Class
  // to regenerate a zoomed image
  m_pireofilereader->SetZoomedImageOrigin(m_zoom->GetOriginX(),
                                          m_zoom->GetOriginY());
  // Get the new zoomed image
  if (m_pireofilereader->Update())
    std::cout<<"[Application::SetZoom] No possible update "<<std::endl;
}

/********************************************************************/
/*             Display Window with registration parameters          */
/********************************************************************/
void PireoApplication::ViewRegistrationParametersGUI()
{
  if ((!m_pireofilereader->FixedImageLoaded()) ||
      (!m_pireofilereader->MovingImageLoaded()))
    return;

  int fixnumber = 4;
  int movnumber = 5;
  if (m_filter->SelectFixed())
    fixnumber = 8;
  else
    fixnumber = 4;
  if (m_filter->SelectMoving())
    movnumber = 9;
  else
    movnumber = 5;

  // Set fixed and moving image into the RegistrationMethod Class
  m_registration->
  SetFixedImage(m_pireoFilter->
                CastImageToRegister(m_pireofilereader->
                                    GetImage(fixnumber)));
  m_registration->
  SetMovingImage(m_pireoFilter->
                 CastImageToRegister(m_pireofilereader->
                                     GetImage(movnumber)));
  m_registration->Show();
}

/********************************************************************/
/*             Launch Registration Process                          */
/********************************************************************/
void PireoApplication::StartRegistration()
{
  if (m_registration->GetParams().Size() == 0)
  {
    std::cout<<"Please select Registration Parameters before start registration."<<std::endl;
    return;
  }
  // Reset parameters list
  m_ListParams.clear();
  // Start Registration
  m_registration->StartRegistration();
}

void PireoApplication::PauseRegistration(bool status)
{
  if (status)
  {
    m_registration->StopRegistration();
  }
  else
  {
    ParametersList::reverse_iterator  it = m_ListParams.rbegin();
    std::cout<<"result : "<<*it<<std::endl;
    m_registration->SetRegistrationTransformParameters(*it);
    m_registration->DoRegister();
  }
}

void PireoApplication::StopRegistration()
{
  this->GetRegistration()->StopRegistration();
  //this->ResetVTKRenderers();
  //this->SetBlenderRenderer();
}

void PireoApplication::InputImagesIntoRegistration()
{
  // Be sure that fixed and moving images have already been loaded
  if ((!m_pireofilereader->FixedImageLoaded()) ||
      (!m_pireofilereader->MovingImageLoaded()))
    return;

  int fixnumber = 4;
  int movnumber = 5;
  if (m_filter->SelectFixed())
    fixnumber = 8;
  else
    fixnumber = 4;
  if (m_filter->SelectMoving())
    movnumber = 9;
  else
    movnumber = 5;

  // Set fixed and moving image into the Registration Class
  m_registration->
  SetFixedImage(m_pireoFilter->
                CastImageToRegister(m_pireofilereader->
                                    GetImage(fixnumber)));
  m_registration->
  SetMovingImage(m_pireoFilter->
                 CastImageToRegister(m_pireofilereader->
                                     GetImage(movnumber)));
}

/********************************************************************/
/*   When registration process is stopped, Vtk windows are reset    */
/********************************************************************/
void PireoApplication::ResetVTKRenderers()
{
  // Remove actor if it has already been used in the right-down VTK
  // window.
  if (m_VecRen != NULL)
  {
    m_VecWin->RemoveRenderer(m_VecRen);
    m_VecRen->Delete();
    m_VecRen = vtkRenderer::New();
    m_VecRen->SetAmbient(0.1, 0.1, 0.1);
  }

  m_VecWin->AddRenderer(m_VecRen);
  m_VecWin->Render();
  // Remove actor if it has already been used in the left-down VTK
  // window.
  /*if(m_ren != NULL)
    m_renwin->RemoveRenderer(m_ren); */

  //m_VecWin->Render();
}

/********************************************************************/
/*    Save registration results (filtered fixed image, filtered     */
/*    moving image, deformed image).                                */
/********************************************************************/
void PireoApplication::SaveRegistrationResults(int n, const char* filename)
{
  // Save the down-sampled fixed image
  if (n==0)
    m_save->SaveAsITKImage(m_pireofilereader->GetImage(4), filename);
  // Save the down-sampled moving image
  else if (n==1)
    m_save->SaveAsITKImage(m_pireofilereader->GetImage(5), filename);
  // Save the deformed image given at the end of the registration process
  else if (n==2)
  {
    m_save->SaveAsITKImage(
      this->GetPireoFiltering()->CastImageToInitialState(
        this->GetRegistration()->GetRegistrationResult()), filename);
  }
  else if (n==3)
    m_save->SaveAsVTKImage(m_transWin->GetBlender(), filename);
}

/********************************************************************/
/*    Save parameters of the registration process in a text file    */
/********************************************************************/
void PireoApplication::SaveRegistrationParameters(const char* filename)
{
  std::string ext(filename);
  ext = ext.substr(ext.rfind("."), ext.length());

  if ( strcmp((char*)ext.c_str(),".txt")!=0 )
    return;

  unsigned int j;
  FILE * pFile;

  pFile = fopen (filename,"w+");
  if (pFile!=NULL)
  {
    fputs ("Registration Parameters File\n\n", pFile);

    // Transform type used
    fputs("Transform: \n", pFile);
    fputs(m_registration->GetTransformType().c_str(), pFile);
    fputs("\n", pFile);
    // Interpolator type used
    fputs("Interpolator: \n", pFile);
    fputs(m_registration->GetInterpolatorType().c_str(), pFile);
    fputs("\n", pFile);
    // Metric type used
    fputs("Metric: \n", pFile);
    fputs(m_registration->GetMetricType().c_str(), pFile);
    fputs("\n", pFile);
    // Optimizer type used
    fputs("Optimizer: \n", pFile);
    fputs(m_registration->GetOptimizerType().c_str(), pFile);
    fputs("\n", pFile);

    ParametersListIterator  it = m_ListParams.begin();
    int i=0;

    while (it!=m_ListParams.end())
    {
      fprintf (pFile, "Iteration No: %d ", i);
      fputs ("Transform : [", pFile);

      for ( j=0; j < (*it).size()-1; j++)
        fprintf (pFile, "%f, ", (*it)[j]);

      fprintf (pFile, "%f] \n\n", (*it)[j]);
      it++;
      i++;
    }
  }
  fclose (pFile);
}

/********************************************************************/
/*           Generate a list of registration parameters             */
/*                  at each process iteration                       */
/********************************************************************/
void PireoApplication::InsertNewRegistrationParameters()
{
  m_ListParams.push_back(this->GetRegistration()->GetOptimizer()->GetCurrentPosition());
}

/********************************************************************/
/*           Display fixed and moving images in transparency        */
/********************************************************************/
void PireoApplication::InputImagesIntoBlender()
{
  // Be sure that fixed and moving images have already been loaded
  if (!m_pireofilereader->FixedImageLoaded() ||
      !m_pireofilereader->MovingImageLoaded())
    return;

  int fixnumber = 4;
  int movnumber = 5;
  if (m_filter->SelectFixed())
    fixnumber = 8;
  else
    fixnumber = 4;
  if (m_filter->SelectMoving())
    movnumber = 9;
  else
    movnumber = 5;

  // Transform ITK fixed and moving images into VTK images
  vtkImageData* m_vtkFixIm = m_save->
                             ITKToVTKTransform(m_pireofilereader->GetImage(fixnumber));
  vtkImageData* m_vtkMovIm = m_save->
                             ITKToVTKTransform(m_pireoFilter->ResampleToFixedImageSize(m_pireofilereader->GetImage(movnumber), m_pireofilereader->GetImage(fixnumber)));

  // Set VTK files into TransparencyWindow Class
  m_transWin->SetMaster(m_vtkFixIm);
  m_transWin->SetSlave(m_vtkMovIm);
  // Manage the actor of the first VTK window
  m_transWin->SetRenderer();
}

void PireoApplication::SetBlenderRenderer()
{
  // Set images to blender
  InputImagesIntoBlender();
  // Set default values for registration
  m_registration->SetTransformType(TRANSLATION);
  m_registration->SetInterpolatorType(LINEAR);
  m_registration->SetMetricType(MEANSQUARE);
  m_registration->SetOptimizerType(RSG);

  // Remove actor if already existing
  if (m_ren!=NULL)
    m_renwin->RemoveRenderer(m_ren);

  // Get back the renderer
  m_ren = m_transWin->GetRenderer();
  m_ren->ResetCamera();
  m_ren->GetActiveCamera()->Zoom(1.58);
  // Add renderer to render window
  m_renwin->AddRenderer(m_ren);
  // Redraw
  m_renwin->Render();
  // To know that a view in transparency has already been displayed
  WinStatu = true;
}

/********************************************************************/
/*           Update vector field during registration process        */
/********************************************************************/
void PireoApplication::UpdateVectorField()
{
  if ((!m_pireofilereader->FixedImageLoaded()) ||
      (!m_pireofilereader->MovingImageLoaded()))
    return;

  // Remove actor if it has already been used
  if (m_VecRen != NULL)
    m_VecWin->RemoveRenderer(m_VecRen);

  // Get back the renderer
  m_VecRen = m_vectorField->GetRenderer();
  m_VecRen->ResetCamera();
  m_VecRen->GetActiveCamera()->Zoom(1.58);
  // Add renderer to render window
  m_VecWin->AddRenderer(m_VecRen);
  // Redraw
  m_VecWin->Render();
}

/********************************************************************/
/*           Update grid during registration process                */
/********************************************************************/
void PireoApplication::UpdateGrid()
{
  if ((!m_pireofilereader->FixedImageLoaded()) ||
      (!m_pireofilereader->MovingImageLoaded()))
    return;

  // Remove actor if it has already been used
  if (m_VecRen != NULL)
    m_VecWin->RemoveRenderer(m_VecRen);

  // Get back the renderer
  m_VecRen = m_grid->GetRenderer();

  m_VecRen->ResetCamera();
  m_VecRen->GetActiveCamera()->Zoom(1.58);
  // Add renderer to render window
  m_VecWin->AddRenderer(m_VecRen);

  // Redraw
  m_VecWin->Render();
}

/********************************************************************/
/*      Update Transparency Window (input new value for opacity)    */
/********************************************************************/
void PireoApplication::SetTransparencyWindowValueSlider(float transparency)
{
  // Value range of the interface slider : 0 to 100
  // So it's necessary to set it between 0 and 1
  transparency = (float)(transparency/100);

//  std::cout<<"transparency slicer "<<transparency<<std::endl;
  // Be sure that Transparency Window has already been displayed
  if (!WinStatu)
    return;
  // Remove actor if it has already been used
  if (m_ren!=NULL)
    m_renwin->RemoveRenderer(m_ren);

  // Set new value for opacity
  m_transWin->SetTransparency(transparency);
  std::cout<<"transparency slicer "<<transparency<<std::endl;
  // Update TransparencyWindow Class
  m_transWin->SetRenderer();
  // Get back renderer
  m_ren = m_transWin->GetRenderer();
  // Add renderer
  m_renwin->AddRenderer(m_ren);
  // Redraw
  m_renwin->Render();
}

/********************************************************************/
/*                  Manage Flick button                             */
/*        Enable to display either the fixed image only or          */
/*  the moving image only when they are viewed in transparency      */
/********************************************************************/
void PireoApplication::TransparencyRendererFlick(bool statebutton)
{
  if (statebutton)
    SetTransparencyWindowValueSlider(0);
  else
    SetTransparencyWindowValueSlider(100);
}
/********************************************************************/
/* Manage user button that enables to display in equal transparency */
/* the fixed and the moving images                                  */
/********************************************************************/
void PireoApplication::ResetTransparencyWindowValueSlider()
{
  SetTransparencyWindowValueSlider(50);
}

/********************************************************************/
/*   Manage the cursor that enables the user to move into the full  */
/*   resolution image by changing automatically the zoomed image    */
/*   displayed                                                      */
/********************************************************************/
void PireoApplication::MoveZoomedImagePositionTo(int position, bool TypeIm)
{
  // Be sure that fixed and moving images have already been loaded
  if (((!m_pireofilereader->FixedImageLoaded()) && TypeIm==true)
      ||((!m_pireofilereader->MovingImageLoaded())
         && TypeIm==false))
    return;
  // Type image we are working on (fixed or moving)
  m_pireofilereader->SetImageType(TypeIm);
  // Set Parameters to ZoomManager class to calculate in pixel the new
  // origin of zoomed image
  m_zoom->SetResampledImageDimensions(m_pireoFilter->GetWinDimensionX(),
                                      m_pireoFilter->GetWinDimensionY());
  m_zoom->SetFullResolutionImageDimensions(
    m_pireofilereader->GetFullResolutionImageDimensionX(),
    m_pireofilereader->GetFullResolutionImageDimensionY());
  m_zoom->SetPosition(
    m_pireofilereader->GetZoomedImageOriginX(),
    m_pireofilereader->GetZoomedImageOriginY());

  // Depending on the user choice, the calculation of the new origin
  // is done
  if (position==0)
    m_zoom->GoDown();
  else if (position==1)
    m_zoom->GoUp();
  else if (position==2)
    m_zoom->GoRight();
  else
    m_zoom->GoLeft();

  // Put the new calculated origin into ViewerImageComputation Class
  // to regenerate a zoomed image
  m_pireofilereader->SetZoomedImageOrigin(m_zoom->GetOriginX(),
                                          m_zoom->GetOriginY());
  // Get the new zoomed image
  if (m_pireofilereader->Update())
    std::cout<<"[Application::SetZoom] No possible update "<<std::endl;

}

#ifdef _MSC_VER
#pragma warning(pop)
#endif

