/*=========================================================================

  $Author: allain $
  $Date: 2005/07/12 15:20:13 $
  $Revision: 1.10 $
  $Id: PireoApplication.h, v 1.10 2005/07/12 15:20:13 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/PireoApplication.h, v $
  $Log: PireoApplication.h, v $
  Revision 1.10  2005/07/12 15:20:13  allain
  Add some functionalities

  Revision 1.9  2005/07/04 15:46:33  allain
  Add results auto save

  Revision 1.8  2005/06/24 15:40:09  allain
  Add metric values graph and started registration levels observer

  Revision 1.7  2005/06/15 14:41:20  allain
  Add registration level observer

  Revision 1.6  2005/06/10 15:56:42  allain
  Add some functionalities for registration

  Revision 1.5  2005/06/06 16:16:56  allain
  Gave new method names, add commentary and implement new params

  Revision 1.4  2005/05/09 15:52:57  allain
  Add pause and stop buttons for generic registration

  Revision 1.3  2005/05/02 15:33:01  allain
  Generic Registration and Configuration File installed

  Revision 1.2  2005/04/29 13:44:53  allain
  Add Save Registration Parameters Method

  Revision 1.1  2005/04/27 13:15:11  allain
  Change most of class names

  Revision 1.12  2005/03/24 16:14:28  allain
  Pre-Processing structure done

  Revision 1.11  2005/03/23 17:34:11  allain
  Pre-Processing is working

  Revision 1.10  2005/03/22 17:40:18  allain
  Grid is working and Pre-processing starts

  Revision 1.9  2005/03/21 14:35:09  allain
  Interface modified

  Revision 1.8  2005/03/16 17:09:08  allain
  Add some commentaries

  Revision 1.7  2005/03/15 17:28:47  allain
  Add Grid class

  Revision 1.6  2005/03/15 11:04:47  allain
  Add Vector Field Window

  Revision 1.5  2005/03/07 16:17:27  allain
  Registration with GDAL image starts working

  Revision 1.4  2005/03/03 16:52:44  allain
  Registration start and its observer

  Revision 1.3  2005/02/25 15:33:10  allain
  Transparency Window Set

  Revision 1.2  2005/02/23 17:48:29  allain
  ITK To VTK IMage start

  Revision 1.1  2005/02/23 16:12:14  allain
  Add missing files


=========================================================================*/

#ifndef __PireoApplication_H__
#define __PireoApplication_H__

/* C++ Libraries */
#include <list>
#include <stdio.h>

/* VTK Libraries */
#include "vtkCamera.h"
#include "vtkRenderWindow.h"

/* Code Related */
#include "PireoFileReader.h"
#include "ZoomManager.h"
#include "TransparencyWindowManager.h"
#include "PireoFileWriter.h"
#include "PireoFiltering.h"
#include "RegistrationMethod.h"
#include "VectorField.h"
#include "Grid.h"
#include "Structures.h"
#include "PreProcessFiltering.h"
#include "ConfigurationFileReader.h"

/* Class PireoApplication
   It is from this class that everything that is not concerning the
   GUI is encapsulated (i.e grid, vectoer field, registration process,
   ...).
   This class contains the main method of the pireo software.
*/
class PireoApplication
{
public:
  /** Registration method type.*/
  typedef RegistrationMethod     RegistrationMethodType;

  /** List of registration transform parameters.*/
  typedef std::list<RegistrationMethodType::ParametersType>
  ParametersList;
  /** Iterator on the list of registration transform parameters.*/
  typedef std::list<RegistrationMethodType::ParametersType>
  ::iterator  ParametersListIterator;

  /** Construstor: create all objects for Pireo application.*/
  PireoApplication();
  /** Destructor.*/
  ~PireoApplication();

  /****************************************************************/
  /*            Methods (links between user interface and code)   */
  /****************************************************************/

  /** Display new window to enable user to choose pre-process
  parameters.*/
  void ViewPreProcessWindow();
  /** Apply pre-process filter to all images that have been
  selected.*/
  void ApplyPreProcessFilter();
  /** Change zoomed image origin (enable displacement on full
  resolution image).*/
  void MoveZoomedImagePositionTo(int position, bool TypeIm);
  /** Switch between fixed and moving image in the transparency
  window.*/
  void TransparencyRendererFlick(bool statebutton);
  /** Display images in the transparency window with the same
  opacity (put the transparency window slider value to 0.5).*/
  void ResetTransparencyWindowValueSlider();
  /** Load an image*/
  bool OpenFile(const char* file, bool TypeIm);
  /** Generate VTK renderer to display fixed and moving images in
  transparency.*/
  void SetBlenderRenderer();
  /** Set blender renderer.*/
  void InputImagesIntoBlender();
  /** Read a new region of the full resolution image.*/
  void UpdateZoomedImage(float px, float py, bool Type);
  /** Take care of the redrawing when the transparency window slider
  is changed.*/
  void SetTransparencyWindowValueSlider(float transparency);
  /** Display window with registration parameters.*/
  void ViewRegistrationParametersGUI();
  /** Start registration process.*/
  void InputImagesIntoRegistration();
  /** Start registration process.*/
  void StartRegistration();
  /** Stop registration process.*/
  void StopRegistration();
  /** Have a break in the registration process.*/
  void PauseRegistration(bool status);
  /** Reset transparency window and grid window when registration
  process stop*/
  void ResetVTKRenderers();
  /** Save registration results as the deformed image*/
  void SaveRegistrationResults(int n, const char* filename);
  /** Autmotically save registration results*/
  void AutomaticSave(const char* filename, int n);
  /** Autmotically save registration results*/
  void AutomaticSave(int n);
  /** Deactivate the automatic save of the registration results*/
  void DisableAutoSave();
  /** Enable ths user to display or not the metric values while
  * processing registration.*/
  void SetMetricObserver(bool a);

  /******************************************************************/
  /*                       Other Methods                            */
  /******************************************************************/

  /** Get back the real size of the down-left VTK Window.*/
  void GetVTKWindowSize(int* SizeWin);
  /** Update vector field during registration process.*/
  void UpdateVectorField();
  /** Update grid during registration process.*/
  void UpdateGrid();
  /** Need to set a new renderer for refreshing the GUI during the
  registration process (the renderer of the transparency window).*/
  void SetRenderer(vtkRenderer* m_renderer)
  {
    m_ren = m_renderer;
  }
  /** Get registration parameters at each iteration.*/
  void InsertNewRegistrationParameters();
  /** Get Metric value at each iteration.*/
  //void InsertNewMetricValue(double v);
  /** Save list of the registration parameters in a text file.*/
  void SaveRegistrationParameters(const char* filename);
  /** Set registration parameters from a text file.*/
  void ReadRegistrationParametersFromFile(const char* filename);

  /******************************************************************/
  /*                        Return Methods                          */
  /*   To get all the objects encapsulated in Application Class     */
  /*                   Used by PireoViewer Object                   */
  /******************************************************************/

  /** Return PireoFileReader Object.*/
  PireoFileReader* GetPireoFileReader()
  {
    return m_pireofilereader;
  }
  /** Return RegistrationMethod Object.*/
  RegistrationMethod* GetRegistration()
  {
    return m_registration;
  }
  /** Return TransparencyWindowManager Object.*/
  TransparencyWindowManager* GetTransparencyWindow()
  {
    return m_transWin;
  }
  /** Return PireoFiltering Object.*/
  PireoFiltering* GetPireoFiltering()
  {
    return m_pireoFilter;
  }
  /** Return PireoFileWriter Object.*/
  PireoFileWriter* GetPireoFileWriter()
  {
    return m_save;
  }
  /** Return VectorField Object.*/
  VectorField* GetVectorField()
  {
    return m_vectorField;
  }
  /** Return Grid Object.*/
  Grid* GetGrid()
  {
    return m_grid;
  }
  /** Return PreProcessFilter Object.*/
  PreProcessFiltering* GetPreProcessFiltering()
  {
    return m_filter;
  }
  /** Return the VTK render window of the down-left window
  (transparency).*/
  vtkRenderWindow* GetRenderWindow()
  {
    return m_renwin;
  }
  /** Return the VTK renderer of the first window (transparency).*/
  vtkRenderer* GetRenderer()
  {
    return m_ren;
  }
  /** Return registration metric values list*/
  //MetricValuesList& GetMetricValuesList(){return m_metricValues; }

protected:
  // Encapsulated objects
  /** PireoFileReader.*/
  PireoFileReader* m_pireofilereader;
  /** ZoomManager.*/
  ZoomManager* m_zoom;
  /** TransparencyWindowManager.*/
  TransparencyWindowManager* m_transWin;
  /** PireoFileWriter.*/
  PireoFileWriter* m_save;
  /** PireoFiltering.*/
  PireoFiltering* m_pireoFilter;
  /** RegistrationMethod.*/
  RegistrationMethod* m_registration;
  /** VectorField.*/
  VectorField* m_vectorField;
  /** Grid.*/
  Grid* m_grid;
  /** PreProcessFiltering.*/
  PreProcessFiltering* m_filter;
  /** ConfigurationFileReader.*/
  ConfigurationFileReader* m_configurationfilereader;

  /** VTK renderer of the down-left window.*/
  vtkRenderer* m_ren;
  /** VTK renderer of the down-right window.*/
  vtkRenderer* m_VecRen;
  /** VTK render window of the down-left window.*/
  vtkRenderWindow* m_renwin;
  /** VTK render window of the right-left window.*/
  vtkRenderWindow* m_VecWin;

  /** Booleen used to know if the view in transparency has already
  been displayed.*/
  bool WinStatu;

  //bool process;

  /** Regsitration parameters list*/
  ParametersList  m_ListParams;

  /** Automatic saves while processing registration*/
  const char *m_saveFilename;
  int m_saveType;
};

#endif
