/*=========================================================================

  $Author: allain $
  $Date: 2005/06/15 14:41:20 $
  $Revision: 1.3 $
  $Id: PireoFileReader.cxx, v 1.3 2005/06/15 14:41:20 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/PireoFileReader.cxx, v $
  $Log: PireoFileReader.cxx, v $
  Revision 1.3  2005/06/15 14:41:20  allain
  Add registration level observer

  Revision 1.2  2005/06/06 16:16:56  allain
  Gave new method names, add commentary and implement new params

  Revision 1.1  2005/04/27 13:15:11  allain
  Change most of class names

  Revision 1.22  2005/03/24 16:14:28  allain
  Pre-Processing structure done

  Revision 1.21  2005/03/23 17:34:11  allain
  Pre-Processing is working

  Revision 1.20  2005/03/22 17:40:18  allain
  Grid is working and Pre-processing starts

  Revision 1.19  2005/03/21 14:35:09  allain
  Interface modified

  Revision 1.18  2005/03/16 17:09:08  allain
  Add some commentaries

  Revision 1.17  2005/03/15 11:04:47  allain
  Add Vector Field Window

  Revision 1.16  2005/03/08 13:11:39  allain
  Set MultiResolutionImageRegistrationMethod

  Revision 1.15  2005/03/07 16:17:27  allain
  Registration with GDAL image starts working

  Revision 1.14  2005/03/04 13:49:42  allain
  GLSliceView file modified for 2D Image

  Revision 1.13  2005/03/03 16:52:44  allain
  Registration start and its observer

  Revision 1.12  2005/02/25 15:33:10  allain
  Transparency Window Set

  Revision 1.11  2005/02/23 16:09:58  allain
  New Class Hierarchy

  Revision 1.10  2005/02/22 14:53:08  allain
  Display image with flip option

  Revision 1.9  2005/02/21 17:09:31  allain
  New InterFace to display image

  Revision 1.8  2005/02/15 15:20:17  allain
  Read a region of an image woth GDAL and ITK

  Revision 1.7  2005/02/14 16:48:59  allain
  CanReadFile() validated

  Revision 1.6  2005/02/14 10:15:59  allain
  Sauvegarde avant ajout memoire

  Revision 1.5  2005/02/11 16:27:21  allain
  Mouse interaction start ..missed files in cvs

  Revision 1.4  2005/02/11 08:49:46  allain
  IHM beginning

  Revision 1.3  2005/02/10 07:51:27  allain
  Add itkGDALImageIO and the header of each file


=========================================================================*/

#include "PireoFileReader.h"
#include <string>

/********************************************************************/
/*                        Constructor                               */
/*  Set default value to 1024x1024 for the region of the read image */
/*                 No image loaded at this time                     */
/********************************************************************/
PireoFileReader::PireoFileReader()
{
  // Define zoomed region size
  regionZoomx = 1024;
  regionZoomy = 1024;

  m_TypeIm = true;
  m_FixLoaded = false;
  m_MovLoaded = false;
}

/********************************************************************/
/*                        Destructor                                */
/********************************************************************/
PireoFileReader::~PireoFileReader()
{
}

/********************************************************************/
/* Methods that returns the dimensions of the full resolution image */
/********************************************************************/
unsigned int PireoFileReader::GetFullResolutionImageDimensionX()
{
  if (m_TypeIm)
    return m_fixIm->GetLargestPossibleRegion().GetSize()[0];
  else
    return m_movIm->GetLargestPossibleRegion().GetSize()[0];
}
unsigned int PireoFileReader::GetFullResolutionImageDimensionY()
{
  if (m_TypeIm)
    return m_fixIm->GetLargestPossibleRegion().GetSize()[1];
  else
    return m_movIm->GetLargestPossibleRegion().GetSize()[1];
}

/********************************************************************/
/*   Methods that returns the spacing of the full resolution image  */
/********************************************************************/
float PireoFileReader::GetFullResolutionImageSpacingX()
{
  if (m_TypeIm)
    return m_fixIm->GetSpacing()[0];
  else
    return m_movIm->GetSpacing()[0];
}
float PireoFileReader::GetFullResolutionImageSpacingY()
{
  if (m_TypeIm)
    return m_fixIm->GetSpacing()[1];
  else
    return m_movIm->GetSpacing()[1];
}

/********************************************************************/
/*       Methods that returns the dimensions of zoomed image        */
/********************************************************************/
unsigned int PireoFileReader::GetZoomedImageDimensionX()
{
  if (m_TypeIm)
    return m_ZoomfixIm->GetLargestPossibleRegion().GetSize()[0];
  else
    return m_ZoommovIm->GetLargestPossibleRegion().GetSize()[0];
}
unsigned int PireoFileReader::GetZoomedImageDimensionY()
{
  if (m_TypeIm)
    return m_ZoomfixIm->GetLargestPossibleRegion().GetSize()[1];
  else
    return m_ZoommovIm->GetLargestPossibleRegion().GetSize()[1];
}

/********************************************************************/
/*       Methods that returns the dimensions of resampled image     */
/********************************************************************/
unsigned int PireoFileReader::GetResampledImageDimensionX()
{
  if (m_TypeIm)
    return m_SamplefixIm->GetLargestPossibleRegion().GetSize()[0];
  else
    return m_SamplemovIm->GetLargestPossibleRegion().GetSize()[0];
}
unsigned int PireoFileReader::GetResampledImageDimensionY()
{
  if (m_TypeIm)
    return m_SamplefixIm->GetLargestPossibleRegion().GetSize()[1];
  else
    return m_SamplemovIm->GetLargestPossibleRegion().GetSize()[1];
}

/********************************************************************/
/*       Methods that returns the spacing of resampled image        */
/********************************************************************/
float PireoFileReader::GetResampledImageSpacingX()
{
  if (m_TypeIm)
    return m_SamplefixIm->GetSpacing()[0];
  else
    return m_SamplemovIm->GetSpacing()[0];
}
float PireoFileReader::GetResampledImageSpacingY()
{
  if (m_TypeIm)
    return m_SamplefixIm->GetSpacing()[1];
  else
    return m_SamplemovIm->GetSpacing()[1];
}

/********************************************************************/
/*       Set Origin (bottom-left corner) of the zoomed image        */
/********************************************************************/
void PireoFileReader::SetZoomedImageOrigin(int ox, int oy)
{
  XBegin = ox;
  YBegin = oy;
}

/********************************************************************/
/*       Return Origin of the zoomed image                          */
/********************************************************************/
int PireoFileReader::GetZoomedImageOriginX()
{
  return XBegin;
}
int PireoFileReader::GetZoomedImageOriginY()
{
  return YBegin;
}

/********************************************************************/
/*       Says if we are working on a fixed or a moving image        */
/********************************************************************/
void PireoFileReader::SetImageType(bool Type)
{
  m_TypeIm = Type;
}
bool PireoFileReader::GetImageType()
{
  return m_TypeIm;
}

/********************************************************************/
/*                        Update zoomed image                       */
/********************************************************************/
bool PireoFileReader::Update()
{
  return ReadZoomedImage();
}

/********************************************************************/
/*       Return the status of the loaded fixed image                */
/********************************************************************/
bool PireoFileReader::FixedImageLoaded()
{
  return m_FixLoaded;
}
/********************************************************************/
/*       Return the status of the loaded moving image               */
/********************************************************************/
bool PireoFileReader::MovingImageLoaded()
{
  return m_MovLoaded;
}

/********************************************************************/
/*    Read zoomed image (thus set the m_ZoomfixIm or m_ZoommovIm)   */
/********************************************************************/
bool PireoFileReader::ReadZoomedImage()
{
  // Check first if it's necessary to build a zoomed image
  if ((GetFullResolutionImageDimensionX() < static_cast<unsigned int>(regionZoomx)) &&
      (GetFullResolutionImageDimensionY() < static_cast<unsigned int>(regionZoomy)) )
    return true;

  // Buffer size set to 1024x1024
  SizeXRead = regionZoomx;
  SizeYRead = regionZoomy;
  // Size of the read region set to 1024x1024
  XSample = regionZoomx;
  YSample = regionZoomy;

  // Extract a 1024x1024 region of the full resolution image
  ImageType::SizeType size0 = {{XSample, YSample}};
  ImageType::IndexType index0 = {{XBegin, YBegin}};
  ImageType::RegionType region0;
  region0.SetSize( size0 );
  region0.SetIndex( index0 );

  m_RegionReader = FilterRegionType::New();
  m_RegionReader->SetRegionOfInterest(region0);

  // Input the full resolution images
  if (m_TypeIm)
    m_RegionReader->SetInput(m_fixIm);
  else
    m_RegionReader->SetInput(m_movIm);

  // Using itkRegionOfInterestImageFilter
  try
  {
    m_RegionReader -> Update();
  }
  catch ( itk::ExceptionObject &err )
  {
    std::cout<<" (!) exception caught "<<std::endl;
    std::cout<< err <<std::endl;
    return true;
  }

  ImageType::Pointer m_Im = m_RegionReader->GetOutput();

  // Set zoomed image
  if (m_TypeIm)
  {
    m_ZoomfixIm =  m_Im;
    m_PZoomfixIm = m_Im;
  }
  else
  {
    m_ZoommovIm = m_Im;
    m_PZoommovIm = m_Im;
  }

  return false;
}

/********************************************************************/
/*       First check if ITK Factory recognizes the loaded image     */
/********************************************************************/
bool PireoFileReader::CanReadFileWithITK(const char *file)
{
  // Reader initialized
  ImageReaderType::Pointer read = ImageReaderType::New();
  read -> SetFileName( file );

  // If the image type belongs to ITK Factory or not
  try
  {
    read->Update();
  }
  catch ( itk::ExceptionObject &err )
  {
    std::cerr << err << std::endl;
    return false;
  }
  return true;
}

/********************************************************************/
/*                Check if filename extension is .tif               */
/*                 If it is so read image with GDAL                 */
/********************************************************************/
bool PireoFileReader::TiffFormat(const char* file)
{
  // Get filename extension
  std::string m_string(file);
  m_string = m_string.substr(m_string.rfind("."), m_string.length());
  char* ext =  (char*)m_string.c_str();
  // Is it a .tif file ?
  if (strcmp(ext,".tif")==0)
    return true;
  else
    return false;
}

/********************************************************************/
/*                      Read GDAL image                             */
/*   If working on fixed image so m_ZoomfixIm and m_fixIm set       */
/*   If working on moving image so m_ZoommovIm and m_movIm set      */
/********************************************************************/
bool PireoFileReader::ReadImage(const char *file, bool FixIm)
{
  if (file == NULL)
    return true;

  m_TypeIm = FixIm;

  // Reader initialized
  ImageReaderType::Pointer m_Reader = ImageReaderType::New();
  m_Reader->SetFileName( file );
  try
  {
    m_Reader->Update();
  }
  catch ( itk::ExceptionObject &err )
  {
    std::cout<<" (!) exception caught "<<std::endl;
    std::cout<< err <<std::endl;
  }
  ImageType::Pointer m_Im = m_Reader->GetOutput();

  // Get the full resolution image
  if (m_TypeIm)
  {
    m_fixIm = m_Im;
    filenameFI = file;
  }
  else
  {
    std::cout<<"moving"<<std::endl;
    m_movIm = m_Im;
    filenameMI = file;
  }

  // Check if the image dimensions are below or above 1024x1024 which
  // represents the zoomed image dimensions
  if ( (GetFullResolutionImageDimensionX() < static_cast<unsigned int>(regionZoomx)) &&
       (GetFullResolutionImageDimensionY() < static_cast<unsigned int>(regionZoomy)) )
  {
    // If below
    if (m_TypeIm)
    {
      m_ZoomfixIm =  m_fixIm;
      m_PZoomfixIm = m_fixIm;
    }
    else
    {
      m_ZoommovIm = m_movIm;
      m_PZoommovIm = m_movIm;
    }
  }
  // If above
  else
  {
    XBegin = 0;
    YBegin = 0;
    if (ReadZoomedImage())
      return true;
  }

  // Set booleen loaded image
  if (m_TypeIm)
    m_FixLoaded = true;
  else
    m_MovLoaded = true;

  return false;
}

/********************************************************************/
/*                        Return one of the images set              */
/********************************************************************/
PireoFileReader::ImageType::Pointer
PireoFileReader::GetImage(int imagenumber)
{
  switch (imagenumber)
  {
  case 0:
    return m_fixIm;
  case 1:
    return m_movIm;
  case 2:
    return m_ZoomfixIm;
  case 3:
    return m_ZoommovIm;
  case 4:
    return m_SamplefixIm;
  case 5:
    return m_SamplemovIm;
    // To work on it without modifing data set shown above
  case 6:
    return m_PZoomfixIm;
  case 7:
    return m_PZoommovIm;
  case 8:
    return m_PSamplefixIm;
  case 9:
    return m_PSamplemovIm;
  }
  return NULL;
}

/********************************************************************/
/*        Set resampled images from PireoApplication Class          */
/*             Used for the resample filter pipeline                */
/********************************************************************/
void PireoFileReader::SetImage(ImageType* m_Im, int number)
{
  switch (number)
  {
  case 4:
  {
    m_SamplefixIm = m_Im;
    break;
  }
  case 5:
  {
    m_SamplemovIm = m_Im;
    break;
  }
  case 6:
  {
    m_PZoomfixIm = m_Im;
    break;
  }
  case 7:
  {
    m_PZoommovIm = m_Im;
    break;
  }
  case 8:
  {
    m_PSamplefixIm = m_Im;
    break;
  }
  case 9:
  {
    m_PSamplemovIm = m_Im;
    break;
  }
  default:
  {
    std::cout<<"No access to this dataset."<<std::endl;
    break;
  }
  }
}
