/*=========================================================================

  $Author: allain $
  $Date: 2005/06/15 14:41:20 $
  $Revision: 1.3 $
  $Id: PireoFileReader.h, v 1.3 2005/06/15 14:41:20 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/PireoFileReader.h, v $
  $Log: PireoFileReader.h, v $
  Revision 1.3  2005/06/15 14:41:20  allain
  Add registration level observer

  Revision 1.2  2005/06/06 16:16:56  allain
  Gave new method names, add commentary and implement new params

  Revision 1.1  2005/04/27 13:15:11  allain
  Change most of class names

  Revision 1.20  2005/03/23 17:34:11  allain
  Pre-Processing is working

  Revision 1.19  2005/03/22 17:40:18  allain
  Grid is working and Pre-processing starts

  Revision 1.18  2005/03/21 14:35:09  allain
  Interface modified

  Revision 1.17  2005/03/16 17:09:08  allain
  Add some commentaries

  Revision 1.16  2005/03/15 11:04:47  allain
  Add Vector Field Window

  Revision 1.15  2005/03/08 13:11:39  allain
  Set MultiResolutionImageRegistrationMethod

  Revision 1.14  2005/03/07 16:17:27  allain
  Registration with GDAL image starts working

  Revision 1.13  2005/03/04 13:49:42  allain
  GLSliceView file modified for 2D Image

  Revision 1.12  2005/03/03 16:52:44  allain
  Registration start and its observer

  Revision 1.11  2005/02/25 15:33:10  allain
  Transparency Window Set

  Revision 1.10  2005/02/23 17:48:29  allain
  ITK To VTK IMage start

  Revision 1.9  2005/02/23 16:09:58  allain
  New Class Hierarchy

  Revision 1.8  2005/02/22 14:53:08  allain
  Display image with flip option

  Revision 1.7  2005/02/21 17:09:31  allain
  New InterFace to display image

  Revision 1.6  2005/02/15 15:20:17  allain
  Read a region of an image woth GDAL and ITK

  Revision 1.5  2005/02/14 16:48:59  allain
  CanReadFile() validated

  Revision 1.4  2005/02/11 08:49:46  allain
  IHM beginning

  Revision 1.3  2005/02/10 07:51:27  allain
  Add itkGDALImageIO and the header of each file


=========================================================================*/

#ifndef __PireoFileReader_H__
#define __PireoFileReader_H__

/* ITK / OTB Libraries */
#include "otbImage.h"
#include "otbImageFileReader.h"
#include "itkImageRegionIterator.h"
#include "itkRegionOfInterestImageFilter.h"
#include "itkFlipImageFilter.h"

/* Class PireoFileReader
   Its stocks all images needed for the Pireo application. That is to
   say nine images.
   - 0 = full resolution fixed image
   - 1 = full resolution moving image
   - 2 = zoomed fixed image
   - 3 = zoomed moving image
   - 4 = resampled fixed image
   - 5 = resampled moving image
   ... and also in order to work without modifing data set shown above
   - 6 = zoomed fixed image on which preprocessing is applyed
   - 7 = zoomed moving image on which preprocessing is applyed
   - 8 = resampled fixed image on which preprocessing is applyed
   - 9 = resampled moving image on which preprocessing is applyed

   This class reads loaded images (only ITK and GDAL recognized
   formats). That's why it encapsulates itkGDALImageIO class that
   enables to read file with GDAL.
*/
class PireoFileReader
{
public :
  /** Initialized ITK tools.*/
  typedef    double   InputPixelType;
  typedef otb::Image <InputPixelType, 2 >      ImageType;
  /** Image file reader.*/
  typedef otb::ImageFileReader< ImageType >    ImageReaderType;

  /** Used to filp image along Y as ITK and GDAL do not read the same
  * way*/
  typedef itk::FlipImageFilter< ImageType >  FilterType;

  /** Smart pointers to the nine images needed.*/
  /** Full resolution fixed image.*/
  ImageType::Pointer m_fixIm;
  /** Full resolution moving image.*/
  ImageType::Pointer m_movIm;
  /** Resampled fixed image (set to the real size of the VTK Window
      where both images are displayed in transparency).*/
  ImageType::Pointer m_SamplefixIm;
  /** Resampled moving image (set to the real size of the VTK Window
      where both images are displayed in transparency).*/
  ImageType::Pointer m_SamplemovIm;
  /** Zoomed fixed image that represents a 1024x1024 region of the
  full resolution fixed image.*/
  ImageType::Pointer m_ZoomfixIm;
  /** Zoomed fixed image that represents a 1024x1024 region of the
  full resolution moving image.*/
  ImageType::Pointer m_ZoommovIm;
  /** Zoomed fixed image used for pre-processing filtering.*/
  ImageType::Pointer m_PZoomfixIm;
  /** Zoomed moving image used for pre-processing filtering.*/
  ImageType::Pointer m_PZoommovIm;
  /** Resampled fixed image used for pre-processing filtering.*/
  ImageType::Pointer m_PSamplefixIm;
  // Resampled moving image used for pre-processing filtering.*/
  ImageType::Pointer m_PSamplemovIm;

  /** Region Of Interest Image Filter.
      Used to read a 1024x1024 region of an full resolution image.*/
  typedef itk::RegionOfInterestImageFilter<ImageType, ImageType>
  FilterRegionType;
  /** Region Of Interest Image Filter.
      Used to read a 1024x1024 region of an full resolution image.*/
  FilterRegionType::Pointer m_RegionReader;


  /** Construtor: set default booleen value and set zoomed image
  dimensions to 1024x1024*/
  PireoFileReader();
  /** Destructor: empty*/
  ~PireoFileReader();

  /** Return useful informations about dimensions of the images.
  Used only to set correct parameters to ZoomManager class.*/
  unsigned int GetFullResolutionImageDimensionX();
  /** Return useful informations about dimensions of the images.
  Used only to set correct parameters to ZoomManager class.*/
  unsigned int GetFullResolutionImageDimensionY();
  /** Return useful informations about dimensions of the images.
  Used only to set correct parameters to ZoomManager class.*/
  unsigned int GetResampledImageDimensionX();
  /** Return useful informations about dimensions of the images.
  Used only to set correct parameters to ZoomManager class.*/
  unsigned int GetResampledImageDimensionY();
  /** Return useful informations about dimensions of the images.
  Used only to set correct parameters to ZoomManager class.*/
  unsigned int GetZoomedImageDimensionX();
  /** Return useful informations about dimensions of the images.
  Used only to set correct parameters to ZoomManager class.*/
  unsigned int GetZoomedImageDimensionY();
  /** Return useful informations about dimensions of the images.
  Used only to set correct parameters to ZoomManager class.*/
  float GetFullResolutionImageSpacingX();
  /** Return useful informations about dimensions of the images.
  Used only to set correct parameters to ZoomManager class.*/
  float GetFullResolutionImageSpacingY();
  /** Return useful informations about dimensions of the images.
  Used only to set correct parameters to ZoomManager class.*/
  float GetResampledImageSpacingX();
  /** Return useful informations about dimensions of the images.
  Used only to set correct parameters to ZoomManager class.*/
  float GetResampledImageSpacingY();

  /** Set the dimensions of the resampled image to match those of the VTK
      Window.*/
  void SetResampledImageDimensions(int dimx, int dimy);

  /** Set origin of the 1024x1024 read region.*/
  void SetZoomedImageOrigin(int ox, int oy);
  /** Update the 1024x1024 Region when origin or dimensions change.*/
  bool Update();

  /** Set booleen m_TypeIm which says if we are working on the fixed or
      the moving image (true -> fixed and false -> moving).*/
  void SetImageType(bool Type);
  /** Return booleen m_TypeIm which says if we are working on the fixed
      or the moving image (true -> fixed and false -> moving).*/
  bool GetImageType();

  /** Read zoomed image which corresponds to a 1024x1024 region of the
      full image.*/
  bool ReadZoomedImage();
  /** Read full resolution image and from then set the two other images:
      resampled image and zoomed image.*/
  bool ReadImage(const char *file, bool FixIm);
  /** Check if loaded image belongs to the ITK Factory.*/
  bool CanReadFileWithITK(const char *file);
  /** Check if the filename extension is .tif (cause so far, no way is
      implemented to differentiate Geotiff and normal tiff formats).*/
  bool TiffFormat(const char* file);
  /** Return one of the nine image smart pointers used, depending on
  selected image number.*/
  ImageType::Pointer GetImage(int imagenumber);

  /** Booleen to know whether on not a fixed image has been loaded.*/
  bool FixedImageLoaded();
  /** Booleen to know whether on not a moving image has been loaded.*/
  bool MovingImageLoaded();

  /** Return the origin of the 1024x1024 read region (bottom-left
      corner of the read region).*/
  int GetZoomedImageOriginX();
  /** Return the origin of the 1024x1024 read region (bottom-left
      corner of the read region).*/
  int GetZoomedImageOriginY();

  /** Use to set the resampled and pre-processing images (that are
  determined with PireoFiltering object).*/
  void SetImage(ImageType* m_Im, int number);

protected:
  /** Origin of the 1024x1024 read region (bottom-left corner).*/
  int XBegin;
  int YBegin;
  /** Size of the buffer used in GDAL IO.*/
  int SizeXRead;
  int SizeYRead;
  /** Dimensions of the resampled image.*/
  int XSample;
  int YSample;
  /** Size of the region that is read (set to 1024x1024).*/
  int regionZoomx;
  int regionZoomy;
  /** Filename of the fixed image loaded.*/
  const char* filenameFI;
  /** Filename of the moving image loaded.*/
  const char* filenameMI;
  /** Booleen to know on which image we are working on: fixed or moving.*/
  bool m_TypeIm;
  /** Booleen to know if a fixed image has already been loaded.*/
  bool m_FixLoaded;
  /** Booleen to know if a moving image has already been loaded.*/
  bool m_MovLoaded;

};

#endif
