/*=========================================================================

  $Author: allain $
  $Date: 2005/07/12 15:20:13 $
  $Revision: 1.9 $
  $Id: PireoFileWriter.cxx, v 1.9 2005/07/12 15:20:13 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/PireoFileWriter.cxx, v $
  $Log: PireoFileWriter.cxx, v $
  Revision 1.9  2005/07/12 15:20:13  allain
  Add some functionalities

  Revision 1.8  2005/07/06 13:49:16  allain
  Update

  Revision 1.7  2005/07/06 13:32:21  allain
  Blender/Resampled Images changed

  Revision 1.6  2005/06/24 15:40:09  allain
  Add metric values graph and started registration levels observer

  Revision 1.5  2005/06/20 15:16:21  allain
  Add ITK registration parameters in the save text file

  Revision 1.4  2005/06/16 15:28:08  allain
  Add new functionalities

  Revision 1.3  2005/06/10 15:56:42  allain
  Add some functionalities for registration

  Revision 1.2  2005/06/06 16:16:56  allain
  Gave new method names, add commentary and implement new params

  Revision 1.1  2005/04/27 13:15:11  allain
  Change most of class names

  Revision 1.3  2005/03/04 13:49:42  allain
  GLSliceView file modified for 2D Image

  Revision 1.2  2005/03/03 16:52:44  allain
  Registration start and its observer

  Revision 1.1  2005/02/25 15:36:19  allain
   Add Savefile


=========================================================================*/

#include "PireoFileWriter.h"

#include "vtkImageShiftScale.h"
#include "otbImageFileReader.h"
#include "otbImage.h"
#include "vtkImageData.h"
#include "vtkJPEGWriter.h"
#include "vtkImageCast.h"

PireoFileWriter::PireoFileWriter()
{
  // Used to write a VTK file (save images of the blender)
  m_Vtkwriter = vtkStructuredPointsWriter::New();
}

PireoFileWriter::~PireoFileWriter()
{
  m_Vtkwriter->Delete();
  /*if(vtkImporter!=NULL)
    vtkImporter->Delete(); */
  if (buffer!=NULL)
    delete buffer;
}

vtkImageData* PireoFileWriter::ITKToVTKTransform(ImageType* m_Im)
{
  // If image pixel value are very small, the VTK image will be black,
  // so we need to rescale the intensity of the image
  RescalerFilterType::Pointer rescale = RescalerFilterType::New();
  rescale->SetInput((ImageType::Pointer)m_Im);
  rescale->SetOutputMaximum(255);
  rescale->SetOutputMinimum(0);
  try
  {
    rescale->Update();
  }
  catch ( itk::ExceptionObject & err )
  {
    std::cout << "ExceptionObject caught !" << std::endl;
    std::cout << err << std::endl;
  }
  vtkImageImport* vtkImporter = vtkImageImport::New();

  ImageType::RegionType region = rescale->GetOutput()->GetLargestPossibleRegion();
  ImageType::SizeType size = region.GetSize();
  int NbPts = (size[0]*size[1]);

  /******************************* Whole Extent !! *******************************************/
  vtkImporter->SetWholeExtent(1 , (int)(size[0]) , 1 , (int)( size[1]), 1 , 1);
  vtkImporter->SetDataExtent(1 , (int)(size[0]) , 1 , (int)( size[1]), 1 , 1);
  vtkImporter->SetDataScalarTypeToUnsignedChar();
  // Set Number of bands in the image
  vtkImporter->SetNumberOfScalarComponents(1);

  typedef  itk::ImageRegionIteratorWithIndex<ImageType> myIteratorType;

  myIteratorType it( rescale->GetOutput(), region);

  it.GoToBegin();

  unsigned int i=0;
  //unsigned int j=size[1]-1;
  buffer = new unsigned char[NbPts];

  while ( !it.IsAtEnd() )
  {
    // Read all information in the ITK file and set it into the buffer
    buffer[i] = (unsigned char)(it.Get());
    // buffer[i+size[0]*j] = (unsigned char)(it.Get());
    ++it;
    i++;
    /*if(i==size[0])
      {
      i = 0;
      j--;
      }*/
  }

  // Input buffer into a VTK importer
  vtkImporter->SetImportVoidPointer(buffer);

  int* tmp = new int[2];
  tmp[0] = size[0];
  tmp[1] = size[1];
  vtkImporter->GetOutput()->SetDimensions(tmp);

  // Generate VTK file
  vtkImageData* m_vtkIm = vtkImporter->GetOutput();

  return m_vtkIm;
}

void PireoFileWriter::SaveAsITKImage(ImageType::Pointer m_Im, const char* filename)
{
  // ITK image file writer
  m_Writer = ImageWriterType::New();
  m_Writer->SetFileName(filename);
  m_Writer->SetInput(m_Im);
  try
  {
    m_Writer->Update();
  }
  catch ( itk::ExceptionObject &err )
  {
    std::cout<<" (!) exception caught "<<std::endl;
    std::cout<< err <<std::endl;
  }
  std::cout<<"[SaveAsITKImage] Itk Image written "<<std::endl;
}

void PireoFileWriter::SaveITKAsVTKImage(ImageType::Pointer m_Im, const char* filename)
{
  /*vtkImageData* m_Dat = ITKToVTKTransform(m_Im);
  m_Vtkwriter->SetInput(m_Dat);
  if(filename!=NULL)
    m_Vtkwriter->SetFileName(filename);
  else
    m_Vtkwriter->SetFileName("Data.vtk");
  m_Vtkwriter->SetFileTypeToBinary();
  m_Vtkwriter->Update();
  m_Vtkwriter->Write();

  std::cout<<"[SaveITKAsVTKImage] Vtk Image written "<<std::endl; */
}


void PireoFileWriter::SaveAsVTKImage(vtkImageData* m_Dat, const char* filename)
{
  std::string ext(filename);
  ext = ext.substr(ext.rfind("."), ext.length());

  if ( strcmp((char*)ext.c_str(),".jpg")==0 )
  {
    vtkImageCast* cast = vtkImageCast::New();
    cast->SetInput(m_Dat);
    cast->SetOutputScalarTypeToUnsignedChar();
    cast->Update();
    vtkJPEGWriter* tiffWriter = vtkJPEGWriter::New();
    tiffWriter->SetInput(cast->GetOutput());
    tiffWriter->SetFileName(filename);
    //tiffWriter->SetFileDimensionality(2);
    tiffWriter->Write();
    std::cout<<"[PireoFileWriter::SaveAsVTKImage] File written."<<std::endl;
    tiffWriter->Delete();
    cast->Delete();
  }
  else
  {

    /*float c;
    int dim = m_Dat->GetDataDimension();
    std::cout<<"DataDim : "<<dim<<std::endl;

    m_Vtkwriter->SetInput(m_Dat);

    if(filename!=NULL)
      m_Vtkwriter->SetFileName(filename);
    else
      m_Vtkwriter->SetFileName("Data.vtk");

    m_Vtkwriter->SetFileTypeToBinary();

    m_Vtkwriter->Update();

    m_Vtkwriter->Write();

    std::cout<<"[SaveAsVTKImage] Vtk Image written "<<std::endl; */
  }
}

