/*=========================================================================

  $Autor$
  $Date: 2005/07/06 13:32:21 $
  $Revision: 1.4 $
  $Id: PireoFileWriter.h, v 1.4 2005/07/06 13:32:21 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/PireoFileWriter.h, v $
  $Log: PireoFileWriter.h, v $
  Revision 1.4  2005/07/06 13:32:21  allain
  Blender/Resampled Images changed

  Revision 1.3  2005/06/10 15:56:42  allain
  Add some functionalities for registration

  Revision 1.2  2005/06/06 16:16:56  allain
  Gave new method names, add commentary and implement new params

  Revision 1.1  2005/04/27 13:15:11  allain
  Change most of class names

  Revision 1.3  2005/03/04 13:49:42  allain
  GLSliceView file modified for 2D Image

  Revision 1.2  2005/03/03 16:52:44  allain
  Registration start and its observer

  Revision 1.1  2005/02/25 15:36:19  allain
   Add Savefile


=========================================================================*/

#ifndef __PireoFileWriter_H__
#define __PireoFileWriter_H__

/* C++ Libraries */
#include <iostream>
#include <string.h>

/* ITK Librairies */
#include "otbImage.h"
#include "itkVTKImageExport.h"
#include "itkVTKImageImport.h"
#include "itkImageRegionIteratorWithIndex.h"
#include "otbImageFileWriter.h"
#include "otbImageFileReader.h"
#include "itkRescaleIntensityImageFilter.h"

/* VTK Libraries */
#include "vtkImageImport.h"
#include "vtkImageExport.h"
#include "vtkImageData.h"
#include "vtkStructuredPointsWriter.h"

/** Class PireoFileWriter
    Take care of saving desired images either as a VTK file or a ITK
    file.
    Contains the important method that converts VTK into ITK file,
    which can also be used whithout needing to write an image. So it
    is the case to generate ITK images from the blender of the
    TransparencyWindowManager object. It is done in the method
    InputImagesIntoBlender() of the PireoApplication class and also in
    the method RefreshGUI() of the PireoObserverManager class.
*/
class PireoFileWriter
{
public:
  /** Constructor: instantiate a VTK structured points writer.*/
  PireoFileWriter();
  /** Destructor.*/
  ~PireoFileWriter();

  /** Pixel type of the ITK image.*/
  typedef    double  InputPixelType;
  /** ITK image type.*/
  typedef otb::Image<InputPixelType, 2> ImageType;
  /** ITK to VTK pipeline connection.*/
  typedef itk::VTKImageExport<ImageType> ImageExportType;

  /** ITK image file reader.*/
  typedef otb::ImageFileReader< ImageType >    ImageReaderType;
  /** ITK image file writer.*/
  typedef otb::ImageFileWriter< ImageType >    ImageWriterType;

  /** Rescaler to be used to avoid black VTK image when ITK image has
  * low pixel value.*/
  typedef itk::RescaleIntensityImageFilter<ImageType, ImageType>  RescalerFilterType;

  /** Convert an ITK image into a VTK file.*/
  vtkImageData* ITKToVTKTransform(ImageType* m_Im);
  /** Save an ITK image.*/
  void SaveAsITKImage(ImageType::Pointer m_Im, const char* filename);
  /** Save an ITK image as a VTK file (not working yet).*/
  void SaveITKAsVTKImage(ImageType::Pointer m_Im, const char* filename);
  /** Save a VTK file.*/
  void SaveAsVTKImage(vtkImageData* m_Dat, const char* filename);

protected:
  /** ITK image file reader.*/
  ImageReaderType::Pointer m_Reader;
  /** ITK image file writer.*/
  ImageWriterType::Pointer m_Writer;
  /** VTK file writer.*/
  vtkStructuredPointsWriter* m_Vtkwriter;
  /** ITK to VTK importer.*/
  //vtkImageImport* vtkImporter;
  /** Used to convert ITK file into VTK file.*/
  unsigned char *buffer;

};

#endif
