/*=========================================================================

  $Autor$
  $Date: 2005/07/06 13:49:16 $
  $Revision: 1.6 $
  $Id: PireoFiltering.cxx, v 1.6 2005/07/06 13:49:16 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/PireoFiltering.cxx, v $
  $Log: PireoFiltering.cxx, v $
  Revision 1.6  2005/07/06 13:49:16  allain
  Update

  Revision 1.5  2005/07/06 13:32:21  allain
  Blender/Resampled Images changed

  Revision 1.4  2005/06/10 15:56:42  allain
  Add some functionalities for registration

  Revision 1.3  2005/06/06 16:16:56  allain
  Gave new method names, add commentary and implement new params

  Revision 1.2  2005/05/02 15:33:01  allain
  Generic Registration and Configuration File installed

  Revision 1.1  2005/04/27 13:15:11  allain
  Change most of class names

  Revision 1.7  2005/03/23 17:34:11  allain
  Pre-Processing is working

  Revision 1.6  2005/03/22 17:40:18  allain
  Grid is working and Pre-processing starts

  Revision 1.5  2005/03/21 14:35:09  allain
  Interface modified

  Revision 1.4  2005/03/08 13:11:39  allain
  Set MultiResolutionImageRegistrationMethod

  Revision 1.3  2005/03/07 16:17:27  allain
  Registration with GDAL image starts working

  Revision 1.2  2005/03/04 13:49:42  allain
  GLSliceView file modified for 2D Image

  Revision 1.1  2005/03/04 07:59:11  allain
  Add Missing files


=========================================================================*/

#include "PireoFiltering.h"
#include "stdlib.h"

PireoFiltering::PireoFiltering()
{
}

PireoFiltering::~PireoFiltering()
{
}

// Set window size of the resampled image windows (down-left VTK window on
// the GUI) of the user interface.
// Method called directly in the main (StartPireo.cxx)
void PireoFiltering::SetWinDimensions(int dimx, int dimy)
{
  winsx = dimx;
  winsy = dimy;
}
// Only way to get back the real size of down-left VTK window in code
int PireoFiltering::GetWinDimensionX()
{
  return winsx;
}

// Only way to get back the real size of down-left VTK window in code
int PireoFiltering::GetWinDimensionY()
{
  return winsy;
}

PireoFiltering::ImageType2D*
PireoFiltering::ResampleToGUIWindowSize(ImageType2D* m_Im)
{
  t = TransformType::New();
  res = ResampleFilterType::New();

  // Set resample image size to corresponding GUI window size
  ImageType2D::SizeType size;
  // Set corresponding spacing
  double spacing[2];

  if (m_Im->GetLargestPossibleRegion().GetSize()[0] == m_Im->GetLargestPossibleRegion().GetSize()[1])
  {
    size[0] = winsx;
    size[1] = winsy;
    spacing[0] = m_Im->GetLargestPossibleRegion().GetSize()[0]*(m_Im->GetSpacing()[0])/size[0];
    spacing[1] = m_Im->GetLargestPossibleRegion().GetSize()[1]*(m_Im->GetSpacing()[1])/size[1];
  }
  else
  {
    // Resize keeping the proportions
    if (m_Im->GetLargestPossibleRegion().GetSize()[0]<m_Im->GetLargestPossibleRegion().GetSize()[1])
    {
      size[1] = winsy;
      size[0] = winsx*(m_Im->GetLargestPossibleRegion().GetSize()[0])/(m_Im->GetLargestPossibleRegion().GetSize()[1]);

      spacing[0] = m_Im->GetLargestPossibleRegion().GetSize()[0]*(m_Im->GetSpacing()[0])/size[0];
      spacing[1] = m_Im->GetLargestPossibleRegion().GetSize()[1]*(m_Im->GetSpacing()[1])/size[1];
    }
    else
    {
      size[0] = winsx;
      size[1] = winsy*(m_Im->GetLargestPossibleRegion().GetSize()[1])/(m_Im->GetLargestPossibleRegion().GetSize()[0]);
      spacing[0] = m_Im->GetLargestPossibleRegion().GetSize()[0]*(m_Im->GetSpacing()[0])/size[0];
      spacing[1] = m_Im->GetLargestPossibleRegion().GetSize()[1]*(m_Im->GetSpacing()[1])/size[1];
    }
  }
  // Set identity transform to resample image filter
  t->SetIdentity();
  res->SetTransform( t );
  // Resample input image
  res->SetInput( m_Im );

  res->SetSize( size);
  res->SetOutputOrigin( m_Im->GetOrigin());
  res->SetOutputSpacing( spacing);

  res->SetDefaultPixelValue( 0 );

  try
  {
    res->Update();
  }
  catch ( itk::ExceptionObject & err )
  {
    std::cout << "ExceptionObject caught !" << std::endl;
    std::cout << err << std::endl;
  }

  return res->GetOutput();
}

PireoFiltering::ImageType2D::Pointer
PireoFiltering::ResampleToFixedImageSize(ImageType2D* m_Im, ImageType2D* fixedIm)
{
  t = TransformType::New();
  ResampleFilterType::Pointer restofix = ResampleFilterType::New();

  // Set identity transform to resample image filter
  t->SetIdentity();
  restofix->SetTransform( t );
  // Resample input image
  restofix->SetInput( m_Im );

  restofix->SetSize( fixedIm->GetLargestPossibleRegion().GetSize());
  restofix->SetOutputOrigin( fixedIm->GetOrigin());
  restofix->SetOutputSpacing( fixedIm->GetSpacing());

  restofix->SetDefaultPixelValue( 0 );

  try
  {
    restofix->Update();
  }
  catch ( itk::ExceptionObject & err )
  {
    std::cout << "ExceptionObject caught !" << std::endl;
    std::cout << err << std::endl;
  }

  return restofix->GetOutput();
}

// Cast image from unsigned char pixel type to double pixel type
PireoFiltering::ImageRegistrationType2D::Pointer
PireoFiltering::CastImageToRegister(ImageType2D* m_Im)
{
  typedef itk::RescaleIntensityImageFilter< ImageType2D, ImageRegistrationType2D>  RescalerFilterType;
  RescalerFilterType::Pointer m_Rescaler = RescalerFilterType::New();
  m_Rescaler->SetOutputMinimum(0);
  m_Rescaler->SetOutputMaximum(255);
  m_Rescaler->SetInput(m_Im);

  try
  {
    m_Rescaler->Update();
  }
  catch ( itk::ExceptionObject & err )
  {
    std::cout << "ExceptionObject caught !" << std::endl;
    std::cout << err << std::endl;
  }
  return m_Rescaler->GetOutput();
}

// Cast image from double pixel type to unsigned char pixel type
PireoFiltering::ImageType2D::Pointer
PireoFiltering::CastImageToInitialState(ImageRegistrationType2D* m_Im)
{
  typedef itk::RescaleIntensityImageFilter< ImageRegistrationType2D, ImageType2D>  RescalerFilterType;
  RescalerFilterType::Pointer m_Rescaler = RescalerFilterType::New();
  m_Rescaler->SetOutputMinimum(0);
  m_Rescaler->SetOutputMaximum(255);
  m_Rescaler->SetInput(m_Im);

  try
  {
    m_Rescaler->Update();
  }
  catch ( itk::ExceptionObject & err )
  {
    std::cout << "ExceptionObject caught !" << std::endl;
    std::cout << err << std::endl;
  }

  return m_Rescaler->GetOutput();
}

// Normalize images (used for mutual information metric)
/*PireoFiltering::ImageType2D::Pointer
PireoFiltering::Normalize(ImageType2D::Pointer m_Im)
{
  typedef itk::NormalizeImageFilter<ImageType2D, ImageType2D>   NormalizeFilterType;
  NormalizeFilterType::Pointer filter = NormalizeFilterType::New();
  filter->SetInput(m_Im);
  try
    {
    filter->Update();
    }
  catch( itk::ExceptionObject & err )
    {
    std::cout << "ExceptionObject caught !" << std::endl;
    std::cout << err << std::endl;
    }
  return filter->GetOutput();
}

// Low-pass filtering (increase robustness against noisefor mutual information
// metric)
PireoFiltering::ImageType2D::Pointer
PireoFiltering::Smooth(ImageType2D::Pointer m_Im)
{
  typedef itk::DiscreteGaussianImageFilter<ImageType2D, ImageType2D>  GaussianFilterType;
  GaussianFilterType::Pointer filter = GaussianFilterType::New();
  filter->SetInput(m_Im);
  filter->SetVariance(2.0);
  try
    {
    filter->Update();
    }
  catch( itk::ExceptionObject & err )
    {
    std::cout << "ExceptionObject caught !" << std::endl;
    std::cout << err << std::endl;
    }
  return filter->GetOutput();
}*/

// Reduce Image Size
/*PireoFiltering::ImageType2D*
PireoFiltering::GetShrinkedImage(ImageType2D* m_Im)
{
  unsigned int factor[2];

  m_shrink = ImageFilterType::New();
  ImageType2D::SizeType size = m_Im->GetLargestPossibleRegion().GetSize();

  factor[0] = (unsigned int)(floor(size[0]/winsx)) + 1;
  factor[1] = (unsigned int)(floor(size[1]/winsy)) + 1;

  m_shrink->SetInput(m_Im);
  m_shrink->SetShrinkFactors(factor);

  try
    {
    m_shrink->Update();
    }
  catch( itk::ExceptionObject &err )
    {
    std::cout<<" (!) exception caught "<<std::endl;
    std::cout<< err <<std::endl;
    }

  return m_shrink->GetOutput();
}*/

/*PireoFiltering::ImageType2D*
PireoFiltering::ResampleImage(ImageType2D* m_Im, ImageType2D* m_ref)
{
  transform     = TransformType::New();
  resample = ResampleFilterType::New();

  transform->SetIdentity();

  resample->SetTransform( transform );
  resample->SetInput( m_Im );

  std::cout<<"Dim im origin : "<<m_ref->GetLargestPossibleRegion().GetSize()[0]<<"  "<<m_ref->GetLargestPossibleRegion().GetSize()[1]<<std::endl;
  resample->SetSize(    m_ref->GetLargestPossibleRegion().GetSize() );
  resample->SetOutputOrigin(  m_ref->GetOrigin() );
  resample->SetOutputSpacing( m_ref->GetSpacing() );
  resample->SetDefaultPixelValue( 100 );

  try
    {
    resample->Update();
    }
  catch( itk::ExceptionObject & err )
    {
    std::cout << "ExceptionObject caught !" << std::endl;
    std::cout << err << std::endl;
    }

  return resample->GetOutput();
}*/
