/*=========================================================================

  $Autor$
  $Date: 2005/07/06 13:32:21 $
  $Revision
  $Id: PireoFiltering.h, v 1.4 2005/07/06 13:32:21 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/PireoFiltering.h, v $
  $Log: PireoFiltering.h, v $
  Revision 1.4  2005/07/06 13:32:21  allain
  Blender/Resampled Images changed

  Revision 1.3  2005/06/10 15:56:42  allain
  Add some functionalities for registration

  Revision 1.2  2005/06/06 16:16:56  allain
  Gave new method names, add commentary and implement new params

  Revision 1.1  2005/04/27 13:15:11  allain
  Change most of class names

  Revision 1.5  2005/03/21 14:35:09  allain
  Interface modified

  Revision 1.4  2005/03/08 13:11:39  allain
  Set MultiResolutionImageRegistrationMethod

  Revision 1.3  2005/03/07 16:17:27  allain
  Registration with GDAL image starts working

  Revision 1.2  2005/03/04 13:49:42  allain
  GLSliceView file modified for 2D Image

  Revision 1.1  2005/03/04 07:59:11  allain
  Add Missing files


=========================================================================*/
#ifndef __PireoFiltering_H__
#define __PireoFiltering_H__

/* C++ Libraries */
#include <iostream>
#include <string.h>
#include <math.h>

/* ITK Librairies */
#include "otbImage.h"
#include "itkShrinkImageFilter.h"
#include "itkResampleImageFilter.h"
#include "itkSimilarity2DTransform.h"
#include "itkRescaleIntensityImageFilter.h"
/*#include "itkNormalizeImageFilter.h"
#include "itkDiscreteGaussianImageFilter.h"*/

/** Class PireoFiltering
    This class uses different filters to operate directly on images.
    It enables to cast image (indeed the interface work with unsigned
    char whereas the registration process asks for double).
    It also takes care of resampling image processes (uses to create the
    resampled images displayed in the down-left VTK window -and also
    in the smallest GL windows- of the GUI).
*/
class PireoFiltering
{
public:
  /** Output image pixel type.*/
  typedef double PixelType;
  /** Input image pixel type.*/
  typedef double RegistrationType;
  /** Interface image type.*/
  typedef otb::Image<PixelType, 2> ImageType2D;
  /** Registration image type.*/
  typedef otb::Image<RegistrationType, 2> ImageRegistrationType2D;

  /** Shrink image filter.*/
  typedef itk::ShrinkImageFilter<ImageType2D, ImageType2D> ImageFilterType;
  /** Shrink image filter.*/
  ImageFilterType::Pointer m_shrink;

  /** Resample image filter.*/
  typedef itk::ResampleImageFilter< ImageType2D, ImageType2D >    ResampleFilterType;
  /** ITK transform (used with resample image filter).*/
  typedef itk::Similarity2DTransform< double> TransformType;

  /** Constructor: empty.*/
  PireoFiltering();
  /** Destructor: empty.*/
  ~PireoFiltering();


  /** Set dimensions of the down-left VTK window to be used to resample
  loaded images to the same size as this window.*/
  void SetWinDimensions(int dimx, int dimy);
  /** Return dimension of the down-left VTK window.*/
  int GetWinDimensionX();
  /** Return dimension of the down-left VTK window.*/
  int GetWinDimensionY();
  /** Create a resampled image (at the GUI window real size i.e
  240x240) from the loaded image (full resolution image).*/
  ImageType2D* ResampleToGUIWindowSize(ImageType2D* m_Im);
  /** */
  ImageType2D::Pointer ResampleToFixedImageSize(ImageType2D* m_Im, ImageType2D* fixedIm);
  /** Cast image in order to apply to it the registration process
  (from unsigned char to double pixel type).*/
  ImageRegistrationType2D::Pointer CastImageToRegister(ImageType2D* m_Im );
  /** Cast image in order to display it on the interface (from double
  to unsigned char pixel type).*/
  ImageType2D::Pointer CastImageToInitialState(ImageRegistrationType2D* m_Im );


  /** Gaussian filter used to blur images for mutual information
  * metric use*/
  //ImageType2D::Pointer Smooth(ImageType2D::Pointer m_Im);
  /** Normalize image for mutual information metric use*/
  //ImageType2D::Pointer Normalize(ImageType2D::Pointer m_Im);

  /** Shrinked image (process to reduce loaded image size (not used
  anymore).*/
  //ImageType2D* GetShrinkedImage(ImageType2D* m_Im);
  /** Resample image after having shrinking imagefilter (not used
  anymore).*/
  //ImageType2D* ResampleImage(ImageType2D* m_Im, ImageType2D* m_ref);

protected:
  /** Real size of the down-left VTK window in the interface (240x240).*/
  int winsx;
  /** Real size of the down-left VTK window in the interface (240x240).*/
  int winsy;


  /** ITK transform (only useful for the resample image filter).*/
  TransformType::Pointer t;
  /** Resample image filter smart pointer.*/
  ResampleFilterType::Pointer res;

  /** ITK transform (only useful for the resample image filter).*/
  //TransformType::Pointer      transform;
  /** Resample image filter smart pointer*/
  //ResampleFilterType::Pointer resample;

};

#endif
