/*=========================================================================

  $Author: allain $
  $Date: 2005/07/13 13:33:50 $
  $Revision: 1.14 $
  $Id: PireoObserverManager.cxx, v 1.14 2005/07/13 13:33:50 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/PireoObserverManager.cxx, v $
  $Log: PireoObserverManager.cxx, v $
  Revision 1.14  2005/07/13 13:33:50  allain
  Pause button

  Revision 1.13  2005/07/12 15:20:13  allain
  Add some functionalities

  Revision 1.12  2005/07/06 16:07:14  allain
  still a bug with the grid

  Revision 1.11  2005/07/06 13:32:21  allain
  Blender/Resampled Images changed

  Revision 1.10  2005/07/04 15:46:33  allain
  Add results auto save

  Revision 1.9  2005/06/28 11:28:58  allain
  Update

  Revision 1.8  2005/06/24 15:40:09  allain
  Add metric values graph and started registration levels observer

  Revision 1.7  2005/06/15 14:41:20  allain
  Add registration level observer

  Revision 1.6  2005/06/14 15:38:03  allain
  Translation, Affine, OPOEO are working now

  Revision 1.5  2005/06/10 15:56:42  allain
  Add some functionalities for registration

  Revision 1.4  2005/06/06 16:16:56  allain
  Gave new method names, add commentary and implement new params

  Revision 1.3  2005/05/02 15:33:01  allain
  Generic Registration and Configuration File installed

  Revision 1.2  2005/04/29 13:44:53  allain
  Add Save Registration Parameters Method

  Revision 1.1  2005/04/27 13:15:11  allain
  Change most of class names

  Revision 1.11  2005/04/27 07:53:34  allain
  Add Save Registration Parameters

  Revision 1.10  2005/03/24 16:14:28  allain
  Pre-Processing structure done

  Revision 1.9  2005/03/23 17:34:11  allain
  Pre-Processing is working

  Revision 1.8  2005/03/22 17:40:18  allain
  Grid is working and Pre-processing starts

  Revision 1.7  2005/03/21 14:35:09  allain
  Interface modified

  Revision 1.6  2005/03/15 17:28:47  allain
  Add Grid class

  Revision 1.5  2005/03/15 11:04:47  allain
  Add Vector Field Window

  Revision 1.4  2005/03/08 13:11:39  allain
  Set MultiResolutionImageRegistrationMethod

  Revision 1.3  2005/03/07 16:17:27  allain
  Registration with GDAL image starts working

  Revision 1.2  2005/03/04 13:49:42  allain
  GLSliceView file modified for 2D Image

  Revision 1.1  2005/03/04 07:59:11  allain
  Add Missing files


=========================================================================*/

// Optimizer
#include "itkRegularStepGradientDescentOptimizer.h"
#include "itkConjugateGradientOptimizer.h"
#include "itkOnePlusOneEvolutionaryOptimizer.h"
#include "itkNormalVariateGenerator.h"
#include "itkGradientDescentOptimizer.h"
//#include "itkLBFGSModifiedOptimizer.h"

#include "PireoObserverManager.h"
#include "PireoViewer.h"

#include "vtkImageData.h"


PireoObserverManager::PireoObserverManager()
{
  // Give the number of the current iteration
  IterationNumber = 0;
  RefreshNumber = 1;
  // Choose to refresh Grid or VectorField
  choice = GRID;

  this->InitializeMetricValuesWindow();

  m_autoSave = false;
  m_displayMetric = false;
}

PireoObserverManager::~PireoObserverManager()
{
  if (win!=NULL)
    delete win;
}

void PireoObserverManager::InitializeMetricValuesWindow()
{
  // Concerns the metric values graph
  // New window
  if (win!=NULL)
    delete win;

  win=new Fl_Double_Window(380, 380,"Metric values");

  // New graph
  plot=new Fl_PlotXY(0, 0, 380, 380);
  plot->box(FL_UP_BOX);
  plot->align(FL_ALIGN_TOP);
  Fl_Group::current()->resizable(plot);
  // Line1 represents the metric values (color=red)
  Line1=plot->newline(0.0, 0.0, 0.0, 0.0, FL_PLOTXY_AUTO, FL_RED,"");
  Line2=plot->newline();
  plot->drawmode(Line2, FL_PLOTXY_DOT);
  Line3=plot->newline();
  Line4=plot->newline();
  plot->add(Line3, 0.0, 0.0);
  plot->add(Line4, 0.0, 0.0);
  plot->xmarkstep(Line1, 1);
  plot->xmarklabel(Line1,"Iteration No.");
  plot->ymarklabel(Line1,"Metric");
}

// When an iterative event occurs on the registration process
// optimizer, automatically this method is called
void PireoObserverManager::Execute(itk::Object *caller, const itk::EventObject & event)
{
  Execute( (const itk::Object *)caller, event);
}

// When an iterative event occurs on the registration process
// optimizer, automatically this method is called
void PireoObserverManager::Execute(const itk::Object * object, const itk::EventObject & event)
{
  IterationNumber++;

  // Get back the optimizer to be able to display its own information
  OptimizerPointer optimizer =
    dynamic_cast< OptimizerPointer >( object );

  if ( typeid( event ) != typeid( itk::IterationEvent ) )
  {

    return;
  }

  // Update registration metric values graph
  if (m_displayMetric)
    this->GetMetricValue(object);

  if (m_autoSave)
    m_pireoviewer->AutomaticSave(IterationNumber);

  if (IterationNumber%RefreshNumber!= 0)
  {
    return;
  }

  this->RefreshTransparencyWindow();

  this->GenerateParametersList();

  // Display optimizer information
  std::cout << "Iteration "<<IterationNumber;
  std::cout<<" Parameters : "<<optimizer->GetCurrentPosition() << std::endl;

  // Choose to display either the grid or the vector field
  switch (choice)
  {
  case VECTOR:
    this->SetVectorField();
    break;
  case GRID:
    this->RefreshGrid();
    break;
  }
}

void PireoObserverManager::ResetCounter()
{
  IterationNumber = 0;
}

void PireoObserverManager::SetRefreshStep(int a)
{
  RefreshNumber = a;
}

void PireoObserverManager::GenerateParametersList()
{
  m_pireoviewer->InsertNewRegistrationParameters();
}

void PireoObserverManager::SetDisplayChoice(DisplayType c)
{
  choice = c;
}

// Attache PireoViewer object to this observer
void PireoObserverManager::SetPireoViewer(PireoViewer* vi)
{
  m_pireoviewer = vi;
}

void PireoObserverManager::SetVectorField()
{
  m_pireoviewer->GetPireoFileReader()->SetImageType(true);
  // Set needed input to vector field
  m_pireoviewer->GetVectorField()->SetImageDimensions(m_pireoviewer->GetPireoFileReader()->GetResampledImageDimensionX(),
      m_pireoviewer->GetPireoFileReader()->GetResampledImageDimensionY());
  m_pireoviewer->GetVectorField()->Initialize();
  // From registration method get the transformed points
  m_pireoviewer->GetRegistration()->TransformPointSet(m_pireoviewer->GetVectorField()->GetPointSet(), (float)((m_pireoviewer->GetVectorField()->GetStepX())/4),
      (float)((m_pireoviewer->GetVectorField()->GetStepY())/4));
  // Get back the vector field renderer
  m_pireoviewer->GetVectorField()->SetRenderer();
  // Display new vector field obtained
  m_pireoviewer->UpdateVectorField();
}

void PireoObserverManager::RefreshGrid()
{
  m_pireoviewer->GetPireoFileReader()->SetImageType(true);
  // Set needed input to grid
  m_pireoviewer->GetGrid()->SetImageDimensions(m_pireoviewer->GetPireoFileReader()->GetResampledImageDimensionX(),
      m_pireoviewer->GetPireoFileReader()->GetResampledImageDimensionY());

  std::cout<<m_pireoviewer->GetPireoFileReader()->GetImage(5)->GetLargestPossibleRegion().GetSize()<<std::endl;

  m_pireoviewer->GetGrid()->Initialize();
  // From registration method get the transformed points
  m_pireoviewer->GetRegistration()->TransformGrid(m_pireoviewer->GetGrid()->GetPointSet());
  // Get back the grid renderer
  m_pireoviewer->GetGrid()->SetRenderer();
  // Display new grid obtained
  m_pireoviewer->UpdateGrid();
}

void PireoObserverManager::RefreshTransparencyWindow()
{
  // Needed to resample the moving image to the size of
  // the fixed image
  int fixnumber = 4;
  if (m_pireoviewer->GetPreProcessFiltering()->SelectFixed())
    fixnumber = 8;
  else
    fixnumber = 4;

  // From registration method get the new deformed image
  vtkImageData* slave = m_pireoviewer->GetPireoFileWriter()->
                        ITKToVTKTransform(m_pireoviewer->GetPireoFiltering()->ResampleToFixedImageSize(m_pireoviewer
                                          ->GetPireoFiltering()->CastImageToInitialState(m_pireoviewer->GetRegistration()->GetRegistrationResult()),
                                          m_pireoviewer->GetPireoFileReader()->GetImage(fixnumber)));
  // Update deformed image into the blender
  m_pireoviewer->GetTransparencyWindow()->SetSlave(slave);

  // If blender renderer has already been set, remove it
  if (m_pireoviewer->GetRenderer()!=NULL)
    m_pireoviewer->GetRenderWindow()->RemoveRenderer(m_pireoviewer->GetRenderer());
  // Update transparency window
  m_pireoviewer->GetTransparencyWindow()->SetRenderer();

  // Set new renderer into PireoApplication class
  m_pireoviewer->SetRenderer(m_pireoviewer->GetTransparencyWindow()->GetRenderer());
  // Add it to the render window of PireoApplication class
  m_pireoviewer->GetRenderWindow()->AddRenderer(m_pireoviewer->GetRenderer());
  // Update GUI
  m_pireoviewer->UpdateRegistrationWindow();

}

void PireoObserverManager::SetAutoSave(bool s)
{
  m_autoSave = s;
}

void PireoObserverManager::DisplayMetric(bool a)
{
  m_displayMetric = a;
}

void PireoObserverManager::GetMetricValue(const itk::Object * object)
{
  // Get back registration optimizer type
  RegistrationOptimizerParameter OP;
  m_pireoviewer->GetRegistration()->GetOptimizerType(&OP);

  double v(0.); //, iter;
  // Get metric value
  switch (OP)
  {
  case RSG :
  {
    const itk::RegularStepGradientDescentOptimizer* RSGO = dynamic_cast< const itk::RegularStepGradientDescentOptimizer* >( object );
    std::cout<<"METRIC VALUE: "<<RSGO->GetValue()<<std::endl;
    v = RSGO->GetValue();
    //iter = RSGO->GetCurrentIteration();
    break;
  }
  case LBFGS :
  {
//    const  itk::LBFGSModifiedOptimizer* LBFGSO = dynamic_cast< const itk::LBFGSModifiedOptimizer* >( object );
    break;
  }
  case CG:
  {
//    const  itk::ConjugateGradientOptimizer* CGO = dynamic_cast< const itk::ConjugateGradientOptimizer* >( object );
    break;
  }
  case OPOE:
  {
    const  itk::OnePlusOneEvolutionaryOptimizer* OPOEO = dynamic_cast< const itk::OnePlusOneEvolutionaryOptimizer* >( object );
    std::cout<<"METRIC VALUE: "<<OPOEO->GetValue()<<std::endl;
    v = OPOEO->GetValue();
    //iter = OPOEO->GetCurrentIteration();
    break;
  }
  case GD:
  {
    const  itk::GradientDescentOptimizer* GDO = dynamic_cast< const itk::GradientDescentOptimizer* >( object );
    std::cout<<"METRIC VALUE: "<<GDO->GetValue()<<std::endl;
    v = GDO->GetValue();
    //iter = GDO->GetCurrentIteration();
    break;
  }
  }

  // Display graph that represents the registration metric values,
  // adding the new value for the current iteration
  this->SetUpWindow(v, IterationNumber);
}

// Create a FLTK graph window (displays the registration metric values
// over iterations)
void PireoObserverManager::SetUpWindow(double v, double iter)
{
  plot->add(Line1, iter, v);
  plot->add(Line2, iter, v);

  double X=0, Y=0, X1=0, Y1=0;
  double step = 10;

  plot->xmarkon(0, FL_PLOTXY_MARK_ON|FL_PLOTXY_MARK_LABEL|FL_PLOTXY_MARK_VAL);
  plot->ymarkon(0, FL_PLOTXY_MARK_ON|FL_PLOTXY_MARK_LABEL|FL_PLOTXY_MARK_VAL);
  plot->getrange(0, &X, &Y, &X1, &Y1);
  if (iter<10)
    plot->xmarkstep(0, 1.0);
  else if (iter<100)
    plot->xmarkstep(0, 10.0);
  else
    plot->xmarkstep(0, 100.0);

  plot->ymarkstep(0, (Y1-Y)/step);

  plot->add(Line3, 1.0, 0.0); //(X1-X)/step, 0.0);
  plot->add(Line4, 0.0, (Y1-Y)/step);

  win->show();
  win->redraw();
}
