/*=========================================================================

  $Author: allain $
  $Date: 2005/07/12 15:20:13 $
  $Revision: 1.9 $
  $Id: PireoObserverManager.h, v 1.9 2005/07/12 15:20:13 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/PireoObserverManager.h, v $
  $Log: PireoObserverManager.h, v $
  Revision 1.9  2005/07/12 15:20:13  allain
  Add some functionalities

  Revision 1.8  2005/07/04 15:46:33  allain
  Add results auto save

  Revision 1.7  2005/06/24 15:40:09  allain
  Add metric values graph and started registration levels observer

  Revision 1.6  2005/06/15 14:41:20  allain
  Add registration level observer

  Revision 1.5  2005/06/14 15:38:03  allain
  Translation, Affine, OPOEO are working now

  Revision 1.4  2005/06/06 16:16:56  allain
  Gave new method names, add commentary and implement new params

  Revision 1.3  2005/05/02 15:33:01  allain
  Generic Registration and Configuration File installed

  Revision 1.2  2005/04/29 13:44:53  allain
  Add Save Registration Parameters Method

  Revision 1.1  2005/04/27 13:15:11  allain
  Change most of class names

  Revision 1.6  2005/04/27 07:53:34  allain
  Add Save Registration Parameters

  Revision 1.5  2005/03/22 17:40:18  allain
  Grid is working and Pre-processing starts

  Revision 1.4  2005/03/15 17:28:47  allain
  Add Grid class

  Revision 1.3  2005/03/15 11:04:47  allain
  Add Vector Field Window

  Revision 1.2  2005/03/07 16:17:27  allain
  Registration with GDAL image starts working

  Revision 1.1  2005/03/04 07:59:11  allain
  Add Missing files


=========================================================================*/

#ifndef __PireoObserverManager_H__
#define __PireoObserverManager_H__

#include "itkCommand.h"
#include "itkOptimizer.h"

#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>
#include "Fl_PlotXY.H"
#include "Structures.h"
#include "stdio.h"

// This class has access to everything of the Pireo process
class PireoViewer;

/** Class PireoObserverManager
    It inherits from itkCommand.
    This class is the key object to observe the ITK registration process
    at each iteration. It is set to the optimizer of the
    RegistrationMethod as an observer. Therefore, it enables to
    refresh GUI or do simple operations at each process iteration.
    This class mainly takes care of updating the GUI during the registration
    process.
*/
class PireoObserverManager : public itk::Command
{
public:
  /** ITK implementation.*/
  typedef  PireoObserverManager     Self;
  /** ITK implementation.*/
  typedef  itk::Command             Superclass;
  /** ITK implementation.*/
  typedef itk::SmartPointer<Self>  Pointer;

  /** ITK implementation.*/
  itkNewMacro( Self );

protected:
  /** Construtor.*/
  PireoObserverManager();
  /** Destructor.*/
  ~PireoObserverManager();


public:
  /** As we set the observer to the registration optimizer we already
  know that it will be an optimizer.*/
  typedef itk::Optimizer  OptimizerType;
  /** As we set the observer to the registration optimizer we already
  know that it will be an optimizer.*/
  typedef const OptimizerType                         *OptimizerPointer;

  /** Execute method call when iterative event occurs.*/
  virtual void Execute(itk::Object *caller, const itk::EventObject & event);
  /** Execute method call when iterative event occurs.*/
  virtual void Execute(const itk::Object * object, const itk::EventObject & event);

  /** Initialize metric values Fl window*/
  void InitializeMetricValuesWindow();
  /** Get Metric value at each iteration.*/
  void GetMetricValue(const itk::Object * object);
  /** Display Graph that represents the metric values over iterations.*/
  void SetUpWindow(double v, double iter);
  /** Enable to link the PireoViewer object with this class.*/
  void SetPireoViewer(PireoViewer* vi);
  /** Reset iteration counter*/
  void ResetCounter();
  /** Refresh GUI for each iteration that is a multiple of
  * RefreshNumber.*/
  void SetRefreshStep(int a);
  /** Generate iteration after iteration the registration parameters
  list.*/
  void GenerateParametersList();
  /** Refresh grid at each iteration.*/
  void RefreshGrid();
  /** Refresh vector field at each iteration.*/
  void SetVectorField();
  /** Refresh transparency window.*/
  void RefreshTransparencyWindow();
  /** Set the choice to display either the grid or the vector field.*/
  void SetDisplayChoice(DisplayType c);
  /** Get the choice made by user to display either the grid or the
  vector field.*/
  DisplayType GetDisplayChoice()
  {
    return choice;
  }
  /** Automatically save registration results.*/
  void SetAutoSave(bool s);
  /** Set up metric status: display metric value or not while
  * processing registration.*/
  void DisplayMetric(bool a);

protected:
  /** PireoViewer object (get back from the pireo process).*/
  PireoViewer* m_pireoviewer;
  /** Choice object: display grid or vector field ?.*/
  DisplayType  choice;
  /** Used to refresh GUI only for a certain number of iteration and
  not for each iteration.*/
  int RefreshNumber;
  /** Used to have access to the number of the current iteration.*/
  int IterationNumber;
  /** Booleen to know if results must be saved at each iteration.*/
  bool m_autoSave;
  /** Booleen to display or not the metric value while processing
  * registration.*/
  bool m_displayMetric;

  /** Fl window to display metric values graph.*/
  Fl_Double_Window* win;
  /** Fl graph to display metric values.*/
  Fl_PlotXY* plot;
  /** Object that contains the metric values.*/
  int Line1;
  /** Object that contains the metric values.*/
  int Line2;
  /** Object used to display graph borders.*/
  int Line3;
  /** Object used to display graph borders.*/
  int Line4;

};

#endif
