/*=========================================================================

  $Author: allain $
  $Date: 2005/06/24 15:40:09 $
  $Revision: 1.2 $
  $Id: PireoRegistrationLevelObserver.h, v 1.2 2005/06/24 15:40:09 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/PireoRegistrationLevelObserver.h, v $
  $Log: PireoRegistrationLevelObserver.h, v $
  Revision 1.2  2005/06/24 15:40:09  allain
  Add metric values graph and started registration levels observer

  Revision 1.1  2005/06/15 14:41:20  allain
  Add registration level observer


=========================================================================*/

#ifndef __PireoRegistrationLevelObserver_H__
#define __PireoRegistrationLevelObserver_H__

#include "itkMultiResolutionImageRegistrationMethod.h"
#include "itkCommand.h"
#include "itkOptimizer.h"

// This class has access to everything of the Pireo process
class PireoViewer;

/** Class PireoRegistrationLevelObserver
    It inherits from PireoObserverManager.
    This class enables to change registration parameters while
    processing.
*/
class PireoRegistrationLevelObserver: public itk::Command
{
public:
  /** ITK implementation.*/
  typedef  PireoRegistrationLevelObserver     Self;
  /** ITK implementation.*/
  typedef  itk::Command            Superclass;
  /** ITK implementation.*/
  typedef itk::SmartPointer<Self>  Pointer;

  /** ITK implementation.*/
  itkNewMacro( Self );

protected:
  /** Construtor.*/
  PireoRegistrationLevelObserver();
  /** Destructor.*/
  ~PireoRegistrationLevelObserver();


public:
  /** Registration image pixel type (can only be double or float).*/
  typedef float PixelType;
  /** Registration image type.*/
  typedef itk::Image<PixelType, 2> ImageType2D;

  /** As we set the observer to the registration optimizer we already
  know that it will be a multi-resolution registration.*/
  typedef itk::MultiResolutionImageRegistrationMethod< ImageType2D, ImageType2D >    RegistrationType;
  typedef   RegistrationType *                         RegistrationPointer;

  /** As we set the observer to the registration optimizer we already
  know that it will be an optimizer.*/
  typedef itk::Optimizer  OptimizerType;
  /** As we set the observer to the registration optimizer we already
  know that it will be an optimizer.*/
  typedef const OptimizerType                         *OptimizerPointer;

  /** Execute method call when iterative event occurs.*/
  virtual void Execute(itk::Object *caller, const itk::EventObject & event);
  /** Execute method call when iterative event occurs.*/
  virtual void Execute(const itk::Object * object, const itk::EventObject & event);

  /** Enable to link the PireoViewer object with this class.*/
  void SetPireoViewer(PireoViewer* vi);

  /** Get registration optimizer type*/
  //void GetOptimizer(const itk::Object * object);

protected:
  /** PireoViewer object (get back from the pireo process).*/
  PireoViewer* m_pireoviewer;
};

#endif
