/*====================================================================

  $Author: allain $
  $Date: 2005/06/15 14:41:20 $
  $Revision: 1.5 $
  $Id: PireoViewer.cxx, v 1.5 2005/06/15 14:41:20 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/PireoViewer.cxx, v $
  $Log: PireoViewer.cxx, v $
  Revision 1.5  2005/06/15 14:41:20  allain
  Add registration level observer

  Revision 1.4  2005/06/14 15:38:03  allain
  Translation, Affine, OPOEO are working now

  Revision 1.3  2005/06/06 16:16:56  allain
  Gave new method names, add commentary and implement new params

  Revision 1.2  2005/05/02 15:33:01  allain
  Generic Registration and Configuration File installed

  Revision 1.1  2005/04/27 13:15:11  allain
  Change most of class names

  Revision 1.20  2005/03/24 16:14:28  allain
  Pre-Processing structure done

  Revision 1.19  2005/03/23 17:34:11  allain
  Pre-Processing is working

  Revision 1.18  2005/03/21 14:35:09  allain
  Interface modified

  Revision 1.17  2005/03/16 17:09:08  allain
  Add some commentaries

  Revision 1.16  2005/03/15 11:04:47  allain
  Add Vector Field Window

  Revision 1.15  2005/03/04 13:49:42  allain
  GLSliceView file modified for 2D Image

  Revision 1.14  2005/03/03 16:52:44  allain
  Registration start and its observer

  Revision 1.13  2005/02/25 15:33:10  allain
  Transparency Window Set

  Revision 1.12  2005/02/23 17:48:29  allain
  ITK To VTK IMage start

  Revision 1.11  2005/02/23 16:09:58  allain
  New Class Hierarchy

  Revision 1.10  2005/02/22 14:53:08  allain
  Display image with flip option

  Revision 1.9  2005/02/21 17:09:31  allain
  New InterFace to display image

  Revision 1.8  2005/02/15 15:20:17  allain
  Read a region of an image woth GDAL and ITK

  Revision 1.7  2005/02/14 16:48:59  allain
  CanReadFile() validated

  Revision 1.6  2005/02/14 10:15:59  allain
  Sauvegarde avant ajout memoire

  Revision 1.5  2005/02/11 16:27:21  allain
  Mouse interaction start ..missed files in cvs

  Revision 1.4  2005/02/11 08:49:46  allain
  IHM beginning

  Revision 1.3  2005/02/10 10:44:35  allain
  Changed Cmake

  Revision 1.2  2005/02/10 07:51:27  allain
  Add itkGDALImageIO and the header of each file


====================================================================*/


// About Resizable Window !!!
/*Here are a few hints. Read them carefully then try again. Good luck!

- to make things more predictable, it helps to fill groups with
  widgets only in one direction: either horizontally or vertically.
  (this also helps explain the following hints).

- If you need to fill groups both horizontally and vertically,
  fill a group WITH GROUPS in one direction, then those 'sub'groups
  in the other direction.

- In a group, at most one widget can be set to "resizable".
  Attempts to setting several widgets to 'resizable' causes
  only the last one to be marked resizable.

- Setting a widget to "resizable" means that that widget can be
  resized BOTH horizontally and vertically, not that it is the
  only resizable widget in the group.

- all other widgets in the group may resize along proportionally to
  the size of the group, but only in one direction (if the group
  is populated horizontally, 'nonresizable' widgets only resize
  vertically, only the 'resizable' widget resizes in both directions.

- if a group is only resizable in one direction, only the resizable
  widget will resize, all other widgets will stay the way they are.
*/

#include "PireoViewer.h"

// Disabling deprecation warning
#ifdef _MSC_VER
#pragma warning(push)
#pragma warning(disable:4996)
#endif

/********************************************************************/
/*                        Constructor                               */
/*                     Initialize the user interface (HMI)          */
/********************************************************************/
PireoViewer :: PireoViewer(): PireoViewerGUI()
{
  // Booleen to check if an image has been loaded
  m_imageloaded = false;

  // Registration Observer(used to have access to every class from
  // PireoObserverManager class where the GUI is refreshed during
  // the registration process)
  this->GetRegistration()->GetObserver()->SetPireoViewer(this);
  this->GetRegistration()->GetRegistrationLevelObserver()->SetPireoViewer(this);
}

/********************************************************************/
/*                        Destructor                                */
/********************************************************************/
PireoViewer :: ~PireoViewer()
{
}

/********************************************************************/
/*                        Quit Application                          */
/********************************************************************/
void PireoViewer :: Quit()
{
  //window -> hide();
  exit(-1);
}

/********************************************************************/
/*                        Update each FLTK Window                   */
/*         Reload the desired image for each FLTK Window            */
/********************************************************************/
// Reload the resampled fixed image and the zoomed fixed image
// The image called zoomed image represents a region only of the fixed
// image

// Concerns the first window (up-left)
void PireoViewer::UpdateFixedWindow()
{
  Display(2);
  Display(4);
}

// Concerns the second window (up-right)
void PireoViewer::UpdateMovingWindow()
{
  Display(3);
  Display(5);
}

// Concerns both GL windows (up-left and up-right)
void PireoViewer::UpdateGL()
{
  if (m_pireofilereader->FixedImageLoaded())
    UpdateFixedWindow();
  if (m_pireofilereader->MovingImageLoaded())
    UpdateMovingWindow();
}

// Reload the two VTK images representing the fixed and moving
// images, view in transparency
// Concerns the third window (down-left)
void PireoViewer::UpdateRegistrationWindow()
{
  Fl::check();
  m_vtkWin->redraw();
  Fl::check();
}

/********************************************************************/
/*                        Manage the flipping process               */
/********************************************************************/
// Depending if it concerns the fixed image or the moving image
// the resampled image and the zoomed image are flipped along OX
void PireoViewer::FlipX(bool flip, bool FixIm)
{
  if (FixIm)
  {
    tkZoomFixedWin -> flipX(flip);
    tkZoomFixedWin->winZoom(1.0);
    tkFixedWin -> flipX(flip);
    tkFixedWin->winZoom(1.0);
    Display(2);
    Display(4);
  }
  else
  {
    tkZoomMovingWin -> flipX(flip);
    tkMovingWin -> flipX(flip);
    Display(3);
    Display(5);
  }
}
// Depending if it concerns the fixed image or the moving image
// the sample image and the zoom image are flipped along OY
void PireoViewer::FlipY(bool flip, bool FixIm)
{
  if (FixIm)
  {
    tkZoomFixedWin -> flipY(flip);
    tkFixedWin -> flipY(flip);
    Display(2);
    Display(4);
  }
  else
  {
    tkZoomMovingWin -> flipY(flip);
    tkMovingWin -> flipY(flip);
    Display(3);
    Display(5);
  }
}

/********************************************************************/
/*                        Create Scene                              */
/********************************************************************/
// To fill the Rendering Scene when first displayed
void PireoViewer::CreateScene()
{
  // Create an image to draw the background of the renderer in black
  // when the window is generated
  ViewedImageType::SizeType size0 = {{128, 128}};
  ViewedImageType::IndexType index0 = {{0, 0}};
  ViewedImageType::RegionType region0;
  region0.SetSize( size0 );
  region0.SetIndex( index0 );
  ViewedImageType::Pointer Image_init = ViewedImageType::New();
  Image_init->SetLargestPossibleRegion( region0 );
  Image_init->SetBufferedRegion( region0);
  Image_init->SetRequestedRegion( region0);
  Image_init->Allocate();
  Image_init->FillBuffer(0);

  double spacing[2];
  spacing[0]=1;
  spacing[1]=1;
  Image_init->SetSpacing(spacing);

  // Input this image to the FLTK-ITK windows where images are
  // displayed
  tkFixedWin -> SetInputImage( Image_init );
  tkZoomFixedWin -> SetInputImage( Image_init );
  tkMovingWin -> SetInputImage( Image_init );
  tkZoomMovingWin -> SetInputImage( Image_init );
  // Decide to display or not the image details (size, range value...)
  // for each window and the coordinates of the point fixed by the
  // green cursor
  tkFixedWin->viewDetails(false);
  tkFixedWin->viewValue(true);
  tkZoomFixedWin->viewDetails(false);
  tkZoomFixedWin->viewValue(true);
  tkMovingWin->viewValue(true);
  tkMovingWin->viewDetails(false);
  tkZoomMovingWin->viewDetails(false);
  tkZoomMovingWin->viewValue(true);
  // Show Windows
  tkFixedWin -> show();
  tkFixedWin->update();
  tkZoomFixedWin -> show();
  tkZoomFixedWin->update();
  tkMovingWin -> show();
  tkMovingWin -> update();
  tkZoomMovingWin -> show();
  tkZoomMovingWin -> update();
  // Show the GLSliceView windows (where fixed and moving images will
  // be set)
  Fl::check();
  tkFixedWin ->redraw();
  tkZoomFixedWin ->redraw();
  tkMovingWin ->redraw();
  tkZoomMovingWin ->redraw();
  Fl::check();
  // Init the VTK Windows (view in transparency and grid of
  // registration process) by setting a black renderer
  m_vtkWin->show();
  m_vtkWin->SetRenderWindow(m_renwin);
  window->resizable(m_vtkWin);
  m_VectorWin->show();
  m_VectorWin->SetRenderWindow(m_VecWin);
  window->resizable(m_VectorWin);
  window->show();
  m_vtkWin->Initialize();
  m_VectorWin->Initialize();

}

/********************************************************************/
/*                        Display Images                            */
/********************************************************************/
void PireoViewer::Display(int number)
{
  switch (number)
  {
  case 0:
  case 1:
  {
    std::cout<<"[PireoViewer::Display] Can't be displayed yet.\n";
    break;
  }
  case 2:
  {
    if (!this->GetPireoFileReader()->FixedImageLoaded())
      return;

    //Input fixed zoomed image
    if (this->GetPreProcessFiltering()->SelectFixed())
      number = 6;
    else
      number = 2;
    std::cout<<"zoom number: "<< number<<std::endl;
    RescalerType::Pointer rescaler = RescalerType::New();
    rescaler->SetOutputMinimum(0);
    rescaler->SetOutputMaximum(255);
    rescaler->SetInput(this->GetPireoFileReader()
                       ->GetImage(number));
    rescaler->Update();
    tkZoomFixedWin -> SetInputImage(rescaler->GetOutput() );
    tkZoomFixedWin -> update();
    tkZoomFixedWin -> show();

    Fl::check();
    tkZoomFixedWin ->redraw();
    Fl::check();

    break;
  }
  case 3:
  {
    if (!this->GetPireoFileReader()->MovingImageLoaded())
      return;

    // Input moving zoomed image
    if (this->GetPreProcessFiltering()->SelectMoving())
      number = 7;
    else
      number = 3;
    RescalerType::Pointer rescaler = RescalerType::New();
    rescaler->SetOutputMinimum(0);
    rescaler->SetOutputMaximum(255);
    rescaler->SetInput(this->GetPireoFileReader()
                       ->GetImage(number));
    rescaler->Update();
    tkZoomMovingWin -> SetInputImage(rescaler->GetOutput() );
    tkZoomMovingWin -> update();
    tkZoomMovingWin -> show();

    Fl::check();
    tkZoomMovingWin ->redraw();
    Fl::check();
    break;
  }
  case 4:
  {
    if (!this->GetPireoFileReader()->FixedImageLoaded())
      return;

    // Input fixed resampled image
    if (this->GetPreProcessFiltering()->SelectFixed())
      number = 8;
    else
      number = 4;
    std::cout<<"resample number: "<< number<<std::endl;
    RescalerType::Pointer rescaler = RescalerType::New();
    rescaler->SetOutputMinimum(0);
    rescaler->SetOutputMaximum(255);
    rescaler->SetInput(this->GetPireoFileReader()
                       ->GetImage(number));
    rescaler->Update();
    tkFixedWin -> SetInputImage(rescaler->GetOutput() );
    tkFixedWin -> update();
    tkFixedWin -> show();

    Fl::check();
    tkFixedWin ->redraw();
    Fl::check();
    break;
  }
  case 5:
  {
    if (!this->GetPireoFileReader()->MovingImageLoaded())
      return;

    // Input moving resampled image
    if (this->GetPreProcessFiltering()->SelectMoving())
      number = 9;
    else
      number = 5;
    RescalerType::Pointer rescaler = RescalerType::New();
    rescaler->SetOutputMinimum(0);
    rescaler->SetOutputMaximum(255);
    rescaler->SetInput(this->GetPireoFileReader()
                       ->GetImage(number));
    rescaler->Update();
    tkMovingWin -> SetInputImage(rescaler->GetOutput());
    tkMovingWin -> update();
    tkMovingWin -> show();

    Fl::check();
    tkMovingWin ->redraw();
    Fl::check();
    break;
  }
  default:
  {
    std::cout<<"[PireoViewer::Display] No existing data.\n";
    break;
  }
  }
}
/********************************************************************/
/*             Check if the filename selected is correct            */
/*             And return the name of the file as a char            */
/********************************************************************/
bool PireoViewer::CheckFileName(Fl_File_Chooser* fc, char file[500])
{
  if (fc == NULL)
    return false;

  if (fc->count()!=1)
    return false;
  else
  {
    // Get filename
    strcpy(file, fc->value());
    delete fc;

    if ( (file==NULL) || (strlen(file)<1) )
      return false;
    else
      return true;
  }
}

/********************************************************************/
/*                        Read GDAL/ITK formats File                */
/********************************************************************/
void PireoViewer::ReadFile(const char* fc, bool TypeIm)
{
  bool m_problem = false;

  // Call method OpenFile implemented in PireoApplication Class
  if (this->OpenFile(fc, TypeIm))
    m_problem = true;

  if (m_problem)
  {
    std::cout<<"[PireoViewer::ReadGDALFile] Error Reading File. \n";
    return;
  }

  m_imageloaded = !m_problem;

  // Insert filenames into the GUI
  Fl_Text_Buffer* intext = new Fl_Text_Buffer(500);
  intext->text(fc);
  if (TypeIm)
    this->m_InputFIm->buffer(intext);
  else
    this->m_InputMIm->buffer(intext);

}

/********************************************************************/
/*                        Load Image                                */
/*   Booleen used to determined if image to load is fixed or moving */
/********************************************************************/
void PireoViewer::LoadImage(bool TypeIm)
{
  // To let the user choose the file in no matter directory
  // If later we want to choose the two files at the same time :
  // change SINGLE into MULTI

  const char * ImageName = fl_file_chooser("Image:", "*.*",".");

  if (ImageName == NULL || strlen(ImageName)<1)
  {
    return;
  }

  ReadFile(ImageName, TypeIm);

  if (m_imageloaded)
  {
    if (TypeIm)
    {
      Display(2);
      Display(4);
    }
    else
    {
      Display(3);
      Display(5);
    }
  }
  else
    std::cout<<"[PireoViewer::LoadGDALImage] Cannot read file.\n";
}

/********************************************************************/
/*                        Load Configuration File                   */
/*             Load registration parameters from a text file        */
/********************************************************************/
void PireoViewer::LoadConfigurationFile()
{
  char file[500];

  Fl_File_Chooser* cf = new Fl_File_Chooser("./", "*.txt*",
      Fl_File_Chooser::SINGLE,
      "Load Configuration File");
  cf->show();

  while (cf->visible())
    Fl::wait();

  if (!CheckFileName(cf, file))
    std::cout<<"[PireoViewer::LoadConfigurationFile] Error in loading file. \n";
  else
    ReadRegistrationParametersFromFile(file);
}

#ifdef _MSC_VER
#pragma warning(pop)
#endif
