/*=========================================================================

  $Author: allain $
  $Date: 2005/06/06 16:16:56 $
  $Revision: 1.3 $
  $Id: PireoViewer.h, v 1.3 2005/06/06 16:16:56 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/PireoViewer.h, v $
  $Log: PireoViewer.h, v $
  Revision 1.3  2005/06/06 16:16:56  allain
  Gave new method names, add commentary and implement new params

  Revision 1.2  2005/05/02 15:33:01  allain
  Generic Registration and Configuration File installed

  Revision 1.1  2005/04/27 13:15:11  allain
  Change most of class names

  Revision 1.19  2005/03/23 17:34:11  allain
  Pre-Processing is working

  Revision 1.18  2005/03/21 14:35:09  allain
  Interface modified

  Revision 1.17  2005/03/16 17:09:08  allain
  Add some commentaries

  Revision 1.16  2005/03/15 11:04:47  allain
  Add Vector Field Window

  Revision 1.15  2005/03/04 13:49:42  allain
  GLSliceView file modified for 2D Image

  Revision 1.14  2005/03/03 16:52:44  allain
  Registration start and its observer

  Revision 1.13  2005/02/25 15:33:10  allain
  Transparency Window Set

  Revision 1.12  2005/02/23 17:48:29  allain
  ITK To VTK IMage start

  Revision 1.11  2005/02/23 16:09:58  allain
  New Class Hierarchy

  Revision 1.10  2005/02/22 14:53:08  allain
  Display image with flip option

  Revision 1.9  2005/02/21 17:09:31  allain
  New InterFace to display image

  Revision 1.8  2005/02/15 15:20:17  allain
  Read a region of an image woth GDAL and ITK

  Revision 1.7  2005/02/14 10:15:59  allain
  Sauvegarde avant ajout memoire

  Revision 1.6  2005/02/11 16:27:21  allain
  Mouse interaction start ..missed files in cvs

  Revision 1.5  2005/02/11 08:49:46  allain
  IHM beginning

  Revision 1.4  2005/02/10 14:33:12  allain
  Add InsightApplications used librairies

  Revision 1.3  2005/02/10 10:44:35  allain
  Changed Cmake

  Revision 1.2  2005/02/10 07:51:27  allain
  Add itkGDALImageIO and the header of each file


=========================================================================*/

#ifndef __PireoViewer_H__
#define __PireoViewer_H__

/* FLTK Library */
#include "FL/Fl_File_Chooser.H"
#include <FL/Fl_Text_Buffer.H>
/* OTB Library */
#include "otbImage.h"
/* Code Related */
#include "PireoViewerGUI.h"
// Manage ITK/FLTK Windows
#include "GLSliceView.h"
#include "itkRescaleIntensityImageFilter.h"


/** Class PireoViewer
    It enables to access the GUI and all its elements directly.
    Thus it's easy to refresh the GUI or to modify directly some
    component. Moreover, from this class you get access to all the objects
    used for this tool. In fact, it's the link for refreshing the GUI
    during the registration process. Therefore it inherits from
    PireoViewerGUI class (which inherits itself from PireoApplication
    class).
*/

class PireoViewer : public PireoViewerGUI
{
public :
  /** Initialized ITK tools.*/
  typedef    double   InputPixelType;
  typedef unsigned char ViewerPixelType;
  /** Initialized ITK tools.*/
  typedef otb::Image <InputPixelType, 2 >      ImageType;
  typedef otb::Image<ViewerPixelType, 2> ViewedImageType;
  typedef itk::RescaleIntensityImageFilter<ImageType, ViewedImageType> RescalerType;


  /** Initialized FLTK tools.
      Enable user to choose file to load.*/
  Fl_File_Chooser *ImageName;

  /** Filename of the two loaded images (fixed and moving).*/
  //char fixIm[500], movIm[500];

  /** Construtor: set this object itself to PireoObserverManager to be
  able to refresh GUI at each iteration of the registration process.*/
  PireoViewer();
  /** Destructor: empty.*/
  virtual ~PireoViewer();

  /** Flip axes depending if it's a fixed or moving image and its
  current set.*/
  void FlipX(bool flip, bool FixIm);
  /** Flip axes depending if it's a fixed or moving image and its
  current set.*/
  void FlipY(bool flip, bool FixIm);
  /** Return true if the name of the image to load exists.*/
  bool CheckFileName(Fl_File_Chooser* fc, char *file);
  /** Load Image depending if it's a fixed or moving image.
      Load either image which can be read by ITK or image which can be
      read by GDAL.*/
  void LoadImage(bool TypeIm);
  /** Load a text file to set registration parameters.*/
  void LoadConfigurationFile();
  /** Read GDAL or recognize ITK formats image.*/
  void ReadFile(const char* fc, bool TypeIm);
  /** Quit the user interface*/
  void Quit();
  /** Display images depending of their status : resampled image or
  zoomed image.*/
  void Display(int number);
  /** Update the FL fixed image window.*/
  void UpdateFixedWindow();
  /** Update the FL moving image window.*/
  void UpdateMovingWindow();
  /** Update VTK blender window (view in transparency of the two
  images: fixed and moving).*/
  void UpdateRegistrationWindow();
  /** Update both fixed and moving image windows.*/
  void UpdateGL();
  /** Fill the rendering scene when first displayed.*/
  void CreateScene();

private:
  /** Used to know if an image has already been loaded*/
  bool m_imageloaded;
};
#endif
