/*=========================================================================

  $Author: allain $
  $Date: 2005/07/12 15:20:13 $
  $Revision: 1.5 $
  $Id: PreProcessFiltering.cxx, v 1.5 2005/07/12 15:20:13 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/PreProcessFiltering.cxx, v $
  $Log: PreProcessFiltering.cxx, v $
  Revision 1.5  2005/07/12 15:20:13  allain
  Add some functionalities

  Revision 1.4  2005/06/14 15:38:03  allain
  Translation, Affine, OPOEO are working now

  Revision 1.3  2005/06/10 15:56:42  allain
  Add some functionalities for registration

  Revision 1.2  2005/06/06 16:16:56  allain
  Gave new method names, add commentary and implement new params

  Revision 1.1  2005/04/27 13:15:11  allain
  Change most of class names

  Revision 1.3  2005/03/24 16:14:28  allain
  Pre-Processing structure done

  Revision 1.2  2005/03/23 17:34:11  allain
  Pre-Processing is working

  Revision 1.1  2005/03/22 17:42:27  allain
  Add Pre-Processing files

=========================================================================*/

#include "PreProcessFiltering.h"
#include "otbImageFileWriter.h"

PreProcessFiltering::PreProcessFiltering()
{
  filtertype = NONE;
  m_selectIm = FIXED_DI;
}

PreProcessFiltering::~PreProcessFiltering()
{
}

bool PreProcessFiltering::SelectFixed()
{
  if ((m_selectIm == FIXED_DI || m_selectIm == BOTH_DI))// && (filtertype != NONE))
    return true;
  else
    return false;
}

bool PreProcessFiltering::SelectMoving()
{
  if ((m_selectIm == MOVING_DI || m_selectIm == BOTH_DI))// && (filtertype != NONE))
    return true;
  else
    return false;
}

void PreProcessFiltering::SelectImage(DisplayImage t)
{
  m_selectIm = t;
}

void PreProcessFiltering::SetFilterType(FilterType f)
{
  filtertype = f;
  // Only there to enable the use this class from a command line without
  // tuoching the GUI.
  switch (filtertype)
  {
  case NONE:
  {
    this->m_InFilter->value(0);
    break;
  }
  case BLURRING:
  {
    this->m_InFilter->value(1);
    break;
  }
  case NORMALIZE:
  {
    this->m_InFilter->value(2);
    break;
  }
  }
}

// Create filter that corresponds to type asked for by the user.
void PreProcessFiltering::UpdateParameters()
{
  switch (filtertype)
  {
  case NONE:
  {
    /*InPlaceFilterType::Pointer InP = InPlaceFilterType::New();
    InP->SetScale(2.0);
    InP->SetShift(0.0);
    m_filter = InP; */
    break;
  }
  case NORMALIZE:
  {
    NormalizeFilterType::Pointer normalize = NormalizeFilterType::New();
    m_filter = normalize;
    break;
  }
  case BLURRING:
  {
    BlurringFilter::Pointer Blur = BlurringFilter::New();
    Blur->SetVariance(m_Variance->value());
    Blur->SetMaximumKernelWidth((int)m_maxKernelWidth->value());
    m_filter = Blur;
    break;
  }
  }
}

// Hide pre_process GUI
void PreProcessFiltering::quit()
{
  this -> m_PreProcessWindow -> hide();
}

// Input image to process filtering
PreProcessFiltering::ImageReaderType::Pointer
PreProcessFiltering::DoFiltering(ImageReaderType::Pointer m_Im)
{
  if (filtertype == NONE)
    return m_Im;

  RescalerFilterType::Pointer m_Rescaler = RescalerFilterType::New();
  m_Rescaler->SetOutputMinimum(0);
  m_Rescaler->SetOutputMaximum(255);
  m_Rescaler->SetInput(m_Im);
  std::cout<<"region : "<<m_Im->GetRequestedRegion()<<std::endl;
  m_filter->SetInput(m_Rescaler->GetOutput());

  ImageRangeFilterType::Pointer m_range = ImageRangeFilterType::New();
  m_range->SetInput(m_filter->GetOutput());
  try
  {
    m_range->Update();
  }
  catch ( itk::ExceptionObject & err )
  {
    std::cout << "ExceptionObject caught !" << std::endl;
    std::cout << err << std::endl;
  }
  std::cout<<"min and max : "<<m_range->GetMinimum()<<"  "<<m_range->GetMaximum()<<std::endl;
  InverseRescalerType::Pointer m_inv = InverseRescalerType::New();
  m_inv->SetOutputMinimum(static_cast<unsigned char>(m_range->GetMinimum()));
  m_inv->SetOutputMaximum(static_cast<unsigned char>(m_range->GetMaximum()));
  m_inv->SetInput(m_filter->GetOutput());

  try
  {
    m_inv->Update();
  }
  catch ( itk::ExceptionObject & err )
  {
    std::cout << "ExceptionObject caught !" << std::endl;
    std::cout << err << std::endl;
  }

//   typedef otb::ImageFileWriter< ImageFilterType >  WriterType;
//
//   WriterType::Pointer      writer =  WriterType::New();
//   writer->SetFileName( "toto.hdr" );
//   writer->SetInput( m_filter->GetOutput()   );
//
//   try
//     {
//     writer->Update();
//     }
//   catch( itk::ExceptionObject & err )
//     {
//     std::cout << "ExceptionObject caught !" << std::endl;
//     std::cout << err << std::endl;
//     }

  return m_inv->GetOutput();
}

// Show pre_process GUI
void PreProcessFiltering::Show()
{
  this->m_PreProcessWindow->show();
  this->m_PreProcessWindow->activate();
  this->m_PreProcessWindow->make_current();
  while (m_PreProcessWindow->visible())
    Fl::wait();
}

