/*=========================================================================

  $Author: allain $
  $Date: 2005/06/14 15:38:03 $
  $Revision: 1.4 $
  $Id: PreProcessFiltering.h, v 1.4 2005/06/14 15:38:03 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/PreProcessFiltering.h, v $
  $Log: PreProcessFiltering.h, v $
  Revision 1.4  2005/06/14 15:38:03  allain
  Translation, Affine, OPOEO are working now

  Revision 1.3  2005/06/10 15:56:42  allain
  Add some functionalities for registration

  Revision 1.2  2005/06/06 16:16:56  allain
  Gave new method names, add commentary and implement new params

  Revision 1.1  2005/04/27 13:15:11  allain
  Change most of class names

  Revision 1.3  2005/03/24 16:14:28  allain
  Pre-Processing structure done

  Revision 1.2  2005/03/23 17:34:11  allain
  Pre-Processing is working

  Revision 1.1  2005/03/22 17:42:27  allain
  Add Pre-Processing files


=========================================================================*/

#ifndef __PreProcessFiltering_H__
#define __PreProcessFiltering_H__

#include "otbImage.h"
/* ITK Librairies */
#include "itkImageToImageFilter.h"
#include "itkDiscreteGaussianImageFilter.h"
#include "itkRescaleIntensityImageFilter.h"
//#include "itkBinaryThresholdImageFilter.h"
#include "itkNormalizeImageFilter.h"
#include "itkCastImageFilter.h"
#include "itkMinimumMaximumImageFilter.h"

/* Code related */
#include "Structures.h"
#include "PreProcessParametersGUI.h"

/* Class PreProcessFiltering
   Inherits from PreProcessParametersGUI which a user interface to
   choose filter to apply.
   Enables user to do a filtering operation of the images before
   registration. This can be use either to generate test images or to
   improve images quality.
   At this time, this class only contain a blurring filter. So it is
   not really useful for the moment.
   Anyway, this architecture is ok, i.e new filter can easily be set
   in this class by implementing corresponding code into
   UpdateParameters() method.
*/
class PreProcessFiltering: public PreProcessParametersGUI
{
public:
  /** Output image pixel type.*/
  typedef float PixelFilterType;
  /** Input image pixel type.*/
  typedef double ReaderType;
  /** Output image type.*/
  typedef otb::Image<PixelFilterType, 2>   ImageFilterType;
  /** Input image type*/
  typedef otb::Image<ReaderType, 2> ImageReaderType;
  /** Generic filter (from this type you can generate any one of the
  ITK filters).*/
  typedef itk::ImageToImageFilter<ImageFilterType, ImageFilterType>  GenericFilter;

  /** Filter which casts Input image pixel type into output image
  pixel type.*/
  typedef itk::RescaleIntensityImageFilter< ImageReaderType, ImageFilterType>  RescalerFilterType;
  /** Filter which casts output image pixel type into input image
  pixel type.*/
  typedef itk::RescaleIntensityImageFilter< ImageFilterType, ImageReaderType>  InverseRescalerType;

  /** Filter that computes the maximum and the minimum value of an
  * image*/
  typedef itk::MinimumMaximumImageFilter<ImageFilterType>     ImageRangeFilterType;

  /** Used to cast iamges to be able to apply a filter on it*/
  //typedef itk::CastImageFilter<ImageFilterType, ImageReaderType>  CastFilterType;
  /** Used to cast iamges to be able to apply a filter on it*/
  //typedef itk::CastImageFilter<ImageReaderType, ImageFilterType>  RescalerFilterType;
  /** Blurring image filter*/
  typedef itk::DiscreteGaussianImageFilter<ImageFilterType, ImageFilterType>  BlurringFilter;
  /** Binary image filter*/
  //typedef itk::BinaryThresholdImageFilter<ImageFilterType, ImageFilterType> BinaryFilter;

  /** Filter that normalizes an image i.e computes statistical gray
  * levels to get e zero mean and a variance equal to one (useful for
  * mutual information metric).*/
  typedef itk::NormalizeImageFilter<ImageFilterType, ImageFilterType>  NormalizeFilterType;

  /** Constructor.*/
  PreProcessFiltering();
  /** Destructor.*/
  virtual ~PreProcessFiltering();

  /** Is the moving image that was selected to apply the filter on ?.*/
  bool SelectMoving();
  /** Is the fixed image that was selected to apply the filter on ?.*/
  bool SelectFixed();
  /** Select the image on which the filter is applyed: fixed or
  moving.*/
  void SelectImage(DisplayImage t);
  /** Return the selected image on which the filter was applyed.*/
  DisplayImage GetSelectedImage()
  {
    return m_selectIm;
  }
  /** Filter selected images.*/
  ImageReaderType::Pointer DoFiltering(ImageReaderType::Pointer m_Im);
  /** Set the filter type.*/
  void SetFilterType(FilterType f);
  /** Update the filter type.*/
  void UpdateParameters();
  /** Show pre_processing GUI*/
  void Show();
  /** Hide pre-processing GUI*/
  void quit();
  /** Return the filter used for pre_processing.*/
  GenericFilter::Pointer GetFilter()
  {
    return m_filter;
  }

protected:
  /** Type of the filter*/
  FilterType filtertype;
  /** Generic filter to set pre_process filter.*/
  GenericFilter::Pointer m_filter;
  /** Which images are selected to be filtered.*/
  DisplayImage m_selectIm;

};

#endif
