/*=========================================================================

  $Author: allain $
  $Date: 2005/07/13 13:33:50 $
  $Revision: 1.15 $
  $Id: RegistrationMethod.cxx, v 1.15 2005/07/13 13:33:50 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/RegistrationMethod.cxx, v $
  $Log: RegistrationMethod.cxx, v $
  Revision 1.15  2005/07/13 13:33:50  allain
  Pause button

  Revision 1.14  2005/07/11 08:48:00  allain
  grid debugged

  Revision 1.13  2005/07/06 16:07:14  allain
  still a bug with the grid

  Revision 1.12  2005/06/28 11:28:58  allain
  Update

  Revision 1.11  2005/06/24 15:40:09  allain
  Add metric values graph and started registration levels observer

  Revision 1.10  2005/06/20 15:16:21  allain
  Add ITK registration parameters in the save text file

  Revision 1.9  2005/06/16 15:28:08  allain
  Add new functionalities

  Revision 1.8  2005/06/15 14:41:20  allain
  Add registration level observer

  Revision 1.7  2005/06/14 15:38:03  allain
  Translation, Affine, OPOEO are working now

  Revision 1.6  2005/06/10 15:56:42  allain
  Add some functionalities for registration

  Revision 1.5  2005/06/06 16:16:56  allain
  Gave new method names, add commentary and implement new params

  Revision 1.4  2005/05/09 15:52:57  allain
  Add pause and stop buttons for generic registration

  Revision 1.3  2005/05/02 15:33:01  allain
  Generic Registration and Configuration File installed

  Revision 1.7  2005/03/22 17:40:18  allain
  Grid is working and Pre-processing starts

  Revision 1.6  2005/03/16 17:09:08  allain
  Add some commentaries

  Revision 1.5  2005/03/15 17:28:47  allain
  Add Grid class

  Revision 1.4  2005/03/15 11:04:47  allain
  Add Vector Field Window

  Revision 1.3  2005/03/08 13:11:39  allain
  Set MultiResolutionImageRegistrationMethod

  Revision 1.2  2005/03/07 16:17:27  allain
  Registration with GDAL image starts working

  Revision 1.1  2005/03/04 07:59:11  allain
  Add Missing files


=========================================================================*/

#include "RegistrationMethod.h"
#include "otbImageFileReader.h"

RegistrationMethod::RegistrationMethod()
{
  // Pireo optimizer observer (observes registration process)
  observer = PireoObserverManager::New();
  // Pireo registration observer (observes registration process)
  command = PireoRegistrationLevelObserver::New();
  // Set registration optimizer default value
  m_optimType = RSG;
  // Default number of iterations for the registration process
  m_NumberOfIterations = 10;

  // Initial parameters default values
  sr00 = 1.0;
  sr01 = 1.0;
  sr10 = 1.0;
  sr11 = 1.0;
  scx = 1.0;
  scy = 1.0;
  stx = 1.0;
  sty = 1.0;

  sa = 1.0;

  tx = 0.0;
  ty = 0.0;

  sx = 1.0;
  sy = 1.0;

  m_fdev = 0.4;
  m_mdev = 0.4;
  m_NumberOfSamples = 50;

  maximize = false;
  initgen=0;

  m_maxStepLength = 4.0;
  m_minStepLength = 0.001;

  m_learningrate = 20;

  m_NumberOfBins = 20;

  m_storder = 1;
  m_nbEvaluations = 1000;
  m_gct = 0.05;
  m_lsa = 0.9;
  m_dsl = 1.5;

  m_nbBins = 50;

  m_angle = 0.0;

  InitializeWithMoments = false;
  geometry = false;

  m_lambda = 1.0;

  /*  transformParameter = TRANSLATION;
    interpolatorParameter = NEARESTNEIGHBOR;
    metricParameter = MEANSQUARE;
    optimizerParameter = RSG; */
}

RegistrationMethod::~RegistrationMethod()
{
}

// Hide registration parameters GUI
void RegistrationMethod::quit()
{
  this->m_RegistrationWindow->hide();
}

// Show registration parameters GUI
void RegistrationMethod::Show()
{
  this->m_RegistrationWindow->show();
  this->m_RegistrationWindow->activate();
  this->m_RegistrationWindow->make_current();
}

// Enable to get back the new parameters set between each level of the
// multi-resolution registration.
void RegistrationMethod::ChangeParametersFromRegistrationLevelObserver()
{
  m_BtnAccept->activate();
  while (!m_BtnAccept->changed())
    Fl::wait();
  m_BtnAccept->do_callback();
  m_BtnAccept->deactivate();
}

// Set vectors field corresponding to the points transformed by
// the registration transform
void RegistrationMethod::TransformPointSet(
  vtkPointSet* m_ptset, float maxLengthVectorX, float maxLengthVectorY)
{
  // Return the current parameters of the registration transform
  ParametersType finalParameters = optimizer->GetCurrentPosition();
  // Set the current transform of the registration process
  finalTransform->SetParameters( finalParameters );

  int i;

  // point
  ImageType2D::PointType point;
  // transformed point
  ImageType2D::PointType TransformedPoint;
  ImageType2D::IndexType  pixelIndex;
  // vector corresponding to point displacement
  ImageType2D::PointType::VectorType vect;

  int Numpts = (int)(m_ptset->GetNumberOfPoints());
  // For all the points, get their homologous point and create vector
  // between those two points
  for ( i = 0; i < Numpts; i++ )
  {
    pixelIndex[0] = (long int)m_ptset->GetPoints()->GetPoint(i)[0];
    pixelIndex[1] = (long int)m_ptset->GetPoints()->GetPoint(i)[1];
    m_MovingIm->TransformIndexToPhysicalPoint(pixelIndex, point);
    // Get transformed point
    TransformedPoint = finalTransform->TransformPoint(point);
    m_ptset->GetPoints()->SetPoint(i, point[0], point[1], 0);
    vect = TransformedPoint - point;
    float* v = new float[3];
    // In order not to have huge vectors displayed
    if (vect[0] > maxLengthVectorX)
      vect[0] = maxLengthVectorX;
    if (vect[1] > maxLengthVectorY)
      vect[1] = maxLengthVectorY;
    v[0] = vect[0];
    v[1] = vect[1];
    v[2] = 0;
    // set new vector
    m_ptset->GetPointData()->GetVectors()->SetTuple(i, v);
  }
}

// Transform points
void RegistrationMethod::TransformGrid(vtkPointSet* m_ptset)
{
  ParametersType finalParameters = optimizer->GetCurrentPosition();

  finalTransform->SetParameters( finalParameters );

  int i;

  ImageType2D::PointType point;
  ImageType2D::PointType TransformedPoint;
  ImageType2D::IndexType  pixelIndex;
  ImageType2D::PointType::VectorType vect;

  int Numpts = (int)(m_ptset->GetNumberOfPoints());

  for ( i = 0; i < Numpts; i++ )
  {
    pixelIndex[0] = (long int)m_ptset->GetPoints()->GetPoint(i)[0];
    pixelIndex[1] = (long int)m_ptset->GetPoints()->GetPoint(i)[1];

    m_MovingIm->TransformIndexToPhysicalPoint(pixelIndex, point);
    TransformedPoint = finalTransform->TransformPoint(point);
    m_ptset->GetPoints()->SetPoint(i, TransformedPoint[0], TransformedPoint[1], 0);
  }
}

// Stop registration process
void RegistrationMethod::StopRegistration()
{
  if (m_optimType == RSG)
    RSGO->StopOptimization();
  else if (m_optimType == GD)
    GDO->StopOptimization();
  else if (m_optimType == OPOE)
    OPOEO->StopOptimization();
  else
    std::cout<<"Cannot stop it, sorry"<<std::endl;

  SetInitialRegistration();
}

// Start registration process
void RegistrationMethod::StartRegistration()
{
  // Reset observer iteration counter
  observer->ResetCounter();
  // Reset metric values FL window
  observer->InitializeMetricValuesWindow();
  // Deactivate Accept button in registration parameters GUI
  m_BtnAccept->deactivate();
  // Add observer for the registration level set (enables to change
  // registration parameters for each level)
  //registration->AddObserver( itk::IterationEvent(), command );

  // Register
  this->DoRegister();

  // Activate Accept button in registration parameters GUI
  m_BtnAccept->activate();
}

// Set initial registration transform parameters
void RegistrationMethod::SetInitialRegistration()
{
  this->SetRegistrationTransformParameters(Params);
}

void RegistrationMethod::SetNumberOfIterations(int v)
{
  m_NumberOfIterations = v;
}

// Set the frequency for refreshing GUI
void RegistrationMethod::SetRefreshFrequency(int v)
{
  observer->SetRefreshStep(v);
}

void RegistrationMethod::SetInitialTransformParameters(double t1, double t2)
{
  tx = t1;
  ty = t2;
}

void RegistrationMethod::SetInitialRigidTransformParameters(double angle, bool b, bool c)
{
  m_angle = angle;
  InitializeWithMoments = b;
  geometry = c;
}

void RegistrationMethod::SetInitialCenteredAffineTransformParameters(bool b)
{
  geometry = b;
}

void RegistrationMethod::SetInitialScalingParameters(double s1, double s2)
{
  sx = s1;
  sy = s2;
}

void RegistrationMethod::SetInitialOptimizerParameters(double r0, double r1, double r2, double r3, double c1, double c2, double t1, double t2)
{
  sr00 = r0;
  sr01 = r1;
  sr10 = r2;
  sr11 = r3;
  scx = c1;
  scy = c2;
  stx = t1;
  sty = t2;
}

void RegistrationMethod::SetInitialOptimizerParameters(double m0, double m1, double m2, double m3, double t1, double t2)
{
  sr00 = m0;
  sr01 = m1;
  sr10 = m2;
  sr11 = m3;
  stx = t1;
  sty = t2;
}

void RegistrationMethod::SetInitialOptimizerParameters(double a, double t1, double t2, double c1, double c2)
{
  sa = a;
  stx = t1;
  sty = t2;
  scx = c1;
  scy = c2;
}

void RegistrationMethod::SetInitialOptimizerParameters(double t1, double t2)
{
  stx = t1;
  sty = t2;
}

void RegistrationMethod::SetMutualInfoMetricParameters(double a, double b, int n)
{
  m_fdev = a;
  m_mdev = b;
  m_NumberOfSamples = n;
}

void RegistrationMethod::SetMattesMutualInfoMetricParameters(int a, int b)
{
  m_NumberOfSamples = a;
  m_NumberOfBins = b;
}

void RegistrationMethod::SetInitialOPOEOParameters(int init, bool m)
{
  initgen = init;
  maximize = m;
}

void RegistrationMethod::SetInitialRGDOParameters(double a, double b, bool c)
{
  m_maxStepLength = a;
  m_minStepLength = b;
  maximize = c;
}

void RegistrationMethod::SetInitialGDOParameters(double a, bool b)
{
  m_learningrate = a;
  maximize = b;

}

void RegistrationMethod::SetBSplineOrderToDeformableTransform(unsigned int o)
{
  m_storder = o;
}

// Here is the implementation of the different registration transforms
// used in Pireo
void RegistrationMethod::SetTransformType(RegistrationTransformParameter TP)
{
  std::stringstream oss;

  switch (TP)
  {
  case AFFINE:
  {
    itk::AffineTransform<double, 2>::Pointer CAT = itk::AffineTransform<double, 2>::New();
    CAT->SetIdentity();
    this->SetTransform((TransformType::Pointer)CAT);
    // Used to write result in a text file in PireoApplication
    oss<<"AFFINE (initialize to identity)";
    break;
  }
  case CENTEREDAFFINE:
  {
    itk::CenteredAffineTransform<double, 2>::Pointer CAT = itk::CenteredAffineTransform<double, 2>::New();
    itk::CenteredTransformInitializer<itk::CenteredAffineTransform<double, 2>, ImageType2D, ImageType2D>::Pointer initializer =
      itk::CenteredTransformInitializer<itk::CenteredAffineTransform<double, 2>, ImageType2D, ImageType2D>::New();
    initializer->SetTransform(CAT);
    initializer->SetFixedImage(this->GetFixedImage());
    initializer->SetMovingImage(this->GetMovingImage());
    if (geometry)
      initializer->GeometryOn();
    else
      initializer->MomentsOn();
    initializer->InitializeTransform();
    this->SetTransform((TransformType::Pointer)CAT);
    oss<<"CENTEREDAFFINE ";
    if (geometry)
      oss<<"(geometry initialization)";
    else
      oss<<"(moments initialization)";
    break;
  }
  case TRANSLATION:
  {
    itk::TranslationTransform<double, 2>::Pointer TT = itk::TranslationTransform<double, 2>::New();
    itk::TranslationTransform<double, 2>::ParametersType params(TT->GetNumberOfParameters());
    params[0] = tx;
    params[1] = ty;
    TT->SetParameters(params);
    this->SetTransform((TransformType::Pointer)TT);
    oss<<"TRANLATION (initialize to "<<tx<<" "<<ty<<")";
    break;
  }
  case CENTEREDRIGID:
  {
    itk::CenteredRigid2DTransform<double>::Pointer CRT = itk::CenteredRigid2DTransform<double>::New();
    if (!InitializeWithMoments)
    {
      itk::CenteredRigid2DTransform<double>::InputPointType centerFixed;
      centerFixed[0] = static_cast<double>(m_FixedIm->GetOrigin()[0] + (m_FixedIm->GetSpacing()[0])*(m_FixedIm->GetLargestPossibleRegion().GetSize()[0])/2);
      centerFixed[1] = static_cast<double>(m_FixedIm->GetOrigin()[1] + (m_FixedIm->GetSpacing()[1])*(m_FixedIm->GetLargestPossibleRegion().GetSize()[1])/2);
      itk::CenteredRigid2DTransform<double>::InputPointType centerMoving;
      centerMoving[0] = static_cast<double>(m_MovingIm->GetOrigin()[0] + (m_MovingIm->GetSpacing()[0])*(m_MovingIm->GetLargestPossibleRegion().GetSize()[0])/2);
      centerMoving[1] = static_cast<double>(m_MovingIm->GetOrigin()[1] + (m_MovingIm->GetSpacing()[1])*(m_MovingIm->GetLargestPossibleRegion().GetSize()[1])/2);
      std::cout<<"center : "<<centerFixed<<"  "<<centerMoving<<std::endl;
      CRT->SetCenter(centerFixed);
      CRT->SetTranslation(centerMoving - centerFixed);
    }
    else
    {
      itk::CenteredTransformInitializer<itk::CenteredRigid2DTransform<double>, ImageType2D, ImageType2D>::Pointer initializer =
        itk::CenteredTransformInitializer<itk::CenteredRigid2DTransform<double>, ImageType2D, ImageType2D>::New();
      initializer->SetTransform(CRT);
      initializer->SetFixedImage(this->GetFixedImage());
      initializer->SetMovingImage(this->GetMovingImage());
      if (geometry)
        initializer->GeometryOn();
      else
        initializer->MomentsOn();
      initializer->InitializeTransform();
    }

    CRT->SetAngle(m_angle);
    this->SetTransform((TransformType::Pointer)CRT);
    oss<<"CENTEREDRIGID ";
    if (!InitializeWithMoments)
      oss<<"(initialize by computing the center of the image)";
    else
      oss<<"(initialize with moments)";
    break;
  }
  case SCALE:
  {
    itk::ScaleTransform<double, 2>::Pointer ST = itk::ScaleTransform<double, 2>::New();
    itk::ScaleTransform<double, 2>::ParametersType params(ST->GetNumberOfParameters());
    params[0] = sx;
    params[1] = sy;
    ST->SetParameters(params);
    this->SetTransform((TransformType::Pointer)ST);
    oss<<"SCALE (initialize to "<<sx<<" "<<sy<<")";
    break;
  }
  case DEFORMABLE:
  {
    typedef itk::BSplineDeformableTransform<
    double, 2, 3 >     TransformType2;

    // Software Guide : BeginCodeSnippet
    typedef TransformType2::RegionType RegionType;
    RegionType bsplineRegion;
    RegionType::SizeType   gridSizeOnImage;
    RegionType::SizeType   gridBorderSize;
    RegionType::SizeType   totalGridSize;

    gridSizeOnImage.Fill( 5 );
    gridBorderSize.Fill( 3 );    // Border for spline order = 3 ( 1 lower, 2 upper )
    totalGridSize = gridSizeOnImage + gridBorderSize;

    bsplineRegion.SetSize( totalGridSize );

    typedef TransformType2::SpacingType SpacingType;
    SpacingType spacing = m_FixedIm->GetSpacing();

    typedef TransformType2::OriginType OriginType;
    OriginType origin = m_FixedIm->GetOrigin(); ;

    ImageType2D::RegionType fixedRegion = m_FixedIm->GetBufferedRegion();
    ImageType2D::SizeType fixedImageSize = fixedRegion.GetSize();

    for (unsigned int r=0; r<2; r++)
    {
      spacing[r] *= floor( static_cast<double>(fixedImageSize[r] - 1)  /
                           static_cast<double>(gridSizeOnImage[r] - 1) );
      origin[r]  -=  spacing[r];
    }
    TransformType2::Pointer t = TransformType2::New();
    t->SetGridSpacing( spacing );
    t->SetGridOrigin( origin );
    t->SetGridRegion( bsplineRegion );


    typedef TransformType2::ParametersType     ParametersType2;

    const unsigned int numberOfParameters =
      t->GetNumberOfParameters();

    ParametersType2 parameters( numberOfParameters );

    parameters.Fill( 0.0 );

    t->SetParameters( parameters );

    this->SetTransform((TransformType::Pointer)t);
    oss<<"DEFORMABLE ";
    break;
  }
  }
  transformParameter = oss.str();
}

// Here is the implementation of the different registration interpolators
// used in Pireo
void RegistrationMethod::SetInterpolatorType(RegistrationInterpolatorParameter IP)
{
  std::stringstream oss;

  switch (IP)
  {
  case LINEAR:
  {
    itk::LinearInterpolateImageFunction<ImageType2D, double>::Pointer LI
    = itk::LinearInterpolateImageFunction<ImageType2D, double>::New();
    this->SetInterpolator((InterpolatorType::Pointer)LI);
    oss<<"LINEAR";
    break;
  }
  case BSPLINE:
  {
    itk::BSplineInterpolateImageFunction<ImageType2D, double>::Pointer BI
    = itk::BSplineInterpolateImageFunction<ImageType2D, double>::New();
    this->SetInterpolator((InterpolatorType::Pointer)BI);
    oss<<"BSPLINE";
    break;
  }
  case NEARESTNEIGHBOR:
  {
    itk::NearestNeighborInterpolateImageFunction<ImageType2D, double>::Pointer NNI
    =  itk::NearestNeighborInterpolateImageFunction<ImageType2D, double>::New();
    this->SetInterpolator((InterpolatorType::Pointer)NNI);
    oss<<"NEARESTNEIGHBOR";
    break;
  }
  }
  interpolatorParameter = oss.str();
}

// Here is the implementation of the different registration metrics
// used in Pireo
void RegistrationMethod::SetMetricType(RegistrationMetricParameter MP)
{
  std::stringstream oss;

  switch (MP)
  {
  case MUTUALINFO:
  {
    itk::MutualInformationImageToImageMetric<ImageType2D, ImageType2D>::Pointer MIM
    = itk::MutualInformationImageToImageMetric<ImageType2D, ImageType2D>::New();
    MIM->SetFixedImageStandardDeviation(m_fdev);
    MIM->SetMovingImageStandardDeviation(m_mdev);
    MIM->SetNumberOfSpatialSamples(m_NumberOfSamples);
    this->SetMetric((MetricType::Pointer)MIM);
    oss<<"MUTUALINFO (Fixed standard deviation="<<m_fdev<<", Moving standard deviation="<<m_mdev<<" and Number of spatial samples="<<m_NumberOfSamples<<")";
    break;
  }
  case  MATTESMUTUALINFO:
  {
    itk::MattesMutualInformationImageToImageMetric<ImageType2D, ImageType2D>::Pointer MMIM
    = itk::MattesMutualInformationImageToImageMetric<ImageType2D, ImageType2D>::New();
    MMIM->SetNumberOfSpatialSamples(m_NumberOfSamples);
    MMIM->SetNumberOfHistogramBins(m_nbBins);
    this->SetMetric((MetricType::Pointer)MMIM);
    oss<<"MATTESMUTUALINFO (Number od spatial samples="<<m_NumberOfSamples<<", Number of hsitogram bins="<<m_nbBins<<")";
    break;
  }
  case MEANSQUARE:
  {
    itk::MeanSquaresImageToImageMetric<ImageType2D, ImageType2D>::Pointer MSM
    = itk::MeanSquaresImageToImageMetric<ImageType2D, ImageType2D>::New();
    this->SetMetric((MetricType::Pointer)MSM);
    oss<<"MEANSQUARE";
    break;
  }
  case MEANRECIPROCALSQUARE:
  {
    itk::MeanReciprocalSquareDifferenceImageToImageMetric<ImageType2D, ImageType2D>::Pointer MRSDM
    = itk::MeanReciprocalSquareDifferenceImageToImageMetric<ImageType2D, ImageType2D>::New();
    // capture the radius of this metric
    MRSDM->SetLambda(m_lambda);
    this->SetMetric((MetricType::Pointer)MRSDM);
    oss<<"MEANRECIPROCALSQUARE (lambda = "<<m_lambda<<")";
    break;
  }
  case CORRELATION:
  {
    itk::NormalizedCorrelationImageToImageMetric<ImageType2D, ImageType2D>::Pointer NCM
    = itk::NormalizedCorrelationImageToImageMetric<ImageType2D, ImageType2D>::New();
    this->SetMetric((MetricType::Pointer)NCM);
    oss<<"CORRELATION";
    break;
  }
  }
  metricParameter = oss.str();
}

// Here is the implementation of the different registration optimizers
// used in Pireo
void RegistrationMethod::SetOptimizerType(RegistrationOptimizerParameter OP)
{
  m_optimType = OP;

  std::stringstream oss;

  switch (OP)
  {
  case RSG :
  {
    RSGO =  itk::RegularStepGradientDescentOptimizer::New();
    itk::RegularStepGradientDescentOptimizer::ScalesType  optimizerScales( transform->GetNumberOfParameters() );
    if (transform->GetNumberOfParameters()==2)
    {
      optimizerScales[0] =  stx;
      optimizerScales[1] =  sty;
    }
    else if (transform->GetNumberOfParameters()==5)
    {
      optimizerScales[0] =  sa;
      optimizerScales[1] =  stx;
      optimizerScales[2] =  sty;
      optimizerScales[3] =  scx;
      optimizerScales[4] =  scy;
    }
    else if (transform->GetNumberOfParameters()==6)
    {
      optimizerScales[0] =  sr00;
      optimizerScales[1] =  sr01;
      optimizerScales[2] =  sr10;
      optimizerScales[3] =  sr11;
      optimizerScales[4] =  stx;
      optimizerScales[5] =  sty;
    }
    else
    {
      optimizerScales[0] =  sr00;
      optimizerScales[1] =  sr01;
      optimizerScales[2] =  sr10;
      optimizerScales[3] =  sr11;
      optimizerScales[4] =  scx;
      optimizerScales[5] =  scy;
      optimizerScales[6] =  stx;
      optimizerScales[7] =  sty;
    }
    std::cout<<"opt params: "<<optimizerScales<<"  "<<m_maxStepLength<<"  "<< m_minStepLength<<std::endl;
    RSGO->SetScales( optimizerScales );
    RSGO->SetMaximumStepLength( m_maxStepLength );
    RSGO->SetMinimumStepLength( m_minStepLength );
    RSGO->SetNumberOfIterations( m_NumberOfIterations );
    if (maximize)
      RSGO->MaximizeOn();
    else
      RSGO->MinimizeOn();
    RSGO->AddObserver( itk::IterationEvent(), observer );
    this->SetOptimizer((OptimizerType::Pointer)RSGO);
    oss<<"RSGD (initialization "<<optimizerScales<<" Max step length = "<<m_maxStepLength<<" Min step length = "<<m_minStepLength <<")";
    break;
  }
  case LBFGS:
  {
    break;
  }
  /*    case LBFXGS:
      {
      LBFGSO = itk::LBFGSModifiedOptimizer::New();
      LBFGSO->SetGradientConvergenceTolerance(0.05);
      LBFGSO->SetLineSearchAccuracy(0.9);
      LBFGSO->SetDefaultStepLength(1.5);
      LBFGSO->TraceOn();
      LBFGSO->SetMaximumNumberOfFunctionEvaluations(1000);
      //LBFGSO->AddIterationObserver((itk::Command::Pointer)observer);
      this->SetOptimizer((OptimizerType::Pointer)LBFGSO);
      oss<<"LBFGS";
      break;
      }*/
  case CG:
  {
    CGO = itk::ConjugateGradientOptimizer::New();
    itk::ConjugateGradientOptimizer::ScalesType  optimizerScales( transform->GetNumberOfParameters() );
    if (transform->GetNumberOfParameters()==2)
    {
      optimizerScales[0] =  stx;
      optimizerScales[1] =  sty;
    }
    else if (transform->GetNumberOfParameters()==5)
    {
      optimizerScales[0] =  sa;
      optimizerScales[1] =  stx;
      optimizerScales[2] =  sty;
      optimizerScales[3] =  scx;
      optimizerScales[4] =  scy;
    }
    else if (transform->GetNumberOfParameters()==6)
    {
      optimizerScales[0] =  sr00;
      optimizerScales[1] =  sr01;
      optimizerScales[2] =  sr10;
      optimizerScales[3] =  sr11;
      optimizerScales[4] =  stx;
      optimizerScales[5] =  sty;
    }
    else
    {
      optimizerScales[0] =  sr00;
      optimizerScales[1] =  sr01;
      optimizerScales[2] =  sr10;
      optimizerScales[3] =  sr11;
      optimizerScales[4] =  scx;
      optimizerScales[5] =  scy;
      optimizerScales[6] =  stx;
      optimizerScales[7] =  sty;
    }

    CGO->SetScales( optimizerScales );

    CGO->AddObserver( itk::IterationEvent(), observer );
    this->SetOptimizer((OptimizerType::Pointer)CGO);
    oss<<"CG (initialization "<<optimizerScales<<")";
    break;
  }
  case OPOE:
  {
    OPOEO = itk::OnePlusOneEvolutionaryOptimizer::New();
    /*itk::OnePlusOneEvolutionaryOptimizer::ScalesType  optimizerScales( transform->GetNumberOfParameters() );
    if(transform->GetNumberOfParameters()==2)
      {
      optimizerScales[0] =  stx;
      optimizerScales[1] =  sty;
      }
    else if(transform->GetNumberOfParameters()==5)
      {
      optimizerScales[0] =  sa;
      optimizerScales[1] =  stx;
      optimizerScales[2] =  sty;
      optimizerScales[3] =  scx;
      optimizerScales[4] =  scy;
      }
    else if(transform->GetNumberOfParameters()==6)
      {
      optimizerScales[0] =  sr00;
      optimizerScales[1] =  sr01;
      optimizerScales[2] =  sr10;
      optimizerScales[3] =  sr11;
      optimizerScales[4] =  stx;
      optimizerScales[5] =  sty;
      }
    else
      {
      optimizerScales[0] =  sr00;
      optimizerScales[1] =  sr01;
      optimizerScales[2] =  sr10;
      optimizerScales[3] =  sr11;
      optimizerScales[4] =  scx;
      optimizerScales[5] =  scy;
      optimizerScales[6] =  stx;
      optimizerScales[7] =  sty;
      }

    OPOEO->SetScales( optimizerScales ); */

    itk::Statistics::NormalVariateGenerator::Pointer generator =  itk::Statistics::NormalVariateGenerator::New();
    generator->Initialize(12345);

    OPOEO->SetNormalVariateGenerator( generator );
    OPOEO->Initialize( 10 );
    OPOEO->SetEpsilon( 1.0 );
    OPOEO->SetMaximumIteration(4000);
    if (maximize)
      OPOEO->MaximizeOn();
    else
      OPOEO->MaximizeOff();
    OPOEO->AddObserver( itk::IterationEvent(), observer );
    this->SetOptimizer((OptimizerType::Pointer)OPOEO);
    oss<<"OPOE";
    break;
  }
  case GD:
  {
    GDO = itk::GradientDescentOptimizer::New();
    /*itk::GradientDescentOptimizer::ScalesType  optimizerScales( transform->GetNumberOfParameters() );
    if(transform->GetNumberOfParameters()==2)
      {
      optimizerScales[0] =  stx;
      optimizerScales[1] =  sty;
      }
    else if(transform->GetNumberOfParameters()==5)
      {
      optimizerScales[0] =  sa;
      optimizerScales[1] =  stx;
      optimizerScales[2] =  sty;
      optimizerScales[3] =  scx;
      optimizerScales[4] =  scy;
      }
    else if(transform->GetNumberOfParameters()==6)
      {
      optimizerScales[0] =  sr00;
      optimizerScales[1] =  sr01;
      optimizerScales[2] =  sr10;
      optimizerScales[3] =  sr11;
      optimizerScales[4] =  stx;
      optimizerScales[5] =  sty;
      }
    else
      {
      optimizerScales[0] =  sr00;
      optimizerScales[1] =  sr01;
      optimizerScales[2] =  sr10;
      optimizerScales[3] =  sr11;
      optimizerScales[4] =  scx;
      optimizerScales[5] =  scy;
      optimizerScales[6] =  stx;
      optimizerScales[7] =  sty;
      }
    GDO->SetScales( optimizerScales ); */
    GDO->SetLearningRate( m_learningrate );
    GDO->SetNumberOfIterations( m_NumberOfIterations );
    if (maximize)
      GDO->MaximizeOn();
    else
      GDO->MinimizeOn();
    GDO->AddObserver( itk::IterationEvent(), observer );
    this->SetOptimizer((OptimizerType::Pointer)GDO);
    oss<<"GD (initialization LearningRate = "<<m_learningrate<<")";
    break;
  }
  }
}


