/*=========================================================================

  $Author: allain $
  $Date: 2005/06/24 15:40:09 $
  $Revision: 1.10 $
  $Id: RegistrationMethod.h, v 1.10 2005/06/24 15:40:09 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/RegistrationMethod.h, v $
  $Log: RegistrationMethod.h, v $
  Revision 1.10  2005/06/24 15:40:09  allain
  Add metric values graph and started registration levels observer

  Revision 1.9  2005/06/20 15:16:21  allain
  Add ITK registration parameters in the save text file

  Revision 1.8  2005/06/16 15:28:08  allain
  Add new functionalities

  Revision 1.7  2005/06/15 14:41:20  allain
  Add registration level observer

  Revision 1.6  2005/06/14 15:38:03  allain
  Translation, Affine, OPOEO are working now

  Revision 1.5  2005/06/10 15:56:42  allain
  Add some functionalities for registration

  Revision 1.4  2005/06/06 16:16:56  allain
  Gave new method names, add commentary and implement new params

  Revision 1.3  2005/05/09 15:52:57  allain
  Add pause and stop buttons for generic registration

  Revision 1.2  2005/05/02 15:33:01  allain
  Generic Registration and Configuration File installed

  Revision 1.6  2005/03/22 17:40:18  allain
  Grid is working and Pre-processing starts

  Revision 1.5  2005/03/16 17:09:08  allain
  Add some commentaries

  Revision 1.4  2005/03/15 11:04:47  allain
  Add Vector Field Window

  Revision 1.3  2005/03/08 13:11:39  allain
  Set MultiResolutionImageRegistrationMethod

  Revision 1.2  2005/03/07 16:17:27  allain
  Registration with GDAL image starts working

  Revision 1.1  2005/03/04 07:59:11  allain
  Add Missing files


=========================================================================*/

#ifndef __REGISTRATIONMETHOD_H__
#define __REGISTRATIONMETHOD_H__

/* C++ Libraries */
#include <iostream>
#include <string.h>
#include <math.h>

/* ITK Librairies */
// Transform
#include "itkScaleTransform.h"
#include "itkCenteredRigid2DTransform.h"
#include "itkTranslationTransform.h"
#include "itkCenteredAffineTransform.h"
#include "itkCenteredTransformInitializer.h"
#include "itkBSplineDeformableTransform.h"
#include "itkAffineTransform.h"
// Interpolator
#include "itkNearestNeighborInterpolateImageFunction.h"
#include "itkBSplineInterpolateImageFunction.h"
#include "itkLinearInterpolateImageFunction.h"
// Metric
#include "itkMeanSquaresImageToImageMetric.h"
#include "itkNormalizedCorrelationImageToImageMetric.h"
#include "itkMeanReciprocalSquareDifferenceImageToImageMetric.h"
#include "itkMattesMutualInformationImageToImageMetric.h"
#include "itkMutualInformationImageToImageMetric.h"
// Optimizer
#include "itkRegularStepGradientDescentOptimizer.h"
#include "itkConjugateGradientOptimizer.h"
#include "itkOnePlusOneEvolutionaryOptimizer.h"
#include "itkNormalVariateGenerator.h"
#include "itkGradientDescentOptimizer.h"

//#include "itkLBFGSModifiedOptimizer.h"

#include "itkNormalizeImageFilter.h"
#include "itkDiscreteGaussianImageFilter.h"

/* VTK Librairies */
#include "vtkPoints.h"
#include "vtkFloatArray.h"
#include "vtkPointSet.h"
#include "vtkPointData.h"

/* Code related */
#include "GenericRegistration.h"
#include "RegistrationParametersGUI.h"
#include "PireoObserverManager.h"
#include "PireoRegistrationLevelObserver.h"

/* Class RegistrationMethod
   Inherits from GenericRegistration which takes care of the
   multi-resolution registration process with the method Doregister()
   and RegistrationParametersGUI which is a FL window that enables
   user to choose ITK registration transform, ITK registration metric,
   ITK registration optimizer, ITK registration interpolator and other
   ITK registration parameters.
   It is from this class that the registration process is launch from
   PireoApplication class.
*/
class RegistrationMethod :
      public GenericRegistration, RegistrationParametersGUI
{
public:
  /** Construtor.*/
  RegistrationMethod();
  /** Destructor.*/
  virtual ~RegistrationMethod();

  /** Display registration parameters guide (window).*/
  void Show();
  /** Hide registration parameters guide (window).*/
  virtual void quit();
  /** Call by observer to change parameters when changing
  * multi-resolution registration level*/
  void ChangeParametersFromRegistrationLevelObserver();
  /** Set the frequency to refresh GUI while registering*/
  virtual void SetRefreshFrequency(int v);
  /** Set number of levels of the multi-resolution registration.*/
  virtual void SetNumberOfLevels(int l)
  {
    m_NumberOfLevels = l;
  }
  /** Set number of iterations of the registration process.*/
  virtual void SetNumberOfIterations(int v);
  /** Set registration transform type.*/
  virtual void SetTransformType(RegistrationTransformParameter TP);
  /** Set registration interpolator type.*/
  virtual void SetInterpolatorType(RegistrationInterpolatorParameter IP);
  /** Set registration metric type.*/
  virtual void SetMetricType(RegistrationMetricParameter MP);
  /** Set registration optimizer type.*/
  virtual void SetOptimizerType(RegistrationOptimizerParameter OP);
  /** Generate a set of vectors that represents the displacement of a
  set of points transformed by the current registration transform.*/
  virtual void TransformPointSet(
    vtkPointSet* m_ptset, float maxLengthVectorX, float maxLengthVectorY);
  /** Transform a set of points with the current registration
  transform.*/
  virtual void TransformGrid(vtkPointSet* m_ptset);
  /** Launch registration process.*/
  virtual void StartRegistration();
  /** Set Bspline order for the deformable transform*/
  virtual void SetBSplineOrderToDeformableTransform(unsigned int o);
  /** Set intial scaling for the scale transform.*/
  virtual void SetInitialScalingParameters(double s1, double s2);
  /** Set initial rotation angle for the rigid transform.*/
  virtual void SetInitialRigidTransformParameters(double angle, bool b, bool c);
  /** Initialize with image geometry the centered affine transform.*/
  virtual void SetInitialCenteredAffineTransformParameters(bool b);
  /** Set initial transform for the translation transform.*/
  virtual void SetInitialTransformParameters(double t1, double t2);
  /** When the affine transform is choosen, the optimizer require
  eight parameters.*/
  virtual void SetInitialOptimizerParameters(
    double r0, double r1, double r2, double r3, double c1,
    double c2, double t1, double t2);
  /** When the rigid transform is choosen, the optimizer require
  five parameters.*/
  void SetInitialOptimizerParameters(double m0, double m1, double m2, double m3, double t1, double t2);
  /** When the rigid transform is choosen, the optimizer require
  five parameters.*/
  virtual void SetInitialOptimizerParameters(double a, double t1, double t2, double c1, double c2);
  /** When the translation transform is choosen, the optimzer require
  only two parameters.*/
  virtual void SetInitialOptimizerParameters(double t1, double t2);
  /** Set initial paremeters of the One Plus One Evolution optimizer.*/
  virtual void SetInitialOPOEOParameters(int init, bool m);
  /** Set initial parameters of the Regular Gradient Descent optimizer.*/
  virtual void SetInitialRGDOParameters(double a, double b, bool c);
  /** Set initial parameters of the Gradient Descent optimizer.*/
  virtual void SetInitialGDOParameters(double a, bool b);
  /** Set initial parameters of the Mattes Mutual Information metric.*/
  virtual void SetMattesMutualInfoMetricParameters(int a, int b);
  /** Set initial parameters of the Mutual Information metric.*/
  virtual void SetMutualInfoMetricParameters(double a, double b, int n);
  /** Stop registration process (in fact stop it from the optimizer).*/
  virtual void StopRegistration();
  /** Reset registration transform parameters into the initial ones */
  virtual void SetInitialRegistration();
  /** Return observer (which enables to refresh GUI during
  registration process).*/
  PireoObserverManager::Pointer GetObserver()
  {
    return observer;
  }
  /** Return observer (which enables to change registration
  * parameters while processing).*/
  PireoRegistrationLevelObserver::Pointer GetRegistrationLevelObserver()
  {
    return command;
  }
  /** Return transform type used for registration*/
  std::string& GetTransformType()
  {
    return transformParameter;
  }
  //RegistrationTransformParameter GetTransformType(){return transformParameter; }
  /** Return interpolator type used for registration*/
  std::string& GetInterpolatorType()
  {
    return interpolatorParameter;
  }
  //RegistrationInterpolatorParameter GetInterpolatorType(){return interpolatorParameter; }
  /** Return metric type used for registration*/
  std::string& GetMetricType()
  {
    return metricParameter;
  }
  //RegistrationMetricParameter GetMetricType(){return metricParameter; }
  /** Return optimizer type used for registration*/
  std::string& GetOptimizerType()
  {
    return optimizerParameter;
  }
  void GetOptimizerType(RegistrationOptimizerParameter *optim)
  {
    *optim = m_optimType;
  }

protected:
  /** Observer*/
  PireoObserverManager::Pointer observer;
  /** Observer*/
  PireoRegistrationLevelObserver::Pointer command;
  /** Optimizer type implemented in this Pireo tool(necessary to keep
  it as smart pointer in order to stop registration process).*/
  itk::RegularStepGradientDescentOptimizer::Pointer RSGO;
  /** Optimizer type implemented in this Pireo tool.*/
  itk::ConjugateGradientOptimizer::Pointer CGO;
  /** Optimizer type implemented in this Pireo tool.*/
  itk::OnePlusOneEvolutionaryOptimizer::Pointer OPOEO;
  /** Optimizer type implemented in this Pireo tool.*/
  itk::GradientDescentOptimizer::Pointer GDO;
  /** Optimizer type implemented in this Pireo tool.*/
//  itk::LBFGSModifiedOptimizer::Pointer LBFGSO;

  /** Useful to return the type of transform used for registration.*/
  std::string transformParameter;
  //RegistrationTransformParameter transformParameter;
  /** Useful to return the type of interpolator used for
  * registration.*/
  std::string interpolatorParameter;
  //RegistrationInterpolatorParameter interpolatorParameter;
  /** Useful to return the type of metric used for registration.*/
  std::string metricParameter;
  //RegistrationMetricParameter metricParameter;
  /** Useful to return the type of metric used for registration.*/
  std::string optimizerParameter;
  RegistrationOptimizerParameter m_optimType;

  /** Number of iterations of the registration process.*/
  int m_NumberOfIterations;
  /** Scaling angle of rotation with optimizer*/
  double sa;
  /** */
  double sr00;
  /** */
  double sr01;
  /** */
  double sr10;
  /** */
  double sr11;
  /** Scalang center of rotation with optimizer.*/
  double scx;
  /** Scaling center of rotation with optimizer.*/
  double scy;
  /** Scaling translation with optimizer.*/
  double stx;
  /** Scaling translation with optimizer.*/
  double sty;
  /** Registration scale transform parameters.*/
  double sx;
  /** Registration scale transform parameters.*/
  double sy;
  /** Registration translation transform parameters.*/
  double tx;
  /** Registration translation transform parameters.*/
  double ty;
  /** Mutual information metric parameters.*/
  double m_fdev;
  /** Mutual information metric parameters.*/
  double m_mdev;
  /** Mutual information metric parameters.*/
  int m_NumberOfSamples;

  /** OPOE optimizer parameter.*/
  int initgen;
  /** General optimizer parameter.*/
  bool maximize;
  /** RSGD optimizer parameter.*/
  double m_maxStepLength;
  /** RSGD optimizer parameter.*/
  double m_minStepLength;
  /** GD optimizer parameter.*/
  double m_learningrate;
  /** Mattes mutual information metric parameter.*/
  int m_NumberOfBins;
  /** Mattes mutual information metric parameter.*/
  int m_nbBins;

  /** Spline order of the deformable transform.*/
  unsigned int m_storder;
  int m_nbEvaluations;
  double m_gct;
  double m_lsa;
  double m_dsl;

  /** Rigid transform parameter.*/
  double m_angle;

  /** Initialize transform with image moments.*/
  bool InitializeWithMoments;
  /** Initialize transform with geometry.*/
  bool geometry;

  /** Capture radius of mean reciprocal square metric.*/
  double m_lambda;
};

#endif
