/*=========================================================================

  $Autor$
  $Date: 2005/06/10 15:56:42 $
  $Revision: 1.4 $
  $Id: StartPireo.cxx, v 1.4 2005/06/10 15:56:42 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/StartPireo.cxx, v $
  $Log: StartPireo.cxx, v $
  Revision 1.4  2005/06/10 15:56:42  allain
  Add some functionalities for registration

  Revision 1.3  2005/06/06 16:16:56  allain
  Gave new method names, add commentary and implement new params

  Revision 1.2  2005/05/02 15:33:01  allain
  Generic Registration and Configuration File installed

  Revision 1.1  2005/04/27 13:15:11  allain
  Change most of class names

  Revision 1.12  2005/04/27 07:53:34  allain
  Add Save Registration Parameters

  Revision 1.11  2005/03/22 17:40:18  allain
  Grid is working and Pre-processing starts

  Revision 1.10  2005/03/21 14:35:09  allain
  Interface modified

  Revision 1.9  2005/03/03 16:52:44  allain
  Registration start and its observer

  Revision 1.8  2005/02/25 15:33:10  allain
  Transparency Window Set

  Revision 1.7  2005/02/23 16:09:58  allain
  New Class Hierarchy

  Revision 1.6  2005/02/21 17:09:31  allain
  New InterFace to display image

  Revision 1.5  2005/02/11 16:27:21  allain
  Mouse interaction start ..missed files in cvs

  Revision 1.4  2005/02/11 08:49:46  allain
  IHM beginning

  Revision 1.3  2005/02/10 14:33:12  allain
  Add InsightApplications used librairies

  Revision 1.2  2005/02/10 10:44:35  allain
  Changed Cmake

  Revision 1.1  2005/02/10 09:20:17  allain
  Add main


=========================================================================*/
/* MAIN */
/* Launch the user interface */
#include <iostream>
using namespace std;

/* FLTK Librairies */
#include <FL/Fl.H>
#include <FL/Fl_Window.H>

#include "otbCommandLineArgumentParser.h"

/* Code related */
#include "PireoViewer.h"
#include "PireoViewerGUI.h"

PireoViewer* gui;

int main(int argc, char* argv[])
{
  // Parse command line parameters
  typedef otb::CommandLineArgumentParser ParserType;
  ParserType::Pointer parser = ParserType::New();
  parser->AddOption("--FixeInputImage","Set the fixed input image name.","-inFix", 1, false);
  parser->AddOption("--MovingInputImage","Set the moving input image name.","-inMov", 1, false);
  parser->AddOption("--OutputImage","Set the output image file name.","-outImage", 1, false);
  parser->AddOption("--OutputFile","Set the output text file name.","-outFile", 1, false);
  parser->AddOption("--ConfigFile","Set the configuration file name.","-c", 1, false);

  typedef otb::CommandLineArgumentParseResult ParserResultType;
  ParserResultType::Pointer  parseResult = ParserResultType::New();

  try
  {
    parser->ParseCommandLine(argc, argv, parseResult);
  }
  catch ( itk::ExceptionObject & err )
  {
    std::string descriptionException = err.GetDescription();
    if (descriptionException.find("ParseCommandLine(): Help Parser") != std::string::npos)
    {
      return EXIT_SUCCESS;
    }
    if (descriptionException.find("ParseCommandLine(): Version Parser") != std::string::npos)
    {
      return EXIT_SUCCESS;
    }
    return EXIT_FAILURE;
  }

  if ( parseResult->IsOptionPresent("--ConfigFile") )
  {
    PireoApplication* m_app = new  PireoApplication();
    if ( parseResult->IsOptionPresent("--FixeInputImage") )
    {
      // Set fixed Image
      m_app->OpenFile(parseResult->GetParameterString("--FixeInputImage").c_str(), true);
    }
    if (parseResult->IsOptionPresent("--MovingInputImage"))
    {
      // Set Moving Image
      m_app->OpenFile(parseResult->GetParameterString("--MovingInputImage").c_str(), false);
    }

    // Registration Process
    m_app->InputImagesIntoRegistration();
    m_app->ReadRegistrationParametersFromFile(parseResult->GetParameterString("--ConfigFile").c_str());
    m_app->StartRegistration(); //BeginRegistration();


    delete m_app;
  }

  else
  {
    gui = new PireoViewer();

    gui->m_TransparencySlider->value(50, 10, 0, 110);
    gui->m_TransparencySlider->step(10);

    gui->window->show();
    gui->window->activate();
    gui->window->make_current();

    gui->CreateScene();
    gui->GetVTKWindowSize(gui->m_vtkWin->GetSize());

    /*
    if ( parseResult->IsOptionPresent("--FixeInputImage") && parseResult->IsOptionPresent("--MovingInputImage") )
    {
    // Set fixed Image
    gui->ReadFile(parseResult->GetParameterString("--FixeInputImage").c_str(), true);
    // Set Moving Image
    gui->ReadFile(parseResult->GetParameterString("--MovingInputImage").c_str(), false);
    gui->Display(2);
    gui->Display(3);
    gui->Display(4);
    gui->Display(5);
    }
    */

    if ( parseResult->IsOptionPresent("--FixeInputImage") || parseResult->IsOptionPresent("--MovingInputImage") )
    {
      if ( parseResult->IsOptionPresent("--FixeInputImage") )
      {
        // Set fixed Image
        gui->ReadFile(parseResult->GetParameterString("--FixeInputImage").c_str(), true);
      }
      if (parseResult->IsOptionPresent("--MovingInputImage"))
      {
        // Set Moving Image
        gui->ReadFile(parseResult->GetParameterString("--MovingInputImage").c_str(), false);
      }
      gui->Display(2);
      gui->Display(3);
      gui->Display(4);
      gui->Display(5);
    }

    // if not testing the app
    if (!parseResult->IsOptionPresent("--OTBTesting"))
    {
      otbGenericMsgDebugMacro(<<"Running GUI ...");
      return Fl::run();
    }
    //delete gui;
  }
  return 0;

}
