/*=========================================================================

  $Author: allain $
  $Date: 2005/07/12 15:20:13 $
  $Revision: 1.4 $
  $Id: TransparencyWindowManager.cxx, v 1.4 2005/07/12 15:20:13 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/TransparencyWindowManager.cxx, v $
  $Log: TransparencyWindowManager.cxx, v $
  Revision 1.4  2005/07/12 15:20:13  allain
  Add some functionalities

  Revision 1.3  2005/06/16 15:28:08  allain
  Add new functionalities

  Revision 1.2  2005/06/06 16:16:56  allain
  Gave new method names, add commentary and implement new params

  Revision 1.1  2005/04/27 13:15:11  allain
  Change most of class names

  Revision 1.3  2005/03/21 14:35:09  allain
  Interface modified

  Revision 1.2  2005/03/08 13:11:39  allain
  Set MultiResolutionImageRegistrationMethod

  Revision 1.1  2005/03/04 07:59:11  allain
  Add Missing files


=========================================================================*/

#include "TransparencyWindowManager.h"

/********************************************************************/
/*                        Constructor                               */
/********************************************************************/
TransparencyWindowManager::TransparencyWindowManager()
{
  // Set transparency default value (= master's opacity)
  transp=0.5;
  // Initialize renderer
  ren=vtkRenderer::New();
  ren->SetBackground(0.1 , 0.2 , 0.4);
  // Blender actor
  ia = vtkImageActor::New();
}

/********************************************************************/
/*                        Destructor                                */
/********************************************************************/
TransparencyWindowManager::~TransparencyWindowManager()
{
  ia->Delete();
  ren->Delete();

  master->Delete();
  slave->Delete();

  // Don't need blender anymore as we get the actor
  /*  if(ib!=NULL)
      ib->Delete(); */
}

/********************************************************************/
/*                        Insert Master Image                       */
/********************************************************************/
void TransparencyWindowManager::SetMaster(vtkImageData* ima)
{
  master = ima;
}

/********************************************************************/
/*                        Insert Slave Image                        */
/********************************************************************/
void TransparencyWindowManager::SetSlave(vtkImageData* ima)
{
  slave = ima;
}

/********************************************************************/
/*    Create actor corresponding to blender and add it to renderer  */
/********************************************************************/
void TransparencyWindowManager::SetRenderer()
{
  // First remove actor if it has already been set to renderer
  if (ia != NULL)
  {
    ren->RemoveActor(ia);
  }

  // Initialize blender
  vtkImageBlend* ib = vtkImageBlend::New();
  // Input images to blender (cannot input more than two images)
  ib->SetInput(0, slave);
  ib->SetInput(1, master);
  // Set master opacity (transparency effect)
  ib->SetOpacity(1, transp);
  // Get the actor corresponding to the blender
  ia->SetInput(ib->GetOutput());

  //Update image actor
  ia->GetInput()->UpdateInformation();
  ia->GetInput()->SetUpdateExtentToWholeExtent();
  ia->GetInput()->Update();

  // Add actor to renderer
  ren->AddActor(ia);

  // Used to write the blender as an image to save results
  result = ib->GetOutput();
  ib->Delete();

}

/********************************************************************/
/*              Return renderer corresponding to blender actor      */
/********************************************************************/
vtkRenderer* TransparencyWindowManager::GetRenderer()
{
  return ren;
}

/********************************************************************/
/*                Update transparency value                         */
/********************************************************************/
void TransparencyWindowManager::SetTransparency(float t)
{
  transp = t;
}

