/*=========================================================================

  $Author: allain $
  $Date: 2005/07/12 15:20:13 $
  $Revision: 1.4 $
  $Id: TransparencyWindowManager.h, v 1.4 2005/07/12 15:20:13 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/TransparencyWindowManager.h, v $
  $Log: TransparencyWindowManager.h, v $
  Revision 1.4  2005/07/12 15:20:13  allain
  Add some functionalities

  Revision 1.3  2005/06/16 15:28:08  allain
  Add new functionalities

  Revision 1.2  2005/06/06 16:16:56  allain
  Gave new method names, add commentary and implement new params

  Revision 1.1  2005/04/27 13:15:11  allain
  Change most of class names

  Revision 1.3  2005/03/21 14:35:09  allain
  Interface modified

  Revision 1.2  2005/03/08 13:11:39  allain
  Set MultiResolutionImageRegistrationMethod

  Revision 1.1  2005/03/04 07:59:11  allain
  Add Missing files


=========================================================================*/

#ifndef __TransparencyWindowManager_H__
#define __TransparencyWindowManager_H__

/* VTK Library */
#include "vtkImageActor.h"
#include "vtkRenderer.h"
#include "vtkRenderWindow.h"
#include "vtkImageData.h"
#include "vtkImageBlend.h"
//#include "vtkCamera.h"

/* Class TransparencyWindowManager
   Take care of everything concerning the renderer of the view in
   transparency window.
   In order to generate a blender to be able to render the fixed and
   moving image in transparency, we need to set:
   - fixed image with the method SetMaster
   - moving image with the method SetSlave
   The only element needed from outside is the corresponding renderer
   which is given by the method GetRenderer().
   Optional: we can set the value of the transparency with
   SetTransparency method (by default it is set to 0.5).
*/
class TransparencyWindowManager
{
public:
  /** Insert master image (fixed image).*/
  void SetMaster(vtkImageData* ima);
  /** Insert slave image (moving image).*/
  void SetSlave(vtkImageData* ima);
  /** Input the two images into a blender and set the renderer.*/
  void SetRenderer();
  /** Set opacity of the first image (to be able to see in transparency
  both images, it is set to 0.5 by default)*/
  void SetTransparency(float t);
  /** Return blender renderer*/
  vtkRenderer* GetRenderer();
  /** Return blender output to save as an image*/
  vtkImageData* GetBlender()
  {
    return result;
  }//ib->GetOutput(); }

  /** Constructor: Instantiate renderer and actor*/
  TransparencyWindowManager();
  /** Destructor*/
  ~TransparencyWindowManager();

protected:
  /** VTK blender (object that encapsulates both images).*/
  //vtkImageBlend* ib;
  /** VTK master image (corresponding to the fixed image)*/
  vtkImageData* master;
  /** VTK slave image (corresponding to the moving image)*/
  vtkImageData* slave;
  /** VTK image blend*/
  vtkImageData* result;
  /** Keep the transparency value (opacity of the master image)*/
  float transp;
  /** VTK renderer of the transparency window (so of the blender)*/
  vtkRenderer* ren;
  /** VTK actor that correspond to a image*/
  vtkImageActor* ia;
};

#endif
