/*=========================================================================

  $Author: allain $
  $Date: 2005/07/11 08:48:00 $
  $Revision: 1.3 $
  $Id: VectorField.cxx, v 1.3 2005/07/11 08:48:00 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/VectorField.cxx, v $
  $Log: VectorField.cxx, v $
  Revision 1.3  2005/07/11 08:48:00  allain
  grid debugged

  Revision 1.2  2005/06/06 16:16:56  allain
  Gave new method names, add commentary and implement new params

  Revision 1.1  2005/04/27 13:15:11  allain
  Change most of class names

  Revision 1.4  2005/03/24 16:14:28  allain
  Pre-Processing structure done

  Revision 1.3  2005/03/23 17:34:11  allain
  Pre-Processing is working

  Revision 1.2  2005/03/22 17:40:18  allain
  Grid is working and Pre-processing starts

  Revision 1.1  2005/03/15 11:05:49  allain
  Add Vector Field Class


=========================================================================*/

#include "VectorField.h"

VectorField::VectorField()
{
  //
  sgrid = vtkStructuredGrid::New();

  arrow = vtkGlyphSource2D::New();
  arrow->SetGlyphTypeToThickArrow();
  arrow->SetScale(1);
  arrow->FilledOn();
  arrow->CrossOff();

  glyph = vtkGlyph3D::New();
  glyph->SetInput(sgrid);
  glyph->SetSource(arrow->GetOutput());
  glyph->ScalingOn();
  glyph->SetScaleModeToScaleByVector();
  glyph->OrientOn();
  glyph->SetVectorModeToUseVector();
  glyph->SetColorModeToColorByVector();

  sgridMapper = vtkPolyDataMapper::New();
  sgridMapper->SetInput(glyph->GetOutput());
  sgridActor = vtkActor::New();
  sgridActor->SetMapper(sgridMapper);
  sgridActor->GetProperty()->SetColor(0, 0, 0);

  renderer = vtkRenderer::New();
  renderer->AddActor(sgridActor);
  renderer->SetBackground(1, 1, 1);

  NumberOfPointsAlongAxes = 5;
}

VectorField::~VectorField()
{
  if (m_PointSet!=NULL)
    m_PointSet->Delete();

  arrow->Delete();
  glyph->Delete();
  sgrid->Delete();
  sgridMapper->Delete();
  sgridActor->Delete();
  renderer->Delete();
}

void VectorField::SetBackgroundColor(float r, float g, float b)
{
  renderer->SetBackground(r, g, b);
}

void VectorField::SetArrowColor(float r, float g, float b)
{
  //sgridActor->GetProperty()->SetColor(r, g, b);
}

void VectorField::SetImageDimensions(int dimx, int dimy)
{
  dims = new int[2];
  dims[0] = dimx;
  dims[1] = dimy;
}

void VectorField::SetParameter(int NbOfPointsAlongAxes)
{
  NumberOfPointsAlongAxes = NbOfPointsAlongAxes;
}

void VectorField::Initialize()
{
  int i, j;
  NbPoints = NumberOfPointsAlongAxes*NumberOfPointsAlongAxes;

  m_PointSet = vtkPolyData::New();
  vtkFloatArray* vectors = vtkFloatArray::New();
  vectors->SetNumberOfComponents(3);
  vectors->SetNumberOfTuples(NbPoints);
  vtkPoints* points = vtkPoints::New();
  points->Allocate(NbPoints);

  stepx = dims[0]/(NumberOfPointsAlongAxes-1);
  stepy = dims[1]/(NumberOfPointsAlongAxes-1);
  int h =0;

  for (i = 0; i <  NumberOfPointsAlongAxes; i++)
  {
    for (j = 0; j <  NumberOfPointsAlongAxes; j++)
    {
      points->InsertPoint(h, static_cast<int>(i*stepx), static_cast<int>(j*stepy), 0);
      float* v = new float[3];
      v[0] = 0;
      v[1] = 0;
      v[2] = 0;
      vectors->InsertTuple(h, v);
      h++;
    }
  }
  m_PointSet->SetPoints(points);
  m_PointSet->GetPointData()->SetVectors(vectors);

  points->Delete();
  vectors->Delete();
}

void VectorField::SetRenderer()
{
  if (sgridActor!=NULL)
    renderer->RemoveActor(sgridActor);

  sgrid->SetDimensions(dims);
  sgrid->SetPoints(m_PointSet->GetPoints());
  sgrid->GetPointData()->SetVectors(m_PointSet->GetPointData()->GetVectors());

  renderer->AddActor(sgridActor);
}
