/*=========================================================================

  $Author: allain $
  $Date: 2005/07/11 08:48:00 $
  $Revision: 1.3 $
  $Id: VectorField.h, v 1.3 2005/07/11 08:48:00 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/VectorField.h, v $
  $Log: VectorField.h, v $
  Revision 1.3  2005/07/11 08:48:00  allain
  grid debugged

  Revision 1.2  2005/06/06 16:16:56  allain
  Gave new method names, add commentary and implement new params

  Revision 1.1  2005/04/27 13:15:11  allain
  Change most of class names

  Revision 1.3  2005/03/23 17:34:11  allain
  Pre-Processing is working

  Revision 1.2  2005/03/22 17:40:18  allain
  Grid is working and Pre-processing starts

  Revision 1.1  2005/03/15 11:05:49  allain
  Add Vector Field Class


=========================================================================*/

#ifndef __VECTORFIELD_H__
#define __VECTORFIELD_H__

#include "vtkPoints.h"
#include "vtkFloatArray.h"
#include "vtkActor.h"
#include "vtkHedgeHog.h"
#include "vtkPolyDataMapper.h"
#include "vtkProperty.h"
#include "vtkRenderer.h"
#include "vtkStructuredGrid.h"
#include "vtkGlyph3D.h"
#include "vtkGlyphSource2D.h"
#include "vtkPointData.h"
#include "vtkPolyData.h"
#include <vector>

/** Class VectorField
    Take care of everything concerning the renderer of the vector
    field.
    In order to generate a vector field, we need to:
    - input the dimensions of the image (method SetImageDimensions)
    - input the desired number of vectors in the grid along each axe
    (method SetParameter)
    - and then intialize the vector field with Initialize() method.
    As the vector field is only here to visualize the registration
    process at each iteration, the vector field is set from the
    PireoObserverManager class. The vector field object instantiated
    in PireoApplication class is only used by this class to update the
    vector field (see UpdateVectorField() method in PireoApplication
    class).
    Everything input in vector field is done from the
    PireoObserverManager object and the only element return by the
    vector field object (i.e the renderer) is used by PireoApplication
    object, where the vector field object is encapsulated).
*/
class VectorField
{
public:
  /** Constructor: instantiate all VTK objects needed.*/
  VectorField();
  /** Destructor.*/
  ~VectorField();

  /** Set the color of the arrow of the vector field (not working
  yet).*/
  void SetArrowColor(float r, float g, float b);
  /** Set the color of the background of the vector field (not working
  yet).*/
  void SetBackgroundColor(float r, float g, float b);
  /** Set grid renderer from all the input information.*/
  void SetRenderer();
  /** Return renderer corresponding to the grid.*/
  vtkRenderer* GetRenderer()
  {
    return renderer;
  }
  /** Need to input the dimensions of image on which the grid is
  created*/
  void SetImageDimensions(int dimx, int dimy);
  /** Set the only parameter of this object i.e the number of points
  in the grid along each axe*/
  void SetParameter(int NbOfPointsAlongAxes);
  /** Generate the grid from the number of points set and the image
  dimensions*/
  void Initialize();
  /** Return the VTK point set that contains grid points and grid cells*/
  vtkPolyData* GetPointSet()
  {
    return m_PointSet;
  }
  /** Return a appropriate maximum value for the length of the vector
  to display*/
  double GetStepX()
  {
    return stepx;
  }
  /** Return a appropriate maximum value for the length of the vector
  to display*/
  double GetStepY()
  {
    return stepy;
  }

  //void SetPointSet(vtkPolyData* ptset){ m_PointSet = ptset; }

protected:
  /** VTK point set*/
  vtkPolyData* m_PointSet;
  /** VTK grid renderer*/
  vtkRenderer* renderer;
  /** VTK unstructuredGrid*/
  vtkStructuredGrid *sgrid;
  /** VTK glyph (used to generate arrows)*/
  vtkGlyphSource2D* arrow;
  /** VTK glyph (used to generate arrows)*/
  vtkGlyph3D* glyph;
  /** VTK vector field mapper*/
  vtkPolyDataMapper *sgridMapper;
  /** VTK vector field actor*/
  vtkActor *sgridActor;

  /** Image dimensions*/
  int* dims;
  /** Number of vectors in the vector field*/
  int NbPoints;
  /** Maximum length of the vector*/
  double stepx;
  /** Maximum length of the vector*/
  double stepy;
  /** Number of points in the grid along each axe*/
  int NumberOfPointsAlongAxes;
};

#endif
