/*=========================================================================

  $Autor$
  $Date: 2005/06/06 16:16:56 $
  $Revision: 1.2 $
  $Id: ZoomManager.cxx, v 1.2 2005/06/06 16:16:56 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/ZoomManager.cxx, v $
  $Log: ZoomManager.cxx, v $
  Revision 1.2  2005/06/06 16:16:56  allain
  Gave new method names, add commentary and implement new params

  Revision 1.1  2005/04/27 13:15:11  allain
  Change most of class names

  Revision 1.4  2005/03/21 14:35:09  allain
  Interface modified

  Revision 1.3  2005/03/07 16:17:27  allain
  Registration with GDAL image starts working

  Revision 1.2  2005/03/03 16:52:44  allain
  Registration start and its observer

  Revision 1.1  2005/02/23 16:15:44  allain
  Remove and Add files


=========================================================================*/

#include "ZoomManager.h"

// Constructor
ZoomManager::ZoomManager()
{
  /*zoomx = 0;
  zoomy = 0; */

  m_spacingx = 1;
  m_spacingy = 1;
  m_spacsx = 1;
  m_spacsy = 1;
  m_originx = 0;
  m_originy = 0;

  m_regiondimx = 1024;
  m_regiondimy = 1024;
  m_dimx = 0;
  m_dimy = 0;
  m_sx = 512;
  m_sy = 512;
}

// Destructor
ZoomManager::~ZoomManager()
{
}

void ZoomManager::SetFullResolutionImageSpacing(float xspac, float yspac)
{
  m_spacingx = xspac;
  m_spacingy = yspac;
}

void ZoomManager::SetResampledImageSpacing(float xspac, float yspac)
{
  m_spacsx = xspac;
  m_spacsy = yspac;
}

void ZoomManager::SetZoomedImageDimensions(unsigned int Xdim, unsigned int Ydim)
{
  m_regiondimx = Xdim;
  m_regiondimy = Ydim;
}

void ZoomManager::SetFullResolutionImageDimensions(unsigned int DimX, unsigned int DimY)
{
  m_dimx = DimX;
  m_dimy = DimY;
}

void ZoomManager::SetResampledImageDimensions(unsigned int sx, unsigned int sy)
{
  m_sx = sx;
  m_sy = sy;
}

// Depending if you use Shrink + Resample as filter to load images or
// if you use Resample alone, the way the origin is calculated is
// different.
// anyway, it doesn't matter anymore as we are using only resample
// filter now.
bool ZoomManager::SetOrigin(float px, float py)
{
  if (m_dimx==0 || m_dimy==0)
  {
    std::cout<<"[Zoom::SetOrigin] Set Image Dimensions First"<<std::endl;
    return false;
  }
  // px and py are the coordinates of the clicked point on the
  // resampled image
  // From their values we calculate the corresponding point on the
  // full resolution image
  // (could be better to add also the origin of the image (ox +...))
  m_originx = (int)floor(px*m_spacsx/m_spacingx - m_regiondimx/2);
  m_originy = (int)floor(py*m_spacsy/m_spacingy - m_regiondimy/2);

  // Careful if the calculation is out of limits
  if (m_originx < 0)
    m_originx = 0;
  else if (static_cast<unsigned int>(m_originx) > m_dimx)
    m_originx = m_dimx - m_regiondimx;
  if (m_originy < 0)
    m_originy = 0;
  else if (static_cast<unsigned int>(m_originy) > m_dimy)
    m_originy = m_dimy - m_regiondimy;

  return true;
}

// Return X coordinate on the full resolution image
int ZoomManager::GetOriginX()
{
  return m_originx;
}

// Return Y coordinate on the full resolution image
int ZoomManager::GetOriginY()
{
  return m_originy;
}

void ZoomManager::GoUp()
{
  // Don't set origin outside image
  if (m_originy + m_regiondimy/2 < m_dimy)
    m_originy += m_regiondimy/2;
}

void ZoomManager::GoDown()
{
  // Don't set origin outside image
  if (m_originy - m_regiondimy/2 >= 0)
    m_originy -= m_regiondimy/2;
}

void ZoomManager::GoRight()
{
  // Don't set origin outside image
  if (m_originx + m_regiondimx/2 < m_dimx)
    m_originx += m_regiondimx/2;
}

void ZoomManager::GoLeft()
{
  // Don't set origin outside image
  if (m_originx - m_regiondimx/2 >= 0)
    m_originx -= m_regiondimx/2;
}

void ZoomManager::SetPosition(int px, int py)
{
  m_originx = px;
  m_originy = py;
}

// Not used anymore
/*float ZoomManager::GetImageSpacingX()
{
  return m_spacingx;
}

float ZoomManager::GetImageSpacingY()
{
  return m_spacingy;
}

unsigned int ZoomManager::GetRegionDimX()
{
  return m_regiondimx;
}

unsigned int ZoomManager::GetRegionDimY()
{
  return m_regiondimy;
}

unsigned int ZoomManager::GetImageDimX()
{
  return m_dimx;
}

unsigned int ZoomManager::GetImageDimY()
{
  return m_dimy;
}

unsigned int ZoomManager::GetSampleDimX()
{
  return m_sx;
}

unsigned int ZoomManager::GetSampleDimY()
{
  return m_sy;
}*/
