/*=========================================================================

  $Autor$
  $Date: 2005/06/06 16:16:56 $
  $Revision: 1.2 $
  $Id: ZoomManager.h, v 1.2 2005/06/06 16:16:56 allain Exp $
  $Source: /usr/local/stok/CVS_REPO/Pireo/Code/Registration/NewClassNames/ZoomManager.h, v $
  $Log: ZoomManager.h, v $
  Revision 1.2  2005/06/06 16:16:56  allain
  Gave new method names, add commentary and implement new params

  Revision 1.1  2005/04/27 13:15:11  allain
  Change most of class names

  Revision 1.4  2005/03/21 14:35:09  allain
  Interface modified

  Revision 1.3  2005/03/07 16:17:27  allain
  Registration with GDAL image starts working

  Revision 1.2  2005/03/03 16:52:44  allain
  Registration start and its observer

  Revision 1.1  2005/02/23 16:15:44  allain
  Remove and Add files


=========================================================================*/

#ifndef __ZoomManager_H__
#define __ZoomManager_H__

/* C++ Libraries */
#include <iostream>
#include <math.h>

/** Class ZoomManager
    Manage everything concerning the bottom-left corner of the zoomed image
    The zoomed image is representing a region of the loaded satellitary
    images.
    Indeed most of the time these images are too big to be supported
    by FLTK.
    In case the loaded images aren't that big the zoomed image is equal
    to the full resolution image.
    In this case, this class is unuseful.

    The only important element returned by this class needed from
    outside) is the origin of the zoomed image which is returned by
    GetOriginX() and GetOriginY().
    To get this element we need to set the full resolution image
    spacing and dimensions, and the resampled image spacing and
    dimensions. Moreover, to use the methods GoLeft, GoRight, GoUp and
    GoDown, we need also to set the current position of the zoomed image.
*/

class ZoomManager
{
public:
  /** Constructor: set default value to all parameters needed by this
  class (particulary origin = (0, 0)).*/
  ZoomManager();
  /** Destructor: empty.*/
  ~ZoomManager();
  /** set the spacing of the full resolution image.*/
  void SetFullResolutionImageSpacing(float xspac, float yspac);
  /** Set the spacing of the resampled image, which is resampled from
  the full resolution image.*/
  void SetResampledImageSpacing(float xspac, float yspac);
  /** Set dimensions of the full resolution image*/
  void SetFullResolutionImageDimensions(unsigned int DimX, unsigned int DimY);
  /** Set dimensions of the zoomed image, which represents a region of
  the full resolution image (1024x1024 by default). Cannot exceed
  2048x2048 as for the moment we are unable to display it with FTLK.*/
  void SetZoomedImageDimensions(unsigned int Xdim, unsigned int Ydim);
  /** Set dimensions of the resampled image, which is resampled from
  the full resolution image.*/
  void SetResampledImageDimensions(unsigned int sx, unsigned int sy);
  /** Set origin, which is calculated from the point clicked by the
  user on the sample image (the two input parameters are the
  coordinates of the clicked point on the sample image ). From then,
  the corresponding point on the full resolution image is calculated.*/
  bool SetOrigin(float px, float py);
  /** Return the X coordinate (corresponding to the clicked point) on
  the full resolution image.*/
  int GetOriginX();
  /** Return the Y coordinate (corresponding to the clicked point) on
  the full resolution image.*/
  int GetOriginY();
  /** Move up the bottom-left corner of the zoomed image, up to region image
  dimensions divided by two.*/
  void GoUp();
  /** Move down the bottom-left corner of the zoomed image, up to region image
  dimensions divided by two.*/
  void GoDown();
  /** Move to the right the bottom-left corner of the zoomed image, up to region
  image dimensions divided by two.*/
  void GoRight();
  /** Move to the left the bottom-left corner of the zoomed image, up to region
  image dimensions divided by two.*/
  void GoLeft();
  /** Set directly the origin coordinates to know where it is
  currently (used only for GoUp(), GoDown(), GoLeft() and GoRight()
  methods.*/
  void SetPosition(int px, int py);

  /** Useful to debug but not used anymore*/
  /*unsigned int GetSampleDimX();
  unsigned int GetSampleDimY();
  unsigned int GetImageDimX();
  unsigned int GetImageDimY();
  unsigned int GetRegionDimX();
  unsigned int GetRegionDimY();
  float GetImageSpacingX();
  float GetImageSpacingY(); */

protected:
  /*int zoomx;
  int zoomy; */
  /** Stock full resolution image spacing.*/
  float m_spacingx;
  /** Stock full resolution image spacing.*/
  float m_spacingy;
  /** Stock resampled image spacing.*/
  float m_spacsx;
  /** Stock resampled image spacing.*/
  float m_spacsy;
  /** Stock zoomed image dimensions (set to 1024x1024). Cannot exceed
  2048x2048.*/
  unsigned int m_regiondimx;
  /** Stock zoomed image dimensions (set to 1024x1024). Cannot exceed
  2048x2048.*/
  unsigned int m_regiondimy;
  /** Stock full resolution image dimensions.*/
  unsigned int m_dimx;
  /** Stock full resolution image dimensions.*/
  unsigned int m_dimy;
  /** Stock resampled image dimensions.*/
  unsigned int m_sx;
  /** Stock resampled image dimensions.*/
  unsigned int m_sy;
  /** Stock the coordinates of the point (clicked on the resampled
  image) on the full resolution image.*/
  int m_originx;
  /** Stock the coordinates of the point (clicked on the resampled
  image) on the full resolution image.*/
  int m_originy;
};

#endif
