/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbPolarimetricSynthesisApplicationController_h
#define __otbPolarimetricSynthesisApplicationController_h

#include "otbPolarimetricSynthesisApplicationControllerInterface.h"
#include "otbPolarimetricSynthesisApplicationViewGUI.h"

namespace otb
{
class PolarimetricSynthesisApplicationController
      : public PolarimetricSynthesisApplicationControllerInterface
{
public:
  /** Standard class typedefs */
  typedef PolarimetricSynthesisApplicationController Self;
  typedef itk::Object                       Superclass;
  typedef itk::SmartPointer<Self>           Pointer;
  typedef itk::SmartPointer<const Self>     ConstPointer;

  typedef otb::PolarimetricSynthesisApplicationViewGUI ViewType;
  typedef ViewType::Pointer                            ViewPointerType;

  /** Standard type macros */
  itkTypeMacro(PolarimetricSynthesisApplicationController, Superclass);
  itkNewMacro(Self);

  /** Set the view */
  virtual void SetView(ViewPointerType view);

  /** User action */
  virtual void LoadImages();
  virtual void ChangeRGB(bool value);
  virtual void ChangeGain(double value);
  virtual void ChangeGrayPsiI(double value);
  virtual void ChangeGrayPsiR(double value);
  virtual void ChangeGrayKhiI(double value);
  virtual void ChangeGrayKhiR(double value);
  virtual void ChangeGrayPolarizationMode(PolarizationMode mode);
  virtual void ChangeRedPsiI(double value);
  virtual void ChangeRedPsiR(double value);
  virtual void ChangeRedKhiI(double value);
  virtual void ChangeRedKhiR(double value);
  virtual void ChangeRedPolarizationMode(PolarizationMode mode);
  virtual void ChangeGreenPsiI(double value);
  virtual void ChangeGreenPsiR(double value);
  virtual void ChangeGreenKhiI(double value);
  virtual void ChangeGreenKhiR(double value);
  virtual void ChangeGreenPolarizationMode(PolarizationMode mode);
  virtual void ChangeBluePsiI(double value);
  virtual void ChangeBluePsiR(double value);
  virtual void ChangeBlueKhiI(double value);
  virtual void ChangeBlueKhiR(double value);
  virtual void ChangeBluePolarizationMode(PolarizationMode mode);
  virtual void IndexClicked(const IndexType& index);
  virtual void SaveResult(const char * fname);
  virtual void MouseIn();
  virtual void MouseOut();
  virtual void PixelHovered(const IndexType & index);

protected:
  /** Constructor */
  PolarimetricSynthesisApplicationController();
  /** Destructor */
  ~PolarimetricSynthesisApplicationController();

private:
  PolarimetricSynthesisApplicationController(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

  /** Pointer to the view */
  ViewPointerType m_View;
};
} //end namespace otb

#endif
