/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include "otbPolarimetricSynthesisApplicationViewGUI.h"
#include <FL/Fl_File_Chooser.H>
#include "otbFltkFilterWatcher.h"
#include "otbMacro.h"

namespace otb
{

PolarimetricSynthesisApplicationViewGUI::PolarimetricSynthesisApplicationViewGUI()
{
  m_PolarimetricSynthesisApplicationModel = PolarimetricSynthesisApplicationModel::GetInstance();
  m_PolarimetricSynthesisApplicationModel->RegisterListener(this);

  m_VectorCastFilter = VectorCastFilterType::New();
  m_QuicklookVectorCastFilter = VectorCastFilterType::New();

  this->Build();
  wOpenImageWindow->hide();
  wMainWindow->show();

  m_InterfaceColor[0]=1;
  m_InterfaceColor[1]=0;
  m_InterfaceColor[2]=0;
  m_InterfaceColor[3]=1;

  m_PoincareSphereWidget = PoincareSphereWidgetType::New();
  m_PoincareSphereWidget->SetPsiI(m_PolarimetricSynthesisApplicationModel->GetRedPsiI());
  m_PoincareSphereWidget->SetPsiR(m_PolarimetricSynthesisApplicationModel->GetRedPsiR());
  m_PoincareSphereWidget->SetKhiI(m_PolarimetricSynthesisApplicationModel->GetRedKhiI());
  m_PoincareSphereWidget->SetKhiR(m_PolarimetricSynthesisApplicationModel->GetRedKhiR());
  bRGB->value(m_PolarimetricSynthesisApplicationModel->GetRGB());
  if (m_PolarimetricSynthesisApplicationModel->GetRGB())
  {
    m_PoincareSphereWidget->SetPsiI(m_PolarimetricSynthesisApplicationModel->GetRedPsiI());
    m_PoincareSphereWidget->SetPsiR(m_PolarimetricSynthesisApplicationModel->GetRedPsiR());
    m_PoincareSphereWidget->SetKhiI(m_PolarimetricSynthesisApplicationModel->GetRedKhiI());
    m_PoincareSphereWidget->SetKhiR(m_PolarimetricSynthesisApplicationModel->GetRedKhiR());
    gGrayTabs->hide();
    gRGBTabs->show();
  }
  else
  {
    m_PoincareSphereWidget->SetPsiI(m_PolarimetricSynthesisApplicationModel->GetGrayPsiI());
    m_PoincareSphereWidget->SetPsiR(m_PolarimetricSynthesisApplicationModel->GetGrayPsiR());
    m_PoincareSphereWidget->SetKhiI(m_PolarimetricSynthesisApplicationModel->GetGrayKhiI());
    m_PoincareSphereWidget->SetKhiR(m_PolarimetricSynthesisApplicationModel->GetGrayKhiR());
    gRGBTabs->hide();
    gGrayTabs->show();
  }

  m_PoincareSphereWidget->Init(gSphere->x(), gSphere->y(), gSphere->w(), gSphere->h(),"Poincare sphere");
  gSphere->add(m_PoincareSphereWidget);
  gSphere->box(FL_NO_BOX);
  m_PoincareSphereWidget->show();
  m_PoincareSphereWidget->redraw();
}


void PolarimetricSynthesisApplicationViewGUI::SetPolarimetricSynthesisApplicationController(PolarimetricSynthesisApplicationControllerInterface * controller)
{
  m_PolarimetricSynthesisApplicationController = controller;
}

void PolarimetricSynthesisApplicationViewGUI::RefreshViewer()
{
  BoxType::Pointer box = BoxType::New();
  SizeType boxSize;
  IndexType boxIndex;

  ImageType::Pointer image, quicklook;

  if (m_PolarimetricSynthesisApplicationModel->GetRGB())
  {
    image = m_PolarimetricSynthesisApplicationModel->GetOutputVectorImage();
    image->UpdateOutputInformation();
    quicklook = m_PolarimetricSynthesisApplicationModel->GetQuicklookOutputVectorImage();
    quicklook->Update();
  }
  else
  {
    m_VectorCastFilter->SetInput(m_PolarimetricSynthesisApplicationModel->GetOutputImage());
    m_VectorCastFilter->GetOutput()->UpdateOutputInformation();
    image = m_VectorCastFilter->GetOutput();
    m_QuicklookVectorCastFilter->SetInput(m_PolarimetricSynthesisApplicationModel->GetQuicklookOutputImage());
    m_QuicklookVectorCastFilter->Update();
    quicklook = m_QuicklookVectorCastFilter->GetOutput();
  }


  // Defining the full widget
  if (m_FullWidget.IsNull())
  {
    m_FullWidget  = FullWidgetType::New();
    m_FullWidget->SetInput(image);
    m_FullWidget->SetController(m_PolarimetricSynthesisApplicationController);
    m_FullWidget->Init(gFullResolution->x(), gFullResolution->y(), gFullResolution->w(), gFullResolution->h(),"Full resolution");
    gFullResolution->box(FL_NO_BOX);
    gFullResolution->add(m_FullWidget);

    if (!m_PolarimetricSynthesisApplicationModel->GetRGB() )
    {
      m_FullWidget->GetTransferFunctionList()->PushBack(m_FullWidget->GetTransferFunctionList()->GetNthElement(0));
      m_FullWidget->GetTransferFunctionList()->PushBack(m_FullWidget->GetTransferFunctionList()->GetNthElement(0));
    }
    else
    {
      m_FullWidget->GetTransferFunctionList()->SetNthElement(1, m_FullWidget->GetTransferFunctionList()->GetNthElement(0));
      m_FullWidget->GetTransferFunctionList()->SetNthElement(2, m_FullWidget->GetTransferFunctionList()->GetNthElement(0));
    }
    m_FullWidget->Show();
  }
  else
  {
    m_FullWidget->SetInput(image);
    m_FullWidget->ClearBufferedRegion();
  }

  // Defining the box of the viewed full region
  boxSize[0]=(m_FullWidget->GetViewedRegion().GetSize()[0]+2);
  boxSize[1]=(m_FullWidget->GetViewedRegion().GetSize()[1]+2);
  boxIndex[0]=(m_FullWidget->GetViewedRegion().GetIndex()[0]-1);
  boxIndex[1]=(m_FullWidget->GetViewedRegion().GetIndex()[1]-1);
  box->SetSize(boxSize);
  box->SetIndex(boxIndex);
  box->SetColor(m_InterfaceColor);


  // defining the scroll widget
  if (m_ScrollWidget.IsNull())
  {
    m_ScrollWidget  = ScrollWidgetType::New();
    m_ScrollWidget->SetController(m_PolarimetricSynthesisApplicationController);
    m_ScrollWidget->SetInput(quicklook);
    m_ScrollWidget->Init(gScroll->x(), gScroll->y(), gScroll->w(), gScroll->h(),"Scroll");
    m_ScrollWidget->SetFormOverlayVisible(true);
    gScroll->add(m_ScrollWidget);
    gScroll->box(FL_NO_BOX);
    m_ScrollWidget->GetFormList()->PushBack(box);
    m_ScrollWidget->GetTransferFunctionList()->SetNthElement(0, m_FullWidget->GetTransferFunctionList()->GetNthElement(0));
    if (!m_PolarimetricSynthesisApplicationModel->GetRGB())
    {
      m_ScrollWidget->GetTransferFunctionList()->PushBack(m_FullWidget->GetTransferFunctionList()->GetNthElement(0));
      m_ScrollWidget->GetTransferFunctionList()->PushBack(m_FullWidget->GetTransferFunctionList()->GetNthElement(0));
    }
    else
    {
      m_ScrollWidget->GetTransferFunctionList()->SetNthElement(1, m_FullWidget->GetTransferFunctionList()->GetNthElement(0));
      m_ScrollWidget->GetTransferFunctionList()->SetNthElement(2, m_FullWidget->GetTransferFunctionList()->GetNthElement(0));
    }
    m_ScrollWidget->Show();
  }
  else
  {
    m_ScrollWidget->GetFormList()->SetNthElement(0, box);
    m_ScrollWidget->SetInput(quicklook);
    m_ScrollWidget->ClearBufferedRegion();
  }
  m_ScrollWidget->SetSubSamplingRate(m_PolarimetricSynthesisApplicationModel->GetShrinkFactor());

  m_FullWidget->GetTransferFunctionList()->GetNthElement(0)->SetLowerBound(m_PolarimetricSynthesisApplicationModel->GetMin());
  m_FullWidget->GetTransferFunctionList()->GetNthElement(0)->SetUpperBound(m_PolarimetricSynthesisApplicationModel->GetMax());

  if (m_PolarimetricSynthesisApplicationModel->GetRGB())
  {

    m_FullWidget->SetViewModel(FullWidgetType::RGB);
    m_ScrollWidget->SetViewModel(ScrollWidgetType::RGB);

  }
  else
  {
    m_FullWidget->SetViewModel(FullWidgetType::GRAYSCALE);
    m_ScrollWidget->SetViewModel(ScrollWidgetType::GRAYSCALE);
  }

  m_ScrollWidget->redraw();
  m_FullWidget->redraw();

  //  if(m_HistogramWidget.IsNull())
//       {
//   m_HistogramWidget = HistogramWidgetType::New();
//   m_HistogramWidget->SetTransferFunction(m_FullWidget->GetTransferFunctionList()->GetNthElement(0));
//   m_HistogramWidget->SetHistogram(m_PolarimetricSynthesisApplicationModel->GetOutputHistogram());
//   m_HistogramWidget->resize(gHistogram->x(), gHistogram->y(), gHistogram->w(), gHistogram->h());
//   gHistogram->add(m_HistogramWidget);
//   m_HistogramWidget->show();
//       }
//     m_HistogramWidget->SetHistogram(m_PolarimetricSynthesisApplicationModel->GetOutputHistogram());
//     m_HistogramWidget->redraw();
}

void PolarimetricSynthesisApplicationViewGUI::Notify()
{
  this->UpdateInterface();
  if (m_PolarimetricSynthesisApplicationModel->GetValid())
  {
    this->RefreshViewer();
  }
}

void PolarimetricSynthesisApplicationViewGUI::VectorBrowserCallback()
{
  const char * filename = NULL;

  filename = fl_file_chooser("Pick an image file", "*.*",".");

  if (filename == NULL || !strcmp(filename, ""))
  {
    otbMsgDebugMacro(<<"Empty file name!");
    return;
  }

  fVectorInputImage->value(filename);
}
void PolarimetricSynthesisApplicationViewGUI::HHBrowserCallback()
{
  const char * filename = NULL;

  filename = fl_file_chooser("Pick an image file", "*.*",".");

  if (filename == NULL || !strcmp(filename, ""))
  {
    otbMsgDebugMacro(<<"Empty file name!");
    return;
  }
  fHHInputImage->value(filename);
}
void PolarimetricSynthesisApplicationViewGUI::HVBrowserCallback()
{
  const char * filename = NULL;

  filename = fl_file_chooser("Pick an image file", "*.*",".");

  if (filename == NULL || !strcmp(filename, ""))
  {
    otbMsgDebugMacro(<<"Empty file name!");
    return;
  }

  fHVInputImage->value(filename);
}
void PolarimetricSynthesisApplicationViewGUI::VHBrowserCallback()
{
  const char * filename = NULL;

  filename = fl_file_chooser("Pick an image file", "*.*",".");

  if (filename == NULL || !strcmp(filename, ""))
  {
    otbMsgDebugMacro(<<"Empty file name!");
    return;
  }

  fVHInputImage->value(filename);
}
void PolarimetricSynthesisApplicationViewGUI::VVBrowserCallback()
{
  const char * filename = NULL;

  filename = fl_file_chooser("Pick an image file", "*.*",".");

  if (filename == NULL || !strcmp(filename, ""))
  {
    otbMsgDebugMacro(<<"Empty file name!");
    return;
  }
  fVVInputImage->value(filename);
}

void PolarimetricSynthesisApplicationViewGUI::SaveResultCallback()
{
  const char * filename = NULL;

  filename = fl_file_chooser("Pick an image file", "*.*",".");

  if (filename == NULL || !strcmp(filename, ""))
  {
    otbMsgDebugMacro(<<"Empty file name!");
    return;
  }

  otb::FltkFilterWatcher  * watcher;

  if (m_PolarimetricSynthesisApplicationModel->GetRGB())
  {
    watcher = new FltkFilterWatcher(m_PolarimetricSynthesisApplicationModel->GetWriterRGB(), wMainWindow->x(), wMainWindow->y(), 200, 20, "Saving result ...");
  }
  else
  {
    watcher = new FltkFilterWatcher(m_PolarimetricSynthesisApplicationModel->GetWriterGray(), wMainWindow->x(), wMainWindow->y(), 200, 20, "Saving result ...");
  }
  m_PolarimetricSynthesisApplicationController->SaveResult(filename);
  delete watcher;
}

void PolarimetricSynthesisApplicationViewGUI::HEmissionCallback()
{
  if (bHEmission->value())
  {
    m_PolarimetricSynthesisApplicationModel->SetHEmissionMode(true);
    bOpenImageOk->activate();
    fHVInputImage->activate();
    bHVBrowser->activate();
    fHHInputImage->activate();
    bHHBrowser->activate();
    if (bVEmission->value())
    {
      vGrayPsiI->activate();
      vGrayKhiI->activate();
      vRedPsiI->activate();
      vRedKhiI->activate();
      vGreenPsiI->activate();
      vGreenKhiI->activate();
      vBluePsiI->activate();
      vBlueKhiI->activate();
    }
  }
  else
  {
    m_PolarimetricSynthesisApplicationModel->SetHEmissionMode(false);
    if (!bVEmission->value())
    {
      bOpenImageOk->deactivate();
    }
    vGrayPsiI->deactivate();
    vGrayKhiI->deactivate();
    vRedPsiI->deactivate();
    vRedKhiI->deactivate();
    vGreenPsiI->deactivate();
    vGreenKhiI->deactivate();
    vBluePsiI->deactivate();
    vBlueKhiI->deactivate();
    fHVInputImage->deactivate();
    bHVBrowser->deactivate();
    fHHInputImage->deactivate();
    bHHBrowser->deactivate();
  }
}

void PolarimetricSynthesisApplicationViewGUI::VEmissionCallback()
{
  if (bVEmission->value())
  {
    bOpenImageOk->activate();
    fVVInputImage->activate();
    bVVBrowser->activate();
    fVHInputImage->activate();
    bVHBrowser->activate();
    if (bHEmission->value())
    {
      vGrayPsiI->activate();
      vGrayKhiI->activate();
      vRedPsiI->activate();
      vRedKhiI->activate();
      vGreenPsiI->activate();
      vGreenKhiI->activate();
      vBluePsiI->activate();
      vBlueKhiI->activate();
    }
  }
  else
  {
    if (!bHEmission->value())
    {
      bOpenImageOk->deactivate();
    }
    vGrayPsiI->deactivate();
    vGrayKhiI->deactivate();
    vRedPsiI->deactivate();
    vRedKhiI->deactivate();
    vGreenPsiI->deactivate();
    vGreenKhiI->deactivate();
    vBluePsiI->deactivate();
    vBlueKhiI->deactivate();
    fVVInputImage->deactivate();
    bVVBrowser->deactivate();
    fVHInputImage->deactivate();
    bVHBrowser->deactivate();
  }
}
void PolarimetricSynthesisApplicationViewGUI::OpenVectorImageCallback()
{
  gGroupImages->deactivate();
  gGroupVectorImage->activate();
  bOpenVectorImage->set();
  bOpenImages->clear();
}
void PolarimetricSynthesisApplicationViewGUI::OpenImagesCallback()
{
  gGroupImages->activate();
  gGroupVectorImage->deactivate();
  bOpenImages->set();
  bOpenVectorImage->clear();

}
void PolarimetricSynthesisApplicationViewGUI::OpenImageOkCallback()
{
  otb::FltkFilterWatcher  watcher(m_PolarimetricSynthesisApplicationModel->GetShrinkFilter(), wOpenImageWindow->x(), wOpenImageWindow->y()-20, 200, 20, "Generating Quicklook ...");
  bOpenImageOk->deactivate();
  m_PolarimetricSynthesisApplicationController->LoadImages();
  bOpenImageOk->activate();
  wOpenImageWindow->hide();
}
void PolarimetricSynthesisApplicationViewGUI::OpenImageCancelCallback()
{
  wOpenImageWindow->hide();
}
void PolarimetricSynthesisApplicationViewGUI::OpenCallback()
{
  wOpenImageWindow->show();
}
void PolarimetricSynthesisApplicationViewGUI::QuitCallback()
{
  wMainWindow->hide();
  wOpenImageWindow->hide();
}

void PolarimetricSynthesisApplicationViewGUI::UpdateInterface()
{

  if (gGrayGroup->visible_r())
  {

    vGrayPsiI->value(m_PolarimetricSynthesisApplicationModel->GetGrayPsiI());
    vGrayPsiR->value(m_PolarimetricSynthesisApplicationModel->GetGrayPsiR());
    vGrayKhiI->value(m_PolarimetricSynthesisApplicationModel->GetGrayKhiI());
    vGrayKhiR->value(m_PolarimetricSynthesisApplicationModel->GetGrayKhiR());
    m_PoincareSphereWidget->SetPsiI(m_PolarimetricSynthesisApplicationModel->GetGrayPsiI());
    m_PoincareSphereWidget->SetPsiR(m_PolarimetricSynthesisApplicationModel->GetGrayPsiR());
    m_PoincareSphereWidget->SetKhiI(m_PolarimetricSynthesisApplicationModel->GetGrayKhiI());
    m_PoincareSphereWidget->SetKhiR(m_PolarimetricSynthesisApplicationModel->GetGrayKhiR());
    m_PoincareSphereWidget->redraw();
  }
  else if (gRedGroup->visible_r())
  {

    vRedPsiI->value(m_PolarimetricSynthesisApplicationModel->GetRedPsiI());
    vRedPsiR->value(m_PolarimetricSynthesisApplicationModel->GetRedPsiR());
    vRedKhiI->value(m_PolarimetricSynthesisApplicationModel->GetRedKhiI());
    vRedKhiR->value(m_PolarimetricSynthesisApplicationModel->GetRedKhiR());
    m_PoincareSphereWidget->SetPsiI(m_PolarimetricSynthesisApplicationModel->GetRedPsiI());
    m_PoincareSphereWidget->SetPsiR(m_PolarimetricSynthesisApplicationModel->GetRedPsiR());
    m_PoincareSphereWidget->SetKhiI(m_PolarimetricSynthesisApplicationModel->GetRedKhiI());
    m_PoincareSphereWidget->SetKhiR(m_PolarimetricSynthesisApplicationModel->GetRedKhiR());
    m_PoincareSphereWidget->redraw();
  }
  else if (gGreenGroup->visible_r())
  {
    vGreenPsiI->value(m_PolarimetricSynthesisApplicationModel->GetGreenPsiI());
    vGreenPsiR->value(m_PolarimetricSynthesisApplicationModel->GetGreenPsiR());
    vGreenKhiI->value(m_PolarimetricSynthesisApplicationModel->GetGreenKhiI());
    vGreenKhiR->value(m_PolarimetricSynthesisApplicationModel->GetGreenKhiR());
    m_PoincareSphereWidget->SetPsiI(m_PolarimetricSynthesisApplicationModel->GetGreenPsiI());
    m_PoincareSphereWidget->SetPsiR(m_PolarimetricSynthesisApplicationModel->GetGreenPsiR());
    m_PoincareSphereWidget->SetKhiI(m_PolarimetricSynthesisApplicationModel->GetGreenKhiI());
    m_PoincareSphereWidget->SetKhiR(m_PolarimetricSynthesisApplicationModel->GetGreenKhiR());
    m_PoincareSphereWidget->redraw();
  }
  else if (gBlueGroup->visible_r())
  {
    vBluePsiI->value(m_PolarimetricSynthesisApplicationModel->GetBluePsiI());
    vBluePsiR->value(m_PolarimetricSynthesisApplicationModel->GetBluePsiR());
    vBlueKhiI->value(m_PolarimetricSynthesisApplicationModel->GetBlueKhiI());
    vBlueKhiR->value(m_PolarimetricSynthesisApplicationModel->GetBlueKhiR());
    m_PoincareSphereWidget->SetPsiI(m_PolarimetricSynthesisApplicationModel->GetBluePsiI());
    m_PoincareSphereWidget->SetPsiR(m_PolarimetricSynthesisApplicationModel->GetBluePsiR());
    m_PoincareSphereWidget->SetKhiI(m_PolarimetricSynthesisApplicationModel->GetBlueKhiI());
    m_PoincareSphereWidget->SetKhiR(m_PolarimetricSynthesisApplicationModel->GetBlueKhiR());
    m_PoincareSphereWidget->redraw();
  }

  if (m_PolarimetricSynthesisApplicationModel->GetValid())
  {
    bSaveResult->activate();
  }
}
void PolarimetricSynthesisApplicationViewGUI::RGBButtonCallback()
{
  m_PolarimetricSynthesisApplicationController->ChangeRGB(bRGB->value());
  if (bRGB->value())
  {
    gGrayTabs->hide();
    gRGBTabs->show();
    this->RGBGroupCallback();
  }
  else
  {
    gRGBTabs->hide();
    gGrayTabs->show();
    this->GrayGroupCallback();
  }
}
void PolarimetricSynthesisApplicationViewGUI::DialGainCallback()
{
  m_PolarimetricSynthesisApplicationController->ChangeGain(vGain->value());
  iGain->value(vGain->value());
}
void PolarimetricSynthesisApplicationViewGUI::InputGainCallback()
{
  m_PolarimetricSynthesisApplicationController->ChangeGain(iGain->value());
  vGain->value(iGain->value());
}
void PolarimetricSynthesisApplicationViewGUI::GrayGroupCallback()
{


  m_PolarimetricSynthesisApplicationController->ChangeGrayPsiI(vGrayPsiI->value());
  m_PolarimetricSynthesisApplicationController->ChangeGrayKhiI(vGrayKhiI->value());
  m_PolarimetricSynthesisApplicationController->ChangeGrayPsiR(vGrayPsiR->value());
  m_PolarimetricSynthesisApplicationController->ChangeGrayKhiR(vGrayKhiR->value());

  // Update the sphere
  m_PoincareSphereWidget->SetPsiI(m_PolarimetricSynthesisApplicationModel->GetGrayPsiI());
  m_PoincareSphereWidget->SetPsiR(m_PolarimetricSynthesisApplicationModel->GetGrayPsiR());
  m_PoincareSphereWidget->SetKhiI(m_PolarimetricSynthesisApplicationModel->GetGrayKhiI());
  m_PoincareSphereWidget->SetKhiR(m_PolarimetricSynthesisApplicationModel->GetGrayKhiR());

}
void PolarimetricSynthesisApplicationViewGUI::RGBGroupCallback()
{

  if (gRGBTabs->value()==gRedGroup)
  {
    m_PolarimetricSynthesisApplicationController->ChangeRedPsiI(vRedPsiI->value());
    m_PolarimetricSynthesisApplicationController->ChangeRedKhiI(vRedKhiI->value());
    m_PolarimetricSynthesisApplicationController->ChangeRedPsiR(vRedPsiR->value());
    m_PolarimetricSynthesisApplicationController->ChangeRedKhiR(vRedKhiR->value());
    // Update the sphere
    m_PoincareSphereWidget->SetPsiI(m_PolarimetricSynthesisApplicationModel->GetRedPsiI());
    m_PoincareSphereWidget->SetPsiR(m_PolarimetricSynthesisApplicationModel->GetRedPsiR());
    m_PoincareSphereWidget->SetKhiI(m_PolarimetricSynthesisApplicationModel->GetRedKhiI());
    m_PoincareSphereWidget->SetKhiR(m_PolarimetricSynthesisApplicationModel->GetRedKhiR());

  }
  else if (gRGBTabs->value()==gGreenGroup)
  {

    m_PolarimetricSynthesisApplicationController->ChangeGreenPsiI(vGreenPsiI->value());
    m_PolarimetricSynthesisApplicationController->ChangeGreenKhiI(vGreenKhiI->value());
    m_PolarimetricSynthesisApplicationController->ChangeGreenPsiR(vGreenPsiR->value());
    m_PolarimetricSynthesisApplicationController->ChangeGreenKhiR(vGreenKhiR->value());
    // Update the sphere
    m_PoincareSphereWidget->SetPsiI(m_PolarimetricSynthesisApplicationModel->GetGreenPsiI());
    m_PoincareSphereWidget->SetPsiR(m_PolarimetricSynthesisApplicationModel->GetGreenPsiR());
    m_PoincareSphereWidget->SetKhiI(m_PolarimetricSynthesisApplicationModel->GetGreenKhiI());
    m_PoincareSphereWidget->SetKhiR(m_PolarimetricSynthesisApplicationModel->GetGreenKhiR());

  }
  else if (gRGBTabs->value()==gBlueGroup)
  {

    m_PolarimetricSynthesisApplicationController->ChangeBluePsiI(vBluePsiI->value());
    m_PolarimetricSynthesisApplicationController->ChangeBlueKhiI(vBlueKhiI->value());
    m_PolarimetricSynthesisApplicationController->ChangeBluePsiR(vBluePsiR->value());
    m_PolarimetricSynthesisApplicationController->ChangeBlueKhiR(vBlueKhiR->value());
    // Update the sphere
    m_PoincareSphereWidget->SetPsiI(m_PolarimetricSynthesisApplicationModel->GetBluePsiI());
    m_PoincareSphereWidget->SetPsiR(m_PolarimetricSynthesisApplicationModel->GetBluePsiR());
    m_PoincareSphereWidget->SetKhiI(m_PolarimetricSynthesisApplicationModel->GetBlueKhiI());
    m_PoincareSphereWidget->SetKhiR(m_PolarimetricSynthesisApplicationModel->GetBlueKhiR());

  }
}

// -------------------------- RED  -------------------------- //
void PolarimetricSynthesisApplicationViewGUI::RedPsiICallback()
{
  m_PolarimetricSynthesisApplicationController->ChangeRedPsiI(vRedPsiI->value());
}
void PolarimetricSynthesisApplicationViewGUI::RedPsiRCallback()
{
  m_PolarimetricSynthesisApplicationController->ChangeRedPsiR(vRedPsiR->value());
}
void PolarimetricSynthesisApplicationViewGUI::RedKhiICallback()
{
  m_PolarimetricSynthesisApplicationController->ChangeRedKhiI(vRedKhiI->value());
}
void PolarimetricSynthesisApplicationViewGUI::RedKhiRCallback()
{
  m_PolarimetricSynthesisApplicationController->ChangeRedKhiR(vRedKhiR->value());
}
void PolarimetricSynthesisApplicationViewGUI::RedCoPolarizationCallback()
{
  bRedCoPolarization->set();
  bRedCrossPolarization->clear();
  bRedAnyPolarization->clear();
  if (!bVEmission->value() | !bHEmission->value())
  {
    vRedPsiR->deactivate();
    vRedKhiR->deactivate();
  }
  else
  {
    vRedPsiR->activate();
    vRedKhiR->activate();
  }
  m_PolarimetricSynthesisApplicationController->ChangeRedPolarizationMode(CO_POLAR);
}
void PolarimetricSynthesisApplicationViewGUI::RedCrossPolarizationCallback()
{
  bRedCrossPolarization->set();
  bRedCoPolarization->clear();
  bRedAnyPolarization->clear();
  if (!bVEmission->value() | !bHEmission->value())
  {
    vRedPsiR->deactivate();
    vRedKhiR->deactivate();
  }
  else
  {
    vRedPsiR->activate();
    vRedKhiR->activate();
  }
  m_PolarimetricSynthesisApplicationController->ChangeRedPolarizationMode(CROSS_POLAR);
}
void PolarimetricSynthesisApplicationViewGUI::RedAnyPolarizationCallback()
{
  bRedAnyPolarization->set();
  bRedCrossPolarization->clear();
  bRedCoPolarization->clear();
  vRedPsiR->activate();
  vRedKhiR->activate();
  m_PolarimetricSynthesisApplicationController->ChangeRedPolarizationMode(ANY_POLAR);
}

// -------------------------- GREEN  -------------------------- //
void PolarimetricSynthesisApplicationViewGUI::GreenPsiICallback()
{
  m_PolarimetricSynthesisApplicationController->ChangeGreenPsiI(vGreenPsiI->value());
}
void PolarimetricSynthesisApplicationViewGUI::GreenPsiRCallback()
{
  m_PolarimetricSynthesisApplicationController->ChangeGreenPsiR(vGreenPsiR->value());
}
void PolarimetricSynthesisApplicationViewGUI::GreenKhiICallback()
{
  m_PolarimetricSynthesisApplicationController->ChangeGreenKhiI(vGreenKhiI->value());
}
void PolarimetricSynthesisApplicationViewGUI::GreenKhiRCallback()
{
  m_PolarimetricSynthesisApplicationController->ChangeGreenKhiR(vGreenKhiR->value());
}
void PolarimetricSynthesisApplicationViewGUI::GreenCoPolarizationCallback()
{
  bGreenCoPolarization->set();
  bGreenCrossPolarization->clear();
  bGreenAnyPolarization->clear();
  if (!bVEmission->value() | !bHEmission->value())
  {
    vGreenPsiR->deactivate();
    vGreenKhiR->deactivate();
  }
  else
  {
    vGreenPsiR->activate();
    vGreenKhiR->activate();
  }
  m_PolarimetricSynthesisApplicationController->ChangeGreenPolarizationMode(CO_POLAR);
}
void PolarimetricSynthesisApplicationViewGUI::GreenCrossPolarizationCallback()
{
  bGreenCrossPolarization->set();
  bGreenCoPolarization->clear();
  bGreenAnyPolarization->clear();
  if (!bVEmission->value() | !bHEmission->value())
  {
    vGreenPsiR->deactivate();
    vGreenKhiR->deactivate();
  }
  else
  {
    vGreenPsiR->activate();
    vGreenKhiR->activate();
  }
  m_PolarimetricSynthesisApplicationController->ChangeGreenPolarizationMode(CROSS_POLAR);
}
void PolarimetricSynthesisApplicationViewGUI::GreenAnyPolarizationCallback()
{
  bGreenAnyPolarization->set();
  bGreenCrossPolarization->clear();
  bGreenCoPolarization->clear();
  vGreenPsiR->activate();
  vGreenKhiR->activate();
  m_PolarimetricSynthesisApplicationController->ChangeGreenPolarizationMode(ANY_POLAR);
}

// -------------------------- BLUE  -------------------------- //
void PolarimetricSynthesisApplicationViewGUI::BluePsiICallback()
{
  m_PolarimetricSynthesisApplicationController->ChangeBluePsiI(vBluePsiI->value());
}
void PolarimetricSynthesisApplicationViewGUI::BluePsiRCallback()
{
  m_PolarimetricSynthesisApplicationController->ChangeBluePsiR(vBluePsiR->value());
}
void PolarimetricSynthesisApplicationViewGUI::BlueKhiICallback()
{
  m_PolarimetricSynthesisApplicationController->ChangeBlueKhiI(vBlueKhiI->value());
}
void PolarimetricSynthesisApplicationViewGUI::BlueKhiRCallback()
{
  m_PolarimetricSynthesisApplicationController->ChangeBlueKhiR(vBlueKhiR->value());
}
void PolarimetricSynthesisApplicationViewGUI::BlueCoPolarizationCallback()
{
  bBlueCoPolarization->set();
  bBlueCrossPolarization->clear();
  bBlueAnyPolarization->clear();
  if (!bVEmission->value() | !bHEmission->value())
  {
    vBluePsiR->deactivate();
    vBlueKhiR->deactivate();
  }
  else
  {
    vBluePsiR->activate();
    vBlueKhiR->activate();
  }
  m_PolarimetricSynthesisApplicationController->ChangeBluePolarizationMode(CO_POLAR);
}
void PolarimetricSynthesisApplicationViewGUI::BlueCrossPolarizationCallback()
{
  bBlueCrossPolarization->set();
  bBlueCoPolarization->clear();
  bBlueAnyPolarization->clear();
  if (!bVEmission->value() | !bHEmission->value())
  {
    vBluePsiR->deactivate();
    vBlueKhiR->deactivate();
  }
  else
  {
    vBluePsiR->activate();
    vBlueKhiR->activate();
  }
  m_PolarimetricSynthesisApplicationController->ChangeBluePolarizationMode(CROSS_POLAR);
}
void PolarimetricSynthesisApplicationViewGUI::BlueAnyPolarizationCallback()
{
  bBlueAnyPolarization->set();
  bBlueCrossPolarization->clear();
  bBlueCoPolarization->clear();
  vBluePsiR->activate();
  vBlueKhiR->activate();
  m_PolarimetricSynthesisApplicationController->ChangeBluePolarizationMode(ANY_POLAR);
}

// -------------------------- GRAYSCALE  -------------------------- //
void PolarimetricSynthesisApplicationViewGUI::GrayPsiICallback()
{
  m_PolarimetricSynthesisApplicationController->ChangeGrayPsiI(vGrayPsiI->value());
}
void PolarimetricSynthesisApplicationViewGUI::GrayPsiRCallback()
{
  m_PolarimetricSynthesisApplicationController->ChangeGrayPsiR(vGrayPsiR->value());
}
void PolarimetricSynthesisApplicationViewGUI::GrayKhiICallback()
{
  m_PolarimetricSynthesisApplicationController->ChangeGrayKhiI(vGrayKhiI->value());
}
void PolarimetricSynthesisApplicationViewGUI::GrayKhiRCallback()
{
  m_PolarimetricSynthesisApplicationController->ChangeGrayKhiR(vGrayKhiR->value());
}
void PolarimetricSynthesisApplicationViewGUI::GrayCoPolarizationCallback()
{
  bGrayCoPolarization->set();
  bGrayCrossPolarization->clear();
  bGrayAnyPolarization->clear();
  if (!bVEmission->value() | !bHEmission->value())
  {
    vGrayPsiR->deactivate();
    vGrayKhiR->deactivate();
  }
  else
  {
    vGrayPsiR->activate();
    vGrayKhiR->activate();
  }
  m_PolarimetricSynthesisApplicationController->ChangeGrayPolarizationMode(CO_POLAR);
}
void PolarimetricSynthesisApplicationViewGUI::GrayCrossPolarizationCallback()
{
  bGrayCrossPolarization->set();
  bGrayCoPolarization->clear();
  bGrayAnyPolarization->clear();
  if (!bVEmission->value() | !bHEmission->value())
  {
    vGrayPsiR->deactivate();
    vGrayKhiR->deactivate();
  }
  else
  {
    vGrayPsiR->activate();
    vGrayKhiR->activate();
  }
  m_PolarimetricSynthesisApplicationController->ChangeGrayPolarizationMode(CROSS_POLAR);
}
void PolarimetricSynthesisApplicationViewGUI::GrayAnyPolarizationCallback()
{
  bGrayAnyPolarization->set();
  bGrayCrossPolarization->clear();
  bGrayCoPolarization->clear();
  vGrayPsiR->activate();
  vGrayKhiR->activate();
  m_PolarimetricSynthesisApplicationController->ChangeGrayPolarizationMode(ANY_POLAR);
}

}
