/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbPolarimetricSynthesisApplicationViewGUI_h
#define __otbPolarimetricSynthesisApplicationViewGUI_h
// Disabling deprecation warning
#ifdef _MSC_VER
#pragma warning(push)
#pragma warning(disable:4996)
#endif
#include "otbPolarimetricSynthesisApplicationViewGroup.h"
#ifdef _MSC_VER
#pragma warning(pop)
#endif
#include "otbListenerBase.h"
#include "otbPolarimetricSynthesisApplicationModel.h"
#include "otbPolarimetricSynthesisApplicationControllerInterface.h"

#include "otbInteractiveScrollWidget.h"
#include "otbPolarimetricSynthesisFullWidget.h"
#include "otbImageWidgetBoxForm.h"
#include "otbImageToVectorImageCastFilter.h"
#include "otbPoincareSphereWidget.h"

#include "itkObject.h"

#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Tabs.H>

namespace otb
{
/** \class PolarimetricSynthesisApplicationViewGUI
 *
 */
class PolarimetricSynthesisApplicationViewGUI
      : public ListenerBase, public PolarimetricSynthesisApplicationViewGroup, public itk::Object
{
public:
  /** Standard class typedefs */
  typedef PolarimetricSynthesisApplicationViewGUI Self;
  typedef itk::Object                    Superclass;
  typedef itk::SmartPointer<Self>        Pointer;
  typedef itk::SmartPointer<const Self>  ConstPointer;

  // Pixel type
  typedef double PixelType;
  typedef otb::InteractiveScrollWidget<PixelType, PolarimetricSynthesisApplicationControllerInterface>  ScrollWidgetType;
  typedef otb::PolarimetricSynthesisFullWidget<PixelType, PolarimetricSynthesisApplicationControllerInterface>    FullWidgetType;
  typedef FullWidgetType::ImageType                          ImageType;
  typedef ImageType::IndexType                               IndexType;
  typedef ImageType::SizeType                                SizeType;
  typedef ImageType::RegionType                              RegionType;
  typedef ImageWidgetBoxForm                                 BoxType;
  typedef BoxType::ColorType                                 ColorType;
  typedef PolarimetricSynthesisApplicationModel::OutputImageType SingleImageType;
  typedef ImageToVectorImageCastFilter<SingleImageType, ImageType> VectorCastFilterType;
  typedef PolarimetricSynthesisApplicationModel::HistogramType HistogramType;
  /*   typedef PolarimetricSynthesisHistogramWidget<HistogramType, PixelType> HistogramWidgetType; */
  typedef otb::PoincareSphereWidget                          PoincareSphereWidgetType;

  /** Standards macros */
  itkNewMacro(Self);
  itkTypeMacro(PolarimetricSynthesisApplicationViewGUI, Object);

  /** Set the controller */
  virtual void SetPolarimetricSynthesisApplicationController(PolarimetricSynthesisApplicationControllerInterface * controller);

  itkGetObjectMacro(FullWidget, FullWidgetType);

  /** Event from the model */
  virtual void Notify();

  /** Buttons callback */
  virtual void VectorBrowserCallback();
  virtual void HHBrowserCallback();
  virtual void HVBrowserCallback();
  virtual void VHBrowserCallback();
  virtual void VVBrowserCallback();
  virtual void HEmissionCallback();
  virtual void VEmissionCallback();
  virtual void OpenVectorImageCallback();
  virtual void OpenImagesCallback();
  virtual void OpenImageOkCallback();
  virtual void OpenImageCancelCallback();
  virtual void OpenCallback();
  virtual void SaveResultCallback();
  virtual void QuitCallback();
  virtual void DialGainCallback();
  virtual void InputGainCallback();

  virtual void RGBButtonCallback();
  virtual void RGBGroupCallback();
  virtual void GrayGroupCallback();

  virtual void RedPsiICallback();
  virtual void RedPsiRCallback();
  virtual void RedKhiICallback();
  virtual void RedKhiRCallback();
  virtual void RedCoPolarizationCallback();
  virtual void RedCrossPolarizationCallback();
  virtual void RedAnyPolarizationCallback();

  virtual void GreenPsiICallback();
  virtual void GreenPsiRCallback();
  virtual void GreenKhiICallback();
  virtual void GreenKhiRCallback();
  virtual void GreenCoPolarizationCallback();
  virtual void GreenCrossPolarizationCallback();
  virtual void GreenAnyPolarizationCallback();

  virtual void BluePsiICallback();
  virtual void BluePsiRCallback();
  virtual void BlueKhiICallback();
  virtual void BlueKhiRCallback();
  virtual void BlueCoPolarizationCallback();
  virtual void BlueCrossPolarizationCallback();
  virtual void BlueAnyPolarizationCallback();

  virtual void GrayPsiICallback();
  virtual void GrayPsiRCallback();
  virtual void GrayKhiICallback();
  virtual void GrayKhiRCallback();
  virtual void GrayCoPolarizationCallback();
  virtual void GrayCrossPolarizationCallback();
  virtual void GrayAnyPolarizationCallback();


protected:
  /** Constructor */
  PolarimetricSynthesisApplicationViewGUI();
  /** Destructor */
  ~PolarimetricSynthesisApplicationViewGUI() {};

  /** Refresh the image widgets */
  virtual void RefreshViewer();
  virtual void UpdateInterface();

private:
  PolarimetricSynthesisApplicationViewGUI(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

  /** Pointer to the model */
  PolarimetricSynthesisApplicationModel::Pointer m_PolarimetricSynthesisApplicationModel;
  /** Pointer to the controller */
  PolarimetricSynthesisApplicationControllerInterface::Pointer m_PolarimetricSynthesisApplicationController;

  ColorType m_InterfaceColor;

  /// Vector cast filters
  VectorCastFilterType::Pointer m_VectorCastFilter;
  VectorCastFilterType::Pointer m_QuicklookVectorCastFilter;
  ScrollWidgetType::Pointer     m_ScrollWidget;
  FullWidgetType::Pointer       m_FullWidget;
  PoincareSphereWidgetType::Pointer m_PoincareSphereWidget;
  /*   HistogramWidgetType::Pointer  m_HistogramWidget; */

};
}//end namespace otb

#endif
