/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbRoadExtractionController_h
#define __otbRoadExtractionController_h

#include "otbRoadExtractionControllerInterface.h"
#include "otbRoadExtractionViewGUI.h"

namespace otb
{
class RoadExtractionController
      : public RoadExtractionControllerInterface
{
public:
  /** Standard class typedefs */
  typedef RoadExtractionController Self;
  typedef itk::Object                       Superclass;
  typedef itk::SmartPointer<Self>           Pointer;
  typedef itk::SmartPointer<const Self>     ConstPointer;

  typedef otb::RoadExtractionViewGUI ViewType;
  typedef ViewType::Pointer                   ViewPointerType;

  /** Standard type macros */
  itkTypeMacro(RoadExtractionController, Superclass);
  itkNewMacro(Self);

  /** Set the view */
  virtual void SetView(ViewPointerType view);

  /** User action */
  virtual void ViewRegionChanged();
  virtual void PixelClicked(const IndexType & index);
  virtual void SelectFileName(std::string filename);
  virtual void ChangeChannel(unsigned int channel);
  virtual void ChangeRedChannel(unsigned int channel);
  virtual void ChangeGreenChannel(unsigned int channel);
  virtual void ChangeUseImageIntensity();
  virtual void ChangeUseImageChannel();
  virtual void ChangeUseSpectralAngle();
  virtual void ChangeUseWaterIndex();
  virtual void SaveResults(std::string filename);
  virtual void ImportSegments(std::string filename);
  virtual void ClearSegments();

  virtual void ChangeAlpha(double alpha);
  virtual void ChangeResolution(double resolution);
  virtual void ChangeTolerance(double tolerance );
  virtual void ChangeMaxAngle(double maxAngle);
  virtual void ChangeAngularThreshold(double angularThreshold);
  virtual void ChangeAmplitudeThreshold(double amplitudeThreshold);
  virtual void ChangeDistanceThreshold(double distanceThreshold);
  virtual void ChangeFirstMeanDistanceThreshold(double firstMeanDistanceThreshold);
  virtual void ChangeSecondMeanDistanceThreshold(double secondMeanDistanceThreshold);

  virtual void Run();


protected:
  /** Constructor */
  RoadExtractionController();
  /** Destructor */
  ~RoadExtractionController();

private:
  RoadExtractionController(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

  /** Pointer to the view */
  ViewPointerType m_View;

  bool m_Computing;
};
} //end namespace otb

#endif
