/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbRoadExtractionControllerInterface_h
#define __otbRoadExtractionControllerInterface_h

#include "itkObject.h"
#include "otbRoadExtractionModel.h"
#include "otbImageViewerFullResolutionEventsInterface.h"

namespace otb
{
class RoadExtractionControllerInterface
      : public ImageViewerFullResolutionEventsInterface
{
public:
  /** Standard class typedefs */
  typedef RoadExtractionControllerInterface Self;
  typedef itk::Object                      Superclass;
  typedef itk::SmartPointer<Self>          Pointer;
  typedef itk::SmartPointer<const Self>    ConstPointer;

  typedef RoadExtractionModel::RegionType RegionType;

  /** Standard type macros */
  itkTypeMacro(RoadExtractionControllerInterface, Superclass);

  /** Users actions */
  virtual void ViewRegionChanged() = 0;
  virtual void PixelClicked(const IndexType & index) = 0;
  virtual void SelectFileName(std::string filename) = 0;
  virtual void ChangeChannel(unsigned int channel)=0;
  virtual void ChangeRedChannel(unsigned int channel)=0;
  virtual void ChangeGreenChannel(unsigned int channel)=0;
  virtual void ChangeUseImageIntensity()=0;
  virtual void ChangeUseImageChannel()=0;
  virtual void ChangeUseSpectralAngle()=0;
  virtual void ChangeUseWaterIndex()=0;
  virtual void SaveResults(std::string filename)=0;
  virtual void ImportSegments(std::string filename)=0;
  virtual void ClearSegments()=0;
  virtual void Run()=0;

  virtual void ChangeAlpha(double alpha)=0;
  virtual void ChangeResolution(double resolution)=0;
  virtual void ChangeTolerance(double tolerance)=0;
  virtual void ChangeMaxAngle(double maxAngle)=0;
  virtual void ChangeAngularThreshold(double angularThreshold)=0;
  virtual void ChangeAmplitudeThreshold(double amplitudeThreshold)=0;
  virtual void ChangeDistanceThreshold(double distanceThreshold)=0;
  virtual void ChangeFirstMeanDistanceThreshold(double firstMeanDistanceThreshold)=0;
  virtual void ChangeSecondMeanDistanceThreshold(double secondMeanDistanceThreshold)=0;


protected:
  /** Constructor */
  RoadExtractionControllerInterface()
  {
    m_Model = RoadExtractionModel::GetInstance();
  }
  /** Destructor */
  ~RoadExtractionControllerInterface() {}

  /** The instance of the model (for commodity) */
  RoadExtractionModel * m_Model;

private:
  RoadExtractionControllerInterface(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented
};
} // end namespace otb

#endif
