/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbRoadExtractionViewGUI_h
#define __otbRoadExtractionViewGUI_h

// Disabling deprecation warning
#ifdef _MSC_VER
#pragma warning(push)
#pragma warning(disable:4996)
#endif
#include "otbRoadExtractionViewGroup.h"
#ifdef _MSC_VER
#pragma warning(pop)
#endif
#include "otbListenerBase.h"
#include "otbRoadExtractionModel.h"
#include "otbRoadExtractionControllerInterface.h"
#include "otbImageViewerBase.h"
#include "itkObject.h"

#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Tabs.H>

#include "otbVisualizationMacro.h"

namespace otb
{
/** \class RoadExtractionViewGUI
 *
 */
class RoadExtractionViewGUI
      : public ListenerBase, public RoadExtractionViewGroup, public itk::Object
{
public:
  /** Standard class typedefs */
  typedef RoadExtractionViewGUI Self;
  typedef itk::Object                    Superclass;
  typedef itk::SmartPointer<Self>        Pointer;
  typedef itk::SmartPointer<const Self>  ConstPointer;

  // Pixel type
  typedef double PixelType;
  typedef otb::ImageViewerBase<PixelType>       ImageViewerBaseType;
  typedef ImageViewerBaseType::Pointer          ImageViewerBasePointerType;
  typedef ImageViewerBaseType::FullWidgetType   FullWidgetType;
  typedef FullWidgetType::Pointer               FullWidgetPointerType;
  typedef FullWidgetType::ColorType             ColorType;
  typedef ImageViewerBaseType::ScrollWidgetType ScrollWidgetType;
  typedef ScrollWidgetType::Pointer             ScrollWidgetPointerType;
  typedef ImageViewerBaseType::SizeType         SizeType;

  /** Standards macros */
  itkNewMacro(Self);
  itkTypeMacro(RoadExtractionViewGUI, Object);

  /** Set the controller */
  virtual void SetRoadExtractionController(RoadExtractionControllerInterface * controller);

  /** Event from the model */
  virtual void Notify();

  /** Buttons callback */
  virtual void OpenImageCallback();
  virtual void QuitCallback();
  virtual void UseImageIntensityCallback();
  virtual void UseImageChannelCallback();
  virtual void UseSpectralAngleCallback();
  virtual void UseWaterIndexCallback();
  virtual void ChannelCallback();
  virtual void RedChannelCallback();
  virtual void GreenChannelCallback();
  virtual void SaveResultsCallback();
  virtual void ImportSegmentsCallback();
  virtual void ClearCallback();
  virtual void RunCallback();
  virtual void UpdateInterface();

  virtual void InputAlphaCallback();
  virtual void InputResolutionCallback();
  virtual void InputToleranceCallback();
  virtual void InputMaxAngleCallback();
  virtual void InputAngularThresholdCallback();
  virtual void InputAmplitudeThresholdCallback();
  virtual void InputDistanceThresholdCallback();
  virtual void InputFirstMeanDistanceThresholdCallback();
  virtual void InputSecondMeanDistanceThresholdCallback();

  /** Init the IHM values */
  virtual void InitViewValues();

  itkGetObjectMacro(Viewer, ImageViewerBaseType);

  /** Standard Running macro */
  otbRunningMacro();

protected:
  /** Constructor */
  RoadExtractionViewGUI();
  /** Destructor */
  ~RoadExtractionViewGUI() {};

  /** Refresh the image widgets */
  virtual void RefreshViewer();

private:
  RoadExtractionViewGUI(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

  /** Pointer to the model */
  RoadExtractionModel::Pointer m_RoadExtractionModel;
  /** Pointer to the controller */
  RoadExtractionControllerInterface::Pointer m_RoadExtractionController;
  /** Pointer to the viewer */
  ImageViewerBasePointerType m_Viewer;
  /** colors */
  ColorType m_Color;
};
}//end namespace otb

#endif
