/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
  See OTBCopyright.txt for details.


     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#include <stdlib.h>
#include <iostream>
#include "itkExceptionObject.h"
#include "otbCommandLineArgumentParser.h"
#include "otbImageFileReader.h"
#include "otbRoadExtractionViewer.h"
#include "otbMacro.h"

int main( int argc, char * argv[] )
{
  try
  {
    // Parse command line parameters
    typedef otb::CommandLineArgumentParser ParserType;
    ParserType::Pointer parser = ParserType::New();
    parser->AddInputImage();
    parser->AddOption("--Alpha","Set the alpha parameter of the detection algorithm. Default is 1.2.","-a", 1, false);

    typedef otb::CommandLineArgumentParseResult ParserResultType;
    ParserResultType::Pointer  parseResult = ParserResultType::New();

    try
    {
      parser->ParseCommandLine(argc, argv, parseResult);
    }
    catch ( itk::ExceptionObject & err )
    {
      std::string descriptionException = err.GetDescription();
      if (descriptionException.find("ParseCommandLine(): Help Parser") != std::string::npos)
      {
        return EXIT_SUCCESS;
      }
      if (descriptionException.find("ParseCommandLine(): Version Parser") != std::string::npos)
      {
        return EXIT_SUCCESS;
      }
      return EXIT_FAILURE;
    }

    typedef double PixelType;
    typedef otb::RoadExtractionViewer<PixelType>  RoadExtractionViewerType;
    typedef RoadExtractionViewerType::ImageType ImageType;
    typedef otb::ImageFileReader<ImageType> ReaderType;

    // instantiation
    RoadExtractionViewerType::Pointer viewer = RoadExtractionViewerType::New();

    // check for input images
    ReaderType::Pointer reader = ReaderType::New();
    reader->SetFileName(parseResult->GetInputImage().c_str());
    reader->Update();

    viewer->SetInputImage(reader->GetOutput());

    // Alpha parameter
    if (parseResult->IsOptionPresent("--Alpha"))
      viewer->SetAlpha(parseResult->GetParameterDouble("--Alpha"));

    // build the app
    viewer->Show();

    // if not testing the app
    if (!parseResult->IsOptionOTBTestingPresent())
    {
      otbGenericMsgDebugMacro(<<"Running GUI ...");
      return Fl::run();
    }
  }
  catch ( itk::ExceptionObject & err )
  {
    std::cout << "Following otbException catch :" << std::endl;
    std::cout << err << std::endl;
    return EXIT_FAILURE;
  }
  catch ( std::bad_alloc & err )
  {
    std::cout << "Exception bad_alloc : "<<(char*)err.what()<< std::endl;
    return EXIT_FAILURE;
  }
  catch ( ... )
  {
    std::cout << "Unknown Exception found !" << std::endl;
    return EXIT_FAILURE;
  }
  return EXIT_SUCCESS;
}
