#include "itkExceptionObject.h"
#include "otbRoadExtractionModel.h"
#include "otbRoadExtractionViewGUI.h"
#include "otbRoadExtractionController.h"
#include "otbCommandLineArgumentParser.h"
#include "otbMsgReporter.h"
#include <FL/Fl.H>

#include "otbI18n.h"

using namespace otb;

int main(int argc, char* argv[])
{
  otbI18nMacro();

  // Parse command line parameters
  typedef otb::CommandLineArgumentParser ParserType;
  ParserType::Pointer parser = ParserType::New();

  parser->AddInputImage(false); //Optionnal parameter

  typedef otb::CommandLineArgumentParseResult ParserResultType;
  ParserResultType::Pointer  parseResult = ParserResultType::New();

  typedef otb::MsgReporter ReporterType;
  try
  {
    parser->ParseCommandLine(argc, argv, parseResult);
  }
  catch ( itk::ExceptionObject & err )
  {
    std::string descriptionException = err.GetDescription();
    if (descriptionException.find("ParseCommandLine(): Help Parser") != std::string::npos)
    {
      return EXIT_SUCCESS;
    }
    if (descriptionException.find("ParseCommandLine(): Version Parser") != std::string::npos)
    {
      return EXIT_SUCCESS;
    }
    return EXIT_FAILURE;
  }


  RoadExtractionModel::Pointer model = RoadExtractionModel::GetInstance();
  RoadExtractionController::Pointer controller = RoadExtractionController::New();
  RoadExtractionViewGUI::Pointer view = RoadExtractionViewGUI::New();
  otb::MsgReporter::GetInstance()->SetTitle("RoadExtraction application");

  view->SetRoadExtractionController(controller);
  controller->SetView(view);
  if ( parseResult->IsOptionInputImagePresent() )
    controller->SelectFileName(parseResult->GetInputImage().c_str());

  if (!parseResult->IsOptionOTBTestingPresent())
    otbFlRunMacro(view->IsRunning());

  return EXIT_SUCCESS;
}
