/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbRoadExtractionViewerFullWidget_h
#define __otbRoadExtractionViewerFullWidget_h

#include "otbFullResolutionImageWidget.h"
#include <FL/Fl.H>

namespace otb
{
template <class TPixel> class RoadExtractionViewer;

/**
 * \brief Custom scroll image widget.
 *
 * This class derives from otb::FullResolutionImageWidget, and implements behaviours specific to the road extraction viewer.
 *
 * \sa RoadExtractionViewer, FullResolutionImageWidget
 *
 */
template <class TPixel>
class ITK_EXPORT RoadExtractionViewerFullWidget
      : public FullResolutionImageWidget<TPixel>
{
public:
  /** Standard typedefs */
  typedef RoadExtractionViewerFullWidget    Self;
  typedef FullResolutionImageWidget<TPixel> Superclass;
  typedef itk::SmartPointer<Self>           Pointer;
  typedef itk::SmartPointer<const Self>     ConstPointer;

  typedef TPixel PixelType;
  typedef typename Superclass::IndexType  IndexType;
  typedef typename Superclass::SizeType   SizeType;
  typedef RoadExtractionViewer<PixelType> ParentType;
  typedef ParentType*                     ParentPointerType;

  /** Type macro */
  itkNewMacro(Self);

  /** Creation through object factory macro */
  itkTypeMacro(RoadExtractionViewerFullWidget, FullResolutionImageWidget);

  itkSetMacro(Parent, ParentPointerType);
  itkGetMacro(Parent, ParentPointerType);

protected:
  /** Constructor */
  RoadExtractionViewerFullWidget() {};
  /** Destructor */
  virtual ~RoadExtractionViewerFullWidget()
  {
    m_Parent = NULL;
  };

  /** handle FLTK event */
  virtual int handle(int event)
  {
    switch (event)
    {
    case FL_PUSH:
    {
      int x = Fl::event_x();
      int y = Fl::event_y();
      IndexType clickedIndex;
      clickedIndex[0]=x;
      clickedIndex[1]=y;
      IndexType newIndex = this->WindowToImageCoordinates(clickedIndex);
      m_Parent->RoadSelected(newIndex);
      return 1;
    }
    }
    return 0;
  }
private:
  RoadExtractionViewerFullWidget(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented
  ParentPointerType m_Parent;
};
}// End namespace otb
#endif
