/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbRoadExtractionViewerScrollWidget_h
#define __otbRoadExtractionViewerScrollWidget_h

#include "otbFixedSizeFullImageWidget.h"
#include "otbImageWidgetRectangleForm.h"
#include "otbImageWidgetBoxForm.h"
#include "otbImageWidgetPointForm.h"
#include <FL/Fl.H>

namespace otb
{
template <class TPixel> class RoadExtractionViewer;

/**
 * \brief Custom scroll image widget.
 *
 * This class derives from otb::FixedSizeFullImageWidget, and implements behaviours specific to the road extraction viewer.
 *
 * \sa RoadExtractionViewer, FixedSizeFullImageWidget
 *
 */
template <class TPixel>
class ITK_EXPORT RoadExtractionViewerScrollWidget
      : public FixedSizeFullImageWidget<TPixel>
{
public:
  /** Standard class typedefs */
  typedef RoadExtractionViewerScrollWidget   Self;
  typedef FixedSizeFullImageWidget<TPixel>   Superclass;
  typedef itk::SmartPointer<Self>            Pointer;
  typedef itk::SmartPointer<const Self>      ConstPointer;

  /** Method for creation through the object factory. */
  itkNewMacro(Self);

  /** Run-time type information (and related methods). */
  itkTypeMacro(RoadExtractionViewerScrollWidget, FixedSizeFullImageWidget);

  typedef TPixel                          PixelType;
  typedef typename Superclass::IndexType  IndexType;
  typedef typename Superclass::SizeType   SizeType;
  typedef RoadExtractionViewer<PixelType> ParentType;
  typedef ParentType*                     ParentPointerType;
  typedef otb::ImageWidgetBoxForm         BoxType;
  typedef BoxType::ColorType              ColorType;

  itkSetMacro(Parent, ParentPointerType);
  itkGetMacro(Parent, ParentPointerType);

  /** Handle method */
  virtual int handle(int event)
  {
    switch (event)
    {
    case FL_PUSH:
    {
      int x = Fl::event_x();
      int y = Fl::event_y();
      IndexType clickedIndex;
      clickedIndex[0]=x;
      clickedIndex[1]=y;
      clickedIndex=this->WindowToImageCoordinates(clickedIndex);
      m_Parent->ChangeFullViewedRegion(clickedIndex);
      return 1;
    }
    }
    return 0;
  }
protected:
  /**
   * Constructor.
   */
  RoadExtractionViewerScrollWidget()
  {};
  /**
   * Destructor.
   */
  ~RoadExtractionViewerScrollWidget()
  {
    m_Parent = NULL;
  }

private:
  ParentPointerType m_Parent;
};
} // end namespace otb
#endif
