/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbLevelSetSegmentationController_h
#define __otbLevelSetSegmentationController_h

#include "otbLevelSetSegmentationControllerInterface.h"
#include "otbLevelSetSegmentationViewGUI.h"

namespace otb
{
class LevelSetSegmentationController
      : public LevelSetSegmentationControllerInterface
{
public:
  /** Standard class typedefs */
  typedef LevelSetSegmentationController          Self;
  typedef itk::Object                      Superclass;
  typedef itk::SmartPointer<Self>          Pointer;
  typedef itk::SmartPointer<const Self>    ConstPointer;

  typedef otb::LevelSetSegmentationViewGUI ViewType;
  typedef ViewType::Pointer         ViewPointerType;

  /** Standard type macros */
  itkTypeMacro(LevelSetSegmentationController, Superclass);
  itkNewMacro(Self);

  itkSetObjectMacro(View, ViewType);

  virtual void ChangeSigma(double sigma);
  virtual void ChangeSigmoidAlpha(double alpha);
  virtual void ChangeSigmoidBeta(double beta);
  virtual void ChangeTimeThreshold(double timeThreshold);
  virtual void ChangeStoppingTime(double stoppingTime);
  virtual void ClearSeeds();
  virtual void SeedSelected(IndexType& seed);

protected:
  /** Constructor */
  LevelSetSegmentationController();
  /** Destructor */
  ~LevelSetSegmentationController();

private:
  LevelSetSegmentationController(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

  /** Pointer to the view */
  ViewPointerType m_View;

};
} //end namespace otb

#endif
