/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbLevelSetSegmentationControllerInterface_h
#define __otbLevelSetSegmentationControllerInterface_h

#include "itkObject.h"
#include "otbLevelSetSegmentationModel.h"

namespace otb
{
class LevelSetSegmentationControllerInterface
      : public itk::Object
{
public:
  /** Standard class typedefs */
  typedef LevelSetSegmentationControllerInterface Self;
  typedef itk::Object                      Superclass;
  typedef itk::SmartPointer<Self>          Pointer;
  typedef itk::SmartPointer<const Self>    ConstPointer;

  typedef LevelSetSegmentationModel ModelType;
  typedef ModelType::IndexType      IndexType;

  /** Standard type macros */
  itkTypeMacro(LevelSetSegmentationControllerInterface, Superclass);

  /** Users actions */
  virtual void ChangeSigma(double sigma) =0;
  virtual void ChangeSigmoidAlpha(double alpha)=0;
  virtual void ChangeSigmoidBeta(double beta)=0;
  virtual void ChangeTimeThreshold(double timeThreshold)=0;
  virtual void ChangeStoppingTime(double stoppingTime)=0;
  virtual void ClearSeeds()=0;
  virtual void SeedSelected(IndexType& seed)=0;
  // not implemented since no pixel reporting is needed for levelset segmentation
  virtual void PixelHovered(IndexType& index) {};
  virtual void PixelDragged(IndexType& index) {};
  virtual void MouseIn() {};
  virtual void MouseOut() {};
  virtual void MouseReleased() {};

protected:
  /** Constructor */
  LevelSetSegmentationControllerInterface()
  {
    m_Model = LevelSetSegmentationModel::GetInstance();
  }
  /** Destructor */
  ~LevelSetSegmentationControllerInterface() {}

  /** The instance of the model (for commodity) */
  LevelSetSegmentationModel * m_Model;

private:
  LevelSetSegmentationControllerInterface(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented
};
} // end namespace otb

#endif
