/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbPreprocessingController_h
#define __otbPreprocessingController_h

#include "otbPreprocessingControllerInterface.h"
#include "otbPreprocessingViewGUI.h"

namespace otb
{
class PreprocessingController
      : public PreprocessingControllerInterface
{
public:
  /** Standard class typedefs */
  typedef PreprocessingController          Self;
  typedef itk::Object                      Superclass;
  typedef itk::SmartPointer<Self>          Pointer;
  typedef itk::SmartPointer<const Self>    ConstPointer;

  typedef otb::PreprocessingViewGUI ViewType;
  typedef ViewType::Pointer         ViewPointerType;

  /** Standard type macros */
  itkTypeMacro(PreprocessingController, Superclass);
  itkNewMacro(Self);

  itkSetObjectMacro(View, ViewType);

  virtual void ChangeNumberOfIterations(unsigned int numberOfIterations);
  virtual void ChangeTimeStep(double timeStep);
  virtual void ChangeAlpha(double alpha);
  virtual void EnableSmoothing();
  virtual void DisableSmoothing();
  virtual void EnableEdgeEnhancement();
  virtual void DisableEdgeEnhancement();

protected:
  /** Constructor */
  PreprocessingController();
  /** Destructor */
  ~PreprocessingController();

private:
  PreprocessingController(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

  /** Pointer to the view */
  ViewPointerType m_View;

};
} //end namespace otb

#endif
