/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbPreprocessingControllerInterface_h
#define __otbPreprocessingControllerInterface_h

#include "itkObject.h"
#include "otbPreprocessingModel.h"

namespace otb
{
class PreprocessingControllerInterface
      : public itk::Object
{
public:
  /** Standard class typedefs */
  typedef PreprocessingControllerInterface Self;
  typedef itk::Object                      Superclass;
  typedef itk::SmartPointer<Self>          Pointer;
  typedef itk::SmartPointer<const Self>    ConstPointer;

  /** Standard type macros */
  itkTypeMacro(PreprocessingControllerInterface, Superclass);

  /** Users actions */
  virtual void ChangeNumberOfIterations(unsigned int numberOfIterations) =0;
  virtual void ChangeTimeStep(double timeStep)=0;
  virtual void ChangeAlpha(double alpha)=0;
  virtual void EnableSmoothing()=0;
  virtual void DisableSmoothing()=0;
  virtual void EnableEdgeEnhancement()=0;
  virtual void DisableEdgeEnhancement()=0;

protected:
  /** Constructor */
  PreprocessingControllerInterface()
  {
    m_Model = PreprocessingModel::GetInstance();
  }
  /** Destructor */
  ~PreprocessingControllerInterface() {}

  /** The instance of the model (for commodity) */
  PreprocessingModel * m_Model;

private:
  PreprocessingControllerInterface(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented
};
} // end namespace otb

#endif
