/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include "otbRegionGrowingSegmentationController.h"

namespace otb
{

RegionGrowingSegmentationController::RegionGrowingSegmentationController()
{}

RegionGrowingSegmentationController::~RegionGrowingSegmentationController()
{}

void RegionGrowingSegmentationController::ChangeUpperThreshold(double thresh)
{
  m_Model->SetUpperThreshold(thresh);
}

void RegionGrowingSegmentationController::ChangeLowerThreshold(double thresh)
{
  m_Model->SetLowerThreshold(thresh);
}

void RegionGrowingSegmentationController::ChangeRadius(unsigned int radius)
{
  m_Model->SetRadius(radius);
}

void RegionGrowingSegmentationController::ChangeNumberOfIterations(unsigned int num)
{
  m_Model->SetNumberOfIterations(num);
}

void RegionGrowingSegmentationController::ChangeTolerance(double tolerance)
{
  m_Model->SetTolerance(tolerance);
}

void RegionGrowingSegmentationController::ClearSeeds()
{
  m_Model->ClearSeeds();
}

void RegionGrowingSegmentationController::SeedSelected(IndexType & seed)
{
  if (m_Model->GetRegionGrowingAlgorithm() == ISOLATED_CONNECTED)
  {
    if (m_View->vInsideSeeds->value())
    {
      m_Model->AddInsideSeed(seed);
    }
    else
    {
      m_Model->AddOutsideSeed(seed);
    }
  }
  else
  {
    m_Model->AddInsideSeed(seed);
  }
}

void RegionGrowingSegmentationController::PixelHovered(IndexType & index)
{
  m_View->ReportPixel(index);
}


void RegionGrowingSegmentationController::MouseIn()
{
  m_View->oOutputPixel->activate();
}

void RegionGrowingSegmentationController::MouseOut()
{
  m_View->oOutputPixel->deactivate();
}

void RegionGrowingSegmentationController::ChangeAlgorithm(RegionGrowingAlgorithm algorithm)
{
  switch (algorithm)
  {
  case THRESHOLD_CONNECTED:
    m_View->vLowerThreshold->activate();
    m_View->vLowerThreshold->show();
    m_View->vUpperThreshold->activate();
    m_View->vUpperThreshold->show();
    m_View->vTolerance->deactivate();
    m_View->vTolerance->hide();
    m_View->vNumberOfIterations->deactivate();
    m_View->vNumberOfIterations->hide();
    m_View->vRadius->deactivate();
    m_View->vRadius->hide();
    m_View->vInsideSeeds->deactivate();
    m_View->vInsideSeeds->hide();
    m_View->vOutsideSeeds->deactivate();
    m_View->vOutsideSeeds->hide();
    break;
  case NEIGHBORHOOD_CONNECTED:
    m_View->vLowerThreshold->activate();
    m_View->vLowerThreshold->show();
    m_View->vUpperThreshold->activate();
    m_View->vUpperThreshold->show();
    m_View->vTolerance->deactivate();
    m_View->vTolerance->hide();
    m_View->vNumberOfIterations->deactivate();
    m_View->vNumberOfIterations->hide();
    m_View->vRadius->activate();
    m_View->vRadius->show();
    m_View->vInsideSeeds->deactivate();
    m_View->vInsideSeeds->hide();
    m_View->vOutsideSeeds->deactivate();
    m_View->vOutsideSeeds->hide();
    break;
  case CONFIDENCE_CONNECTED:
    m_View->vLowerThreshold->deactivate();
    m_View->vLowerThreshold->hide();
    m_View->vUpperThreshold->deactivate();
    m_View->vUpperThreshold->hide();
    m_View->vTolerance->activate();
    m_View->vTolerance->show();
    m_View->vNumberOfIterations->activate();
    m_View->vNumberOfIterations->show();
    m_View->vRadius->activate();
    m_View->vRadius->show();
    m_View->vInsideSeeds->deactivate();
    m_View->vInsideSeeds->hide();
    m_View->vOutsideSeeds->deactivate();
    m_View->vOutsideSeeds->hide();
    break;
  case ISOLATED_CONNECTED:
    m_View->vLowerThreshold->activate();
    m_View->vLowerThreshold->show();
    m_View->vUpperThreshold->deactivate();
    m_View->vUpperThreshold->hide();
    m_View->vTolerance->deactivate();
    m_View->vTolerance->hide();
    m_View->vNumberOfIterations->deactivate();
    m_View->vNumberOfIterations->hide();
    m_View->vRadius->deactivate();
    m_View->vRadius->hide();
    m_View->vInsideSeeds->activate();
    m_View->vInsideSeeds->show();
    m_View->vOutsideSeeds->activate();
    m_View->vOutsideSeeds->show();
    break;
  }
  m_Model->SetRegionGrowingAlgorithm(algorithm);
}

void RegionGrowingSegmentationController::SelectInsideSeeds()
{
  m_View->vOutsideSeeds->clear();
}

void RegionGrowingSegmentationController::SelectOutsideSeeds()
{
  m_View->vInsideSeeds->clear();
}
} // end namespace otb
