/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


this software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbRegionGrowingSegmentationController_h
#define __otbRegionGrowingSegmentationController_h

#include "otbRegionGrowingSegmentationControllerInterface.h"
#include "otbRegionGrowingSegmentationViewGUI.h"

namespace otb
{
class RegionGrowingSegmentationController
      : public RegionGrowingSegmentationControllerInterface
{
public:
  /** Standard class typedefs */
  typedef RegionGrowingSegmentationController          Self;
  typedef itk::Object                      Superclass;
  typedef itk::SmartPointer<Self>          Pointer;
  typedef itk::SmartPointer<const Self>    ConstPointer;

  typedef otb::RegionGrowingSegmentationViewGUI ViewType;
  typedef ViewType::Pointer         ViewPointerType;

  /** Standard type macros */
  itkTypeMacro(RegionGrowingSegmentationController, Superclass);
  itkNewMacro(Self);

  itkSetObjectMacro(View, ViewType);

  /** Users actions */
  virtual void ChangeUpperThreshold(double thresh);
  virtual void ChangeLowerThreshold(double thresh);
  virtual void ChangeRadius(unsigned int radius);
  virtual void ChangeNumberOfIterations(unsigned int num);
  virtual void ChangeTolerance(double tolerance);
  virtual void ClearSeeds();
  virtual void SeedSelected(IndexType& seed);
  virtual void PixelHovered(IndexType& index);
  virtual void MouseIn();
  virtual void MouseOut();
  virtual void SelectInsideSeeds();
  virtual void SelectOutsideSeeds();
  virtual void ChangeAlgorithm(RegionGrowingAlgorithm algorithm);

protected:
  /** Constructor */
  RegionGrowingSegmentationController();
  /** Destructor */
  ~RegionGrowingSegmentationController();

private:
  RegionGrowingSegmentationController(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

  /** Pointer to the view */
  ViewPointerType m_View;

};
} //end namespace otb

#endif
