/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbRegionGrowingSegmentationControllerInterface_h
#define __otbRegionGrowingSegmentationControllerInterface_h

#include "itkObject.h"
#include "otbRegionGrowingSegmentationModel.h"

namespace otb
{
class RegionGrowingSegmentationControllerInterface
      : public itk::Object
{
public:
  /** Standard class typedefs */
  typedef RegionGrowingSegmentationControllerInterface Self;
  typedef itk::Object                      Superclass;
  typedef itk::SmartPointer<Self>          Pointer;
  typedef itk::SmartPointer<const Self>    ConstPointer;

  typedef RegionGrowingSegmentationModel ModelType;
  typedef ModelType::IndexType      IndexType;

  /** Standard type macros */
  itkTypeMacro(RegionGrowingSegmentationControllerInterface, Superclass);

  /** Users actions */
  virtual void ChangeUpperThreshold(double thresh) =0;
  virtual void ChangeLowerThreshold(double thresh)=0;
  virtual void ChangeRadius(unsigned int radius)=0;
  virtual void ChangeNumberOfIterations(unsigned int num)=0;
  virtual void ChangeTolerance(double tolerance)=0;
  virtual void ClearSeeds()=0;
  virtual void SeedSelected(IndexType& seed)=0;
  virtual void PixelHovered(IndexType& index)=0;
  virtual void PixelDragged(IndexType& index) {};
  virtual void MouseIn()=0;
  virtual void MouseOut()=0;
  virtual void MouseReleased() {};
  virtual void SelectInsideSeeds()=0;
  virtual void SelectOutsideSeeds()=0;
  virtual void ChangeAlgorithm(RegionGrowingAlgorithm algorithm)=0;

protected:
  /** Constructor */
  RegionGrowingSegmentationControllerInterface()
  {
    m_Model = RegionGrowingSegmentationModel::GetInstance();
  }
  /** Destructor */
  ~RegionGrowingSegmentationControllerInterface() {}

  /** The instance of the model (for commodity) */
  RegionGrowingSegmentationModel * m_Model;

private:
  RegionGrowingSegmentationControllerInterface(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented
};
} // end namespace otb

#endif
