/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include "otbSVMBasedRegionGrowingSegmentationController.h"

namespace otb
{

SVMBasedRegionGrowingSegmentationController::SVMBasedRegionGrowingSegmentationController()
{}

SVMBasedRegionGrowingSegmentationController::~SVMBasedRegionGrowingSegmentationController()
{}

void SVMBasedRegionGrowingSegmentationController::ClearSeeds()
{
  m_Model->ClearSeeds();
  m_Model->Update();
}

void SVMBasedRegionGrowingSegmentationController::ChangeNumberOfSads(unsigned int nb)
{
  m_Model->SetNumberOfSpectralAngleDistanceImages(nb);

  if (m_View->bAutomaticUpdate->value())
  {
    m_Model->Update();
  }
}

void SVMBasedRegionGrowingSegmentationController::ChangeNumberOfThresholds(unsigned int nb)
{
  m_Model->SetNumberOfThresholds(nb);

  if (m_View->bAutomaticUpdate->value())
  {
    m_Model->Update();
  }
}

void SVMBasedRegionGrowingSegmentationController::PixelDragged(IndexType & index)
{
  SVMBasedRegionGrowingSegmentationModel::InputPSType::PointType point;
  point[0]=index[0];
  point[1]=index[1];

  if (m_View->bInsideSeeds->value())
  {

    m_Model->AddInsidePoint(point);
  }
  else
  {
    m_Model->AddOutsidePoint(point);
  }
}

void SVMBasedRegionGrowingSegmentationController::MouseReleased()
{
  if (m_View->bAutomaticUpdate->value())
  {
    m_Model->Update();
  }
  else
  {
    m_View->Notify();
  }
}

void SVMBasedRegionGrowingSegmentationController::ManualUpdate()
{
  m_Model->Update();
}

} // end namespace otb
