/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbSVMBasedRegionGrowingSegmentationControllerInterface_h
#define __otbSVMBasedRegionGrowingSegmentationControllerInterface_h

#include "itkObject.h"
#include "otbSVMBasedRegionGrowingSegmentationModel.h"

namespace otb
{
class SVMBasedRegionGrowingSegmentationControllerInterface
      : public itk::Object
{
public:
  /** Standard class typedefs */
  typedef SVMBasedRegionGrowingSegmentationControllerInterface Self;
  typedef itk::Object                      Superclass;
  typedef itk::SmartPointer<Self>          Pointer;
  typedef itk::SmartPointer<const Self>    ConstPointer;

  typedef SVMBasedRegionGrowingSegmentationModel ModelType;
  typedef ModelType::IndexType      IndexType;

  /** Standard type macros */
  itkTypeMacro(SVMBasedRegionGrowingSegmentationControllerInterface, Object);

  /** Users actions */
  virtual void ClearSeeds()=0;
  virtual void ChangeNumberOfSads(unsigned int nb)=0;
  virtual void ChangeNumberOfThresholds(unsigned int nb)=0;
  virtual void ManualUpdate() = 0;
  virtual void SeedSelected(IndexType& seed) {};
  // not implemented since no pixel reporting is needed for levelset segmentation
  virtual void PixelHovered(IndexType& index) {};
  virtual void PixelDragged(IndexType& index)=0;
  virtual void MouseIn() {};
  virtual void MouseOut() {};
  virtual void MouseReleased()=0;


protected:
  /** Constructor */
  SVMBasedRegionGrowingSegmentationControllerInterface()
  {
    m_Model = SVMBasedRegionGrowingSegmentationModel::GetInstance();
  }
  /** Destructor */
  ~SVMBasedRegionGrowingSegmentationControllerInterface() {}

  /** The instance of the model (for commodity) */
  SVMBasedRegionGrowingSegmentationModel * m_Model;

private:
  SVMBasedRegionGrowingSegmentationControllerInterface(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented
};
} // end namespace otb

#endif
