/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbSegmentationApplicationController_h
#define __otbSegmentationApplicationController_h

#include "otbSegmentationApplicationControllerInterface.h"
#include "otbPreprocessingController.h"
#include "otbRegionGrowingSegmentationController.h"
#include "otbLevelSetSegmentationController.h"
#include "otbSVMBasedRegionGrowingSegmentationController.h"
#include "otbVectorizationController.h"
#include "otbSegmentationApplicationViewGUI.h"

namespace otb
{
class SegmentationApplicationController
      : public SegmentationApplicationControllerInterface
{
public:
  /** Standard class typedefs */
  typedef SegmentationApplicationController Self;
  typedef itk::Object                       Superclass;
  typedef itk::SmartPointer<Self>           Pointer;
  typedef itk::SmartPointer<const Self>     ConstPointer;

  typedef otb::SegmentationApplicationViewGUI ViewType;
  typedef ViewType::Pointer                   ViewPointerType;

  /** Standard type macros */
  itkTypeMacro(SegmentationApplicationController, Superclass);
  itkNewMacro(Self);

  /** Set the view */
  virtual void SetView(ViewPointerType view);

  /** User action */
  virtual void RegionSelected(const RegionType& region);
  virtual void SelectFileName(std::string filename);
  virtual void ChangeAlgorithm(SegmentationAlgorithm algorithm);
  virtual void ChangeChannel(unsigned int channel);
  virtual void ChangeUseImageIntensity();
  virtual void ChangeUseImageChannel();
  virtual void TriggerSegmentation();
  virtual void EndSegmentation();
  virtual void SaveResults(std::string filename);
  virtual void ImportSegments(std::string filename);
  virtual void DeleteSegment(unsigned int index);
  virtual void ClearSegments();

  /**  Get the sub controllers */
  virtual PreprocessingControllerInterface * GetPreprocessingController();
  virtual RegionGrowingSegmentationControllerInterface * GetRegionGrowingSegmentationController();
  virtual LevelSetSegmentationControllerInterface * GetLevelSetSegmentationController();
  virtual SVMBasedRegionGrowingSegmentationControllerInterface * GetSVMBasedRegionGrowingSegmentationController();
  virtual VectorizationControllerInterface * GetVectorizationController();

protected:
  /** Constructor */
  SegmentationApplicationController();
  /** Destructor */
  ~SegmentationApplicationController();

private:
  SegmentationApplicationController(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

  /** Pointer to the view */
  ViewPointerType m_View;

  /// Preprocessing controller interface
  PreprocessingController::Pointer m_PreprocessingController;
  /// Region growing controller interface
  RegionGrowingSegmentationController::Pointer m_RegionGrowingSegmentationController;
  /// Level set controller interface
  LevelSetSegmentationController::Pointer m_LevelSetSegmentationController;
  /// SVM based region growing controller
  SVMBasedRegionGrowingSegmentationController::Pointer m_SVMBasedRegionGrowingSegmentationController;
  /// Vectorization controller interface
  VectorizationController::Pointer m_VectorizationController;


};
} //end namespace otb

#endif
