/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbSegmentationApplicationControllerInterface_h
#define __otbSegmentationApplicationControllerInterface_h

#include "itkObject.h"
#include "otbSegmentationApplicationModel.h"
#include "otbImageViewerFullResolutionEventsInterface.h"
#include "otbPreprocessingControllerInterface.h"
#include "otbLevelSetSegmentationControllerInterface.h"
#include "otbRegionGrowingSegmentationControllerInterface.h"
#include "otbSVMBasedRegionGrowingSegmentationControllerInterface.h"
#include "otbVectorizationControllerInterface.h"

namespace otb
{
class SegmentationApplicationControllerInterface
      : public ImageViewerFullResolutionEventsInterface
{
public:
  /** Standard class typedefs */
  typedef SegmentationApplicationControllerInterface Self;
  typedef itk::Object                      Superclass;
  typedef itk::SmartPointer<Self>          Pointer;
  typedef itk::SmartPointer<const Self>    ConstPointer;

  typedef SegmentationApplicationModel::RegionType RegionType;

  /** Standard type macros */
  itkTypeMacro(SegmentationApplicationControllerInterface, Superclass);

  /** Users actions */
  virtual void RegionSelected(const RegionType& region) =0;
  virtual void SelectFileName(std::string filename) = 0;
  virtual void ChangeAlgorithm(SegmentationAlgorithm algorithm)=0;
  virtual void ChangeChannel(unsigned int channel)=0;
  virtual void ChangeUseImageIntensity()=0;
  virtual void ChangeUseImageChannel()=0;
  virtual void TriggerSegmentation()=0;
  virtual void EndSegmentation()=0;
  virtual void SaveResults(std::string filename)=0;
  virtual void ImportSegments(std::string filename)=0;
  virtual void DeleteSegment(unsigned int index)=0;
  virtual void ClearSegments()=0;

  virtual PreprocessingControllerInterface * GetPreprocessingController() = 0;
  virtual RegionGrowingSegmentationControllerInterface * GetRegionGrowingSegmentationController() = 0;
  virtual LevelSetSegmentationControllerInterface * GetLevelSetSegmentationController() = 0;
  virtual SVMBasedRegionGrowingSegmentationControllerInterface * GetSVMBasedRegionGrowingSegmentationController() = 0;
  virtual VectorizationControllerInterface * GetVectorizationController() = 0;

protected:
  /** Constructor */
  SegmentationApplicationControllerInterface()
  {
    m_Model = SegmentationApplicationModel::GetInstance();
  }
  /** Destructor */
  ~SegmentationApplicationControllerInterface() {}

  /** The instance of the model (for commodity) */
  SegmentationApplicationModel * m_Model;

private:
  SegmentationApplicationControllerInterface(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented
};
} // end namespace otb

#endif
