/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbVectorizationModel_h
#define __otbVectorizationModel_h

#include "otbMVCModel.h"
#include "otbImage.h"

#include "otbPolygon.h"
#include "itkConnectedComponentImageFilter.h"
#include "otbImageToEdgePathFilter.h"
#include "otbSimplifyPathListFilter.h"

#include "otbListenerBase.h"
#include "otbListenerBase.h"

namespace otb
{
/** \class VectorizationModel
 *
 *
 *
 */
class VectorizationModel
      //TODO Change LevelSetListenerBase to a more generic interface
      : public MVCModel<ListenerBase>, public ListenerBase, public itk::Object
{

public:
  /** Standard class typedefs */
  typedef VectorizationModel                         Self;
  typedef otb::MVCModel<ListenerBase> Superclass;
  typedef itk::SmartPointer<Self>                    Pointer;
  typedef itk::SmartPointer<const Self>              ConstPointer;

  typedef unsigned char                                            UCharPixelType;
  typedef otb::Image<UCharPixelType, 2>                             UCharImageType;
  typedef UCharImageType::Pointer                                  UCharImagePointerType;
  typedef double                                                   PixelType;
  typedef otb::Image<PixelType, 2>                                  ImageType;
  typedef ImageType::Pointer                                       ImagePointerType;
  typedef unsigned short                                           LabeledPixelType;
  typedef otb::Image<LabeledPixelType, 2>                           LabeledImageType;
  typedef otb::Polygon<double>                                     PolygonType;
  typedef PolygonType::Pointer                                     PolygonPointerType;
  typedef otb::ObjectList<PolygonType>                             PolygonListType;
  typedef PolygonListType::Pointer                                 PolygonListPointerType;
  typedef otb::ImageToEdgePathFilter<LabeledImageType, PolygonType> PolygonFilterType;
  typedef PolygonFilterType::Pointer                               PolygonFilterPointerType;
  typedef otb::SimplifyPathListFilter<PolygonType>                 SimplifyPolygonFilterType;
  typedef SimplifyPolygonFilterType::Pointer                       SimplifyPolygonFilterPointerType;
  typedef itk::ConnectedComponentImageFilter<UCharImageType, LabeledImageType> ConnectedComponentsFilterType;
  typedef ConnectedComponentsFilterType::Pointer                              ConnectedComponentsFilterPointerType;


  /** Standard type macro */
  itkTypeMacro(VectorizationModel, Object);

  /** Get the unique instance of the model */
  static Pointer GetInstance();

  /** Get the output polygons */
  virtual PolygonListPointerType GetPolygonList();

  /** Set the reference image */
  virtual void SetReferenceImage(ImagePointerType image);
  itkGetObjectMacro(ReferenceImage, ImageType);

  /** Set the segmentation image */
  virtual void SetSegmentationImage(UCharImagePointerType image);
  itkGetObjectMacro(SegmentationImage, UCharImageType);

  /** Get the flags to activate/deactivate the preprocessings */
  itkGetMacro(Valid, bool);

  /** Set/get the sigma gradient parameter */
  void   SetTolerance(double tolerance);
  itkGetMacro(Tolerance, double);

  /** Receive events from the preprocessing model */
  virtual void Notify();

protected:
  /** This is protected for the singleton. Use GetInstance() instead. */
  itkNewMacro(Self);

  /** Constructor */
  VectorizationModel();
  /** Destructor */
  ~VectorizationModel();

private:
  VectorizationModel(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

  /** Update the pipeline */
  void UpdatePipeline(void);

  /** Notify a given listener of changes */
  virtual void NotifyListener(ListenerBase * listener);

  /** Filters */
  SimplifyPolygonFilterPointerType     m_SimplifyPolygonFilter;

  /** Data */
  ImagePointerType       m_ReferenceImage;
  UCharImagePointerType  m_SegmentationImage;

  bool m_Valid;

  /** The instance singleton */
  static Pointer Instance;

  /** parameters */
  double m_Tolerance;
};
}
#endif
