/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbFullWidgetWithSeedsSelection_h
#define __otbFullWidgetWithSeedsSelection_h

#include "otbFullResolutionImageWidget.h"


namespace otb
{
/** \class FullWidgetWithSeedsSelection
 *
 */
template <class TPixel, class TController, class TLabel = double>
class ITK_EXPORT FullWidgetWithSeedsSelection
      : public FullResolutionImageWidget<TPixel>
{
public:
  /** Standard class typedefs */
  typedef FullWidgetWithSeedsSelection       Self;
  typedef FullResolutionImageWidget<TPixel>  Superclass;
  typedef itk::SmartPointer<Self>            Pointer;
  typedef itk::SmartPointer<const Self>      ConstPointer;

  /** Method for creation through the object factory. */
  itkNewMacro(Self);

  /** Run-time type information (and related methods). */
  itkTypeMacro(FullWidgetWithSeedsSelection, FullResolutionImageWidget);

  typedef TController                      ControllerType;
  typedef typename ControllerType::Pointer ControllerPointerType;
  typedef typename Superclass::IndexType   IndexType;

  itkSetObjectMacro(Controller, ControllerType);
  itkGetObjectMacro(Controller, ControllerType);

  /** Default mode handling, without ROI selection */
  virtual int handle(int event)
  {
    switch (event)
    {
    case FL_PUSH:
    {
      // in case of mouse click, change the point of view
      int x = Fl::event_x();
      int y = Fl::event_y();
      IndexType clickedIndex;
      clickedIndex[0]=x;
      clickedIndex[1]=y;
      clickedIndex=this->WindowToImageCoordinates(clickedIndex);
      if (m_Controller.IsNotNull())
      {
        m_Controller->SeedSelected(clickedIndex);
      }
      return 1;
    }
    case FL_ENTER:
    {
      if (m_Controller.IsNotNull())
      {
        m_Controller->MouseIn();
      }
      return 1;
    }
    case FL_LEAVE:
    {
      if (m_Controller.IsNotNull())
      {
        m_Controller->MouseOut();
      }
      return 1;
    }
    case FL_MOVE:
    {
      if (m_MouseMoveCount%m_MoveReportFrequency==0)
      {
        IndexType mousePos;
        mousePos[0]=Fl::event_x();
        mousePos[1]=Fl::event_y();
        IndexType index = this->WindowToImageCoordinates(mousePos);
        if (m_Controller.IsNotNull())
        {
          m_Controller->PixelHovered(index);
        }
        m_MouseMoveCount = 1;
      }
      else
      {
        ++m_MouseMoveCount;
      }
      return 1;
    }
    case FL_DRAG:
    {
      if (m_Controller.IsNotNull())
      {
        IndexType mousePos;
        mousePos[0]=Fl::event_x();
        mousePos[1]=Fl::event_y();
        IndexType index = this->WindowToImageCoordinates(mousePos);
        m_Controller->PixelDragged(index);
      }
      return 1;
    }
    case FL_RELEASE:
    {
      if (m_Controller.IsNotNull())
      {
        m_Controller->MouseReleased();
      }
      return 1;
    }
    }
    return Superclass::handle(event);
  }

protected:
  /**
   * Constructor.
   */
  FullWidgetWithSeedsSelection()
  {
    m_MouseMoveCount = 0;
    m_MoveReportFrequency = 5;
  }
  /**
   * Destructor.
   */
  ~FullWidgetWithSeedsSelection()
  {}

private:
  // Pointer to the controller
  ControllerPointerType m_Controller;
  unsigned int m_MouseMoveCount;
  unsigned int m_MoveReportFrequency;
};

} // end namespace otb
#endif
