/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include "otbLevelSetSegmentationViewGUI.h"

namespace otb
{

LevelSetSegmentationViewGUI::LevelSetSegmentationViewGUI()
    : LevelSetSegmentationViewGroup(0, 0, 670, 350,"Level set")
{
  m_LevelSetSegmentationModel = LevelSetSegmentationModel::GetInstance();
  m_LevelSetSegmentationModel->RegisterListener(this);

  m_GradientVectorCastFilter = VectorCastFilterType::New();
  m_SpeedMapVectorCastFilter = VectorCastFilterType::New();
  m_TimeCrossingVectorCastFilter = VectorCastFilterType::New();
  m_SegmentationVectorCastFilter = UCharVectorCastFilterType::New();

  vSigma->value(m_LevelSetSegmentationModel->GetSigma());
  vSigmoidAlpha->value(m_LevelSetSegmentationModel->GetSigmoidAlpha());
  vSigmoidBeta->value(m_LevelSetSegmentationModel->GetSigmoidBeta());
  vStoppingTime->value(m_LevelSetSegmentationModel->GetStoppingTime());
  vTimeThreshold->value(m_LevelSetSegmentationModel->GetTimeThreshold());
  vTimeThreshold->minimum(0.);
  vTimeThreshold->maximum(m_LevelSetSegmentationModel->GetStoppingTime());

  m_SeedsColor[0]=0.0;
  m_SeedsColor[1]=1.0;
  m_SeedsColor[2]=0.0;
  m_SeedsColor[3]=1.0;

  m_WMargin = 25;
  m_HMargin = 25;
}

void LevelSetSegmentationViewGUI::RefreshViewers()
{
  FormListPointerType formList = FormListType::New();
  for (NodeContainerType::Iterator it = m_LevelSetSegmentationModel->GetNodeContainer()->Begin();
       it !=   m_LevelSetSegmentationModel->GetNodeContainer()->End();
       ++it)
  {
    ImageWidgetCircleFormPointerType new_circle = ImageWidgetCircleFormType::New();
    new_circle->SetCenter(it.Value().GetIndex());
    new_circle->SetRadius(5);
    new_circle->SetSolid(false);
    new_circle->SetColor(m_SeedsColor);
    formList->PushBack(new_circle);
  }

  MinMaxCalculatorType::Pointer GradientMinMax = MinMaxCalculatorType::New();
  GradientMinMax->SetImage(m_LevelSetSegmentationModel->GetGradientMagnitudeImage());
  GradientMinMax->Compute();

  m_GradientVectorCastFilter->SetInput(m_LevelSetSegmentationModel->GetGradientMagnitudeImage());
  m_GradientVectorCastFilter->GetOutput()->UpdateOutputInformation();
  m_GradientVectorCastFilter->GetOutput()->SetRequestedRegionToLargestPossibleRegion();
  m_GradientVectorCastFilter->Update();

  if (m_GradientWidget.IsNull())
  {
    m_GradientWidget  = ImageWidgetType::New();
    m_GradientWidget->SetInput(m_GradientVectorCastFilter->GetOutput());
    m_GradientWidget->Init(gGradientWidgetGroup->x(), gGradientWidgetGroup->y(), gGradientWidgetGroup->w(), gGradientWidgetGroup->h(),"Gradient image");
    m_GradientWidget->SetViewModel(ImageWidgetType::GRAYSCALE);
    m_GradientWidget->SetFormOverlayVisible(true);
    gGradientWidgetGroup->add(m_GradientWidget);
    gGradientWidgetGroup->box(FL_NO_BOX);
  }
  else
  {
    m_GradientWidget->ClearBufferedRegion();
  }

  m_GradientWidget->SetFormListOverlay(formList);
  m_GradientWidget->GetTransferFunctionList()->GetNthElement(0)->SetLowerBound(GradientMinMax->GetMinimum());
  m_GradientWidget->GetTransferFunctionList()->GetNthElement(0)->SetUpperBound(GradientMinMax->GetMaximum());
  m_GradientWidget->redraw();

  MinMaxCalculatorType::Pointer SpeedMapMinMax = MinMaxCalculatorType::New();
  SpeedMapMinMax->SetImage(m_LevelSetSegmentationModel->GetSpeedMapImage());
  SpeedMapMinMax->Compute();

  m_SpeedMapVectorCastFilter->SetInput(m_LevelSetSegmentationModel->GetSpeedMapImage());
  m_SpeedMapVectorCastFilter->GetOutput()->UpdateOutputInformation();
  m_SpeedMapVectorCastFilter->GetOutput()->SetRequestedRegionToLargestPossibleRegion();
  m_SpeedMapVectorCastFilter->Update();

  if (m_SpeedMapWidget.IsNull())
  {
    m_SpeedMapWidget  = ImageWidgetWithSeedsSelectionType::New();
    m_SpeedMapWidget->SetController(m_LevelSetSegmentationController);
    m_SpeedMapWidget->SetInput(m_SpeedMapVectorCastFilter->GetOutput());
    m_SpeedMapWidget->Init(gSpeedMapWidgetGroup->x(), gSpeedMapWidgetGroup->y(), gSpeedMapWidgetGroup->w(), gSpeedMapWidgetGroup->h(),"SpeedMap image");
    m_SpeedMapWidget->SetViewModel(ImageWidgetType::GRAYSCALE);
    m_SpeedMapWidget->SetFormOverlayVisible(true);
    gSpeedMapWidgetGroup->add(m_SpeedMapWidget);
    gSpeedMapWidgetGroup->box(FL_NO_BOX);
  }
  else
  {
    m_SpeedMapWidget->ClearBufferedRegion();
  }

  m_SpeedMapWidget->SetFormListOverlay(formList);
  m_SpeedMapWidget->GetTransferFunctionList()->GetNthElement(0)->SetLowerBound(SpeedMapMinMax->GetMinimum());
  m_SpeedMapWidget->GetTransferFunctionList()->GetNthElement(0)->SetUpperBound(SpeedMapMinMax->GetMaximum());
  m_SpeedMapWidget->redraw();

  m_TimeCrossingVectorCastFilter->SetInput(m_LevelSetSegmentationModel->GetTimeCrossingImage());
  m_TimeCrossingVectorCastFilter->GetOutput()->UpdateOutputInformation();
  m_TimeCrossingVectorCastFilter->GetOutput()->SetRequestedRegionToLargestPossibleRegion();
  m_TimeCrossingVectorCastFilter->Update();

  if (m_TimeCrossingWidget.IsNull())
  {
    m_TimeCrossingWidget  = ImageWidgetType::New();
    m_TimeCrossingWidget->SetInput(m_TimeCrossingVectorCastFilter->GetOutput());
    m_TimeCrossingWidget->Init(gTimeCrossingWidgetGroup->x(), gTimeCrossingWidgetGroup->y(), gTimeCrossingWidgetGroup->w(), gTimeCrossingWidgetGroup->h(),"TimeCrossing image");
    m_TimeCrossingWidget->SetViewModel(ImageWidgetType::GRAYSCALE);
    m_TimeCrossingWidget->SetFormOverlayVisible(true);
    gTimeCrossingWidgetGroup->box(FL_NO_BOX);
    gTimeCrossingWidgetGroup->add(m_TimeCrossingWidget);
  }
  else
  {
    m_TimeCrossingWidget->ClearBufferedRegion();
  }
  m_TimeCrossingWidget->SetFormListOverlay(formList);
  m_TimeCrossingWidget->GetTransferFunctionList()->GetNthElement(0)->SetLowerBound(0);
  m_TimeCrossingWidget->GetTransferFunctionList()->GetNthElement(0)->SetUpperBound(m_LevelSetSegmentationModel->GetStoppingTime());
  m_TimeCrossingWidget->redraw();

  m_SegmentationVectorCastFilter->SetInput(m_LevelSetSegmentationModel->GetOutputImage());
  m_SegmentationVectorCastFilter->GetOutput()->UpdateOutputInformation();
  m_SegmentationVectorCastFilter->GetOutput()->SetRequestedRegionToLargestPossibleRegion();
  m_SegmentationVectorCastFilter->Update();

  if (m_SegmentationWidget.IsNull())
  {
    m_SegmentationWidget  = UCharImageWidgetType::New();
    m_SegmentationWidget->SetInput(m_SegmentationVectorCastFilter->GetOutput());
    m_SegmentationWidget->Init(gSegmentationWidgetGroup->x(), gSegmentationWidgetGroup->y(), gSegmentationWidgetGroup->w(), gSegmentationWidgetGroup->h(),"Segmentation image");
    m_SegmentationWidget->SetViewModel(UCharImageWidgetType::GRAYSCALE);
    m_SegmentationWidget->SetFormOverlayVisible(true);
    gSegmentationWidgetGroup->box(FL_NO_BOX);
    gSegmentationWidgetGroup->add(m_SegmentationWidget);
  }
  else
  {
    m_SegmentationWidget->ClearBufferedRegion();
  }

  m_SegmentationWidget->SetFormListOverlay(formList);
  m_SegmentationWidget->GetTransferFunctionList()->GetNthElement(0)->SetLowerBound(0);
  m_SegmentationWidget->GetTransferFunctionList()->GetNthElement(0)->SetUpperBound(255);
  m_SegmentationWidget->redraw();
}

void LevelSetSegmentationViewGUI::show()
{
  this->Fl_Group::show();
  if (m_GradientWidget.IsNotNull())
  {
    m_GradientWidget->Show();
  }
  if (m_SegmentationWidget.IsNotNull())
  {
    m_SpeedMapWidget->Show();
  }
  if (m_SegmentationWidget.IsNotNull())
  {
    m_SegmentationWidget->Show();
  }
  if (m_TimeCrossingWidget.IsNotNull())
  {
    m_TimeCrossingWidget->Show();
  }
}

LevelSetSegmentationViewGUI::SizeType LevelSetSegmentationViewGUI::GetIdealSize()
{
  ImageType::SizeType size;

  if (m_LevelSetSegmentationModel->GetValid())
  {
    size = m_LevelSetSegmentationModel->GetOutputImage()->GetLargestPossibleRegion().GetSize();
  }
  else
  {
    SizeType defaultSize;
    defaultSize[0]=this->w();
    defaultSize[1]= this->h();
    return defaultSize;
  }

  ImageType::SizeType newSize;
  newSize[0] = 2*size[0]+4*m_WMargin + gInterface->w();
  newSize[1] = std::max(static_cast<int>(2 * size[1]+3*m_HMargin), 2*m_HMargin+gInterface->h());

  return newSize;
}


void LevelSetSegmentationViewGUI::resize(int x, int y, int w, int h)
{
  int m_WMargin = 25;
  int m_HMargin = 25;

  int interfacew = gInterface->w();
  int interfaceh = gInterface->h();

  this->Fl_Group::resize(x, y, w, h);
  int imagew = (w-4*m_WMargin-interfacew)/2;
  int imageh = (h-3*m_HMargin)/2;

  gGradientWidgetGroup->resize(x+m_WMargin, y+m_HMargin, imagew, imageh);
  if (m_GradientWidget.IsNotNull())
  {
    m_GradientWidget->resize(x+m_WMargin, y+m_HMargin, imagew, imageh);
  }

  gSpeedMapWidgetGroup->resize(x+2*m_WMargin+imagew, y+m_HMargin, imagew, imageh);
  if (m_SpeedMapWidget.IsNotNull())
  {
    m_SpeedMapWidget->resize(x+2*m_WMargin+imagew, y+m_HMargin, imagew, imageh);
  }

  gTimeCrossingWidgetGroup->resize(x+m_WMargin, y+2*m_HMargin+imageh, imagew, imageh);
  if (m_TimeCrossingWidget.IsNotNull())
  {
    m_TimeCrossingWidget->resize(x+m_WMargin, y+2*m_HMargin+imageh, imagew, imageh);
  }

  gSegmentationWidgetGroup->resize(x+2*m_WMargin+imagew, y+2*m_HMargin+imageh, imagew, imageh);
  if (m_SegmentationWidget.IsNotNull())
  {
    m_SegmentationWidget->resize(x+2*m_WMargin+imagew, y+2*m_HMargin+imageh, imagew, imageh);
  }
  int interfacex = 3*m_WMargin + 2 * imagew;
  int interfacey = (h-interfaceh)/2;

  gInterface->resize(x+interfacex, y+interfacey, interfacew, interfaceh);
}

void LevelSetSegmentationViewGUI::Notify()
{
  if (m_LevelSetSegmentationModel->GetValid())
  {
    this->RefreshViewers();
  }
  this->redraw();
}

void LevelSetSegmentationViewGUI::SigmaCallback()
{
  m_LevelSetSegmentationController->ChangeSigma(vSigma->value());
}

void LevelSetSegmentationViewGUI::SigmoidAlphaCallback()
{
  m_LevelSetSegmentationController->ChangeSigmoidAlpha(vSigmoidAlpha->value());
}

void LevelSetSegmentationViewGUI::SigmoidBetaCallback()
{
  m_LevelSetSegmentationController->ChangeSigmoidBeta(vSigmoidBeta->value());
}

void LevelSetSegmentationViewGUI::StoppingTimeCallback()
{
  m_LevelSetSegmentationController->ChangeStoppingTime(vStoppingTime->value());
}

void LevelSetSegmentationViewGUI::TimeThresholdCallback()
{
  m_LevelSetSegmentationController->ChangeTimeThreshold(vTimeThreshold->value());
}

void LevelSetSegmentationViewGUI::ClearSeedsCallback()
{
  m_LevelSetSegmentationController->ClearSeeds();
}
}
