/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbLevelSetSegmentationViewGUI_h
#define __otbLevelSetSegmentationViewGUI_h

// Disabling deprecation warning
#ifdef _MSC_VER
#pragma warning(push)
#pragma warning(disable:4996)
#endif
#include "otbLevelSetSegmentationViewGroup.h"
#ifdef _MSC_VER
#pragma warning(pop)
#endif
#include "otbListenerBase.h"
#include "otbLevelSetSegmentationModel.h"
#include "otbLevelSetSegmentationControllerInterface.h"
#include "otbFullResolutionImageWidget.h"
#include "otbFullWidgetWithSeedsSelection.h"
#include "itkMinimumMaximumImageCalculator.h"
#include "otbImageToVectorImageCastFilter.h"
#include "otbImageWidgetCircleForm.h"

namespace otb
{
/** \class LevelSetSegmentationViewGUI
 *
 */
class LevelSetSegmentationViewGUI
      : public ListenerBase, public LevelSetSegmentationViewGroup, public itk::Object
{
public:
  /** Standard class typedefs */
  typedef LevelSetSegmentationViewGUI   Self;
  typedef itk::Object                   Superclass;
  typedef itk::SmartPointer<Self>       Pointer;
  typedef itk::SmartPointer<const Self> ConstPointer;

  // Pixel type
  typedef double PixelType;
  typedef otb::FullResolutionImageWidget<PixelType>                                   ImageWidgetType;
  typedef ImageWidgetType::Pointer                                                    ImageWidgetPointerType;
  typedef ImageWidgetType::ImageType                                                  ImageType;
  typedef ImageType::SizeType                                                         SizeType;
  typedef otb::FullWidgetWithSeedsSelection<PixelType,
  LevelSetSegmentationControllerInterface>                                          ImageWidgetWithSeedsSelectionType;
  typedef ImageWidgetWithSeedsSelectionType::Pointer                                  ImageWidgetWithSeedsSelectionPointerType;
  typedef unsigned char                                                               UCharPixelType;
  typedef otb::FullResolutionImageWidget<UCharPixelType>                              UCharImageWidgetType;
  typedef UCharImageWidgetType::Pointer                                               UCharImageWidgetPointerType;
  typedef UCharImageWidgetType::ImageType                                             UCharImageType;
  typedef ImageType::PixelType                                                        VectorPixelType;
  typedef LevelSetSegmentationModel                                                   LevelSetSegmentationModelType;
  typedef LevelSetSegmentationModelType::ImageType                                    SingleImageType;
  typedef LevelSetSegmentationModelType::UCharImageType                               UCharSingleImageType;
  typedef LevelSetSegmentationModelType::NodeContainerType                            NodeContainerType;
  typedef itk::MinimumMaximumImageCalculator<SingleImageType>                         MinMaxCalculatorType;
  typedef otb::ImageToVectorImageCastFilter<SingleImageType, ImageType>                VectorCastFilterType;
  typedef VectorCastFilterType::Pointer                                               VectorCastFilterPointerType;
  typedef otb::ImageToVectorImageCastFilter<UCharSingleImageType, UCharImageType>      UCharVectorCastFilterType;
  typedef UCharVectorCastFilterType::Pointer                                          UCharVectorCastFilterPointerType;
  /// Type for the overlay list
  typedef ImageWidgetType::FormListType FormListType;
  typedef FormListType::Pointer FormListPointerType;
  typedef FormListType::Iterator FormListIteratorType;
  typedef ImageWidgetCircleForm ImageWidgetCircleFormType;
  typedef ImageWidgetCircleFormType::Pointer ImageWidgetCircleFormPointerType;
  typedef ImageWidgetCircleFormType::ColorType ColorType;

  /** Standards macros */
  itkNewMacro(Self);
  itkTypeMacro(LevelSetSegmentationViewGUI, Object);

  /** Set the controller */
  itkSetObjectMacro(LevelSetSegmentationController, LevelSetSegmentationControllerInterface);

  /** Event from the model */
  virtual void Notify();

  /** Buttons callback */
  virtual void SigmaCallback();
  virtual void SigmoidAlphaCallback();
  virtual void SigmoidBetaCallback();
  virtual void StoppingTimeCallback();
  virtual void TimeThresholdCallback();
  virtual void ClearSeedsCallback();

  /** Handle the resize event.*/
  virtual void resize(int x, int y, int w, int h);
  virtual void show();

  /** Display full images */
  virtual SizeType GetIdealSize();

protected:
  /** Constructor */
  LevelSetSegmentationViewGUI();
  /** Destructor */
  ~LevelSetSegmentationViewGUI() {};

  /** Refresh the image widgets */
  virtual void RefreshViewers();

private:
  LevelSetSegmentationViewGUI(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

  /** Pointer to the model */
  LevelSetSegmentationModel::Pointer m_LevelSetSegmentationModel;
  /** Pointer to the controller */
  LevelSetSegmentationControllerInterface::Pointer m_LevelSetSegmentationController;
  /** Gradient widget */
  ImageWidgetPointerType m_GradientWidget;
  /** Speed map widget */
  ImageWidgetWithSeedsSelectionPointerType  m_SpeedMapWidget;
  /** Time crossing widget */
  ImageWidgetPointerType  m_TimeCrossingWidget;
  /** Segmentation widget */
  UCharImageWidgetPointerType m_SegmentationWidget;
  /** Cast the gradient image to vector image */
  VectorCastFilterPointerType m_GradientVectorCastFilter;
  /** Cast the speed map image to vector image */
  VectorCastFilterPointerType m_SpeedMapVectorCastFilter;
  /** Cast the time crossing image to vector image */
  VectorCastFilterPointerType m_TimeCrossingVectorCastFilter;
  /** Cast the speed map image to vector image */
  UCharVectorCastFilterPointerType m_SegmentationVectorCastFilter;
  /** Seeds color */
  ColorType m_SeedsColor;

  /** margins for the group */
  int m_WMargin;
  int m_HMargin;
};
}//end namespace otb

#endif
