/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbPreprocessingViewGUI_h
#define __otbPreprocessingViewGUI_h

// Disabling deprecation warning
#ifdef _MSC_VER
#pragma warning(push)
#pragma warning(disable:4996)
#endif
#include "otbPreprocessingViewGroup.h"
#ifdef _MSC_VER
#pragma warning(pop)
#endif
#include "otbListenerBase.h"
#include "otbPreprocessingModel.h"
#include "otbPreprocessingControllerInterface.h"
#include "otbFullResolutionImageWidget.h"
#include "itkMinimumMaximumImageCalculator.h"
#include "otbImageToVectorImageCastFilter.h"
#include "itkObject.h"

namespace otb
{
/** \class PreprocessingViewGUI
 *
 */
class PreprocessingViewGUI
      : public ListenerBase, public PreprocessingViewGroup, public itk::Object
{
public:
  /** Standard class typedefs */
  typedef PreprocessingViewGUI          Self;
  typedef itk::Object                   Superclass;
  typedef itk::SmartPointer<Self>       Pointer;
  typedef itk::SmartPointer<const Self> ConstPointer;

  // Pixel type
  typedef double PixelType;
  typedef otb::FullResolutionImageWidget<PixelType> ImageWidgetType;
  typedef ImageWidgetType::Pointer                  ImageWidgetPointerType;
  typedef ImageWidgetType::ImageType                ImageType;
  typedef ImageType::SizeType                       SizeType;
  typedef ImageType::PixelType                      VectorPixelType;
  typedef PreprocessingModel                        PreprocessingModelType;
  typedef PreprocessingModelType::ImageType         SingleImageType;
  typedef itk::MinimumMaximumImageCalculator<SingleImageType> MinMaxCalculatorType;
  typedef otb::ImageToVectorImageCastFilter<SingleImageType, ImageType> VectorCastFilterType;
  typedef VectorCastFilterType::Pointer VectorCastFilterPointerType;

  /** Standards macros */
  itkNewMacro(Self);
  itkTypeMacro(PreprocessingViewGUI, Object);

  /** Set the controller */
  itkSetObjectMacro(PreprocessingController, PreprocessingControllerInterface);

  /** Event from the model */
  virtual void Notify();

  /** Buttons callback */
  virtual void UseSmoothingCallback();
  virtual void UseEdgeEnhancementCallback();
  virtual void NumberOfIterationsCallback();
  virtual void TimeStepCallback();
  virtual void AlphaCallback();

  /** Handle the resize event.*/
  virtual void resize(int x, int y, int w, int h);
  virtual void show();

  /** Display full images */
  virtual SizeType GetIdealSize();

protected:
  /** Constructor */
  PreprocessingViewGUI();
  /** Destructor */
  ~PreprocessingViewGUI() {};

  /** Refresh the image widgets */
  virtual void RefreshViewers();

  /** Center the widgets */
  virtual void CenterSmoothedWidget(int x, int y, int w, int h);
  virtual void CenterEdgeEnhancedWidget(int x, int y, int w, int h);

private:
  PreprocessingViewGUI(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

  /** Pointer to the model */
  PreprocessingModel::Pointer m_PreprocessingModel;
  /** Pointer to the controller */
  PreprocessingControllerInterface::Pointer m_PreprocessingController;
  /** Smoothing widget */
  ImageWidgetPointerType m_SmoothedWidget;
  /** Edge enhancement */
  ImageWidgetPointerType  m_EdgeEnhancedWidget;
  /** Cast the smoothed image to vector image */
  VectorCastFilterPointerType m_SmoothedVectorCastFilter;
  /** Cast the edge enhanced image to vector image */
  VectorCastFilterPointerType m_EdgeEnhancedVectorCastFilter;

  /** margins for the group */
  int m_WMargin;
  int m_HMargin;

};
}//end namespace otb

#endif
