/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include "otbRegionGrowingSegmentationViewGUI.h"

namespace otb
{

RegionGrowingSegmentationViewGUI::RegionGrowingSegmentationViewGUI()
    : RegionGrowingSegmentationViewGroup(0, 0, 670, 200,"Region growing")
{
  m_RegionGrowingSegmentationModel = RegionGrowingSegmentationModel::GetInstance();
  m_RegionGrowingSegmentationModel->RegisterListener(this);

  m_ImageVectorCastFilter = VectorCastFilterType::New();
  m_SegmentationVectorCastFilter = UCharVectorCastFilterType::New();

  vLowerThreshold->value(m_RegionGrowingSegmentationModel->GetLowerThreshold());
  vUpperThreshold->value(m_RegionGrowingSegmentationModel->GetUpperThreshold());
  vTolerance->value(m_RegionGrowingSegmentationModel->GetTolerance());
  vRadius->value(m_RegionGrowingSegmentationModel->GetRadius());
  vNumberOfIterations->value(m_RegionGrowingSegmentationModel->GetNumberOfIterations());

  vAlgorithm->add("Connected threshold");
  vAlgorithm->add("Neigborhood connected");
  vAlgorithm->add("Confidence connected");
  // Commented out until the isolated connected bug is fixed
  // vAlgorithm->add("Isolated connected");
  vAlgorithm->value(0);

  m_InsideSeedsColor[0]=1.0;
  m_InsideSeedsColor[1]=0.0;
  m_InsideSeedsColor[2]=0.0;
  m_InsideSeedsColor[3]=1.0;
  m_OutsideSeedsColor[0]=0.0;
  m_OutsideSeedsColor[1]=0.0;
  m_OutsideSeedsColor[2]=1.0;
  m_OutsideSeedsColor[3]=1.0;

  m_WMargin = 25;
  m_HMargin = 25;

  m_Offset.Fill(0);
}

void RegionGrowingSegmentationViewGUI::RefreshViewers()
{
  ImageWidgetCircleFormPointerType new_circle = ImageWidgetCircleFormType::New();

  FormListPointerType formList = FormListType::New();
  for (SeedsContainerType::iterator it = m_RegionGrowingSegmentationModel->GetInsideSeeds().begin();
       it != m_RegionGrowingSegmentationModel->GetInsideSeeds().end();
       ++it)
  {
    ImageWidgetCircleFormPointerType new_circle = ImageWidgetCircleFormType::New();
    new_circle->SetCenter(*it);
    new_circle->SetRadius(5);
    new_circle->SetSolid(false);
    new_circle->SetColor(m_InsideSeedsColor);
    formList->PushBack(new_circle);
  }

  if (m_RegionGrowingSegmentationModel->GetRegionGrowingAlgorithm() == ISOLATED_CONNECTED)
  {
    for (SeedsContainerType::iterator it = m_RegionGrowingSegmentationModel->GetOutsideSeeds().begin();
         it != m_RegionGrowingSegmentationModel->GetOutsideSeeds().end();
         ++it)
    {
      ImageWidgetCircleFormPointerType new_circle = ImageWidgetCircleFormType::New();
      new_circle->SetCenter(*it);
      new_circle->SetRadius(5);
      new_circle->SetSolid(false);
      new_circle->SetColor(m_OutsideSeedsColor);
      formList->PushBack(new_circle);
    }
  }

  MinMaxCalculatorType::Pointer imageMinMax = MinMaxCalculatorType::New();
  imageMinMax->SetImage(m_RegionGrowingSegmentationModel->GetInputImage());
  imageMinMax->Compute();

  m_ImageVectorCastFilter->SetInput(m_RegionGrowingSegmentationModel->GetInputImage());
  m_ImageVectorCastFilter->GetOutput()->UpdateOutputInformation();
  m_ImageVectorCastFilter->GetOutput()->SetRequestedRegionToLargestPossibleRegion();
  m_ImageVectorCastFilter->Update();

  if (m_ImageWidget.IsNull())
  {
    m_ImageWidget  = ImageWidgetWithSeedsSelectionType::New();
    m_ImageWidget->SetController(m_RegionGrowingSegmentationController);
    m_ImageWidget->SetInput(m_ImageVectorCastFilter->GetOutput());
    m_ImageWidget->Init(gImageWidgetGroup->x(), gImageWidgetGroup->y(), gImageWidgetGroup->w(), gImageWidgetGroup->h(),"Image");
    m_ImageWidget->SetViewModel(ImageWidgetWithSeedsSelectionType::GRAYSCALE);
    m_ImageWidget->SetFormOverlayVisible(true);
    gImageWidgetGroup->add(m_ImageWidget);
    gImageWidgetGroup->box(FL_NO_BOX);
  }
  else
  {
    m_ImageWidget->ClearBufferedRegion();
  }
  m_ImageWidget->SetFormListOverlay(formList);
  m_ImageWidget->GetTransferFunctionList()->GetNthElement(0)->SetLowerBound(imageMinMax->GetMinimum());
  m_ImageWidget->GetTransferFunctionList()->GetNthElement(0)->SetUpperBound(imageMinMax->GetMaximum());
  m_ImageWidget->redraw();


  if ((m_RegionGrowingSegmentationModel->GetRegionGrowingAlgorithm()!=ISOLATED_CONNECTED)
      || !m_RegionGrowingSegmentationModel->GetOutsideSeeds().empty())
  {
    m_SegmentationVectorCastFilter->SetInput(m_RegionGrowingSegmentationModel->GetOutputImage());
    m_SegmentationVectorCastFilter->GetOutput()->UpdateOutputInformation();
    m_SegmentationVectorCastFilter->GetOutput()->SetRequestedRegionToLargestPossibleRegion();
    m_SegmentationVectorCastFilter->Update();

    if (m_SegmentationWidget.IsNull())
    {
      m_SegmentationWidget  = UCharImageWidgetType::New();
      m_SegmentationWidget->SetInput(m_SegmentationVectorCastFilter->GetOutput());
      m_SegmentationWidget->Init(gSegmentationWidgetGroup->x(), gSegmentationWidgetGroup->y(), gSegmentationWidgetGroup->w(), gSegmentationWidgetGroup->h(),"Segmentation image");
      m_SegmentationWidget->SetViewModel(UCharImageWidgetType::GRAYSCALE);
      m_SegmentationWidget->SetFormOverlayVisible(true);
      gSegmentationWidgetGroup->add(m_SegmentationWidget);
      gSegmentationWidgetGroup->box(FL_NO_BOX);
    }
    else
    {
      m_SegmentationWidget->ClearBufferedRegion();
    }
    m_SegmentationWidget->SetFormListOverlay(formList);
    m_SegmentationWidget->GetTransferFunctionList()->GetNthElement(0)->SetLowerBound(0);
    m_SegmentationWidget->GetTransferFunctionList()->GetNthElement(0)->SetUpperBound(255);
    m_SegmentationWidget->redraw();
  }
}

void RegionGrowingSegmentationViewGUI::show()
{
  this->Fl_Group::show();
  if ((m_RegionGrowingSegmentationModel->GetRegionGrowingAlgorithm()!=ISOLATED_CONNECTED)
      || !m_RegionGrowingSegmentationModel->GetOutsideSeeds().empty())
  {
    if (m_SegmentationWidget.IsNotNull())
    {
      m_SegmentationWidget->Show();
    }
  }
  if (m_ImageWidget.IsNotNull())
  {
    m_ImageWidget->Show();
  }
}

RegionGrowingSegmentationViewGUI::SizeType RegionGrowingSegmentationViewGUI::GetIdealSize()
{
  ImageType::SizeType size;

  if (m_RegionGrowingSegmentationModel->GetValid())
  {
    size = m_RegionGrowingSegmentationModel->GetOutputImage()->GetLargestPossibleRegion().GetSize();
  }
  else
  {
    SizeType defaultSize;
    defaultSize[0]=this->w();
    defaultSize[1]= this->h();
    return defaultSize;
  }

  ImageType::SizeType newSize;
  newSize[0] = 2*size[0]+4*m_WMargin + gInterface->w();
  newSize[1] = std::max(static_cast<int>(size[1]+3*m_HMargin+oOutputPixel->h() ), 3*m_HMargin+gInterface->h()+oOutputPixel->h());

  return newSize;
}


void RegionGrowingSegmentationViewGUI::resize(int x, int y, int w, int h)
{
  int interfacew = gInterface->w();
  int interfaceh = gInterface->h();

  this->Fl_Group::resize(x, y, w, h);
  int imagew = (w-4*m_WMargin-interfacew)/2;
  int imageh = (h-3*m_HMargin-oOutputPixel->h());

  gImageWidgetGroup->resize(x+m_WMargin, y+m_HMargin, imagew, imageh);
  if (m_ImageWidget.IsNotNull())
  {
    m_ImageWidget->resize(x+m_WMargin, y+m_HMargin, imagew, imageh);
  }

  gSegmentationWidgetGroup->resize(x+2*m_WMargin+imagew, y+m_HMargin, imagew, imageh);
  if (m_SegmentationWidget.IsNotNull())
  {
    m_SegmentationWidget->resize(x+2*m_WMargin+imagew, y+m_HMargin, imagew, imageh);
  }

  oOutputPixel->resize(x+m_WMargin, y+2*m_HMargin+imageh, m_WMargin+2*imagew, oOutputPixel->h());

  int interfacex = 3*m_WMargin + 2 * imagew;
  int interfacey = (h-interfaceh)/2;

  gInterface->resize(x+interfacex, y+interfacey, interfacew, interfaceh);
}

void RegionGrowingSegmentationViewGUI::Notify()
{

  if (m_RegionGrowingSegmentationModel->GetValid())
  {
    this->RefreshViewers();
  }
}

void RegionGrowingSegmentationViewGUI::AlgorithmCallback()
{
  switch (vAlgorithm->value())
  {
  case 0:
    m_RegionGrowingSegmentationController->ChangeAlgorithm(THRESHOLD_CONNECTED);
    break;
  case 1:
    m_RegionGrowingSegmentationController->ChangeAlgorithm(NEIGHBORHOOD_CONNECTED);
    break;
  case 2:
    m_RegionGrowingSegmentationController->ChangeAlgorithm(CONFIDENCE_CONNECTED);
    break;
  case 3:
    m_RegionGrowingSegmentationController->ChangeAlgorithm(ISOLATED_CONNECTED);
    break;
  }
}

void RegionGrowingSegmentationViewGUI::LowerThresholdCallback()
{
  m_RegionGrowingSegmentationController->ChangeLowerThreshold(vLowerThreshold->value());
}

void RegionGrowingSegmentationViewGUI::UpperThresholdCallback()
{
  m_RegionGrowingSegmentationController->ChangeUpperThreshold(vUpperThreshold->value());
}

void RegionGrowingSegmentationViewGUI::ToleranceCallback()
{
  m_RegionGrowingSegmentationController->ChangeTolerance(vTolerance->value());
}

void RegionGrowingSegmentationViewGUI::NumberOfIterationsCallback()
{
  m_RegionGrowingSegmentationController->ChangeNumberOfIterations(static_cast<unsigned int>(vNumberOfIterations->value()));
}
void RegionGrowingSegmentationViewGUI::RadiusCallback()
{
  m_RegionGrowingSegmentationController->ChangeRadius(static_cast<unsigned int>(vRadius->value()));
}

void RegionGrowingSegmentationViewGUI::InsideSeedsCallback()
{
  m_RegionGrowingSegmentationController->SelectInsideSeeds();
}

void RegionGrowingSegmentationViewGUI::OutsideSeedsCallback()
{
  m_RegionGrowingSegmentationController->SelectOutsideSeeds();
}

void RegionGrowingSegmentationViewGUI::ClearSeedsCallback()
{
  m_RegionGrowingSegmentationController->ClearSeeds();
}
void RegionGrowingSegmentationViewGUI::ReportPixel(IndexType index)
{
  itk::OStringStream oss;
  oss<<"Position: ("<<index[0]+m_Offset[0]<<", "<<index[1]+m_Offset[1]<<"), value: "<<m_RegionGrowingSegmentationModel->GetInputImage()->GetPixel(index);
  oOutputPixel->value(oss.str().c_str());
  oOutputPixel->redraw();
}
}
