/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbRegionGrowingSegmentationViewGUI_h
#define __otbRegionGrowingSegmentationViewGUI_h

// Disabling deprecation warning
#ifdef _MSC_VER
#pragma warning(push)
#pragma warning(disable:4996)
#endif
#include "otbRegionGrowingSegmentationViewGroup.h"
#ifdef _MSC_VER
#pragma warning(pop)
#endif
#include "otbListenerBase.h"
#include "otbRegionGrowingSegmentationModel.h"
#include "otbRegionGrowingSegmentationControllerInterface.h"
#include "otbFullResolutionImageWidget.h"
#include "otbFullWidgetWithSeedsSelection.h"
#include "itkMinimumMaximumImageCalculator.h"
#include "otbImageToVectorImageCastFilter.h"
#include "otbImageWidgetCircleForm.h"

namespace otb
{
/** \class RegionGrowingSegmentationViewGUI
 *
 */
class RegionGrowingSegmentationViewGUI
      : public ListenerBase, public RegionGrowingSegmentationViewGroup, public itk::Object
{
public:
  /** Standard class typedefs */
  typedef RegionGrowingSegmentationViewGUI   Self;
  typedef itk::Object                   Superclass;
  typedef itk::SmartPointer<Self>       Pointer;
  typedef itk::SmartPointer<const Self> ConstPointer;

  // Pixel type
  typedef double PixelType;
  typedef otb::FullWidgetWithSeedsSelection<PixelType,
  RegionGrowingSegmentationControllerInterface>                                     ImageWidgetWithSeedsSelectionType;
  typedef ImageWidgetWithSeedsSelectionType::Pointer                                  ImageWidgetWithSeedsSelectionPointerType;
  typedef ImageWidgetWithSeedsSelectionType::ImageType                                ImageType;
  typedef ImageType::SizeType                                                         SizeType;
  typedef ImageType::IndexType                                                        IndexType;
  typedef unsigned char                                                               UCharPixelType;
  typedef otb::FullResolutionImageWidget<UCharPixelType>                              UCharImageWidgetType;
  typedef UCharImageWidgetType::Pointer                                               UCharImageWidgetPointerType;
  typedef UCharImageWidgetType::ImageType                                             UCharImageType;
  typedef ImageType::PixelType                                                        VectorPixelType;
  typedef RegionGrowingSegmentationModel                                              RegionGrowingSegmentationModelType;
  typedef RegionGrowingSegmentationModelType::ImageType                               SingleImageType;
  typedef RegionGrowingSegmentationModelType::UCharImageType                          UCharSingleImageType;
  typedef RegionGrowingSegmentationModelType::SeedsContainerType                      SeedsContainerType;
  typedef itk::MinimumMaximumImageCalculator<SingleImageType>                         MinMaxCalculatorType;
  typedef otb::ImageToVectorImageCastFilter<SingleImageType, ImageType>                VectorCastFilterType;
  typedef VectorCastFilterType::Pointer                                               VectorCastFilterPointerType;
  typedef otb::ImageToVectorImageCastFilter<UCharSingleImageType, UCharImageType>      UCharVectorCastFilterType;
  typedef UCharVectorCastFilterType::Pointer                                          UCharVectorCastFilterPointerType;
  /// Type for the overlay list
  typedef UCharImageWidgetType::FormListType FormListType;
  typedef FormListType::Pointer FormListPointerType;
  typedef FormListType::Iterator FormListIteratorType;
  typedef ImageWidgetCircleForm ImageWidgetCircleFormType;
  typedef ImageWidgetCircleFormType::Pointer ImageWidgetCircleFormPointerType;
  typedef ImageWidgetCircleFormType::ColorType ColorType;

  /** Standards macros */
  itkNewMacro(Self);
  itkTypeMacro(RegionGrowingSegmentationViewGUI, Object);

  /** Set the controller */
  itkSetObjectMacro(RegionGrowingSegmentationController, RegionGrowingSegmentationControllerInterface);

  /** Get/set the offset */
  itkSetMacro(Offset, IndexType);
  itkGetConstReferenceMacro(Offset, IndexType);

  /** Event from the model */
  virtual void Notify();

  /** Buttons callback */
  virtual void AlgorithmCallback();
  virtual void LowerThresholdCallback();
  virtual void UpperThresholdCallback();
  virtual void ToleranceCallback();
  virtual void NumberOfIterationsCallback();
  virtual void RadiusCallback();
  virtual void InsideSeedsCallback();
  virtual void OutsideSeedsCallback();
  virtual void ClearSeedsCallback();

  /** Print pixel location and value */
  virtual void ReportPixel(IndexType index);

  /** Handle the resize event.*/
  virtual void resize(int x, int y, int w, int h);
  virtual void show();

  /** Display full images */
  virtual SizeType GetIdealSize();

protected:
  /** Constructor */
  RegionGrowingSegmentationViewGUI();
  /** Destructor */
  ~RegionGrowingSegmentationViewGUI() {};

  /** Refresh the image widgets */
  virtual void RefreshViewers();

private:
  RegionGrowingSegmentationViewGUI(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

  /** Pointer to the model */
  RegionGrowingSegmentationModel::Pointer m_RegionGrowingSegmentationModel;
  /** Pointer to the controller */
  RegionGrowingSegmentationControllerInterface::Pointer m_RegionGrowingSegmentationController;
  /** Image widget */
  ImageWidgetWithSeedsSelectionPointerType m_ImageWidget;
  /** Segmentation widget */
  UCharImageWidgetPointerType m_SegmentationWidget;
  /** Cast the gradient image to vector image */
  VectorCastFilterPointerType m_ImageVectorCastFilter;
  /** Cast the speed map image to vector image */
  UCharVectorCastFilterPointerType m_SegmentationVectorCastFilter;
  /** Seeds color */
  ColorType m_InsideSeedsColor;
  ColorType m_OutsideSeedsColor;

  /** margins for the group */
  int m_WMargin;
  int m_HMargin;

  /** Offset for pixel location */
  IndexType m_Offset;
};
}//end namespace otb

#endif
