/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include "otbSVMBasedRegionGrowingSegmentationViewGUI.h"

namespace otb
{

SVMBasedRegionGrowingSegmentationViewGUI::SVMBasedRegionGrowingSegmentationViewGUI()
    : SVMBasedRegionGrowingSegmentationViewGroup(0, 0, 670, 350,"SVM based region growing")
{
  m_SVMBasedRegionGrowingSegmentationModel = SVMBasedRegionGrowingSegmentationModel::GetInstance();
  m_SVMBasedRegionGrowingSegmentationModel->RegisterListener(this);

  m_FeaturesVectorCastFilter = VectorCastFilterType::New();
  m_HyperplanDistanceVectorCastFilter = VectorCastFilterType::New();
  m_SegmentationVectorCastFilter = UCharVectorCastFilterType::New();
  m_ImageRescaleIntensityFilter = VectorRescaleIntensityImageFilterType::New();

  vNumberOfSads->value(m_SVMBasedRegionGrowingSegmentationModel->GetNumberOfSpectralAngleDistanceImages());
  vNumberOfThresholds->value(m_SVMBasedRegionGrowingSegmentationModel->GetNumberOfThresholds());

  m_InsideSeedsColor[0]=1.0;
  m_InsideSeedsColor[1]=0.0;
  m_InsideSeedsColor[2]=0.0;
  m_InsideSeedsColor[3]=1.0;

  m_OutsideSeedsColor[0]=0.0;
  m_OutsideSeedsColor[1]=0.0;
  m_OutsideSeedsColor[2]=1.0;
  m_OutsideSeedsColor[3]=1.0;


  m_ViewedFeature = 0;

  m_WMargin = 25;
  m_HMargin = 25;
}

void SVMBasedRegionGrowingSegmentationViewGUI::UpdateInterface()
{
  itk::OStringStream oss;
  unsigned int featureIndex = 7;
  vFeatures->clear();
  vFeatures->add("Blue channel");
  vFeatures->add("Green channel");
  vFeatures->add("Red channel");
  vFeatures->add("Near infra-red channel");
  vFeatures->add("Ndvi");
  vFeatures->add("Pseudo-ndvi");
  vFeatures->add("water index");
  vFeatures->add("shadows");

  if (m_SVMBasedRegionGrowingSegmentationModel->GetUseLaplacian())
  {
    featureIndex++;
    vFeatures->add("Laplacian");
  }

  for (unsigned int i  = featureIndex+1; i<m_SVMBasedRegionGrowingSegmentationModel->GetFeatureImagesList()->Size(); ++i)
  {
    oss<<"Spectral angle "<<i-featureIndex;
    vFeatures->add(oss.str().c_str());
    oss.str("");
  }

  vFeatures->value(m_ViewedFeature);
}

void SVMBasedRegionGrowingSegmentationViewGUI::RefreshViewers()
{

  FormListPointerType formList = FormListType::New();
  for (SVMBasedRegionGrowingSegmentationModel::ContainerIteratorType it = m_SVMBasedRegionGrowingSegmentationModel->GetPixelsInPS()->GetPoints()->Begin();
       it != m_SVMBasedRegionGrowingSegmentationModel->GetPixelsInPS()->GetPoints()->End(); ++it)
  {
    ImageWidgetPointFormPointerType new_point = ImageWidgetPointFormType::New();
    ImageType::IndexType index;
    index[0] = static_cast<long int>(it.Value()[0]);
    index[1] = static_cast<long int>(it.Value()[1]);
    new_point->SetIndex(index);
    new_point->SetColor(m_InsideSeedsColor);
    formList->PushBack(new_point);
  }

  for (SVMBasedRegionGrowingSegmentationModel::ContainerIteratorType it = m_SVMBasedRegionGrowingSegmentationModel->GetPixelsOutPS()->GetPoints()->Begin();
       it != m_SVMBasedRegionGrowingSegmentationModel->GetPixelsOutPS()->GetPoints()->End(); ++it)
  {
    ImageWidgetPointFormPointerType new_point = ImageWidgetPointFormType::New();
    ImageType::IndexType index;
    index[0] = static_cast<long int>(it.Value()[0]);
    index[1] = static_cast<long int>(it.Value()[1]);
    new_point->SetIndex(index);
    new_point->SetColor(m_OutsideSeedsColor);
    formList->PushBack(new_point);
  }

  m_ImageRescaleIntensityFilter->SetInput(m_SVMBasedRegionGrowingSegmentationModel->GetInputImage());
  ImageType::PixelType min, max;

  min.SetSize(m_SVMBasedRegionGrowingSegmentationModel->GetInputImage()->GetNumberOfComponentsPerPixel());
  min.Fill(0);
  max.SetSize(m_SVMBasedRegionGrowingSegmentationModel->GetInputImage()->GetNumberOfComponentsPerPixel());
  max.Fill(255);

  m_ImageRescaleIntensityFilter->SetOutputMinimum(min);
  m_ImageRescaleIntensityFilter->SetOutputMaximum(max);
  m_ImageRescaleIntensityFilter->GetOutput()->UpdateOutputInformation();
  m_ImageRescaleIntensityFilter->GetOutput()->SetRequestedRegionToLargestPossibleRegion();
  m_ImageRescaleIntensityFilter->Update();

  if (m_ImageWidget.IsNull())
  {
    m_ImageWidget  = ImageWidgetWithSeedsSelectionType::New();
    m_ImageWidget->SetController(m_SVMBasedRegionGrowingSegmentationController);
    m_ImageWidget->SetInput(m_ImageRescaleIntensityFilter->GetOutput());
    m_ImageWidget->Init(gImageWidgetGroup->x(), gImageWidgetGroup->y(), gImageWidgetGroup->w(), gImageWidgetGroup->h(),"Input image");
    m_ImageWidget->SetViewModel(ImageWidgetType::RGB);
    m_ImageWidget->SetRedChannelIndex(m_SVMBasedRegionGrowingSegmentationModel->GetRedChannelIndex());
    m_ImageWidget->SetGreenChannelIndex(m_SVMBasedRegionGrowingSegmentationModel->GetGreenChannelIndex());
    m_ImageWidget->SetBlueChannelIndex(m_SVMBasedRegionGrowingSegmentationModel->GetBlueChannelIndex());
    m_ImageWidget->SetFormOverlayVisible(true);
    gImageWidgetGroup->add(m_ImageWidget);
    gImageWidgetGroup->box(FL_NO_BOX);
  }
  else
  {
    m_ImageWidget->ClearBufferedRegion();
  }
  m_ImageWidget->SetFormListOverlay(formList);
  m_ImageWidget->redraw();

  MinMaxCalculatorType::Pointer featuresMinMax = MinMaxCalculatorType::New();
  featuresMinMax->SetImage(m_SVMBasedRegionGrowingSegmentationModel->GetFeatureImagesList()->GetNthElement(m_ViewedFeature));
  featuresMinMax->Compute();

  m_FeaturesVectorCastFilter->SetInput(m_SVMBasedRegionGrowingSegmentationModel->GetFeatureImagesList()->GetNthElement(m_ViewedFeature));
  m_FeaturesVectorCastFilter->GetOutput()->UpdateOutputInformation();
  m_FeaturesVectorCastFilter->GetOutput()->SetRequestedRegionToLargestPossibleRegion();
  m_FeaturesVectorCastFilter->Update();

  if (m_FeaturesWidget.IsNull())
  {
    m_FeaturesWidget  = ImageWidgetType::New();
    m_FeaturesWidget->SetInput(m_FeaturesVectorCastFilter->GetOutput());
    m_FeaturesWidget->Init(gFeaturesWidgetGroup->x(), gFeaturesWidgetGroup->y(), gFeaturesWidgetGroup->w(), gFeaturesWidgetGroup->h(),"Features image");
    m_FeaturesWidget->SetViewModel(ImageWidgetType::GRAYSCALE);
    m_FeaturesWidget->SetFormOverlayVisible(true);
    gFeaturesWidgetGroup->add(m_FeaturesWidget);
    gFeaturesWidgetGroup->box(FL_NO_BOX);
  }
  else
  {
    m_FeaturesWidget->ClearBufferedRegion();
  }

  m_FeaturesWidget->GetTransferFunctionList()->GetNthElement(0)->SetLowerBound(featuresMinMax->GetMinimum());
  m_FeaturesWidget->GetTransferFunctionList()->GetNthElement(0)->SetUpperBound(featuresMinMax->GetMaximum());
  m_FeaturesWidget->redraw();


  m_HyperplanDistanceVectorCastFilter->SetInput(m_SVMBasedRegionGrowingSegmentationModel->GetHyperplanDistanceImage());
  m_HyperplanDistanceVectorCastFilter->GetOutput()->UpdateOutputInformation();
  m_HyperplanDistanceVectorCastFilter->GetOutput()->SetRequestedRegionToLargestPossibleRegion();
  m_HyperplanDistanceVectorCastFilter->Update();

  if (m_HyperplanDistanceWidget.IsNull())
  {
    m_HyperplanDistanceWidget  = ImageWidgetType::New();
    m_HyperplanDistanceWidget->SetInput(m_HyperplanDistanceVectorCastFilter->GetOutput());
    m_HyperplanDistanceWidget->Init(gHyperplanDistanceWidgetGroup->x(), gHyperplanDistanceWidgetGroup->y(), gHyperplanDistanceWidgetGroup->w(), gHyperplanDistanceWidgetGroup->h(),"Hyperplan distance image");
    m_HyperplanDistanceWidget->SetViewModel(ImageWidgetType::GRAYSCALE);
    m_HyperplanDistanceWidget->SetFormOverlayVisible(true);
    gHyperplanDistanceWidgetGroup->box(FL_NO_BOX);
    gHyperplanDistanceWidgetGroup->add(m_HyperplanDistanceWidget);
  }
  else
  {
    m_HyperplanDistanceWidget->ClearBufferedRegion();
  }
  m_HyperplanDistanceWidget->GetTransferFunctionList()->GetNthElement(0)->SetLowerBound(-1);
  m_HyperplanDistanceWidget->GetTransferFunctionList()->GetNthElement(0)->SetUpperBound(1);
  m_HyperplanDistanceWidget->redraw();

  m_SegmentationVectorCastFilter->SetInput(m_SVMBasedRegionGrowingSegmentationModel->GetOutputImage());
  m_SegmentationVectorCastFilter->GetOutput()->UpdateOutputInformation();
  m_SegmentationVectorCastFilter->GetOutput()->SetRequestedRegionToLargestPossibleRegion();
  m_SegmentationVectorCastFilter->Update();

  if (m_SegmentationWidget.IsNull())
  {
    m_SegmentationWidget  = UCharImageWidgetType::New();
    m_SegmentationWidget->SetInput(m_SegmentationVectorCastFilter->GetOutput());
    m_SegmentationWidget->Init(gSegmentationWidgetGroup->x(), gSegmentationWidgetGroup->y(), gSegmentationWidgetGroup->w(), gSegmentationWidgetGroup->h(),"Segmentation image");
    m_SegmentationWidget->SetViewModel(UCharImageWidgetType::GRAYSCALE);
    m_SegmentationWidget->SetFormOverlayVisible(true);
    gSegmentationWidgetGroup->box(FL_NO_BOX);
    gSegmentationWidgetGroup->add(m_SegmentationWidget);
  }
  else
  {
    m_SegmentationWidget->ClearBufferedRegion();
  }
  m_SegmentationWidget->GetTransferFunctionList()->GetNthElement(0)->SetLowerBound(0);
  m_SegmentationWidget->GetTransferFunctionList()->GetNthElement(0)->SetUpperBound(255);
  m_SegmentationWidget->redraw();
}

void SVMBasedRegionGrowingSegmentationViewGUI::show()
{
  this->Fl_Group::show();
  if (m_ImageWidget.IsNotNull())
  {
    m_ImageWidget->Show();
  }
  if (m_FeaturesWidget.IsNotNull())
  {
    m_FeaturesWidget->Show();
  }
  if (m_SegmentationWidget.IsNotNull())
  {
    m_SegmentationWidget->Show();
  }
  if (m_HyperplanDistanceWidget.IsNotNull())
  {
    m_HyperplanDistanceWidget->Show();
  }
}

SVMBasedRegionGrowingSegmentationViewGUI::SizeType SVMBasedRegionGrowingSegmentationViewGUI::GetIdealSize()
{
  ImageType::SizeType size;

  if (m_SVMBasedRegionGrowingSegmentationModel->GetValid())
  {
    size = m_SVMBasedRegionGrowingSegmentationModel->GetOutputImage()->GetLargestPossibleRegion().GetSize();
  }
  else
  {
    SizeType defaultSize;
    defaultSize[0]=this->w();
    defaultSize[1]= this->h();
    return defaultSize;
  }

  ImageType::SizeType newSize;
  newSize[0] = 2*size[0]+4*m_WMargin + gInterface->w();
  newSize[1] = std::max(static_cast<int>(2 * size[1]+3*m_HMargin), 2*m_HMargin+gInterface->h());

  return newSize;
}


void SVMBasedRegionGrowingSegmentationViewGUI::resize(int x, int y, int w, int h)
{
  int m_WMargin = 25;
  int m_HMargin = 25;

  int interfacew = gInterface->w();
  int interfaceh = gInterface->h();

  this->Fl_Group::resize(x, y, w, h);
  int imagew = (w-4*m_WMargin-interfacew)/2;
  int imageh = (h-3*m_HMargin)/2;

  gImageWidgetGroup->resize(x+m_WMargin, y+m_HMargin, imagew, imageh);
  if (m_ImageWidget.IsNotNull())
  {
    m_ImageWidget->resize(x+m_WMargin, y+m_HMargin, imagew, imageh);
  }

  gFeaturesWidgetGroup->resize(x+2*m_WMargin+imagew, y+m_HMargin, imagew, imageh);
  if (m_FeaturesWidget.IsNotNull())
  {
    m_FeaturesWidget->resize(x+2*m_WMargin+imagew, y+m_HMargin, imagew, imageh);
  }

  gHyperplanDistanceWidgetGroup->resize(x+m_WMargin, y+2*m_HMargin+imageh, imagew, imageh);
  if (m_HyperplanDistanceWidget.IsNotNull())
  {
    m_HyperplanDistanceWidget->resize(x+m_WMargin, y+2*m_HMargin+imageh, imagew, imageh);
  }

  gSegmentationWidgetGroup->resize(x+2*m_WMargin+imagew, y+2*m_HMargin+imageh, imagew, imageh);
  if (m_SegmentationWidget.IsNotNull())
  {
    m_SegmentationWidget->resize(x+2*m_WMargin+imagew, y+2*m_HMargin+imageh, imagew, imageh);
  }
  int interfacex = 3*m_WMargin + 2 * imagew;
  int interfacey = (h-interfaceh)/2;

  gInterface->resize(x+interfacex, y+interfacey, interfacew, interfaceh);
}

void SVMBasedRegionGrowingSegmentationViewGUI::Notify()
{
  if (m_SVMBasedRegionGrowingSegmentationModel->GetValid())
  {
    this->UpdateInterface();
    this->RefreshViewers();
  }
  this->redraw();
}

void SVMBasedRegionGrowingSegmentationViewGUI::FeaturesCallback()
{
  m_ViewedFeature = vFeatures->value();
  if (m_SVMBasedRegionGrowingSegmentationModel->GetValid())
  {
    this->RefreshViewers();
  }
}

void SVMBasedRegionGrowingSegmentationViewGUI::NumberOfSadsCallback()
{
  m_SVMBasedRegionGrowingSegmentationController->ChangeNumberOfSads(static_cast<unsigned int>(vNumberOfSads->value()));
}

void SVMBasedRegionGrowingSegmentationViewGUI::NumberOfThresholdsCallback()
{
  m_SVMBasedRegionGrowingSegmentationController->ChangeNumberOfThresholds(static_cast<unsigned int>(vNumberOfThresholds->value()));
}

void SVMBasedRegionGrowingSegmentationViewGUI::ClearSeedsCallback()
{
  m_SVMBasedRegionGrowingSegmentationController->ClearSeeds();
}

void SVMBasedRegionGrowingSegmentationViewGUI::UpdateCallback()
{
  m_SVMBasedRegionGrowingSegmentationController->ManualUpdate();
}

void SVMBasedRegionGrowingSegmentationViewGUI::AutomaticUpdateCallback()
{
  if (bAutomaticUpdate->value())
  {
    bUpdate->deactivate();
  }
  else
  {
    bUpdate->activate();
  }
}

}
