/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbSVMBasedRegionGrowingSegmentationViewGUI_h
#define __otbSVMBasedRegionGrowingSegmentationViewGUI_h

// Disabling deprecation warning
#ifdef _MSC_VER
#pragma warning(push)
#pragma warning(disable:4996)
#endif
#include "otbSVMBasedRegionGrowingSegmentationViewGroup.h"
#ifdef _MSC_VER
#pragma warning(pop)
#endif
#include "otbListenerBase.h"
#include "otbSVMBasedRegionGrowingSegmentationModel.h"
#include "otbSVMBasedRegionGrowingSegmentationControllerInterface.h"
#include "otbFullResolutionImageWidget.h"
#include "otbFullWidgetWithSeedsSelection.h"
#include "otbImageWidgetPointForm.h"
#include "itkMinimumMaximumImageCalculator.h"
#include "otbImageToVectorImageCastFilter.h"
#include "otbVectorRescaleIntensityImageFilter.h"

namespace otb
{
/** \class SVMBasedRegionGrowingSegmentationViewGUI
 *
 */
class SVMBasedRegionGrowingSegmentationViewGUI
      : public ListenerBase, public SVMBasedRegionGrowingSegmentationViewGroup, public itk::Object
{
public:
  /** Standard class typedefs */
  typedef SVMBasedRegionGrowingSegmentationViewGUI   Self;
  typedef itk::Object                   Superclass;
  typedef itk::SmartPointer<Self>       Pointer;
  typedef itk::SmartPointer<const Self> ConstPointer;

  typedef SVMBasedRegionGrowingSegmentationModel                          SVMBasedRegionGrowingSegmentationModelType;
  typedef SVMBasedRegionGrowingSegmentationModelType::InternImageType     SingleImageType;
  typedef SVMBasedRegionGrowingSegmentationModelType::InputMSImageType    ImageType;
  typedef SVMBasedRegionGrowingSegmentationModelType::InternPixelType     PixelType;
  typedef SVMBasedRegionGrowingSegmentationModelType::UCharPixelType      UCharPixelType;

  typedef otb::FullResolutionImageWidget<PixelType>                    ImageWidgetType;
  typedef ImageWidgetType::Pointer                                     ImageWidgetPointerType;
  typedef ImageType::SizeType                                          SizeType;
  typedef otb::FullWidgetWithSeedsSelection<PixelType,
  SVMBasedRegionGrowingSegmentationControllerInterface>                 ImageWidgetWithSeedsSelectionType;
  typedef ImageWidgetWithSeedsSelectionType::Pointer                   ImageWidgetWithSeedsSelectionPointerType;
  typedef itk::MinimumMaximumImageCalculator<SingleImageType>          MinMaxCalculatorType;
  typedef otb::ImageToVectorImageCastFilter<SingleImageType, ImageType> VectorCastFilterType;
  typedef VectorCastFilterType::Pointer                                VectorCastFilterPointerType;
  typedef VectorRescaleIntensityImageFilter<ImageType, ImageType>       VectorRescaleIntensityImageFilterType;
  typedef VectorRescaleIntensityImageFilterType::Pointer               VectorRescaleIntenistyImageFilterPointerType;
  typedef otb::VectorImage<UCharPixelType, 2>                           UCharImageType;
  typedef otb::Image<UCharPixelType, 2>                                 UCharSingleImageType;
  typedef otb::ImageToVectorImageCastFilter<UCharSingleImageType, UCharImageType> UCharVectorCastFilterType;
  typedef UCharVectorCastFilterType::Pointer                                     UCharVectorCastFilterPointerType;
  typedef otb::FullResolutionImageWidget<UCharPixelType>                         UCharImageWidgetType;
  typedef UCharImageWidgetType::Pointer                                          UCharImageWidgetPointerType;
  /// Type for the overlay list
  typedef ImageWidgetType::FormListType FormListType;
  typedef FormListType::Pointer FormListPointerType;
  typedef FormListType::Iterator FormListIteratorType;
  typedef ImageWidgetPointForm ImageWidgetPointFormType;
  typedef ImageWidgetPointFormType::Pointer ImageWidgetPointFormPointerType;
  typedef ImageWidgetPointFormType::ColorType ColorType;

  /** Standards macros */
  itkNewMacro(Self);
  itkTypeMacro(SVMBasedRegionGrowingSegmentationViewGUI, Object);

  /** Set the controller */
  itkSetObjectMacro(SVMBasedRegionGrowingSegmentationController, SVMBasedRegionGrowingSegmentationControllerInterface);

  /** Event from the model */
  virtual void Notify();

  /** Buttons callback */
  virtual void FeaturesCallback();
  virtual void NumberOfSadsCallback();
  virtual void NumberOfThresholdsCallback();
  virtual void ClearSeedsCallback();
  virtual void UpdateCallback();
  virtual void AutomaticUpdateCallback();

  /** Handle the resize event.*/
  virtual void resize(int x, int y, int w, int h);
  virtual void show();

  /** Display full images */
  virtual SizeType GetIdealSize();

protected:
  /** Constructor */
  SVMBasedRegionGrowingSegmentationViewGUI();
  /** Destructor */
  ~SVMBasedRegionGrowingSegmentationViewGUI() {};

  /** Refresh the image widgets */
  virtual void RefreshViewers();
  virtual void UpdateInterface();

private:
  SVMBasedRegionGrowingSegmentationViewGUI(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

  /** Pointer to the model */
  SVMBasedRegionGrowingSegmentationModel::Pointer m_SVMBasedRegionGrowingSegmentationModel;
  /** Pointer to the controller */
  SVMBasedRegionGrowingSegmentationControllerInterface::Pointer m_SVMBasedRegionGrowingSegmentationController;
  /** Gradient widget */
  ImageWidgetWithSeedsSelectionPointerType m_ImageWidget;
  /** Speed map widget */
  ImageWidgetPointerType m_FeaturesWidget;
  /** Time crossing widget */
  ImageWidgetPointerType  m_HyperplanDistanceWidget;
  /** Segmentation widget */
  UCharImageWidgetPointerType m_SegmentationWidget;
  /** Cast the speed map image to vector image */
  VectorCastFilterPointerType m_FeaturesVectorCastFilter;
  /** Cast the time crossing image to vector image */
  VectorCastFilterPointerType m_HyperplanDistanceVectorCastFilter;
  /** Cast the speed map image to vector image */
  UCharVectorCastFilterPointerType m_SegmentationVectorCastFilter;
  /** Rescale the intensity of input image */
  VectorRescaleIntenistyImageFilterPointerType m_ImageRescaleIntensityFilter;
  /** Seeds color */
  ColorType m_InsideSeedsColor;
  ColorType m_OutsideSeedsColor;
  /// Index of the feature to view
  unsigned int m_ViewedFeature;

  /** margins for the group */
  int m_WMargin;
  int m_HMargin;
};
}//end namespace otb

#endif
