/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbSegmentationApplicationViewGUI_h
#define __otbSegmentationApplicationViewGUI_h

// Disabling deprecation warning
#ifdef _MSC_VER
#pragma warning(push)
#pragma warning(disable:4996)
#endif
#include "otbSegmentationApplicationViewGroup.h"
#ifdef _MSC_VER
#pragma warning(pop)
#endif
#include "otbListenerBase.h"
#include "otbSegmentationApplicationModel.h"
#include "otbSegmentationApplicationControllerInterface.h"
#include "otbPreprocessingViewGUI.h"
#include "otbRegionGrowingSegmentationViewGUI.h"
#include "otbLevelSetSegmentationViewGUI.h"
#include "otbSVMBasedRegionGrowingSegmentationViewGUI.h"
#include "otbVectorizationViewGUI.h"

#include "otbImageViewerBase.h"
#include "itkObject.h"

#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Tabs.H>

namespace otb
{
/** \class SegmentationApplicationViewGUI
 *
 */
class SegmentationApplicationViewGUI
      : public ListenerBase, public SegmentationApplicationViewGroup, public itk::Object
{
public:
  /** Standard class typedefs */
  typedef SegmentationApplicationViewGUI Self;
  typedef itk::Object                    Superclass;
  typedef itk::SmartPointer<Self>        Pointer;
  typedef itk::SmartPointer<const Self>  ConstPointer;

  // Pixel type
  typedef double PixelType;
  typedef otb::ImageViewerBase<PixelType>       ImageViewerBaseType;
  typedef ImageViewerBaseType::Pointer          ImageViewerBasePointerType;
  typedef ImageViewerBaseType::FullWidgetType   FullWidgetType;
  typedef FullWidgetType::Pointer               FullWidgetPointerType;
  typedef FullWidgetType::ColorType             ColorType;
  typedef ImageViewerBaseType::ScrollWidgetType ScrollWidgetType;
  typedef ScrollWidgetType::Pointer             ScrollWidgetPointerType;
  typedef ImageViewerBaseType::SizeType         SizeType;

  /** Standards macros */
  itkNewMacro(Self);
  itkTypeMacro(SegmentationApplicationViewGUI, Object);

  /** Set the controller */
  virtual void SetSegmentationApplicationController(SegmentationApplicationControllerInterface * controller);

  /** Get the subviews */
  itkGetObjectMacro(PreprocessingView, PreprocessingViewGUI);
  itkGetObjectMacro(RegionGrowingView, RegionGrowingSegmentationViewGUI);
  itkGetObjectMacro(LevelSetView, LevelSetSegmentationViewGUI);
  itkGetObjectMacro(SVMBasedRegionGrowingView, SVMBasedRegionGrowingSegmentationViewGUI);
  itkGetObjectMacro(VectorizationView, VectorizationViewGUI);

  /** Event from the model */
  virtual void Notify();

  /** Buttons callback */
  virtual void OpenImageCallback();
  virtual void QuitCallback();
  virtual void UseImageIntensityCallback();
  virtual void UseImageChannelCallback();
  virtual void ChannelCallback();
  virtual void AlgorithmCallback();
  virtual void SegmentCallback();
  virtual void SegmentationOkCallback();
  virtual void SegmentationCancelCallback();
  virtual void FocusCallback();
  virtual void DeleteCallback();
  virtual void ClearCallback();
  virtual void SaveResultsCallback();
  virtual void ImportSegmentsCallback();

  virtual void SetupSegmentationWindow();

protected:
  /** Constructor */
  SegmentationApplicationViewGUI();
  /** Destructor */
  ~SegmentationApplicationViewGUI() {};

  /** Refresh the image widgets */
  virtual void RefreshViewer();
  virtual void UpdateInterface();

private:
  SegmentationApplicationViewGUI(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

  /** Pointer to the model */
  SegmentationApplicationModel::Pointer m_SegmentationApplicationModel;
  /** Pointer to the controller */
  SegmentationApplicationControllerInterface::Pointer m_SegmentationApplicationController;
  /** Pointer to the viewer */
  ImageViewerBasePointerType m_Viewer;
  /** colors */
  ColorType m_PolygonColor;
  ColorType m_RegionColor;

  /// Sub views
  PreprocessingViewGUI::Pointer m_PreprocessingView;
  RegionGrowingSegmentationViewGUI::Pointer m_RegionGrowingView;
  LevelSetSegmentationViewGUI::Pointer m_LevelSetView;
  SVMBasedRegionGrowingSegmentationViewGUI::Pointer m_SVMBasedRegionGrowingView;
  VectorizationViewGUI::Pointer m_VectorizationView;

  Fl_Double_Window * m_SegmentationWindow;
  Fl_Tabs *          m_SegmentationTabs;
};
}//end namespace otb

#endif
