/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include "otbVectorizationViewGUI.h"

namespace otb
{

VectorizationViewGUI::VectorizationViewGUI()
    : VectorizationViewGroup(0, 0, 670, 200,"Vectorization")
{
  m_BinaryVectorCastFilter = UCharVectorCastFilterType::New();
  m_OriginalVectorCastFilter = VectorCastFilterType::New();

  m_VectorizationModel = VectorizationModel::GetInstance();

  vTolerance->value(m_VectorizationModel->GetTolerance());

  m_VectorizationModel->RegisterListener(this);

  m_PolygonColor[0]=1.;
  m_PolygonColor[1]=0;
  m_PolygonColor[2]=0;
  m_PolygonColor[3]=0.5;

  m_PointColor[0]=0.;
  m_PointColor[1]=1.;
  m_PointColor[2]=0;
  m_PointColor[3]=1.;

  m_WMargin = 25;
  m_HMargin = 25;
}

void VectorizationViewGUI::RefreshViewers()
{

  FormListPointerType formList = FormListType::New();

  for (PolygonListType::Iterator pit = m_VectorizationModel->GetPolygonList()->Begin();
       pit!=m_VectorizationModel->GetPolygonList()->End();
       ++pit)
  {
    ImageWidgetPolygonFormPointerType polygon = ImageWidgetPolygonFormType::New();
    polygon->SetPolygon(pit.Get());
    polygon->SetColor(m_PolygonColor);
    polygon->SetSolid(true);
    polygon->SetInternalValueToAlphaChannel(false);

    for (PolygonType::VertexListConstIteratorType vIt = pit.Get()->GetVertexList()->Begin();
         vIt != pit.Get()->GetVertexList()->End(); ++vIt)
    {
      ImageWidgetCircleFormPointerType circle = ImageWidgetCircleFormType::New();
      circle->SetCenter(vIt.Value());
      circle->SetRadius(1);
      circle->SetSolid(false);
      circle->SetColor(m_PointColor);
      formList->PushBack(circle);
    }
    formList->PushBack(polygon);
  }
  m_BinaryVectorCastFilter->SetInput(m_VectorizationModel->GetSegmentationImage());
  m_BinaryVectorCastFilter->GetOutput()->UpdateOutputInformation();
  m_BinaryVectorCastFilter->GetOutput()->SetRequestedRegionToLargestPossibleRegion();
  m_BinaryVectorCastFilter->Update();

  if (m_BinaryWidget.IsNull())
  {
    m_BinaryWidget  = UCharImageWidgetType::New();
    m_BinaryWidget->SetInput(m_BinaryVectorCastFilter->GetOutput());
    m_BinaryWidget->Init(gBinaryWidgetGroup->x(), gBinaryWidgetGroup->y(), gBinaryWidgetGroup->w(), gBinaryWidgetGroup->h(),"Binary image");
    m_BinaryWidget->SetViewModel(UCharImageWidgetType::GRAYSCALE);
    m_BinaryWidget->SetFormOverlayVisible(true);
    m_BinaryWidget->GetTransferFunctionList()->GetNthElement(0)->SetLowerBound(0);
    m_BinaryWidget->GetTransferFunctionList()->GetNthElement(0)->SetUpperBound(255);
    m_BinaryWidget->SetFormListOverlay(formList);
    gBinaryWidgetGroup->box(FL_NO_BOX);
    gBinaryWidgetGroup->add(m_BinaryWidget);
  }
  else
  {
    m_BinaryWidget->GetTransferFunctionList()->GetNthElement(0)->SetLowerBound(0);
    m_BinaryWidget->GetTransferFunctionList()->GetNthElement(0)->SetUpperBound(255);
    m_BinaryWidget->SetFormListOverlay(formList);
    m_BinaryWidget->ClearBufferedRegion();
  }

  MinMaxCalculatorType::Pointer minMax = MinMaxCalculatorType::New();
  minMax->SetImage(m_VectorizationModel->GetReferenceImage());
  minMax->Compute();

  m_OriginalVectorCastFilter->SetInput(m_VectorizationModel->GetReferenceImage());
  m_OriginalVectorCastFilter->GetOutput()->UpdateOutputInformation();
  m_OriginalVectorCastFilter->GetOutput()->SetRequestedRegionToLargestPossibleRegion();
  m_OriginalVectorCastFilter->Update();

  if (m_OriginalWidget.IsNull())
  {
    m_OriginalWidget  = ImageWidgetType::New();
    m_OriginalWidget->SetInput(m_OriginalVectorCastFilter->GetOutput());
    m_OriginalWidget->Init(gOriginalWidgetGroup->x(), gOriginalWidgetGroup->y(), gOriginalWidgetGroup->w(), gOriginalWidgetGroup->h(),"Original image");
    m_OriginalWidget->SetViewModel(ImageWidgetType::GRAYSCALE);
    m_OriginalWidget->SetFormOverlayVisible(true);
    m_OriginalWidget->GetTransferFunctionList()->GetNthElement(0)->SetLowerBound(minMax->GetMinimum());
    m_OriginalWidget->GetTransferFunctionList()->GetNthElement(0)->SetUpperBound(minMax->GetMaximum());
    m_OriginalWidget->SetFormListOverlay(formList);
    gOriginalWidgetGroup->box(FL_NO_BOX);
    gOriginalWidgetGroup->add(m_OriginalWidget);
  }

  else
  {
    m_OriginalWidget->ClearBufferedRegion();
    m_OriginalWidget->GetTransferFunctionList()->GetNthElement(0)->SetLowerBound(minMax->GetMinimum());
    m_OriginalWidget->GetTransferFunctionList()->GetNthElement(0)->SetUpperBound(minMax->GetMaximum());
    m_OriginalWidget->SetFormListOverlay(formList);
  }

  m_BinaryWidget->redraw();
  m_OriginalWidget->redraw();
}

void VectorizationViewGUI::show()
{
  this->Fl_Group::show();
  if (m_BinaryWidget.IsNotNull())
  {
    m_BinaryWidget->Show();
  }
  if (m_OriginalWidget.IsNotNull())
  {
    m_OriginalWidget->Show();
  }
}

VectorizationViewGUI::SizeType VectorizationViewGUI::GetIdealSize()
{
  ImageType::SizeType size;

  if (m_VectorizationModel->GetValid())
  {
    size = m_VectorizationModel->GetReferenceImage()->GetLargestPossibleRegion().GetSize();
  }
  else
  {
    SizeType defaultSize;
    defaultSize[0]=this->w();
    defaultSize[1]= this->h();
    return defaultSize;
  }

  ImageType::SizeType newSize;
  newSize[0] = 2*size[0]+4*m_WMargin + gInterface->w();
  newSize[1] = std::max(static_cast<int>(size[1]+2*m_HMargin), 2*m_HMargin+gInterface->h());

  return newSize;
}


void VectorizationViewGUI::resize(int x, int y, int w, int h)
{
  int m_WMargin = 25;
  int m_HMargin = 25;

  int interfacew = gInterface->w();
  int interfaceh = gInterface->h();

  this->Fl_Group::resize(x, y, w, h);
  int imagew = (w-4*m_WMargin-interfacew)/2;
  int imageh = h-2*m_HMargin;

  gBinaryWidgetGroup->resize(x+m_WMargin, y+m_HMargin, imagew, imageh);
  if (m_BinaryWidget.IsNotNull())
  {
    m_BinaryWidget->resize(x+m_WMargin, y+m_HMargin, imagew, imageh);
  }
  gOriginalWidgetGroup->resize(x+2*m_WMargin+imagew, y+m_HMargin, imagew, imageh);
  if (m_OriginalWidget.IsNotNull())
  {
    m_OriginalWidget->resize(x+2*m_WMargin+imagew, y+m_HMargin, imagew, imageh);
  }

  int interfacex = 3*m_WMargin + 2 * imagew;
  int interfacey = (h-interfaceh)/2;

  gInterface->resize(x+interfacex, y+interfacey, interfacew, interfaceh);
}

void VectorizationViewGUI::Notify()
{
  if (m_VectorizationModel->GetValid())
  {
    this->RefreshViewers();
  }
}

void VectorizationViewGUI::ToleranceCallback()
{
  m_VectorizationController->ChangeTolerance(vTolerance->value());
}
}
