/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbVectorizationViewGUI_h
#define __otbVectorizationViewGUI_h

// Disabling deprecation warning
#ifdef _MSC_VER
#pragma warning(push)
#pragma warning(disable:4996)
#endif
#include "otbVectorizationViewGroup.h"
#ifdef _MSC_VER
#pragma warning(pop)
#endif
#include "otbListenerBase.h"
#include "otbVectorizationModel.h"
#include "otbVectorizationControllerInterface.h"
#include "otbFullResolutionImageWidget.h"
#include "itkMinimumMaximumImageCalculator.h"
#include "otbImageToVectorImageCastFilter.h"
#include "itkObject.h"
#include "otbImageWidgetCircleForm.h"
#include "otbImageWidgetPolygonForm.h"

namespace otb
{
/** \class VectorizationViewGUI
 *
 */
class VectorizationViewGUI
      : public ListenerBase, public VectorizationViewGroup, public itk::Object
{
public:
  /** Standard class typedefs */
  typedef VectorizationViewGUI          Self;
  typedef itk::Object                   Superclass;
  typedef itk::SmartPointer<Self>       Pointer;
  typedef itk::SmartPointer<const Self> ConstPointer;

  // Pixel type
  typedef double        PixelType;
  typedef unsigned char UCharPixelType;
  typedef otb::FullResolutionImageWidget<PixelType>                              ImageWidgetType;
  typedef ImageWidgetType::Pointer                                               ImageWidgetPointerType;
  typedef otb::FullResolutionImageWidget<UCharPixelType>                         UCharImageWidgetType;
  typedef UCharImageWidgetType::Pointer                                          UCharImageWidgetPointerType;
  typedef UCharImageWidgetType::ImageType                                        UCharImageType;
  typedef ImageWidgetType::ImageType                                             ImageType;
  typedef ImageType::SizeType                                                    SizeType;
  typedef ImageType::PixelType                                                   VectorPixelType;
  typedef VectorizationModel                                                     VectorizationModelType;
  typedef VectorizationModelType::ImageType                                      SingleImageType;
  typedef VectorizationModelType::UCharImageType                                 UCharSingleImageType;
  typedef VectorizationModelType::PolygonType                                    PolygonType;
  typedef VectorizationModelType::PolygonListType                                PolygonListType;
  typedef itk::MinimumMaximumImageCalculator<SingleImageType>                    MinMaxCalculatorType;
  typedef otb::ImageToVectorImageCastFilter<SingleImageType, ImageType>           VectorCastFilterType;
  typedef VectorCastFilterType::Pointer                                          VectorCastFilterPointerType;
  typedef otb::ImageToVectorImageCastFilter<UCharSingleImageType, UCharImageType> UCharVectorCastFilterType;
  typedef UCharVectorCastFilterType::Pointer                                     UCharVectorCastFilterPointerType;

  typedef ImageWidgetType::FormListType FormListType;
  typedef FormListType::Pointer FormListPointerType;
  typedef FormListType::Iterator FormListIteratorType;
  typedef ImageWidgetCircleForm ImageWidgetCircleFormType;
  typedef ImageWidgetCircleFormType::Pointer ImageWidgetCircleFormPointerType;
  typedef ImageWidgetCircleFormType::ColorType ColorType;
  typedef ImageWidgetPolygonForm<double> ImageWidgetPolygonFormType;
  typedef ImageWidgetPolygonFormType::Pointer ImageWidgetPolygonFormPointerType;


  /** Standards macros */
  itkNewMacro(Self);
  itkTypeMacro(VectorizationViewGUI, Object);

  /** Set the controller */
  itkSetObjectMacro(VectorizationController, VectorizationControllerInterface);

  /** Event from the model */
  virtual void Notify();

  /** Buttons callback */
  virtual void ToleranceCallback();

  /** Handle the resize event.*/
  virtual void resize(int x, int y, int w, int h);
  virtual void show();

  /** Display full images */
  virtual SizeType GetIdealSize();

protected:
  /** Constructor */
  VectorizationViewGUI();
  /** Destructor */
  ~VectorizationViewGUI() {};

  /** Refresh the image widgets */
  virtual void RefreshViewers();

private:
  VectorizationViewGUI(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

  /** Pointer to the model */
  VectorizationModel::Pointer m_VectorizationModel;
  /** Pointer to the controller */
  VectorizationControllerInterface::Pointer m_VectorizationController;
  /** Smoothing widget */
  UCharImageWidgetPointerType m_BinaryWidget;
  /** Edge enhancement */
  ImageWidgetPointerType  m_OriginalWidget;
  /** Cast the smoothed image to vector image */
  UCharVectorCastFilterPointerType m_BinaryVectorCastFilter;
  /** Cast the edge enhanced image to vector image */
  VectorCastFilterPointerType m_OriginalVectorCastFilter;

  /** margins for the group */
  int m_WMargin;
  int m_HMargin;

  ColorType m_PolygonColor;
  ColorType m_PointColor;

};
}//end namespace otb

#endif
