#include "otbPreprocessingController.h"
#include "otbPreprocessingViewGUI.h"
#include "otbImageFileReader.h"
#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>

#include "otbI18n.h"

int main(int argc, char* argv[])
{
  otbI18nMacro();

  typedef otb::PreprocessingModel ModelType;
  typedef ModelType::ImageType ImageType;
  typedef otb::PreprocessingController ControllerType;
  typedef otb::PreprocessingViewGUI ViewType;
  typedef otb::ImageFileReader<ImageType> ReaderType;

  ReaderType::Pointer reader = ReaderType::New();
  reader->SetFileName(argv[1]);
  reader->Update();

  ControllerType::Pointer controller = ControllerType::New();

  Fl_Double_Window window(450, 330);
  ViewType::Pointer view = ViewType::New();
  controller->SetView(view);
  view->SetPreprocessingController(controller);
  view->resize(0, 0, 450, 330);
  window.resizable(view);
  window.end();
  window.show();
  view->show();

  ModelType::Pointer model = otb::PreprocessingModel::GetInstance();
  model->SetInputImage(reader->GetOutput());

  Fl::run();

  window.remove(view.GetPointer());

  return EXIT_SUCCESS;
}
