#include "otbPreprocessingController.h"
#include "otbPreprocessingViewGUI.h"
#include "otbRegionGrowingSegmentationController.h"
#include "otbRegionGrowingSegmentationViewGUI.h"
#include "otbVectorizationController.h"
#include "otbVectorizationViewGUI.h"
#include "otbImageFileReader.h"
#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>

#include "otbI18n.h"

int main(int argc, char* argv[])
{
  otbI18nMacro();

  typedef otb::PreprocessingModel PreprocessingModelType;
  typedef otb::RegionGrowingSegmentationModel RegionGrowingModelType;
  typedef otb::VectorizationModel VectorizationModelType;
  typedef PreprocessingModelType::ImageType ImageType;
  typedef otb::PreprocessingController PreprocessingControllerType;
  typedef otb::PreprocessingViewGUI PreprocessingViewType;
  typedef otb::RegionGrowingSegmentationController RegionGrowingControllerType;
  typedef otb::RegionGrowingSegmentationViewGUI    RegionGrowingViewType;
  typedef otb::VectorizationController VectorizationControllerType;
  typedef otb::VectorizationViewGUI    VectorizationViewType;
  typedef otb::ImageFileReader<ImageType> ReaderType;

  ReaderType::Pointer reader = ReaderType::New();
  reader->SetFileName(argv[1]);
  reader->Update();

  PreprocessingControllerType::Pointer controller1 = PreprocessingControllerType::New();
  RegionGrowingControllerType::Pointer controller2 = RegionGrowingControllerType::New();
  VectorizationControllerType::Pointer controller3 = VectorizationControllerType::New();

  Fl_Double_Window window1(670, 200,"Preprocessing");
  PreprocessingViewType::Pointer view1 = PreprocessingViewType::New();
  controller1->SetView(view1);
  view1->SetPreprocessingController(controller1);
  view1->resize(0, 0, 670, 200);
  window1.resizable(view1);
  window1.end();
  window1.show();
  view1->show();

  Fl_Double_Window window2(670, 350,"Region growing segmentation");
  RegionGrowingViewType::Pointer view2 = RegionGrowingViewType::New();
  controller2->SetView(view2);
  view2->SetRegionGrowingSegmentationController(controller2);
  view2->resize(0, 0, 670, 350);
  window2.resizable(view2);
  window2.end();
  window2.show();
  view2->show();


  Fl_Double_Window window3(670, 200,"Vectorization");
  VectorizationViewType::Pointer view3 = VectorizationViewType::New();
  controller3->SetView(view3);
  view3->SetVectorizationController(controller3);
  view3->resize(0, 0, 670, 200);
  window3.resizable(view3);
  window3.end();
  window3.show();
  view3->show();

//  otb::PreprocessingModel::GetInstance()->RegisterListener(otb::RegionGrowingSegmentationModel::GetInstance());

  PreprocessingModelType::Pointer model1 = otb::PreprocessingModel::GetInstance();
  RegionGrowingModelType::Pointer model2 = otb::RegionGrowingSegmentationModel::GetInstance();
  VectorizationModelType::Pointer model3 = otb::VectorizationModel::GetInstance();

  model1->SetInputImage(reader->GetOutput());
  model2->SetInputImage(model1->GetOutputImage());
  model3->SetReferenceImage(reader->GetOutput());
  model3->SetSegmentationImage(model2->GetOutputImage());
  model1->RegisterListener(model2);
  model2->RegisterListener(model3);

  ImageType::SizeType size1= view1->GetIdealSize();
  ImageType::SizeType size2= view2->GetIdealSize();
  ImageType::SizeType size3= view3->GetIdealSize();

  window1.resize(0, 0, size1[0], size1[1]);
  window2.resize(window1.x(), window1.y(), size2[0], size2[1]);
  window3.resize(window2.x(), window2.y(), size3[0], size3[1]);

  view1->show();
  view2->show();
  view3->show();

  Fl::run();

  window1.remove(view1.GetPointer());
  window2.remove(view2.GetPointer());
  window3.remove(view3.GetPointer());

  return EXIT_SUCCESS;
}
