#include "itkExceptionObject.h"
#include "otbSegmentationApplicationModel.h"
#include "otbSegmentationApplicationViewGUI.h"
#include "otbSegmentationApplicationController.h"
#include "otbCommandLineArgumentParser.h"
#include <FL/Fl.H>

#include "otbI18n.h"

using namespace otb;

int main(int argc, char* argv[])
{
  otbI18nMacro();

  // Parse command line parameters
  typedef otb::CommandLineArgumentParser ParserType;
  ParserType::Pointer parser = ParserType::New();

  parser->AddInputImage(false); //Optionnal parameter

  typedef otb::CommandLineArgumentParseResult ParserResultType;
  ParserResultType::Pointer  parseResult = ParserResultType::New();

  try
  {
    parser->ParseCommandLine(argc, argv, parseResult);
  }
  catch ( itk::ExceptionObject & err )
  {
    std::string descriptionException = err.GetDescription();
    if (descriptionException.find("ParseCommandLine(): Help Parser") != std::string::npos)
    {
      std::cout << "WARNING : output file pixels are converted in 'unsigned char'" << std::endl;
      return EXIT_SUCCESS;
    }
    if (descriptionException.find("ParseCommandLine(): Version Parser") != std::string::npos)
    {
      return EXIT_SUCCESS;
    }
    return EXIT_FAILURE;
  }


  SegmentationApplicationModel::Pointer model = SegmentationApplicationModel::GetInstance();
  SegmentationApplicationController::Pointer controller = SegmentationApplicationController::New();
  SegmentationApplicationViewGUI::Pointer view = SegmentationApplicationViewGUI::New();
  view->SetSegmentationApplicationController(controller);
  controller->SetView(view);
  if ( parseResult->IsOptionInputImagePresent() )
    model->SetFileName(parseResult->GetInputImage().c_str());

  if (!parseResult->IsOptionOTBTestingPresent())
    Fl::run();

  return EXIT_SUCCESS;
}
