/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include "otbStereoscopicApplicationController.h"

#include "otbFltkFilterWatcher.h"
#include <FL/fl_ask.H>

namespace otb
{

StereoscopicApplicationController::StereoscopicApplicationController()
{
}

StereoscopicApplicationController::~StereoscopicApplicationController()
{}

void StereoscopicApplicationController::OpenStereoCouple(const char * left, const char * right)
{
  unsigned int index = 0;
  try
  {
    index = m_Model->AddCouple(left, right);
  }
  catch (itk::ExceptionObject & err)
  {
    const char* msg = err.GetDescription();
    fl_alert("%s", msg);
  }

  FltkFilterWatcher watcher1(m_Model->GetLeftShrinkFilter(index), m_View->wMainWindow->x(), m_View->wMainWindow->y(), 200, 20, "Loading left image ...");
  FltkFilterWatcher watcher2(m_Model->GetRightShrinkFilter(index), m_View->wMainWindow->x(), m_View->wMainWindow->y(), 200, 20, "Loading right image ...");

  try
  {
    m_Model->GenerateQuicklooksAndNormalizationFactors(index);
  }
  catch (itk::ExceptionObject & err)
  {
    const char* msg = err.GetDescription();
    fl_alert("%s", msg);
  }

  // Updating couples lists
  itk::OStringStream oss;
  oss.str("");


  oss<<"Couple "<<m_Model->GetNumberOfStereoscopicCouples();
  m_View->lMainCouples->add(oss.str().c_str());
  m_View->lInsightCouples->add(oss.str().c_str());
}

void StereoscopicApplicationController::IndexClicked(const IndexType & index)
{
  IndexType newIndex;
  newIndex[0]=m_View->m_Scroll->GetSubSamplingRate()*index[0];
  newIndex[1]=m_View->m_Scroll->GetSubSamplingRate()*index[1];

  m_View->m_Viewer->SetViewedRegionCenter(newIndex);
  m_View->ResetViewer();
  m_View->RefreshViewer();
}

void StereoscopicApplicationController::ChangeNormalizationFactor(double factor)
{
  m_Model->SetClampThreshold(factor/100.);
  m_View->ResetViewer();
  m_View->RefreshViewer();
}


} // end namespace otb
