/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbStereoscopicApplicationController_h
#define __otbStereoscopicApplicationController_h

#include "otbStereoscopicApplicationControllerInterface.h"
#include "otbStereoscopicApplicationViewGUI.h"

namespace otb
{
class StereoscopicApplicationController
      : public StereoscopicApplicationControllerInterface
{
public:
  /** Standard class typedefs */
  typedef StereoscopicApplicationController Self;
  typedef itk::Object                       Superclass;
  typedef itk::SmartPointer<Self>           Pointer;
  typedef itk::SmartPointer<const Self>     ConstPointer;

  typedef otb::StereoscopicApplicationViewGUI ViewType;
  typedef ViewType::Pointer                   ViewPointerType;
  typedef StereoscopicApplicationControllerInterface::IndexType IndexType;

  /** Standard type macros */
  itkTypeMacro(StereoscopicApplicationController, Superclass);
  itkNewMacro(Self);

  /** Set the view */
  itkSetObjectMacro(View, ViewType);

  /** User action */
  virtual void OpenStereoCouple(const char * left, const char * right);

  virtual void IndexClicked(const IndexType & index);
  virtual void ChangeNormalizationFactor(double factor);

protected:
  /** Constructor */
  StereoscopicApplicationController();
  /** Destructor */
  ~StereoscopicApplicationController();

private:
  StereoscopicApplicationController(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

  /** Pointer to the view */
  ViewPointerType m_View;

};
} //end namespace otb

#endif
