/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbStereoscopicApplicationControllerInterface_h
#define __otbStereoscopicApplicationControllerInterface_h

#include "itkObject.h"
#include "otbStereoscopicApplicationModel.h"

namespace otb
{
class StereoscopicApplicationControllerInterface
      : public itk::Object
{
public:
  /** Standard class typedefs */
  typedef StereoscopicApplicationControllerInterface Self;
  typedef itk::Object                      Superclass;
  typedef itk::SmartPointer<Self>          Pointer;
  typedef itk::SmartPointer<const Self>    ConstPointer;

  typedef StereoscopicApplicationModel::ImageType::IndexType IndexType;

  /** Standard type macros */
  itkTypeMacro(StereoscopicApplicationControllerInterface, Superclass);

  /** Users actions */
  virtual void OpenStereoCouple(const char * left, const char * right) = 0;
  virtual void IndexClicked(const IndexType & index)=0;
  virtual void ChangeNormalizationFactor(double factor)=0
;
protected:
  /** Constructor */
  StereoscopicApplicationControllerInterface()
  {
    m_Model = StereoscopicApplicationModel::GetInstance();
  }
  /** Destructor */
  ~StereoscopicApplicationControllerInterface() {}

  /** The instance of the model (for commodity) */
  StereoscopicApplicationModel * m_Model;

private:
  StereoscopicApplicationControllerInterface(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented
};
} // end namespace otb

#endif
