/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbUrbanAreaExtractionController_h
#define __otbUrbanAreaExtractionController_h

#include "otbUrbanAreaExtractionControllerInterface.h"
#include "otbUrbanAreaExtractionViewGUI.h"
#include "otbWidgetResizingActionHandler.h"
#include "otbChangeScaledExtractRegionActionHandler.h"
#include "otbChangeExtractRegionActionHandler.h"
#include "otbChangeScaleActionHandler.h"
#include "otbSelectAreaActionHandler.h"
#include "otbArrowKeyMoveActionHandler.h"
#include "otbChangeOpacityActionHandler.h"
#include "otbPixelDescriptionActionHandler.h"
#include "otbPixelDescriptionModel.h"
#include "otbPixelDescriptionView.h"

namespace otb
{
class ITK_EXPORT UrbanAreaExtractionController
      : public UrbanAreaExtractionControllerInterface
{
public:
  /** Standard class typedefs */
  typedef UrbanAreaExtractionController            Self;
  typedef UrbanAreaExtractionControllerInterface   Superclass;
  typedef itk::SmartPointer<Self>                  Pointer;
  typedef itk::SmartPointer<const Self>            ConstPointer;
  
  typedef UrbanAreaExtractionViewGUI        ViewType;
  typedef ViewType::Pointer                 ViewPointerType;
  typedef Superclass::InputImageType        InputImageType;
  typedef Superclass::InputImagePointerType InputImagePointerType;
  typedef Superclass::ModelType             ModelType;
  typedef ModelType::IndexType              IndexType;
  typedef ModelType::SizeType               SizeType;
  typedef ModelType::OffsetType             OffsetType;
  typedef ModelType::ColorType              ColorType;
  
  /** Visualization typedefs*/
  // Model and view
  typedef ModelType::FocusingModelType           FocusingModelType;
  typedef FocusingModelType::Pointer             FocusingModelPointerType;
  typedef ViewType::FocusingViewType             FocusingViewType;
  typedef FocusingViewType::Pointer              FocusingViewPointerType;
  typedef Superclass::FocusingControllerType     FocusingControllerType;
  typedef FocusingControllerType::Pointer        FocusingControllerPointerType;
  typedef ModelType::VisuModelType               VisuModelType;
  typedef VisuModelType::Pointer                 VisuModelPointerType;
  typedef FocusingViewType::ImageViewType        VisuViewType;
  typedef VisuViewType::Pointer                  VisuViewPointerType;
  typedef VisuViewType::ImageWidgetType          VisuImageWidgetType;
  typedef VisuImageWidgetType::Pointer           VisuImageWidgetPointerType;
  typedef FocusingControllerType::ControllerType VisuControllerType;
  typedef VisuControllerType::Pointer            VisuControllerPointerType;

  // Handlers
  typedef WidgetResizingActionHandler<VisuModelType, VisuViewType>             ResizingHandlerType;
  typedef ChangeScaledExtractRegionActionHandler<VisuModelType, VisuViewType>  ChangeScaledRegionHandlerType;
  typedef ChangeExtractRegionActionHandler<VisuModelType, VisuViewType>        ChangeRegionHandlerType;
  typedef ChangeScaleActionHandler<VisuModelType, VisuViewType>                ChangeScaleHandlerType;
  typedef SelectAreaActionHandler<FocusingModelType, VisuImageWidgetType>     SelectAreaHandlerType;
  typedef ArrowKeyMoveActionHandler<VisuModelType, VisuViewType>              ArrowKeyMoveActionHandlerType;
  typedef ChangeOpacityActionHandler<FocusingModelType, FocusingViewType>      ChangeOpacityHandlerType;

  typedef FocusingModelType::OutputImageType                                  OutputImageType;
  typedef PixelDescriptionModel<OutputImageType>                              PixelDescriptionModelType;
  typedef PixelDescriptionActionHandler<PixelDescriptionModelType, VisuViewType>
                                                                              PixelDescriptionActionHandlerType;
  typedef PixelDescriptionView<PixelDescriptionModelType>                     PixelDescriptionViewType;
  typedef PixelDescriptionViewType::Pointer PixelDescriptionViewPointerType;


  typedef ResizingHandlerType::Pointer                                        ResizingHandlerPointerType;
  typedef ChangeScaledRegionHandlerType::Pointer                              ChangeScaledRegionHandlerPointerType;
  typedef ChangeRegionHandlerType::Pointer                                    ChangeRegionHandlerPointerType;
  typedef ChangeScaleHandlerType::Pointer                                     ChangeScaleHandlerPointerType;
  typedef SelectAreaHandlerType::Pointer                                      SelectAreaHandlerPointerType;
  typedef ArrowKeyMoveActionHandlerType::Pointer                              ArrowKeyMoveActionHandlerPointerType;
  typedef PixelDescriptionActionHandlerType::Pointer                          PixelDescriptionActionHandlerPointerType;


  /** Standard type macros */
  itkTypeMacro(UrbanAreaExtractionController, Superclass);
  itkNewMacro(Self);

  VisuControllerPointerType GetMasterController()
  {
    return m_FocusingController->GetMasterController();
  }
  VisuControllerPointerType GetWorkingController()
  {
    return m_FocusingController->GetWorkingController();
  }
  virtual FocusingControllerPointerType GetFocusingController()
  {
    return m_FocusingController;
  }


  /** Get/Set the view */
  itkSetObjectMacro(View, ViewType);
  itkGetObjectMacro(View, ViewType);

  void AddHandlers();

  virtual void OpenInputImage(const char * filename);
  virtual void OpenInputVector( const char * filename );
  virtual void SaveImageOnExtract(const char * filename);
  virtual void SaveImageOnFull(const char * filename);
  virtual void SaveVectorOnExtract(const char * filename);
  virtual void SaveVectorOnFull(const char * filename);
  virtual void Run();
  virtual void SetThreshold(double value);
  virtual void SetThresholdDensity(double value);
  virtual void SetSobelLowerThreshold(double value);
  virtual void SetSobelUpperThreshold(double value);
  virtual void SetCurrentPolygonIndex(unsigned int index);
  virtual void ClearAllPolygons();
  virtual void DeletePolygon(unsigned int index);
  virtual void SetAlpha(double alpha);
  virtual void SetIndices(unsigned int green, unsigned int red , unsigned int nir);
  virtual void SetMinDetail(unsigned int value);
  virtual void SetSubSampleLevel(unsigned int value);
  virtual void SetVectorDisplayed(bool value);
  virtual void UpdateRGBChannelOrder(unsigned int red, unsigned int green, unsigned int blue);
  virtual void UpdateGrayscaleChannel(unsigned int channel);


protected:
  /** Constructor */
  UrbanAreaExtractionController();
  /** Destructor */
  virtual ~UrbanAreaExtractionController();

private:
  UrbanAreaExtractionController(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

  /** Pointer to the view */
  ViewPointerType m_View;
  /***/
  FocusingControllerPointerType m_FocusingController;
  /***/
  PixelDescriptionModelType::Pointer m_PixelModel;
  PixelDescriptionViewType::Pointer m_PixelView;
  PixelDescriptionModelType::Pointer m_PixelModelWorking;
  PixelDescriptionViewType::Pointer m_PixelViewWorking;


};
} //end namespace otb

#endif
