/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbUrbanAreaExtractionControllerInterface_h
#define __otbUrbanAreaExtractionControllerInterface_h

#include "otbUrbanAreaExtractionModel.h"
#include "itkProcessObject.h"
#include "otbFocusingController.h"

namespace otb
{
class ITK_EXPORT  UrbanAreaExtractionControllerInterface
: public itk::ProcessObject
{
public:
  /** Standard class typedefs */
  typedef UrbanAreaExtractionControllerInterface  Self;
  typedef itk::ProcessObject                      Superclass;
  typedef itk::SmartPointer<Self>                 Pointer;
  typedef itk::SmartPointer<const Self>           ConstPointer;

  typedef UrbanAreaExtractionModel                 ModelType;
  typedef ModelType::InputImageType                InputImageType;
  typedef ModelType::InputImagePointerType         InputImagePointerType;
  typedef ModelType::IndexType                     IndexType;
  typedef ModelType::SizeType                      SizeType;

  typedef ModelType::FocusingModelType             FocusingModelType;
  typedef ModelType::FocusingViewType              FocusingViewType;

/** NewVisu */
  typedef FocusingController<FocusingModelType, FocusingViewType>   FocusingControllerType;
  typedef FocusingControllerType::Pointer          FocusingControllerPointerType;


  /** Standard type macros */
  itkTypeMacro(UrbanAreaExtractionControllerInterface, Superclass);

  /** Users actions */
  virtual void OpenInputImage(const char * filename) =0;
  virtual void OpenInputVector(const char * filename) =0;
  virtual void SaveImageOnExtract(const char * filename) =0;
  virtual void SaveImageOnFull(const char * filename) =0;
  virtual void SaveVectorOnExtract(const char * filename) =0;
  virtual void SaveVectorOnFull(const char * filename) =0;
  virtual void Run() =0;
  virtual void SetThreshold(double value)=0;
  virtual void SetThresholdDensity(double value)=0;
  virtual void SetSobelLowerThreshold(double value)=0;
  virtual void SetSobelUpperThreshold(double value)=0;
  virtual void SetCurrentPolygonIndex(unsigned int index)=0;
  virtual void ClearAllPolygons()=0;
  virtual void DeletePolygon(unsigned int index)=0;
  virtual void SetAlpha(double alpha)=0;
  virtual void SetIndices(unsigned int green, unsigned int red, unsigned int nir)=0;
  virtual void SetMinDetail(unsigned int value)=0;
  virtual void SetSubSampleLevel(unsigned int value)=0;
  virtual void SetVectorDisplayed(bool value)=0;
  virtual void UpdateRGBChannelOrder(unsigned int red, unsigned int green, unsigned int blue)=0;
  virtual void UpdateGrayscaleChannel(unsigned int channel)=0;


/** NewVisu */
  virtual FocusingControllerPointerType GetFocusingController()=0;

protected:
  /** Constructor */
  UrbanAreaExtractionControllerInterface()
  {
    m_Model = ModelType::GetInstance();
  }
  /** Destructor */
  ~UrbanAreaExtractionControllerInterface() {};

  /** The instance of the model (for commodity) */
  ModelType * m_Model;

private:
  UrbanAreaExtractionControllerInterface(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented
};
} // end namespace otb

#endif
