/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbColoredPoly_h
#define __otbColoredPoly_h

#include "itkObject.h"
#include "itkSmartPointer.h"
#include "otbMacro.h"

namespace otb
{

template <class TLabel, class TColor>
class ColoredPoly
      : public itk::Object
{
public:
  /// Standard class typedefs
  typedef ColoredPoly Self;
  typedef itk::Object Superclass;
  typedef itk::SmartPointer<Self> Pointer;
  typedef itk::SmartPointer<const Self> ConstPointer;
  typedef std::vector<unsigned int> ROIIndexVectorType;

  /// Standard macros
  itkNewMacro(Self);
  itkTypeMacro(ColoredPoly, Object);

  /// Label type
  typedef TLabel LabelType;
  /// Color type
  typedef TColor ColorType;

  itkGetMacro(Id, LabelType);
  itkSetMacro(Id, LabelType);
  itkGetMacro(Color, ColorType);
  itkSetMacro(Color, ColorType);

protected:
  /// Constructor
  ColoredPoly() {};
  /// Destructor
  ~ColoredPoly() {};

private:
  ColoredPoly(const Self&); // purposely not implemented
  void operator=(const Self&); // purposely not implemented


  /// ColoredPoly id
  LabelType m_Id;
  /// Color
  ColorType m_Color;
};

} // end namespace otb

#endif
