/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbUrbanAreaExtractionModel_h
#define __otbUrbanAreaExtractionModel_h

#include "otbMVCModel.h"
#include "otbListenerBase.h"
#include "otbImage.h"
#include "otbVectorImage.h"
#include "itkIntensityWindowingImageFilter.h"
#include "otbStreamingShrinkImageFilter.h"
#include "itkExpandImageFilter.h"
#include "otbMultiChannelExtractROI.h"
#include "otbImageFileReader.h"
#include "otbObjectList.h"
#include "otbUrbanAreaDetectionImageFilter.h"

#include "itkRGBAPixel.h"
#include "otbImageLayer.h"
#include "otbImageLayerGenerator.h"
#include "otbImageLayerRenderingModel.h"
#include "otbFocusingView.h"
#include "otbFocusingModel.h"

#include "otbUrbanAreaPolygonsManager.h"

#include "otbImageLayerGenerator.h"
#include "otbImageLayer.h"

#include "otbVectorDataFileReader.h"
#include "otbVectorDataFileWriter.h"
#include "itkImageRegionSplitter.h"
#include "otbStreamingTraits.h"

namespace otb
{
/** \class UrbanAreaExtractionModel
 *
 *
 *
 */

typedef enum
{
  NO_IMAGE,
  IMAGE_OPENED,
}
StateType;

typedef enum
{
  EXTRACT,
  BANDEAUX,
  FULL_IMAGE
}
ImageToWorkType;


class ITK_EXPORT UrbanAreaExtractionModel
      : public MVCModel<ListenerBase>, public itk::Object
{

public:
  /** Standard class typedefs */
  typedef UrbanAreaExtractionModel                       Self;
  typedef MVCModel<ListenerBase>    Superclass;
  typedef itk::SmartPointer<Self>                        Pointer;
  typedef itk::SmartPointer<const Self>                  ConstPointer;

  /** Standard type macro */
  itkTypeMacro(UrbanAreaExtractionModel, MVCModel);

  /** Input Image Type */
  typedef double                                         PixelType;
  typedef Image<PixelType, 2>                             SingleImageType;
  typedef VectorImage<PixelType, 2>                       InputImageType;
  typedef InputImageType::Pointer                        InputImagePointerType;
  typedef InputImageType::PixelType                      InputImagePixelType;
  typedef InputImageType::IndexType                      IndexType;
  typedef InputImageType::SizeType                       SizeType;
  typedef InputImageType::RegionType                     RegionType;
  typedef InputImageType::OffsetType                     OffsetType;

  typedef unsigned int                                   LabelPixelType;
  typedef Image<LabelPixelType, 2>                       LabeledImageType;

  /** Vector Image Output  */
  typedef VectorImage<PixelType, 2>                       OutputImageType;
  typedef OutputImageType::PixelType                     OutputImagePixelType;
  typedef OutputImageType::Pointer                       OutputImagePointerType;


  typedef itk::RGBAPixel<unsigned char>                   RGBPixelType;
  typedef otb::Image<RGBPixelType, 2>                     RGBImageType;

  /** Focusing viewer typedefs */
  typedef FocusingModel<InputImageType, RGBImageType>          FocusingModelType;
  typedef FocusingModelType::Pointer                          FocusingModelPointerType;
  typedef FocusingView<FocusingModelType>                     FocusingViewType;
  typedef FocusingViewType::Pointer                           FocusingViewPointerType;
  typedef FocusingModelType::ImageLayerRenderingModelType     VisuModelType;
  typedef VisuModelType::Pointer                              VisuModelPointerType;
  typedef FocusingViewType::ImageViewType                     VisuViewType;
  typedef VisuViewType::Pointer                               VisuViewPointerType;
  typedef FocusingModelType::ColorType                        ColorType;

  typedef UrbanAreaPolygonsManager<InputImageType>            PolygonsManagerType;
  typedef PolygonsManagerType::Pointer                        PolygonsManagerPointerType;
  typedef PolygonsManagerType::PolygonListType                PolygonListType;
  typedef PolygonsManagerType::PolygonListPointerType         PolygonListPointerType;
  typedef PolygonsManagerType::ColoredPolyListType            ColoredPolyListType;
  typedef PolygonsManagerType::ColoredPolyListPointerType     ColoredPolyListPointerType;
  typedef PolygonsManagerType::VectorDataType                 VectorDataType;
  typedef VectorDataType::Pointer                             VectorDataPointerType;
  typedef VectorDataType::DataTreeType                        DataTreeType;
  typedef VectorDataType::DataNodeType                        DataNodeType;
  typedef DataNodeType::Pointer                               DataNodePointerType;

  /** Model Filters  */
  // UrbanAreaDetectionFilter
  typedef UrbanAreaDetectionImageFilter<InputImageType, LabeledImageType>  UrbanAreaDetectionFilterType;
  typedef UrbanAreaDetectionFilterType::Pointer                           UrbanAreaDetectionFilterPointerType;
  // Resamplers
  typedef StreamingShrinkImageFilter<InputImageType, InputImageType>       StreamingShrinkImageFilterType;
  typedef StreamingShrinkImageFilterType::Pointer                         StreamingShrinkImageFilterPointerType;
  typedef itk::ExpandImageFilter<LabeledImageType, LabeledImageType>       ExpandImageFilterType;
  typedef ExpandImageFilterType::Pointer                                  ExpandImageFilterPointerType;
  // ExtractROI
  typedef MultiChannelExtractROI<PixelType, PixelType>                     ExtractROIFilterType;
  typedef ExtractROIFilterType::Pointer                                   ExtractROIFilterPointerType;
  // Splitter
  typedef itk::ImageRegionSplitter<2>                                     SplitterType;
  typedef SplitterType::Pointer                                           SplitterPointerType;
  typedef StreamingTraits<InputImageType>                                 StreamingTraitsType;


  /** Layer typedefs */
  typedef ImageLayer<LabeledImageType, RGBImageType>       LayerType;
  typedef ImageLayerGenerator<LayerType>                   LayerGeneratorType;

  typedef PolygonsManagerType::VectorImageType             VectorImageType;
  typedef VectorImageType::PixelType                       VectorPixelType;
  typedef ImageLayer<VectorImageType, RGBImageType>        VectorLayerType;
  typedef ImageLayerGenerator<VectorLayerType>             VectorLayerGeneratorType;
  typedef FocusingModelType::RenderingFunctionType         RenderingFunctionType;
  typedef RenderingFunctionType::Pointer                   RenderingFunctionPointerType;
  typedef FocusingModelType::StandardRenderingFunctionType StandardRenderingFunctionType;
  typedef StandardRenderingFunctionType::Pointer           StandardRenderingFunctionPointerType;


  /** Reader */
  typedef ImageFileReader<InputImageType>                  ReaderType;
  typedef ReaderType::Pointer                              ReaderPointerType;
  typedef VectorDataFileReader<VectorDataType>             VectorDataReaderType;
  typedef VectorDataReaderType::Pointer                    VectorDataReaderPointerType;

  /** Writers */
  typedef VectorDataFileWriter<VectorDataType>             VectorDataWriterType;

  /** Enum def */
  typedef ImageToWorkType                                  WhichImageType;


  /** Get the unique instanc1e of the model */
  static Pointer GetInstance();

  /** Input Image Pointer */
  itkSetObjectMacro(InputImage, InputImageType);
  itkGetObjectMacro(InputImage, InputImageType);
  /** Get/Set Image Filename */
  itkSetMacro(ImageFilename, std::string);
  itkGetMacro(ImageFilename, std::string);
  /** Get the Output Image */
  itkGetObjectMacro(OutputImage, OutputImageType);
  /** Get the Reader */
  itkGetObjectMacro(Reader, ReaderType);
  /** Get/Set FocusingModel */
  itkSetObjectMacro(FocusingModel, FocusingModelType);
  itkGetObjectMacro(FocusingModel, FocusingModelType);
  /** Get/Set PolygonsManager */
  itkSetObjectMacro(PolygonsManager, PolygonsManagerType);
  itkGetObjectMacro(PolygonsManager, PolygonsManagerType);
  /** Get/Set Loaded Vector */
  itkSetObjectMacro(LoadedVector, VectorDataType);
  itkGetObjectMacro(LoadedVector, VectorDataType);


  /** Get/Set the rendering Function */
  RenderingFunctionPointerType GetRenderingFunction(){
    return m_FocusingModel->GetRenderingFunction();
  }


  /** Open Input Vector */
  void SetVectorFilename(std::string /*strfilename*/){};

  /** Get the polygons List */
  PolygonListPointerType GetPolygonsList();
  /** Get the colored polygons List */
  ColoredPolyListPointerType GetColoredPolyList();

  VisuModelPointerType GetMasterModel(){
    return m_FocusingModel->GetMasterModel();
  }
  VisuModelPointerType GetWorkingModel(){
    return m_FocusingModel->GetWorkingModel();
  }

  void SetAlpha(double alpha);
  itkGetMacro(HasNewInput, bool);
  itkGetMacro(State, StateType);
  itkGetMacro(VectorDisplayed, bool);
  itkSetMacro(VectorDisplayed, bool);
  itkSetMacro(ThresholdValue, double);
  itkSetMacro(ThresholdDensity, double);
  itkSetMacro(SobelLowerThreshold, double);
  itkSetMacro(SobelUpperThreshold, double);
  itkSetMacro(SubSampleLevel, unsigned int);
  itkSetMacro(MinRegionSize, unsigned int);
  itkSetMacro(SavedIndex, IndexType);
  itkGetMacro(SavedIndex, IndexType);
  itkSetMacro(LoadedVDIndex, unsigned int);
  itkGetMacro(LoadedVDIndex, unsigned int);
  itkSetMacro(GreenIndex, unsigned int);
  itkGetMacro(GreenIndex, unsigned int);
  itkGetMacro(RedIndex, unsigned int);
  itkSetMacro(RedIndex, unsigned int);
  itkSetMacro(NIRIndex, unsigned int);
  itkGetMacro(NIRIndex, unsigned int);


  /** Create a layer and display the Colored Image */
  void DisplayNewLayer();
  /** Find Urban Areas on the selected view */
  void ExtractUrbanAreas();
  /** Open Input Image /Vector */
  void OpenImage();
  void OpenVector(std::string name);
  /** Save results */
  void SaveImage(const char * filename);
  void SaveImageOnFull(const char * filename);
  void SaveVectorOnExtract(const char * filename);
  void SaveVectorOnFull(const char * filename);
  void InitVector();

protected:
  /** This is protected for the singleton. Use GetInstance() instead. */
  itkNewMacro(Self);

  /** Constructor */
  UrbanAreaExtractionModel();
  /** Destructor */
  virtual ~UrbanAreaExtractionModel();

private:
  UrbanAreaExtractionModel(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

  /** Notify a given listener of changes */
  virtual void NotifyListener(ListenerBase * listener);

  /** Methods */
  void InitPipeline();
  void BuildVector();
  void SaveVector(const char * filename);

  /** Create the binary image of the urban areas */
  void FindUrbanAreas();


  /** The instance singleton */
  static Pointer Instance;

  /** Input Image Filename */
  std::string m_ImageFilename;
  /** Input Vector Filename */
  std::string m_VectorFilename;
  /** Output Image Filename */
  std::string m_OutputImageFilename;
  /** Output Vector Filename */
  std::string m_OutputVectorFileName;

  /** Imput Image Pointer */
  InputImagePointerType m_InputImage;
  InputImagePointerType m_CurrentImage;
  /** Output Image Pointer */
  OutputImagePointerType m_OutputImage;

  /** Reader */
  ReaderPointerType m_Reader;
  /** UrbanAreaDetectionFilterType */
  UrbanAreaDetectionFilterPointerType m_UrbanAreaDetectionFilter;
  /** Shrink */
  StreamingShrinkImageFilterPointerType m_ShrinkFilter;
  /** Expand */
  ExpandImageFilterPointerType m_ExpandFilter;
  /** Focusing model */
  FocusingModelPointerType m_FocusingModel;
  /** PolygonsManager */
  PolygonsManagerPointerType m_PolygonsManager;
  /** ExtractROIBase */
  ExtractROIFilterPointerType m_ExtractROIFilter;

  /** StreamingRegion */
  RegionType m_StreamingRegion;
  /** Opened Vector */
  VectorDataPointerType m_LoadedVector;
  /** Full Vector */
  VectorDataPointerType m_FullVector;
  /** Folder Node of the Full Vector */
  DataNodePointerType m_Folder;
  /** Thresholds */
  double m_ThresholdValue;
  double m_ThresholdDensity;
  double m_SobelUpperThreshold;
  double m_SobelLowerThreshold;
  /** Sub Sample Level : ShrinkFactor */
  unsigned int m_SubSampleLevel;
  /** Minimum Size Region value (m²)*/
  unsigned int m_MinRegionSize;

  /** The Input has changed */
  bool m_HasNewInput;
  bool m_HasOutputImage;
  bool m_VectorDisplayed;
  StateType m_State;
  WhichImageType m_WhichImage;
  double m_Alpha;
  IndexType m_SavedIndex;
  unsigned int m_LoadedVDIndex;

  unsigned int m_GreenIndex;
  unsigned int m_RedIndex;
  unsigned int m_NIRIndex;

};

}
#endif
