/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbUrbanAreaExtractionViewGUI_h
#define __otbUrbanAreaExtractionViewGUI_h

#include "otbListenerBase.h"

// Disabling deprecation warning
#ifdef _MSC_VER
#pragma warning(push)
#pragma warning(disable:4996)
#endif
#include "otbUrbanAreaExtractionViewGroup.h"
#ifdef _MSC_VER
#pragma warning(pop)
#endif

#include <vector>
#include "otbUrbanAreaExtractionModel.h"
#include "otbUrbanAreaExtractionControllerInterface.h"

#include "otbFocusingView.h"
#include "otbHistogramCurve.h"
#include "otbPolygon.h"
#include "otbColoredPoly.h"
#include "itkFixedArray.h"

#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Tabs.H>

namespace otb
{
/** \class UrbanAreaExtractionViewGUI
 *
 */
class UrbanAreaExtractionViewGUI
      : public ListenerBase, public UrbanAreaExtractionViewGroup, public itk::Object
{
public:
  /** Standard class typedefs */
  typedef UrbanAreaExtractionViewGUI             Self;
  typedef itk::Object      Superclass;
  typedef itk::SmartPointer<Self>                Pointer;
  typedef itk::SmartPointer<const Self>          ConstPointer;

  /** Standards macros */
  itkNewMacro(Self);
  itkTypeMacro(UrbanAreaExtractionViewGUI, itk::Object);

  typedef double PixelType;

  typedef UrbanAreaExtractionModel                  ModelType;

  typedef ModelType::FocusingModelType              FocusingModelType;
  typedef FocusingModelType::Pointer                FocusingModelPointerType;
  typedef ModelType::FocusingViewType               FocusingViewType;
  typedef FocusingViewType::Pointer                 FocusingViewPointerType;
  typedef ModelType::RenderingFunctionType          RenderingFunctionType;
  typedef ModelType::StandardRenderingFunctionType  StandardRenderingFunctionType;

  typedef ModelType::VisuModelType                  VisuModelType;
  typedef VisuModelType::Pointer                    VisuModelPointerType;
  typedef FocusingViewType::ImageViewType           VisuViewType;
  typedef VisuViewType::Pointer                     VisuViewPointerType;
  typedef VisuViewType::ImageWidgetType             VisuImageWidgetType;
  typedef VisuImageWidgetType::Pointer              VisuImageWidgetPointerType;
  typedef VisuImageWidgetType::ColorType            VisuImageWidgetColorType;
  typedef FocusingModelType::ColorType              ColorType;
  typedef std::vector<ColorType>                    VectorColorType;
  typedef ModelType::RGBPixelType                   RGBPixelType;

  /** Polygons typedef */
  typedef otb::Polygon<double>                      PolygonType;
  typedef ObjectList<PolygonType>                   PolygonListType;
  typedef PolygonListType::Pointer                  PolygonListPointerType;

  /** Colors and map */
  typedef ColoredPoly<unsigned int, ColorType>       ColoredPolyType;
  typedef ColoredPolyType::Pointer                  ColoredPolyPointerType;
  typedef ObjectList<ColoredPolyType>               ColoredPolyListType;
  typedef ColoredPolyListType::Pointer              ColoredPolyListPointerType;


  /** Set the controller */
  virtual void SetUrbanAreaExtractionController(UrbanAreaExtractionControllerInterface * pController);
  /** Set the model */
  virtual void SetUrbanAreaExtractionModel(UrbanAreaExtractionModel * pModel);

  /** Event from the model */
  virtual void Notify();
  virtual void InitConfigIndices();
  virtual void UpdateViewerSetup();

  /** Buttons callback */
  virtual void OpenImageCallback();
  virtual void OpenVectorCallback();
  virtual void SaveImageOnExtractCallback();
  virtual void SaveImageOnFullCallback();
  virtual void SaveVectorOnExtractCallback();
  virtual void SaveVectorOnFullCallback();
  virtual void QuitCallback();
  virtual void MinDetailCallback();
  virtual void SetSubSampleLevelCallback();
  virtual void ThresholdCallback();
  virtual void ThresholdDensityCallback();
  virtual void SobelLowerThresholdCallback();
  virtual void SobelUpperThresholdCallback();
  virtual void DeleteCallback();
  virtual void ClearROIsCallback();
  virtual void FocusCallback();
  virtual void ChangeROIColorCallback();
  virtual void SwitchScrollCallback();
  virtual void RunCallback();
  virtual void DisplayCallback();
  virtual void PolygonSelectedCallback();
  virtual void AlphaCallback();
  virtual void ConfigIndicesCallback();
  virtual void IndicesOkCallback();
  virtual void IndicesCancelCallback();
  virtual void ViewerSetupCallback();
  virtual void ViewerSetupOkCallback();
  virtual void ViewerSetupCancelCallback();
  virtual void GrayscaleSetCallback();
  virtual void RGBSetCallback();


  VisuViewPointerType GetMasterView()
  {
    return m_FocusingView->GetMasterViewer();
  }
  VisuViewPointerType GetWorkingView()
  {
    return m_FocusingView->GetWorkingViewer();
  }
  virtual FocusingViewPointerType GetFocusingView()
  {
    return m_FocusingView;
  }

  void UpdateInformation();
  void InitWidgets();
  void Show();


protected:
  /** Constructor */
  UrbanAreaExtractionViewGUI();
  /** Destructor */
  ~UrbanAreaExtractionViewGUI() {};


private:
  UrbanAreaExtractionViewGUI(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

  /** Pointer to the model */
  UrbanAreaExtractionModel::Pointer m_UrbanAreaExtractionModel;
  /** Pointer to the controller */
  UrbanAreaExtractionControllerInterface::Pointer m_UrbanAreaExtractionController;

  /** Pointer to the focusing view */
  FocusingViewPointerType m_FocusingView;
  /** Store the last path. */
  std::string m_LastPath;
  /** Which kind of view is selected. */
  bool m_MasterViewSelected;
  /** colors */
  ColorType m_ClassColor;
  /** Program is running */
  bool m_Running;
};
}//end namespace otb

#endif
