/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include "itkExceptionObject.h"
#include "otbUrbanAreaExtractionModel.h"
#include "otbUrbanAreaExtractionViewGUI.h"
#include "otbUrbanAreaExtractionController.h"
#include "otbCommandLineArgumentParser.h"
#include <FL/Fl.H>

#include "otbI18n.h"

using namespace otb;

int main(int argc, char* argv[])
{
  otbI18nMacro();

  // Parse command line parameters
  typedef otb::CommandLineArgumentParser ParserType;
  ParserType::Pointer parser = ParserType::New();

  parser->AddInputImage(false); //Optionnal parameter

  typedef otb::CommandLineArgumentParseResult ParserResultType;
  ParserResultType::Pointer  parseResult = ParserResultType::New();

  try
  {
    parser->ParseCommandLine(argc, argv, parseResult);
  }
  catch ( itk::ExceptionObject & err )
  {
    std::string descriptionException = err.GetDescription();
    if (descriptionException.find("ParseCommandLine(): Help Parser") != std::string::npos)
    {
      std::cout << "WARNING : output file pixels are converted in 'unsigned char'" << std::endl;
      return EXIT_SUCCESS;
    }
    if (descriptionException.find("ParseCommandLine(): Version Parser") != std::string::npos)
    {
      return EXIT_SUCCESS;
    }
    return EXIT_FAILURE;
  }


  UrbanAreaExtractionModel::Pointer model = UrbanAreaExtractionModel::GetInstance();
  UrbanAreaExtractionController::Pointer controller = UrbanAreaExtractionController::New();
  UrbanAreaExtractionViewGUI::Pointer view = UrbanAreaExtractionViewGUI::New();
  view->SetUrbanAreaExtractionController(controller);
  view->SetUrbanAreaExtractionModel(model);
  controller->SetView(view);
  controller->AddHandlers();

  if ( parseResult->IsOptionInputImagePresent() )
  {
    model->SetImageFilename(parseResult->GetInputImage().c_str());
    model->OpenImage();
  }

  if (!parseResult->IsOptionOTBTestingPresent())
    Fl::run();

  return EXIT_SUCCESS;
}
