/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
  See OTBCopyright.txt for details.


     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#include "otbLAIFromNDVIImageTransform.h"

#include "otbCommandLineArgumentParser.h"
#include "otbImage.h"
#include "otbImageFileWriter.h"

#include "otbMultiChannelRAndNIRIndexImageFilter.h"
#include "otbImage.h"
#include "otbVectorImage.h"
#include "otbImageFileReader.h"
#include "otbImageFileWriter.h"
#include "otbVegetationIndicesFunctor.h"

namespace otb
{

int LAIFromNDVIImageTransform::Describe(ApplicationDescriptor* descriptor)
{
  descriptor->SetName("LAIFromNDVIImageTransform");

  descriptor->SetDescription("LAI Image transformed based on NDVI index. Input image is a vector data. if no red and nir channel given, "
      "they are respectively set to band 1 and 4");
  descriptor->AddInputImage();
  descriptor->AddOutputImage();

  descriptor->AddOption("RedChannel","red band channel","r", 1, false, ApplicationDescriptor::Integer);
  descriptor->AddOption("NirChannel","NIR band channel","nir", 1, false, ApplicationDescriptor::Integer);
  return EXIT_SUCCESS;
}

int LAIFromNDVIImageTransform::Execute(otb::ApplicationOptionsResult* parseResult)
{

  const unsigned int Dimension = 2;
  typedef otb::VectorImage<double, Dimension> InputImageType;
  typedef otb::Image<double, Dimension> OutputImageType;
  typedef otb::ImageFileReader<InputImageType> ReaderType;
  typedef otb::ImageFileWriter<OutputImageType> WriterType;
  typedef otb::Functor::LAIFromNDVIFormosat2Functor<InputImageType::InternalPixelType,
      InputImageType::InternalPixelType, OutputImageType::PixelType> FunctorType;
  typedef otb::MultiChannelRAndNIRIndexImageFilter<InputImageType, OutputImageType, FunctorType>
      MultiChannelRAndNIRIndexImageFilterType;

  // Instantiating object
  MultiChannelRAndNIRIndexImageFilterType::Pointer filter = MultiChannelRAndNIRIndexImageFilterType::New();
  ReaderType::Pointer reader = ReaderType::New();
  WriterType::Pointer writer = WriterType::New();

  std::string infilename = parseResult->GetInputImage();


  std::string outfilename = parseResult->GetOutputImage();

  reader->SetFileName(infilename);
  writer->SetFileName(outfilename);


  if (parseResult->IsOptionPresent("RedChannel") && parseResult->IsOptionPresent("NirChannel"))
     {
    unsigned int redChannel=parseResult->GetParameterUInt("RedChannel");
    unsigned int nirChannel=parseResult->GetParameterUInt("NirChannel");
    filter->SetRedIndex(redChannel);
    filter->SetNIRIndex(nirChannel);
     }
   else
     {
     filter->SetRedIndex(1);
     filter->SetNIRIndex(4);
     }

  filter->SetInput(reader->GetOutput());
  writer->SetInput(filter->GetOutput());
  writer->Update();

  return EXIT_SUCCESS;
}
}

  


