/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
//#include "otbImageLayerRenderingModel.h"

#include "otbVectorImage.h"
#include "itkRGBPixel.h"
#include "otbImageFileReader.h"

#include "otbImageLayerGenerator.h"
#include "otbImageLayer.h"
#include "otbFocusingModel.h"
#include "otbFocusingView.h"
#include "otbFocusingController.h"
#include <FL/Fl.H>
#include "otbImageWidgetController.h"
#include "otbWidgetResizingActionHandler.h"
#include "otbChangeScaledExtractRegionActionHandler.h"
#include "otbChangeExtractRegionActionHandler.h"
#include "otbChangeScaleActionHandler.h"
#include "otbPixelDescriptionModel.h"
#include "otbPixelDescriptionActionHandler.h"
#include "otbPixelDescriptionView.h"
#include "otbArrowKeyMoveActionHandler.h"
// Vector datas
#include "otbVectorData.h"
#include "otbVectorDataFileReader.h"


int otbFocusingViewDBManipulation(int argc, char* argv[])
{
  // params
  const char * infname           = argv[1];
  const unsigned int scrollSize1 = atoi(argv[2]);
  const unsigned int scrollSize2 = atoi(argv[3]);
  const unsigned int fullSize    = atoi(argv[4]);
  const double       run         = atoi(argv[5]);

  /**************************  END SAME AS otbFocusingViewShow    ****************/
  // typedefs
  typedef double                                          PixelType;
  typedef itk::RGBPixel<unsigned char>                    RGBPixelType;
  typedef otb::Image<RGBPixelType, 2>                      OutputImageType;
  typedef otb::VectorImage<PixelType, 2>                   InputImageType;
  typedef otb::ImageLayer<InputImageType, OutputImageType> LayerType;
  typedef otb::ImageFileReader<InputImageType>            ReaderType;
  typedef otb::ImageLayerGenerator<LayerType>             LayerGeneratorType;

  typedef otb::FocusingModel<InputImageType, OutputImageType>           FocusingModelType;
  typedef otb::FocusingView<FocusingModelType>                         FocusingViewType;
  //typedef otb::FocusingController<FocusingModelType, FocusingViewType> FocusingControllerType;
  typedef FocusingViewType::FocusingControllerType                     FocusingControllerType;

  typedef FocusingModelType::ImageLayerRenderingModelType ModelType;
  typedef FocusingViewType::ImageViewType                 ViewType;
  typedef otb::ImageWidgetController                      ControllerType;

  typedef otb::WidgetResizingActionHandler<ModelType, ViewType>            ResizingHandlerType;
  typedef otb::ChangeScaledExtractRegionActionHandler<ModelType, ViewType> ChangeScaledRegionHandlerType;
  typedef otb::ChangeExtractRegionActionHandler<ModelType, ViewType>       ChangeRegionHandlerType;
  typedef otb::ChangeScaleActionHandler<ModelType, ViewType>               ChangeScaleHandlerType;
  typedef otb::ArrowKeyMoveActionHandler<ModelType, ViewType>             ArrowKeyMoveActionHandlerType;


  // Reading input image
  ReaderType::Pointer reader = ReaderType::New();
  reader->SetFileName(infname);

  // Instantiation
  FocusingModelType::Pointer lFocusingModel = FocusingModelType::New();

  // Saving the input Image
  lFocusingModel->SetInputImage(reader->GetOutput());

  // Build a Focusing View
  FocusingViewType::Pointer lFocusingView = FocusingViewType::New();
  lFocusingView->SetFocusingModel( lFocusingModel );

  // Build a Focusing Controller
  FocusingControllerType::Pointer lFocusingController = FocusingControllerType::New();
  lFocusingView->SetFocusingController(lFocusingController);
  lFocusingController->SetFocusingView(lFocusingView);
  lFocusingController->SetFocusingModel(lFocusingModel);
  lFocusingController->AddSelectAreaHandler();

  // Add the resizing handler
  ResizingHandlerType::Pointer resizingHandlerM = ResizingHandlerType::New();
  resizingHandlerM->SetModel(lFocusingModel->GetMasterModel());
  resizingHandlerM->SetView(lFocusingView->GetMasterViewer());
  lFocusingController->GetMasterController()->AddActionHandler(resizingHandlerM);

  ResizingHandlerType::Pointer resizingHandlerW = ResizingHandlerType::New();
  resizingHandlerW->SetModel(lFocusingModel->GetWorkingModel());
  resizingHandlerW->SetView(lFocusingView->GetWorkingViewer());
  lFocusingController->GetWorkingController()->AddActionHandler(resizingHandlerW);

  // Add the change scaled region handler
  ChangeScaledRegionHandlerType::Pointer changeScaledHandler =ChangeScaledRegionHandlerType::New();
  changeScaledHandler->SetModel(lFocusingModel->GetWorkingModel());
  changeScaledHandler->SetView(lFocusingView->GetWorkingViewer());
  lFocusingController->GetWorkingController()->AddActionHandler(changeScaledHandler);

  // Add the change scaled handler
  ChangeScaleHandlerType::Pointer changeScaleHandler =ChangeScaleHandlerType::New();
  changeScaleHandler->SetModel(lFocusingModel->GetWorkingModel());
  changeScaleHandler->SetView(lFocusingView->GetWorkingViewer());
  lFocusingController->GetWorkingController()->AddActionHandler(changeScaleHandler);

  // Add the change region action handler
  ChangeRegionHandlerType::Pointer changeRegionHandler = ChangeRegionHandlerType::New();
  changeRegionHandler->SetModel(lFocusingModel->GetWorkingModel());
  changeRegionHandler->SetView(lFocusingView->GetWorkingViewer());
  lFocusingController->GetWorkingController()->AddActionHandler(changeRegionHandler);

  // Add the arrow keys move action handler
  ArrowKeyMoveActionHandlerType::Pointer arrowKeyMoveHandler = ArrowKeyMoveActionHandlerType::New();
  arrowKeyMoveHandler->SetModel(lFocusingModel->GetWorkingModel());
  arrowKeyMoveHandler->SetView(lFocusingView->GetWorkingViewer());
  lFocusingController->GetWorkingController()->AddActionHandler(arrowKeyMoveHandler);

  Fl_Window masterScrollWindow(scrollSize1, scrollSize1);
  if(scrollSize1 > 0)
  {
    masterScrollWindow.label("MasterScroll");
    masterScrollWindow.add(lFocusingView->GetMasterScrollWidget());
    masterScrollWindow.resizable(lFocusingView->GetMasterScrollWidget());
    masterScrollWindow.show();
    lFocusingView->GetMasterScrollWidget()->show();
    lFocusingView->GetMasterScrollWidget()->resize(0, 0, scrollSize1, scrollSize1);
  }

  Fl_Window workingScrollWindow(scrollSize2, scrollSize2);
  if(scrollSize2 > 0)
  {
    workingScrollWindow.label("WorkingScroll");
    workingScrollWindow.add(lFocusingView->GetWorkingScrollWidget());
    workingScrollWindow.resizable(lFocusingView->GetWorkingScrollWidget());
    workingScrollWindow.show();
    lFocusingView->GetWorkingScrollWidget()->show();
    lFocusingView->GetWorkingScrollWidget()->resize(0, 0, scrollSize2, scrollSize2);
  }

  Fl_Window fullWindow(fullSize, fullSize);
  if(fullSize > 0)
  {
    fullWindow.label("WorkingFull");
    fullWindow.add(lFocusingView->GetWorkingFullWidget());
    fullWindow.resizable(lFocusingView->GetWorkingFullWidget());
    fullWindow.show();
    lFocusingView->GetWorkingFullWidget()->show();
    lFocusingView->GetWorkingFullWidget()->resize(0, 0, fullSize, fullSize);
  }

  lFocusingModel->UpdateModels();

  /**************************  END SAME AS otbFocusingViewShow    ****************/

  // display 2 vector data
  lFocusingModel->LoadVectorData(argv[6]);

  typedef otb::VectorData<double>                   VectorDataType;
  typedef otb::VectorDataFileReader<VectorDataType> VectorDataReaderType;
  typedef FocusingModelType::ColorType              ColorType;
  ColorType color;
  color[0] = 0;
  color[1] = 1;
  color[2] = 0;
  color[3] = 1;
  VectorDataReaderType::Pointer vectReader = VectorDataReaderType::New();
  vectReader->SetFileName(argv[7]);
  vectReader->Update();
  lFocusingModel->SetColor( color );

  lFocusingModel->AddVectorData( vectReader->GetOutput() );

  // erase the first vector data
  lFocusingModel->ClearVectorData(0);

  // show another one
  color[0] = 1;
  color[1] = 0;
  color[3] = 1;
  lFocusingModel->SetColor( color );
  lFocusingModel->LoadVectorData(argv[6]);

  // clear all
  //lFocusingModel->ClearAllVectorData();


  if(run)
    {
    Fl::run();
    }
  else
    {
    Fl::check();
    }


  masterScrollWindow.remove(lFocusingView->GetMasterScrollWidget());
  workingScrollWindow.remove(lFocusingView->GetWorkingScrollWidget());
  fullWindow.remove(lFocusingView->GetWorkingFullWidget());


  return EXIT_SUCCESS;
}
