/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
  See OTBCopyright.txt for details.


  This software is distributed WITHOUT ANY WARRANTY; without even
  the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
  PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#include "itkExceptionObject.h"

#include "otbCorinLandCoverTextureImageFilter.h"
#include "otbImage.h"
#include "otbVectorImage.h"
#include "otbImageFileReader.h"
#include "otbStreamingImageFileWriter.h"
#include "itkConstNeighborhoodIterator.h"
#include "otbUnaryFunctorNeighborhoodImageFilter.h"


int otbCorinLandCoverTextureFunctorTest(int argc, char * argv[])
{
  typedef double InputPixelType;
  const char * inputfilename(argv[1]);
  const char * outputfilename1(argv[2]);
  const char * outputfilename2(argv[3]);
  const int radius(atoi(argv[4]));

  const int Dimension = 2;
  typedef otb::VectorImage<InputPixelType, Dimension> InputImageType;
  typedef otb::VectorImage<InputPixelType, Dimension> OutputVectorImageType;
  typedef OutputVectorImageType::PixelType OutputVectorImagePixelType;

  typedef otb::ImageFileReader<InputImageType> ReaderType;
  typedef otb::StreamingImageFileWriter<OutputVectorImageType> WriterType;

  typedef otb::CorinLandCoverTextureImageFilter<InputImageType, OutputVectorImageType>  FilterType;

  // Instantiating object
  FilterType::Pointer object = FilterType::New();
  WriterType::Pointer writer1 = WriterType::New();
  ReaderType::Pointer reader = ReaderType::New();

  std::vector<int> listIndexFeature;
  listIndexFeature.push_back(0);
  listIndexFeature.push_back(2);
  listIndexFeature.push_back(4);

  object->SetInput(reader->GetOutput());
  object->SetRadius(radius);
  object->SetTextureIndex(listIndexFeature);
  writer1->SetInput(object->GetOutput());
  reader->SetFileName(inputfilename);

  writer1->SetFileName(outputfilename1);
  writer1->SetAutomaticNumberOfStreamDivisions();

  writer1->Update();

  FilterType::Pointer object2 = FilterType::New();
  WriterType::Pointer writer2 = WriterType::New();

  listIndexFeature.clear();
  listIndexFeature.push_back(0);
  listIndexFeature.push_back(2);
  listIndexFeature.push_back(4);

  object2->SetFillImage(true);

  object2->SetInput(reader->GetOutput());
  object2->SetRadius(radius);
  object2->SetTextureIndex(listIndexFeature);
  writer2->SetInput(object2->GetOutput());

  writer2->SetFileName(outputfilename2);
  writer2->SetAutomaticNumberOfStreamDivisions();
  writer2->Update();

  return EXIT_SUCCESS;
}
