/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
  See OTBCopyright.txt for details.


  This software is distributed WITHOUT ANY WARRANTY; without even
  the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
  PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#include "itkExceptionObject.h"

#include "otbRescaleWithMinMaxVectorImageFilter.h"
#include "otbVectorImage.h"
#include "otbImageFileReader.h"
#include "otbStreamingImageFileWriter.h"

int otbRescaleWithMinMaxVectorImageFilterTest(int argc, char * argv[])
{
  typedef unsigned int                                                      InPixelType;
  typedef double                                                            OutPixelType;
  const int Dimension = 2;
  typedef otb::VectorImage<InPixelType, Dimension>                           InImageType;
  typedef otb::VectorImage<OutPixelType, Dimension>                          OutImageType;
  typedef otb::ImageFileReader<InImageType>                                 ReaderType;
  typedef otb::StreamingImageFileWriter<OutImageType>                       WriterType;
  typedef otb::RescaleWithMinMaxVectorImageFilter<InImageType, OutImageType> FilterType;

  // Instantiating object
  FilterType::Pointer rescale = FilterType::New();
  WriterType::Pointer writer  = WriterType::New();
  ReaderType::Pointer reader  = ReaderType::New();

  reader->SetFileName(argv[1]);
  reader->UpdateOutputInformation();

  rescale->SetInput( reader->GetOutput() );
  OutImageType::PixelType pixOut;
  pixOut.SetSize( reader->GetOutput()->GetNumberOfComponentsPerPixel() );
  pixOut.Fill(-1);
  rescale->SetOutputMinimum(pixOut);
  pixOut.Fill(1);
  rescale->SetOutputMaximum(pixOut);

  OutImageType::PixelType pixIn;
  pixIn.SetSize( reader->GetOutput()->GetNumberOfComponentsPerPixel() );
  pixIn.Fill(0);
  rescale->SetInputMinimum( pixIn );
  pixIn.Fill(255);
  rescale->SetInputMaximum( pixIn );

  std::cout<<"oulala"<<std::endl;

  writer->SetFileName(argv[2]);
  writer->SetInput(rescale->GetOutput());
  writer->Update();

  return EXIT_SUCCESS;
}
