/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include <FL/Fl.H>
#include "otbFineRegistrationApplicationModel.h"
#include "otbFineRegistrationApplicationController.h"
#include "otbFineRegistrationApplicationViewGUI.h"

int otbFineRegistrationApplicationViewTestCase1(int argc, char* argv[])
{

  if (argc!=5)
  {
    std::cerr<<"Usage: "<<argv[0]<<" movingName fixedName disparityMapName registeredImageName"<<std::endl;
    return EXIT_FAILURE;
  }

  const char * movingName = argv[1];
  const char * fixedName = argv[2];
  const char * disparityMapName = argv[3];
  const char * registeredImageName = argv[4];
  const unsigned int nbIterations(1);
  const unsigned int XRadius(1);
  const unsigned int YRadius(1);
  const double XMax(1.);
  const double YMax(1.);


  typedef otb::FineRegistrationApplicationModel ModelType;
  typedef ModelType::ImageType ImageType;
  typedef otb::FineRegistrationApplicationController ControllerType;
  typedef otb::FineRegistrationApplicationViewGUI ViewType;


  ControllerType::Pointer controller = ControllerType::New();

  ViewType::Pointer view = ViewType::New();
  view->SetFineRegistrationApplicationController(controller);
  controller->SetView(view);
  ModelType::Pointer model = otb::FineRegistrationApplicationModel::GetInstance();

  // Scripting
  Fl::check();

  view->OpenPairCallback();
  Fl::check();
  // Push Cancel button
  view->bOpenPairCancel->value(1);
  view->bOpenPairCancel->do_callback();

  view->OpenPairCallback();
  Fl::check();
  view->vMovingImagePath->value(movingName);
  view->vMovingImagePath->do_callback();
  Fl::check();
  view->vFixedImagePath->value(fixedName);
  view->vFixedImagePath->do_callback();
  Fl::check();

  // Push Ok button
  view->bOpenPairOk->value(1);
  view->bOpenPairOk->do_callback();
  Fl::check();

  view->bFixed->value(1);
  view->bFixed->do_callback();
  Fl::check();

  view->bMoving->value(1);
  view->bMoving->do_callback();
  Fl::check();

  view->bNbIterations->value(nbIterations);
  view->bNbIterations->do_callback();
  Fl::check();
  view->bXRadius->value(XRadius);
  view->bXRadius->do_callback();
  Fl::check();
  view->bYRadius->value(YRadius);
  view->bYRadius->do_callback();
  Fl::check();

  view->bXMax->value(XMax);
  view->bXMax->do_callback();
  Fl::check();
  view->bYMax->value(YMax);
  view->bYMax->do_callback();
  Fl::check();


  // Run call back
  view->bRun->value(1);
  view->bRun->do_callback();
  Fl::check();

  view->bXDeformation->value(1);
  view->bXDeformation->do_callback();
  view->bYDeformation->value(1);
  view->bYDeformation->do_callback();
  view->bIntensity->value(1);
  view->bIntensity->do_callback();

  // Save results
  controller->SaveDisparityMap(disparityMapName);
  Fl::check();

  controller->SaveRegisteredImage(registeredImageName);
  Fl::check();

  return EXIT_SUCCESS;
}
