/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include <FL/Fl.H>
#include "otbFineRegistrationApplicationModel.h"
#include "otbFineRegistrationApplicationController.h"
#include "otbFineRegistrationApplicationViewGUI.h"

int otbFineRegistrationApplicationViewTestCase2(int argc, char* argv[])
{

  if (argc!=5)
  {
    std::cerr<<"Usage: "<<argv[0]<<" movingName fixedName disparityMapName registeredImageName"<<std::endl;
    return EXIT_FAILURE;
  }

  const char * movingName = argv[1];
  const char * fixedName = argv[2];
  const char * disparityMapName = argv[3];
  const char * registeredImageName = argv[4];
  const unsigned int nbIterations(1);
  const unsigned int XRadius(1);
  const unsigned int YRadius(1);
  const double XMax(1.);
  const double YMax(1.);


  typedef otb::FineRegistrationApplicationModel ModelType;
  typedef ModelType::ImageType ImageType;
  typedef otb::FineRegistrationApplicationController ControllerType;
  typedef otb::FineRegistrationApplicationViewGUI ViewType;


  ControllerType::Pointer controller = ControllerType::New();

  ViewType::Pointer view = ViewType::New();
  view->SetFineRegistrationApplicationController(controller);
  controller->SetView(view);
  ModelType::Pointer model = otb::FineRegistrationApplicationModel::GetInstance();

  // Scripting
  Fl::check();

  otbTestMenuItemCallback(view, bMenuBar, bOpenImagePair);
  // Push Ok button without initialization
  otbTestCallback(view, bOpenPairOk);
  // Try to trigger the inactive buttons
  otbTestCallbackWithValue(view, bFixed, 1);
  otbTestCallbackWithValue(view, bMoving, 1);
  otbTestCallbackWithValue(view, bNbIterations, nbIterations);
  otbTestCallbackWithValue(view, bXRadius, XRadius);
  otbTestCallbackWithValue(view, bYRadius, YRadius);
  otbTestCallbackWithValue(view, bXMax, XMax);
  otbTestCallbackWithValue(view, bYMax, YMax);
  // Use case: Run call back without initilization
  otbTestCallback(view, bRun);
  otbTestCallbackWithValue(view, bXDeformation, 1);
  otbTestCallbackWithValue(view, bYDeformation, 1);
  otbTestCallbackWithValue(view, bIntensity, 1);

  controller->SaveDisparityMap(disparityMapName);
  Fl::check();

  controller->SaveRegisteredImage(registeredImageName);
  Fl::check();

  // Use Case: Open Pair image and save results without Run execution
  otbTestMenuItemCallback(view, bMenuBar, bOpenImagePair);
  otbTestCallbackWithValue(view, vMovingImagePath, movingName);
  otbTestCallbackWithValue(view, vFixedImagePath, fixedName);

  otbTestCallback(view, bOpenPairOk);
  // With Invalid results filename
  controller->SaveDisparityMap("");
  Fl::check();
  controller->SaveRegisteredImage("");
  Fl::check();

  controller->SaveDisparityMap(disparityMapName);
  Fl::check();
  controller->SaveRegisteredImage(registeredImageName);
  Fl::check();

  return EXIT_SUCCESS;
}
