/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include <FL/Fl.H>
#include "otbFineRegistrationApplicationModel.h"
#include "otbFineRegistrationApplicationController.h"
#include "otbFineRegistrationApplicationViewGUI.h"

// Test the open twice case

int otbFineRegistrationApplicationViewTestCase3(int argc, char* argv[])
{

  if (argc!=5)
  {
    std::cerr<<"Usage: "<<argv[0]<<" movingName1 fixedName1 movingName2 fixedName2 "<<std::endl;
    return EXIT_FAILURE;
  }

  const char * movingName1 = argv[1];
  const char * fixedName1 = argv[2];
  const char * movingName2 = argv[3];
  const char * fixedName2 = argv[4];

  typedef otb::FineRegistrationApplicationModel ModelType;
  typedef ModelType::ImageType ImageType;
  typedef otb::FineRegistrationApplicationController ControllerType;
  typedef otb::FineRegistrationApplicationViewGUI ViewType;


  ControllerType::Pointer controller = ControllerType::New();

  ViewType::Pointer view = ViewType::New();
  view->SetFineRegistrationApplicationController(controller);
  controller->SetView(view);
  ModelType::Pointer model = otb::FineRegistrationApplicationModel::GetInstance();

  // Scripting
  Fl::check();

  // Open first pair
  otbTestMenuItemCallback(view, bMenuBar, bOpenImagePair);
  otbTestCallbackWithValue(view, vMovingImagePath, movingName1);
  otbTestCallbackWithValue(view, vFixedImagePath, fixedName1);
  otbTestCallback(view, bOpenPairOk);

  // Open second pair
  otbTestMenuItemCallback(view, bMenuBar, bOpenImagePair);
  otbTestCallbackWithValue(view, vMovingImagePath, movingName2);
  otbTestCallbackWithValue(view, vFixedImagePath, fixedName2);
  otbTestCallback(view, bOpenPairOk);

  return EXIT_SUCCESS;
}
