/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/

#include "itkExceptionObject.h"
#include "otbImageToDBRegistrationController.h"
#include "otbImageToDBRegistrationViewGUI.h"
#include "itkIndex.h"
#include "otbImageToDBRegistrationEnum.h"

int otbImageToDBRegistrationAlpha(int argc, char* argv[])
{
  typedef otb::ImageToDBRegistrationController ControllerType;
  typedef otb::ImageToDBRegistrationViewGUI ViewType;
 
  ControllerType::Pointer controller = ControllerType::New();
  ViewType::Pointer view = ViewType::New();
  controller->SetView(view);
  controller->AddHandlers();
  view->SetImageToDBRegistrationController(controller);
 
  view->InitWidgets();
  view->Show();
 
  controller->OpenInputImage(argv[1]);
  Fl::check();

  // Without image activated
  itk::Index<> index1, index2;
  index1.Fill(20);
  index2.Fill(50);
  controller->GetFocusingVisuController()->GetFocusingModel()->SetExtractRegionByIndex( index1, index2 );
  controller->GetFocusingVisuController()->GetFocusingModel()->Update();
  Fl::check();
  
  view->RunOverExtract();
  Fl::check();

  // With image activated
  view->gDBRef->value(1);
  view->gDBRef->do_callback();
  Fl::check();

  view->RunOverExtract();
  Fl::check();

  view->vAlpha->value( 0.8 );
  view->vAlpha->do_callback();
  Fl::check();
 
  index1.Fill(50);
  index2.Fill(500);
  controller->GetFocusingVisuController()->GetFocusingModel()->SetExtractRegionByIndex( index1, index2 );
  controller->GetFocusingVisuController()->GetFocusingModel()->Update();
  Fl::check();

  view->vAlpha->value( 0.1 );
  view->vAlpha->do_callback();
  Fl::check();

  view->RunOverExtract();
  Fl::check();

  // With DB activated
  view->gImageRef->value(1);
  view->gImageRef->do_callback();
  Fl::check();

  view->RunOverExtract();
  Fl::check();


  return EXIT_SUCCESS;
}
