/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/

#include "itkExceptionObject.h"
#include "otbImageToDBRegistrationController.h"
#include "otbImageToDBRegistrationViewGUI.h"
#include "itkIndex.h"
#include "otbImageToDBRegistrationEnum.h"

int otbImageToDBRegistrationDeleteClear(int argc, char* argv[])
{
  typedef otb::ImageToDBRegistrationController ControllerType;
  typedef otb::ImageToDBRegistrationViewGUI ViewType;
 
  ControllerType::Pointer controller = ControllerType::New();
  ViewType::Pointer view = ViewType::New();
  controller->SetView(view);
  controller->AddHandlers();
  view->SetImageToDBRegistrationController(controller);
 
  view->InitWidgets();
  view->Show();

  controller->OpenInputImage(argv[1]);
  Fl::check();

  itk::Index<> index1, index2;
  index1.Fill(150);
  index2.Fill(250);
  controller->GetFocusingVisuController()->GetFocusingModel()->SetExtractRegionByIndex( index1, index2 );
  controller->GetFocusingVisuController()->GetFocusingModel()->Update();
  Fl::check();

  view->RunOverExtract();
  Fl::check();

  ViewType::ColorType color;
  color.Fill(1);
  color[0]=0;
  color[1]=0;
  fl_color(static_cast<unsigned char>(255*color[0]),
          static_cast<unsigned char>(255*color[1]),
          static_cast<unsigned char>(255*color[2]));
  view->dROIColor->color(fl_color());
  view->dROIColor->redraw();
  view->dVDList->selection_color(fl_color());
  Fl::check();
  controller->ChangeColor( color );
  Fl::check();

  controller->LoadVectorData(argv[2]);
  Fl::check();
  std::string name = argv[2];
  view->dVDList->add(name.substr( name.find_last_of("/")+1, name.size() ).c_str() );
  view->dVDList->value(view->dVDList->size());
  view->dVDList->redraw();
  Fl::check();

  view->RunOverExtract();
  Fl::check();

  view->dVDList->value(1);
  Fl::check();

  color.Fill(1);
  color[0]=0;
  color[2]=0;
  fl_color(static_cast<unsigned char>(255*color[0]),
          static_cast<unsigned char>(255*color[1]),
          static_cast<unsigned char>(255*color[2]));
  view->dROIColor->color(fl_color());
  view->dROIColor->redraw();
  view->dVDList->selection_color(fl_color());
  view->dVDList->redraw();
  Fl::check();

  controller->ChangeVectorDataColor( view->dVDList->value()-1, color );
  Fl::check();
  

  view->Delete();
  Fl::check();
  
  view->RunOverExtract();
  Fl::check();
  
  
  view->ClearAll();
  Fl::check();

  return EXIT_SUCCESS;
}
