/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/

#include "itkExceptionObject.h"
#include "otbImageToDBRegistrationController.h"
#include "otbImageToDBRegistrationViewGUI.h"
#include "itkIndex.h"
#include "otbImageToDBRegistrationEnum.h"

int otbImageToDBRegistrationRunFull(int argc, char* argv[])
{
  typedef otb::ImageToDBRegistrationController ControllerType;
  typedef otb::ImageToDBRegistrationViewGUI ViewType;
 
  ControllerType::Pointer controller = ControllerType::New();
  ViewType::Pointer view = ViewType::New();
  controller->SetView(view);
  controller->AddHandlers();
  view->SetImageToDBRegistrationController(controller);
 
  view->InitWidgets();
  view->Show();

  if(atoi(argv[2])==0)
    {
      view->gImageRef->value(1);
      view->gImageRef->do_callback();
      Fl::check();
    }
  else
    {
      view->gDBRef->value(1);
      view->gDBRef->do_callback();
      Fl::check();
    }


  controller->OpenInputImage(argv[1]);
  Fl::check();
 
  controller->LoadVectorData(argv[2]);
  std::string name = argv[2];
  view->dVDList->add(name.substr( name.find_last_of("/")+1, name.size() ).c_str() );
  view->dVDList->value(view->dVDList->size());
  view->dVDList->redraw();
  Fl::check();

  itk::Index<> index1, index2;
  index1.Fill(100);
  index2.Fill(200);
  controller->GetFocusingVisuController()->GetFocusingModel()->SetExtractRegionByIndex( index1, index2 );
  controller->GetFocusingVisuController()->GetFocusingModel()->Update();
  Fl::check();

  view->RunOverExtract();
  Fl::check();

  view->RunOverExtract();
  Fl::check();

  index1.Fill(150);
  index2.Fill(250);
  controller->GetFocusingVisuController()->GetFocusingModel()->SetExtractRegionByIndex( index1, index2 );
  controller->GetFocusingVisuController()->GetFocusingModel()->Update();
  Fl::check();

  view->RunOverExtract();
  Fl::check();

  Fl::check();
  if(argc==4)
    {
      if( atoi(argv[3])==0)
       {
         controller->Run(argv[4], otb::DATABASE);
         Fl::check();
       }
      else
       {
         controller->Run(argv[4], otb::IMAGE);
         Fl::check();
       }
    }

  return EXIT_SUCCESS;

}
