/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/

#include "itkExceptionObject.h"
#include "otbImageToDBRegistrationController.h"
#include "otbImageToDBRegistrationViewGUI.h"
#include "itkIndex.h"
#include "otbMsgReporter.h"

int otbImageToDBRegistrationVD(int argc, char* argv[])
{
  typedef otb::ImageToDBRegistrationController ControllerType;
  typedef otb::ImageToDBRegistrationViewGUI ViewType;
 
  ControllerType::Pointer controller = ControllerType::New();
  ViewType::Pointer view = ViewType::New();
  controller->SetView(view);
  controller->AddHandlers();
  view->SetImageToDBRegistrationController(controller);
 
  view->InitWidgets();
  view->Show();

  controller->OpenInputImage(argv[1]);
  Fl::check();

  itk::Index<> index1, index2;
  index1.Fill(20);
  index2.Fill(50);
  controller->GetFocusingVisuController()->GetFocusingModel()->SetExtractRegionByIndex( index1, index2 );
  controller->GetFocusingVisuController()->GetFocusingModel()->Update();
  Fl::check();

  if(argc>2)
    {
      controller->LoadVectorData(argv[2]);
         std::string name = argv[2];
      view->dVDList->add(name.substr( name.find_last_of("/")+1, name.size() ).c_str() );
      view->dVDList->value(view->dVDList->size());
      view->dVDList->redraw();
      Fl::check();
    }

  if(argc>3)
    {
      controller->SetDEMDirectory(argv[3]);
      controller->UseDEM(false);
      controller->UseDEM(true);
      Fl::check();
    }

  view->SwitchScroll();
  Fl::check();

  index1.Fill(150);
  index2.Fill(350);
  controller->GetFocusingVisuController()->GetFocusingModel()->SetExtractRegionByIndex( index1, index2 );
  controller->GetFocusingVisuController()->GetFocusingModel()->Update();
  Fl::check();

  view->SwitchScroll();
  Fl::check();


  return EXIT_SUCCESS;
}
