/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include "otbObjectCountingModel.h"
#include "otbMacro.h"


#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>

int otbObjectCountingModelPolyToSampleTest(int argc, char* argv[])
{
  typedef otb::ObjectCountingModel ModelType;
  typedef ModelType::SizeType SizeType;
  typedef ModelType::IndexType IndexType;
  typedef ModelType::RegionType RegionType;

  ModelType::Pointer model = ModelType::GetInstance();

  long int col, lig;
  col = atoi(argv[2]);
  lig = atoi(argv[3]);

  SizeType qlSize;
  qlSize[0] = 100;
  qlSize[1] = 100;
  model->SetQuicklookSize( qlSize );
  model->OpenImage( argv[1] );
  RegionType region;
  IndexType newIndex;
  newIndex.Fill(0);
  SizeType newSize;
  newSize[0]=100;
  newSize[1]=100;
  region.SetIndex(newIndex);
  region.SetSize(newSize);
  model->SetCurrentRegion(region);
  model->RunImageExtraction();


  IndexType id;
  id[0] = col;
  id[1] = lig;
  model->NewPolygon(id);
  id[0] = col;
  id[1] = lig +5;
  model->AddPointToCurrentPolygon(id);
  id[0] = col +5;
  id[1] = lig +5;
  model->AddPointToCurrentPolygon(id);
  id[0] = col +5;
  id[1] = lig;
  model->AddPointToCurrentPolygon(id);


  model->GenerateInputExampleSampleList();
  std::cout<<model->GetInputSampleList()->Size()<<std::endl;
  for ( unsigned int i=0; i<model->GetInputSampleList()->Size(); i++ )
  {
    std::cout<<model->GetInputSampleList()->GetMeasurementVector(i)<<std::endl;
  }

  return EXIT_SUCCESS;
}
