/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include "otbRoadExtractionModel.h"
#include "otbMacro.h"

int RoadExtractionModelTestCase2(int argc, char* argv[])
{
  if (argc!=4)
  {
    std::cerr<<"Usage: "<<argv[0]<<" inputImageName resultName pixelValue"<<std::endl;
    return EXIT_FAILURE;
  }

  const char * inputImageName = argv[1];
  const char * resultName = argv[2];


  typedef otb::RoadExtractionModel ModelType;

  ModelType::VectorPixelType pixel;
  pixel.SetSize(4);
  pixel.Fill(atoi(argv[3]));

  ModelType::RegionType region;
  ModelType::SizeType size;
  ModelType::IndexType index;
  size[0]=200;
  size[1]=200;
  index[0]=0;
  index[1]=0;
  region.SetSize(size);
  region.SetIndex(index);

  // model instance
  ModelType::Pointer model = otb::RoadExtractionModel::GetInstance();

  otbTestingCheckValidCommand(          model->SetupRoadExtraction( ) );
  otbTestingCheckNotValidCommand(       model->GenerateData() );
  otbTestingCheckNotValidCommand(       model->SaveResults("") );
  otbTestingCheckNotValidCommand(       model->SaveResults(resultName) );
  otbTestingCheckNotValidCommand(       model->SetFileName("") );

  otbTestingCheckValidCommand(          model->SetFileName(inputImageName) );
  otbTestingCheckValidCommand(          model->ClearPathList() );
  otbTestingCheckValidCommand(          model->SetReferencePixel(pixel));
  otbTestingCheckValidCommand(          model->SetRegion(region));
  otbTestingCheckValidCommand(          model->GenerateData() );
  otbTestingCheckValidCommand(          model->SaveResults(resultName) );

  return EXIT_SUCCESS;
}
