/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include <FL/Fl.H>
#include "otbRoadExtractionModel.h"
#include "otbRoadExtractionController.h"
#include "otbRoadExtractionViewGUI.h"

int RoadExtractionViewTestCase1(int argc, char* argv[])
{

  if (argc!=5)
  {
    std::cerr<<"Usage: "<<argv[0]<<" inputName PixelClickedX PixelClickedY outputvectordataName"<<std::endl;
    return EXIT_FAILURE;
  }

  const char * inputName = argv[1];
  const unsigned int PixelClickedX(atoi(argv[2]));
  const unsigned int PixelClickedY(atoi(argv[3]));
  const char * outputvectordataName = argv[4];


  typedef otb::RoadExtractionModel ModelType;
  typedef otb::RoadExtractionController ControllerType;
  typedef otb::RoadExtractionViewGUI ViewType;
  typedef ControllerType::IndexType IndexType;


  ControllerType::Pointer controller = ControllerType::New();

  ViewType::Pointer view = ViewType::New();
  view->SetRoadExtractionController(controller);
  controller->SetView(view);
  ModelType::Pointer model = otb::RoadExtractionModel::GetInstance();

  // Scripting
  Fl::check();

  // Open a filename
  controller->SelectFileName(inputName);
  controller->ChangeChannel(0);
  controller->ChangeRedChannel(1);
  controller->ChangeGreenChannel(2);

  otbTestCallbackWithValue(view, iAlpha, 1.);
  otbTestCallback(view, bUseSpectralAngle);
  otbTestCallback(view, bUseImageIntensity);
  otbTestCallback(view, bUseImageChannel);

  // Simulate a click on the image
  IndexType idPixel;
  idPixel[0] = PixelClickedX;
  idPixel[1] = PixelClickedY;
  controller->PixelClicked(idPixel);

  // Save result
  controller->SaveResults(outputvectordataName);

  otbTestCallback(view, bClear);

  // Import vector data results
  controller->ImportSegments(outputvectordataName);

  return EXIT_SUCCESS;
}
