/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>
#include "otbLevelSetSegmentationController.h"
#include "otbLevelSetSegmentationViewGUI.h"
#include "otbImageFileReader.h"

int otbLevelSetSegmentationControllerAndViewTest(int argc, char* argv[])
{

  if (argc!=9)
  {
    std::cerr<<"Usage: "<<argv[0]<<" fname insidex insidey sigma sigmoidAlpha sigmoidBeta timeThreshold stoppingTime"<<std::endl;
    return EXIT_FAILURE;
  }

  const char * fname = argv[1];
  const unsigned int insidex = atoi(argv[2]);
  const unsigned int insidey = atoi(argv[3]);
  const double    sigma      = atof(argv[4]);
  const double    sigAlpha   = atof(argv[5]);
  const double    sigBeta    = atof(argv[6]);
  const double stoppingTime  = atof(argv[7]);
  const double timeThreshold = atof(argv[8]);


  typedef otb::LevelSetSegmentationModel ModelType;
  typedef ModelType::ImageType ImageType;
  typedef otb::LevelSetSegmentationController ControllerType;
  typedef otb::LevelSetSegmentationViewGUI ViewType;
  typedef otb::ImageFileReader<ImageType> ReaderType;

  ReaderType::Pointer reader = ReaderType::New();
  reader->SetFileName(fname);
  reader->Update();


  ControllerType::Pointer controller = ControllerType::New();

  Fl_Double_Window window(450, 330);
  ViewType::Pointer view = ViewType::New();
  view->SetLevelSetSegmentationController(controller);
  controller->SetView(view);
  ModelType::Pointer model = otb::LevelSetSegmentationModel::GetInstance();
  model->SetInputImage(reader->GetOutput());
  ImageType::SizeType size = view->GetIdealSize();
  view->resize(0, 0, size[0], size[1]);
  window.resize(window.x(), window.y(), size[0], size[1]);
  window.resizable(view);
  window.end();
  window.show();
  view->hide();
  view->show();

  ImageType::IndexType inside;
  inside[0]  = insidex;
  inside[1]  = insidey;


  // Scripting
  Fl::check();

  view->vSigma->value(sigma);
  view->vSigma->do_callback();
  Fl::check();

  view->vSigmoidAlpha->value(sigAlpha);
  view->vSigmoidAlpha->do_callback();
  Fl::check();

  view->vSigmoidBeta->value(sigBeta);
  view->vSigmoidBeta->do_callback();
  Fl::check();

  controller->SeedSelected(inside);
  Fl::check();

  view->vStoppingTime->value(stoppingTime);
  view->vStoppingTime->do_callback();
  Fl::check();

  view->bClearSeeds->do_callback();
  Fl::check();

  controller->SeedSelected(inside);
  Fl::check();

  view->vTimeThreshold->value(timeThreshold);
  view->vTimeThreshold->do_callback();
  Fl::check();

  window.remove(view.GetPointer());

  return EXIT_SUCCESS;
}
