/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include "otbLevelSetSegmentationModel.h"
#include "otbImageFileReader.h"

int otbLevelSetSegmentationModelTest(int argc, char* argv[])
{
  if (argc!=9)
  {
    std::cerr<<"Usage: "<<argv[0]<<" fname insidex insidey sigma sigmoidAlpha sigmoidBeta timeThreshold stoppingTime"<<std::endl;
    return EXIT_FAILURE;
  }

  const char * fname = argv[1];
  const unsigned int insidex = atoi(argv[2]);
  const unsigned int insidey = atoi(argv[3]);
  const double    sigma      = atof(argv[4]);
  const double    sigAlpha   = atof(argv[5]);
  const double    sigBeta    = atof(argv[6]);
  const double timeThreshold = atof(argv[7]);
  const double stoppingTime  = atof(argv[8]);

  typedef otb::LevelSetSegmentationModel ModelType;
  typedef ModelType::ImageType ImageType;
  typedef otb::ImageFileReader<ImageType> ReaderType;
  typedef ImageType::IndexType IndexType;

  ReaderType::Pointer reader = ReaderType::New();
  reader->SetFileName(fname);
  reader->Update();

  // model instance
  ModelType::Pointer model = otb::LevelSetSegmentationModel::GetInstance();
  model->SetInputImage(reader->GetOutput());

  IndexType inside;
  inside[0]  = insidex;
  inside[1]  = insidey;

  //action scripting
  model->SetInputImage(reader->GetOutput());
  model->SetSigma(sigma);
  model->SetSigmoidAlpha(sigAlpha);
  model->SetSigmoidBeta(sigBeta);
  model->AddSeed(inside);
  model->SetTimeThreshold(timeThreshold);
  model->ClearSeeds();
  model->AddSeed(inside);
  model->SetStoppingTime(stoppingTime);

  return EXIT_SUCCESS;
}
