/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>
#include "otbPreprocessingController.h"
#include "otbImageFileReader.h"

int otbPreprocessingControllerAndViewTest(int argc, char* argv[])
{
  if (argc!=5)
  {
    std::cerr<<"Usage: "<<argv[0]<<" fname nbIterations timeStep alpha"<<std::endl;
    return EXIT_FAILURE;
  }

  const char * fname = argv[1];
  const unsigned int nbIterations = atoi(argv[2]);
  const double timeStep = atof(argv[3]);
  const double alpha = atof(argv[4]);

  typedef otb::PreprocessingModel ModelType;
  typedef ModelType::ImageType ImageType;
  typedef otb::PreprocessingController ControllerType;
  typedef otb::PreprocessingViewGUI ViewType;
  typedef otb::ImageFileReader<ImageType> ReaderType;

  ReaderType::Pointer reader = ReaderType::New();
  reader->SetFileName(fname);
  reader->Update();


  ControllerType::Pointer controller = ControllerType::New();

  Fl_Double_Window window(450, 330);
  ViewType::Pointer view = ViewType::New();
  view->SetPreprocessingController(controller);
  controller->SetView(view);
  ModelType::Pointer model = otb::PreprocessingModel::GetInstance();
  model->SetInputImage(reader->GetOutput());
  ImageType::SizeType size = view->GetIdealSize();
  view->resize(0, 0, size[0], size[1]);
  window.resize(window.x(), window.y(), size[0], size[1]);
  window.resizable(view);
  window.end();
  window.show();
  view->hide();
  view->show();

  Fl::check();
  view->vNumberOfIterations->value(nbIterations);
  view->vNumberOfIterations->do_callback();
  Fl::check();
  view->vTimeStep->value(timeStep);
  view->vTimeStep->do_callback();
  Fl::check();
  view->bUseSmoothing->value(0);
  view->bUseSmoothing->do_callback();
  Fl::check();
  view->vAlpha->value(alpha);
  view->vAlpha->do_callback();
  Fl::check();
  view->bUseEdgeEnhancement->value(0);
  view->bUseEdgeEnhancement->do_callback();
  Fl::check();
  view->bUseSmoothing->value(1);
  view->bUseSmoothing->do_callback();
  Fl::check();
  view->bUseEdgeEnhancement->value(1);
  view->bUseEdgeEnhancement->do_callback();
  Fl::check();

  window.remove(view.GetPointer());

  return EXIT_SUCCESS;
}
