/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include "otbPreprocessingModel.h"
#include "otbImageFileReader.h"

int otbPreprocessingModelTest(int argc, char* argv[])
{
  if (argc!=5)
  {
    std::cerr<<"Usage: "<<argv[0]<<" fname nbIterations timeStep alpha"<<std::endl;
    return EXIT_FAILURE;
  }

  const char * fname = argv[1];
  const unsigned int nbIterations = atoi(argv[2]);
  const double timeStep = atof(argv[3]);
  const double alpha = atof(argv[4]);

  typedef otb::PreprocessingModel ModelType;
  typedef ModelType::ImageType ImageType;
  typedef otb::ImageFileReader<ImageType> ReaderType;

  ReaderType::Pointer reader = ReaderType::New();
  reader->SetFileName(fname);
  reader->Update();

  // model instance
  ModelType::Pointer model = otb::PreprocessingModel::GetInstance();
  model->SetInputImage(reader->GetOutput());

  //action scripting
  model->SetInputImage(reader->GetOutput());
  model->SetNumberOfIterations(nbIterations);
  model->SetTimeStep(timeStep);
  model->SetUseSmoothing(false);
  model->SetAlpha(alpha);
  model->SetUseEdgeEnhancement(false);
  model->SetUseSmoothing(true);
  model->SetUseEdgeEnhancement(true);

  return EXIT_SUCCESS;
}
