/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>
#include "otbRegionGrowingSegmentationController.h"
#include "otbRegionGrowingSegmentationViewGUI.h"
#include "otbImageFileReader.h"

int otbRegionGrowingSegmentationControllerAndViewTest(int argc, char* argv[])
{
  if (argc!=11)
  {
    std::cerr<<"Usage: "<<argv[0]<<" fname insidex insidey outsidex outsidey upper lower radius nbIterations tolerance"<<std::endl;
    return EXIT_FAILURE;
  }

  const char * fname = argv[1];
  const unsigned int insidex = atoi(argv[2]);
  const unsigned int insidey = atoi(argv[3]);
  // not used for the moment
//   const unsigned int outsidex = atoi(argv[4]);
//   const unsigned int outsidey = atoi(argv[5]);
  const double upper = atof(argv[6]);
  const double lower = atof(argv[7]);
  const unsigned int radius = atoi(argv[8]);
  const unsigned int nbIterations = atoi(argv[9]);
  const double tolerance = atof(argv[10]);


  typedef otb::RegionGrowingSegmentationModel ModelType;
  typedef ModelType::ImageType ImageType;
  typedef otb::RegionGrowingSegmentationController ControllerType;
  typedef otb::RegionGrowingSegmentationViewGUI ViewType;
  typedef otb::ImageFileReader<ImageType> ReaderType;

  ReaderType::Pointer reader = ReaderType::New();
  reader->SetFileName(fname);
  reader->Update();


  ControllerType::Pointer controller = ControllerType::New();

  Fl_Double_Window window(450, 330);
  ViewType::Pointer view = ViewType::New();
  view->SetRegionGrowingSegmentationController(controller);
  controller->SetView(view);
  ModelType::Pointer model = otb::RegionGrowingSegmentationModel::GetInstance();
  model->SetInputImage(reader->GetOutput());
  ImageType::SizeType size = view->GetIdealSize();
  view->resize(0, 0, size[0], size[1]);
  window.resize(window.x(), window.y(), size[0], size[1]);
  window.resizable(view);
  window.end();
  window.show();
  view->hide();
  view->show();

  ImageType::IndexType inside; //outside;
  inside[0]  = insidex;
  inside[1]  = insidey;
// not used for the moment
//  outside[0] = outsidex;
//   outside[1] = outsidey;

  view->vInsideSeeds->value(1);
  view->vInsideSeeds->do_callback();
  Fl::check();
  controller->SeedSelected(inside);
  Fl::check();
  view->vAlgorithm->value(0);
  view->vAlgorithm->do_callback();
  Fl::check();
  view->vUpperThreshold->value(upper);
  view->vUpperThreshold->do_callback();
  Fl::check();
  view->vLowerThreshold->value(lower);
  view->vLowerThreshold->do_callback();
  Fl::check();
  view->vAlgorithm->value(1);
  view->vAlgorithm->do_callback();
  Fl::check();
  view->vRadius->value(radius);
  view->vRadius->do_callback();
  Fl::check();
  view->vAlgorithm->value(2);
  view->vAlgorithm->do_callback();
  Fl::check();
  view->vNumberOfIterations->value(nbIterations);
  view->vNumberOfIterations->do_callback();
  Fl::check();
  view->vTolerance->value(tolerance);
  view->vTolerance->do_callback();
  Fl::check();

  window.remove(view.GetPointer());

  return EXIT_SUCCESS;
}
