/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include "otbRegionGrowingSegmentationModel.h"
#include "otbImageFileReader.h"

int otbRegionGrowingSegmentationModelTest(int argc, char* argv[])
{
  if (argc!=11)
  {
    std::cerr<<"Usage: "<<argv[0]<<" fname insidex insidey outsidex outsidey upper lower radius nbIterations tolerance"<<std::endl;
    return EXIT_FAILURE;
  }

  const char * fname = argv[1];
  const unsigned int insidex = atoi(argv[2]);
  const unsigned int insidey = atoi(argv[3]);
  const unsigned int outsidex = atoi(argv[4]);
  const unsigned int outsidey = atoi(argv[5]);
  const double upper = atof(argv[6]);
  const double lower = atof(argv[7]);
  const unsigned int radius = atoi(argv[8]);
  const unsigned int nbIterations = atoi(argv[9]);
  const double tolerance = atof(argv[10]);

  typedef otb::RegionGrowingSegmentationModel ModelType;
  typedef ModelType::ImageType ImageType;
  typedef otb::ImageFileReader<ImageType> ReaderType;
  typedef ImageType::IndexType IndexType;

  ReaderType::Pointer reader = ReaderType::New();
  reader->SetFileName(fname);
  reader->Update();

  // model instance
  ModelType::Pointer model = otb::RegionGrowingSegmentationModel::GetInstance();
  model->SetInputImage(reader->GetOutput());

  IndexType inside, outside;
  inside[0]  = insidex;
  inside[1]  = insidey;
  outside[0] = outsidex;
  outside[1] = outsidey;

  //action scripting
  model->SetInputImage(reader->GetOutput());
  model->AddInsideSeed(inside);
  model->AddOutsideSeed(outside);
  model->SetRegionGrowingAlgorithm(otb::THRESHOLD_CONNECTED);
  model->SetUpperThreshold(lower);
  model->SetLowerThreshold(upper);
  model->SetRegionGrowingAlgorithm(otb::NEIGHBORHOOD_CONNECTED);
  model->SetRadius(radius);
  model->SetRegionGrowingAlgorithm(otb::CONFIDENCE_CONNECTED);
  model->SetNumberOfIterations(nbIterations);
  model->SetTolerance(tolerance);

  return EXIT_SUCCESS;
}
