/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>
#include "otbSVMBasedRegionGrowingSegmentationController.h"
#include "otbSVMBasedRegionGrowingSegmentationViewGUI.h"
#include "otbImageFileReader.h"

int otbSVMBasedRegionGrowingSegmentationControllerAndViewTest(int argc, char* argv[])
{
  if (argc!=12)
  {
    std::cerr<<"Usage: "<<argv[0]<<" fname insideStartX insideStartY insideEndX insideEndY outsideStartX outsideStartY outsideEndX outsideEndY nbSpectralAngles nbThresholds "<<std::endl;
    return EXIT_FAILURE;
  }

  const char *       fname         = argv[1];
  const unsigned int insideStartX  = atoi(argv[2]);
  const unsigned int insideStartY  = atoi(argv[3]);
  const unsigned int insideEndX    = atoi(argv[4]);
  const unsigned int insideEndY    = atoi(argv[5]);
  const unsigned int outsideStartX = atoi(argv[6]);
  const unsigned int outsideStartY = atoi(argv[7]);
  const unsigned int outsideEndX   = atoi(argv[8]);
  const unsigned int outsideEndY   = atoi(argv[9]);
  const unsigned int nbSA          = atoi(argv[10]);
  const unsigned int nbThresh      = atoi(argv[11]);

  typedef otb::SVMBasedRegionGrowingSegmentationModel ModelType;
  typedef ModelType::InputMSImageType ImageType;
  typedef otb::SVMBasedRegionGrowingSegmentationController ControllerType;
  typedef otb::SVMBasedRegionGrowingSegmentationViewGUI ViewType;
  typedef otb::ImageFileReader<ImageType> ReaderType;

  ReaderType::Pointer reader = ReaderType::New();
  reader->SetFileName(fname);
  reader->Update();


  ControllerType::Pointer controller = ControllerType::New();

  Fl_Double_Window window(450, 330);
  ViewType::Pointer view = ViewType::New();
  view->SetSVMBasedRegionGrowingSegmentationController(controller);
  controller->SetView(view);
  ModelType::Pointer model = otb::SVMBasedRegionGrowingSegmentationModel::GetInstance();
  model->SetInputImage(reader->GetOutput());
  ImageType::SizeType size = view->GetIdealSize();
  view->resize(0, 0, size[0], size[1]);
  window.resize(window.x(), window.y(), size[0], size[1]);
  window.resizable(view);
  window.end();
  window.show();
  view->hide();
  view->show();

  // Disable automatic update
  view->bAutomaticUpdate->value(0);
  view->bAutomaticUpdate->do_callback();
  Fl::check();

  ControllerType::IndexType start, stop, point;

  // Draw inside line

  view->bInsideSeeds->value(1);
  view->bInsideSeeds->do_callback();
  Fl::check();

  if (insideStartX<insideEndX)
  {
    start[0]=insideStartX;
    start[1]=insideStartY;
    stop[0]=insideEndX;
    stop[1]=insideEndY;
  }
  else
  {
    start[0]=insideEndX;
    start[1]=insideEndY;
    stop[0]=insideStartX;
    stop[1]=insideStartY;
  }
  double scale = static_cast<double>(stop[1]-start[1])/static_cast<double>(stop[0]-start[0]);

  for (unsigned int i = static_cast<unsigned int>(start[0]); i<static_cast<unsigned int>(stop[0]); ++i)
  {
    point[0]=i;
    point[1]=static_cast<unsigned int>(vcl_floor(scale*(i-start[0])+start[1]+0.5));
    std::cout<<"Adding inside point: "<<point<<std::endl;
    controller->PixelDragged(point);
  }

  // Draw outside line

  view->bOutsideSeeds->value(1);
  view->bOutsideSeeds->do_callback();
  Fl::check();

  if (outsideStartX<outsideEndX)
  {
    start[0]=outsideStartX;
    start[1]=outsideStartY;
    stop[0]=outsideEndX;
    stop[1]=outsideEndY;
  }
  else
  {
    start[0]=outsideEndX;
    start[1]=outsideEndY;
    stop[0]=outsideStartX;
    stop[1]=outsideStartY;
  }
  scale = static_cast<double>(stop[1]-start[1])/static_cast<double>(stop[0]-start[0]);
  for (unsigned int i = static_cast<unsigned int>(start[0]); i<static_cast<unsigned int>(stop[0]); ++i)
  {
    point[0]=i;
    point[1]=static_cast<unsigned int>(vcl_floor(scale*(i-start[0])+start[1]+0.5));
    std::cout<<"Adding outside point: "<<point<<std::endl;
    controller->PixelDragged(point);
  }

  view->vNumberOfSads->value(nbSA);
  view->vNumberOfSads->do_callback();
  Fl::check();

  view->vNumberOfThresholds->value(nbThresh);
  view->vNumberOfThresholds->do_callback();
  Fl::check();

  view->bUpdate->do_callback();
  Fl::check();

  for (int i =1; i<=view->vFeatures->size(); ++i)
  {
    view->vFeatures->value(i);
    view->vFeatures->do_callback();
    Fl::check();
  }

  Fl::check();

  window.remove(view.GetPointer());

  return EXIT_SUCCESS;
}
