/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include "otbSVMBasedRegionGrowingSegmentationModel.h"
#include "otbImageFileReader.h"

int otbSVMBasedRegionGrowingSegmentationModelTest(int argc, char* argv[])
{
  if (argc!=12)
  {
    std::cerr<<"Usage: "<<argv[0]<<" fname insideStartX insideStartY insideEndX insideEndY outsideStartX outsideStartY outsideEndX outsideEndY nbSpectralAngles nbThresholds "<<std::endl;
    return EXIT_FAILURE;
  }

  const char *       fname         = argv[1];
  const unsigned int insideStartX  = atoi(argv[2]);
  const unsigned int insideStartY  = atoi(argv[3]);
  const unsigned int insideEndX    = atoi(argv[4]);
  const unsigned int insideEndY    = atoi(argv[5]);
  const unsigned int outsideStartX = atoi(argv[6]);
  const unsigned int outsideStartY = atoi(argv[7]);
  const unsigned int outsideEndX   = atoi(argv[8]);
  const unsigned int outsideEndY   = atoi(argv[9]);
  const unsigned int nbSA          = atoi(argv[10]);
  const unsigned int nbThresh      = atoi(argv[11]);

  typedef otb::SVMBasedRegionGrowingSegmentationModel ModelType;
  typedef ModelType::InputMSImageType ImageType;
  typedef otb::ImageFileReader<ImageType> ReaderType;
  typedef ImageType::IndexType IndexType;

  ReaderType::Pointer reader = ReaderType::New();
  reader->SetFileName(fname);
  reader->Update();

  // model instance
  ModelType::Pointer model = otb::SVMBasedRegionGrowingSegmentationModel::GetInstance();
  model->SetInputImage(reader->GetOutput());

  ModelType::PointType start, stop, point;

  // Draw inside line
  if (insideStartX<insideEndX)
  {
    start[0]=insideStartX;
    start[1]=insideStartY;
    stop[0]=insideEndX;
    stop[1]=insideEndY;
  }
  else
  {
    start[0]=insideEndX;
    start[1]=insideEndY;
    stop[0]=insideStartX;
    stop[1]=insideStartY;
  }
  double scale = static_cast<double>(stop[1]-start[1])/static_cast<double>(stop[0]-start[0]);

  for (unsigned int i = static_cast<unsigned int>(start[0]); i< static_cast<unsigned int>(stop[0]); ++i)
  {
    point[0]=i;
    point[1]=vcl_floor(scale*(i-start[0])+start[1]+0.5);
    model->AddInsidePoint(point);
  }

  // Draw outside line
  if (outsideStartX<outsideEndX)
  {
    start[0]=outsideStartX;
    start[1]=outsideStartY;
    stop[0]=outsideEndX;
    stop[1]=outsideEndY;
  }
  else
  {
    start[0]=outsideEndX;
    start[1]=outsideEndY;
    stop[0]=outsideStartX;
    stop[1]=outsideStartY;
  }
  scale = static_cast<double>(stop[1]-start[1])/static_cast<double>(stop[0]-start[0]);
  for (unsigned int i = static_cast<unsigned int>(start[0]); i<static_cast<unsigned int>(stop[0]); ++i)
  {
    point[0]=i;
    point[1]=vcl_floor(scale*(i-start[0])+start[1]+0.5);
    model->AddOutsidePoint(point);
  }

  model->SetNumberOfSpectralAngleDistanceImages(nbSA);
  model->SetNumberOfThresholds(nbThresh);
  model->Update();

  return EXIT_SUCCESS;
}
