/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>
#include "otbVectorizationController.h"
#include "otbVectorizationViewGUI.h"
#include "otbImageFileReader.h"

int otbVectorizationControllerAndViewTest(int argc, char* argv[])
{
  if (argc!=6)
  {
    std::cerr<<"Usage: "<<argv[0]<<" reffname segfname toleranceStart toleranceEnd toleranceStep"<<std::endl;
    return EXIT_FAILURE;
  }

  const char * reffname       =      argv[1];
  const char * segfname       =      argv[2];
  const double toleranceStart = atof(argv[3]);
  const double toleranceEnd   = atof(argv[4]);
  const double toleranceStep  = atof(argv[5]);

  typedef otb::VectorizationModel ModelType;
  typedef ModelType::ImageType ImageType;
  typedef ModelType::UCharImageType UCharImageType;
  typedef otb::VectorizationController ControllerType;
  typedef otb::VectorizationViewGUI ViewType;

  typedef otb::ImageFileReader<ImageType> ReaderType;
  ReaderType::Pointer reader = ReaderType::New();
  reader->SetFileName(reffname);
  reader->Update();

  typedef otb::ImageFileReader<UCharImageType> UCharReaderType;
  UCharReaderType::Pointer ucharReader = UCharReaderType::New();
  ucharReader->SetFileName(segfname);
  ucharReader->Update();

  ControllerType::Pointer controller = ControllerType::New();

  Fl_Double_Window window(450, 330);
  ViewType::Pointer view = ViewType::New();
  view->SetVectorizationController(controller);
  controller->SetView(view);
  ModelType::Pointer model = otb::VectorizationModel::GetInstance();
  model->SetReferenceImage(reader->GetOutput());
  model->SetSegmentationImage(ucharReader->GetOutput());
  ImageType::SizeType size = view->GetIdealSize();
  view->resize(0, 0, size[0], size[1]);
  window.resize(window.x(), window.y(), size[0], size[1]);
  window.resizable(view);
  window.end();
  window.show();
  view->hide();
  view->show();

  Fl::check();

  for (double tolerance = toleranceStart; tolerance<toleranceEnd; tolerance += toleranceStep)
  {
    view->vTolerance->value(tolerance);
    view->vTolerance->do_callback();
    Fl::check();
  }

  window.remove(view.GetPointer());

  return EXIT_SUCCESS;
}
