/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/
#include "otbVectorizationModel.h"
#include "otbImageFileReader.h"

int otbVectorizationModelTest(int argc, char* argv[])
{
  if (argc!=6)
  {
    std::cerr<<"Usage: "<<argv[0]<<" reffname segfname toleranceStart toleranceEnd toleranceStep"<<std::endl;
    return EXIT_FAILURE;
  }

  const char * reffname       =      argv[1];
  const char * segfname       =      argv[2];
  const double toleranceStart = atof(argv[3]);
  const double toleranceEnd   = atof(argv[4]);
  const double toleranceStep  = atof(argv[5]);

  typedef otb::VectorizationModel ModelType;
  typedef ModelType::ImageType ImageType;
  typedef ModelType::UCharImageType UCharImageType;


  typedef otb::ImageFileReader<ImageType> ReaderType;
  ReaderType::Pointer reader = ReaderType::New();
  reader->SetFileName(reffname);
  reader->Update();

  typedef otb::ImageFileReader<UCharImageType> UCharReaderType;
  UCharReaderType::Pointer ucharReader = UCharReaderType::New();
  ucharReader->SetFileName(segfname);
  ucharReader->Update();

  // model instance
  ModelType::Pointer model = otb::VectorizationModel::GetInstance();
  model->SetReferenceImage(reader->GetOutput());
  model->SetSegmentationImage(ucharReader->GetOutput());

  for (double tolerance = toleranceStart; tolerance<toleranceEnd; tolerance += toleranceStep)
  {
    model->SetTolerance(tolerance);
  }

  return EXIT_SUCCESS;
}
