/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


    This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE,  See the above copyright notices for more information.

=========================================================================*/

#include "itkExceptionObject.h"
#include "otbUrbanAreaExtractionController.h"
#include "otbUrbanAreaExtractionViewGUI.h"
#include "itkIndex.h"


int otbUrbanAreaExtractionRun(int argc, char* argv[])
{
  typedef otb::UrbanAreaExtractionController ControllerType;
  typedef otb::UrbanAreaExtractionViewGUI ViewType;
 
  ControllerType::Pointer controller = ControllerType::New();
  ViewType::Pointer view = ViewType::New();
  controller->SetView(view);
  controller->AddHandlers();
  view->SetUrbanAreaExtractionController(controller);
 
  view->InitWidgets();
  view->Show();


  controller->OpenInputImage(argv[1]);
  Fl::check();
 
  controller->OpenInputVector(argv[2]);
  Fl::check();
  
  itk::Index<> index1, index2;
  index1.Fill(20);
  index2.Fill(50);
  controller->GetFocusingController()->GetFocusingModel()->SetExtractRegionByIndex( index1, index2 );
  controller->GetFocusingController()->GetFocusingModel()->Update();
  Fl::check();

  view->RunCallback();
  Fl::check();


  index1.Fill(200);
  index2.Fill(3000);
  controller->GetFocusingController()->GetFocusingModel()->SetExtractRegionByIndex( index1, index2 );
  controller->GetFocusingController()->GetFocusingModel()->Update();
  Fl::check();


  controller->SetAlpha(0.2);
  controller->SetThreshold(250);
  controller->GetFocusingController()->GetFocusingModel()->Update();

  view->RunCallback();
  Fl::check();

  view->DisplayCallback();
  Fl::check();
  view->DisplayCallback();
  Fl::check();

  view->SwitchScrollCallback();
  Fl::check();
  view->SwitchScrollCallback();
  Fl::check();

  view->ClearROIsCallback();
  Fl::check();

  if(argc==5)
    {
      if( atoi(argv[3])==0)
    {
      controller->SaveVectorOnExtract(argv[4]);
    }
      else
    {
      controller->SaveImageOnExtract(argv[4]);
    }
    }

  return EXIT_SUCCESS;

}
